*DECK DOORAV
      SUBROUTINE DOORAV (CDOOR,IPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRP,NREG,
     1 NBMIX,NANI,MAT,VOL,KNORM,LEAKSW,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* assembly of system matrices. Vectorial version.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* CDOOR   name of the geometry/solution operator.
* IPSYS   pointer to the PIJ LCM object (L_PIJ signature). IPSYS is a
*         list of NGRP directories.
* NPSYS   index array pointing to the IPSYS list component corresponding
*         to each energy group. Set to zero if a group is not to be
*         processed. Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  unit of the sequential binary tracking file.
* IMPX    print flag (equal to zero for no print).
* NGRP    number of energy groups.
* NREG    total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures (NBMIX=max(MAT(i))).
* NANI    number of Legendre orders.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   normalization scheme.
* LEAKSW  leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* TITR    title.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER CDOOR*12,TITR*72
      LOGICAL LEAKSW
      INTEGER IPSYS,NPSYS(NGRP),IPTRK,IFTRAK,IMPX,NGRP,NREG,NBMIX,NANI,
     > MAT(NREG),KNORM
      REAL VOL(NREG)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40)
      INTEGER ISTATE(NSTATE)
      COMMON BASE(1)
*
      IF (CDOOR.EQ.'SYBIL') THEN
         CALL SETARA(BASE,NBMIX+1,ISGAR)
         CALL SETARA(BASE,(NBMIX+1)*NANI,ISGAS)
         DO 100 IGR=1,NGRP
            IOFSET=NPSYS(IGR)
            IF(IOFSET.NE.0) THEN
               KPSYS=LCMGIL(IPSYS,IOFSET)
               CALL LCMLEN(KPSYS,'DRAGON-TXSC',ILONG,ITYLCM)
               IF(ILONG.NE.NBMIX+1) CALL XABORT('DOORAV: INVALID TXSC '
     1         //'LENGTH(1).')
               CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
               CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
               CALL SYBILA(KPSYS,IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,
     1                     MAT,VOL,KNORM,BASE(ISGAR),BASE(ISGAS),
     2                     LEAKSW,TITR)
            ENDIF
 100     CONTINUE
         CALL RLSARA(BASE(ISGAS))
         CALL RLSARA(BASE(ISGAR))
      ELSE IF (CDOOR.EQ.'SN') THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
         IF(ISTATE(19).EQ.1) THEN
*           SYNTHETIC ACCELERATION.
            IF(IMPX.GT.0) WRITE (6,'(/29H DOORAV: SYNTHETIC ACCELERATI,
     1      20HON ASSEMBLY FOLLOWS:)')
            CALL LCMSIX(IPTRK,'DSA',1)
            CALL SETARA(BASE,NBMIX+1,ISGAR)
            CALL SETARA(BASE,(NBMIX+1)*NANI,ISGAS)
            CALL SETARA(BASE,NBMIX+1,ISGAD)
            DO 150 IGR=1,NGRP
               IOFSET=NPSYS(IGR)
               IF(IOFSET.NE.0) THEN
                  KPSYS=LCMGIL(IPSYS,IOFSET)
                  CALL LCMLEN(KPSYS,'DRAGON-TXSC',ILONG,ITYLCM)
                  IF(ILONG.NE.NBMIX+1) CALL XABORT('DOORAV: INVALID TX'
     1            //'SC LENGTH(2).')
                  CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
                  CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
                  CALL BIVAA(KPSYS,IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,
     1                       MAT,VOL,KNORM,BASE(ISGAR),BASE(ISGAS),
     2                       BASE(ISGAD),LEAKSW,TITR)
               ENDIF
 150        CONTINUE
            CALL RLSARA(BASE(ISGAD))
            CALL RLSARA(BASE(ISGAS))
            CALL RLSARA(BASE(ISGAR))
            CALL LCMSIX(IPTRK,' ',2)
         ENDIF
      ELSE IF (CDOOR.EQ.'BIVAC') THEN
         CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
         NLF=ISTATE(14)
         CALL SETARA(BASE,NBMIX+1,ISGAR)
         CALL SETARA(BASE,(NBMIX+1)*NANI,ISGAS)
         CALL SETARA(BASE,NBMIX+1,ISGAD)
         DO 200 IGR=1,NGRP
            IOFSET=NPSYS(IGR)
            IF(IOFSET.NE.0) THEN
               KPSYS=LCMGIL(IPSYS,IOFSET)
               CALL LCMLEN(KPSYS,'DRAGON-TXSC',ILONG,ITYLCM)
               IF(ILONG.NE.NBMIX+1) CALL XABORT('DOORAV: INVALID TXSC '
     1         //'LENGTH(3).')
               CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
               CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
	       IF(NLF.EQ.0) CALL LCMGET(KPSYS,'DRAGON-DIFF',BASE(ISGAD))
               CALL BIVAA(KPSYS,IPTRK,IFTRAK,IMPX,NREG,NBMIX,NANI,MAT,
     1                    VOL,KNORM,BASE(ISGAR),BASE(ISGAS),BASE(ISGAD),
     2                    LEAKSW,TITR)
            ENDIF
 200     CONTINUE
         CALL RLSARA(BASE(ISGAD))
         CALL RLSARA(BASE(ISGAS))
         CALL RLSARA(BASE(ISGAR))
      ELSE IF ((CDOOR.EQ.'MCCG').OR.(CDOOR.EQ.'EXCELL')) THEN
         CALL MCCGA(IPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRP,NREG,NBMIX,
     1              NANI,MAT,VOL,KNORM,LEAKSW,TITR)
      ELSE
         CALL XABORT('DOORAV: UNKNOWN DOOR:'//CDOOR//'.')
      ENDIF
      RETURN
      END
