*DECK EDIHFC
      SUBROUTINE EDIHFC(IPEDIT,IPLIB,NGROUP,NGCOND,NREGIO,NMERGE,NBISO,
     >                  NDEPL,NDFI,NDFP,NREAC,MATCOD,VOLUME,INADPL,
     >                  ISONAM,ISONRF,MIX,FLUXES,SPH,DEN,IDEPL,CURNAM,
     >                  IGCOND,IMERGE,KDRI,RER,RRD,FIYI,EMEVF2,EMEVG2,
     >                  DECAY,YIELD,PIFI,PYIELD,SIG,HFACT,FLXMER,WORK,
     >                  FIRA,PFIRA,INDX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* evaluate H-factors using information recovered from the reference
* internal library and store them on LCM.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): G. Marleau
*
*Parameters: input
* IPEDIT  pointer to the edition LCM object.
* IPLIB   pointer to the reference internal library LCM object.
* NGROUP  number of groups.
* NGCOND  number of groups condensed.
* NREGIO  number of regions.
* NMERGE  number of regions merged.
* NBISO   number of isotopes.
* NDEPL   number of depleting isotopes.
* NDFI    number of direct fissile isotopes.
* NDFP    number of direct fission products.
* NREAC   number of depletion reactions.
* MATCOD  material per region.
* VOLUME  volume of region.
* INADPL  name of depleting isotopes.
* ISONAM  isotopes names.
* ISONRF  library name of isotopes.
* MIX     mixture associated to isotopes.
* FLUXES  multigroup fluxes.
* SPH     SPH equivalence factors.
* DEN     isotope density.
* IDEPL   non depleting flag (=1 to stop depletion).
* CURNAM  name of LCM directory where the merged/condensed cross
*         sections are stored.
* IGCOND  limit condensed groups.
* IMERGE  index of merged region.
* KDRI    depletion identifiers.
* RER     fission and capture production energy (MeV/reaction).
* RRD     radioactive decay constants.
* FIYI    fission yields.
*
*Parameters: output
* EMEVF2  fission production energy by isotope.
* EMEVG2  capture production energy by isotope.
* DECAY   radioactive decay constants.
* YIELD   condensed fission product yield  (group ordered).
* PIFI    fissile isotope index in IPLIB.
* PYIELD  condensed fission product yield (fissile isotope ordered).
*
*Parameters: scratch
* SIG     fission/capture cross sections.
* HFACT   H-factor in a macrogroup.
* FLXMER  merged and condensed flux.
* WORK    H-factors.
* FIRA    fission rates.
* PFIRA   fission rates.
* INDX    depleting isotope index.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER     IPEDIT,IPLIB,NGROUP,NGCOND,NREGIO,NMERGE,NBISO,
     >            NDEPL,NDFI,NDFP,NREAC,MATCOD(NREGIO),INADPL(3,NDEPL),
     >            ISONAM(3,NBISO),ISONRF(3,NBISO),MIX(NBISO),
     >            IDEPL(NBISO),IGCOND(NGCOND),IMERGE(NREGIO),
     >            KDRI(NREAC,NDEPL),PIFI(NDFI,NMERGE),INDX(NBISO)
      REAL        VOLUME(NREGIO),FLUXES(NREGIO,NGROUP),DEN(NBISO),
     >            SPH(NMERGE,NGCOND),RER(NREAC,NDEPL),RRD(NDEPL),
     >            FIYI(NDFI,NDFP),SIG(NGROUP),HFACT(NMERGE),
     >            EMEVF2(NBISO),EMEVG2(NBISO),DECAY(NBISO),
     >            YIELD(NGCOND+1,NBISO,NMERGE),FIRA(NGCOND+1,NMERGE),
     >            PYIELD(NDFI,NBISO,NMERGE),PFIRA(NDFI,NMERGE)
      DOUBLE PRECISION FLXMER(NMERGE,NGCOND),WORK(NMERGE,NGCOND)
      CHARACTER   CURNAM*12
*----
*  LOCAL VARIABLES
*----
      INTEGER     IGAR(3)
      CHARACTER   HNISOR*12,HNAMIS*12,TEXT12*12
      LOGICAL     L1,L2
      DOUBLE PRECISION GAR
*----
*  COMPUTE H-FACTOR
*----
      IZFISS=0
      DO 20 IGC=1,NGCOND
        DO 10 IMR=1,NMERGE
          WORK(IMR,IGC)=0.0D0
          FLXMER(IMR,IGC)=0.0D0
  10    CONTINUE
  20  CONTINUE
      DO 30 IMR=1,NMERGE
      DO 25 IFI=1,NDFI
        PIFI(IFI,IMR)=0
        PFIRA(IFI,IMR)=0.0
        DO 25 IBISO=1,NBISO
          PYIELD(IFI,IBISO,IMR)=0.0
  25  CONTINUE
      DO 30 IGC=1,NGCOND+1
        FIRA(IGC,IMR)=0.0
        DO 30 IBISO=1,NBISO
          YIELD(IGC,IBISO,IMR)=0.0
  30  CONTINUE
*----
*  COMPUTE THE DEPLETING ISOTOPE INDEX
*----
      DO 50 ISO=1,NBISO
        IF(IDEPL(ISO).NE.1) THEN
          WRITE(HNISOR,'(3A4)') (ISONRF(I0,ISO),I0=1,3)
          I1=INDEX(HNISOR,'_')
          IF(I1.EQ.0) THEN
            TEXT12=HNISOR
          ELSE
            TEXT12=HNISOR(:I1-1)
          ENDIF
          READ(TEXT12,'(3A4)') (IGAR(I0),I0=1,3)
          DO 40 IDP=1,NDEPL
            L1=((ISONRF(1,ISO).EQ.INADPL(1,IDP)).AND.
     1          (ISONRF(2,ISO).EQ.INADPL(2,IDP)).AND.
     2          (ISONRF(3,ISO).EQ.INADPL(3,IDP)))
            L2=((IGAR(1).EQ.INADPL(1,IDP)).AND.
     1          (IGAR(2).EQ.INADPL(2,IDP)).AND.
     2          (IGAR(3).EQ.INADPL(3,IDP)))
            IF(L1.OR.L2) THEN
              INDX(ISO)=IDP
              GO TO 50
            ENDIF
  40      CONTINUE
        ENDIF
        INDX(ISO)=0
  50  CONTINUE
*----
*  MAIN ISOTOPIC LOOP
*----
      DO 160 ISO=1,NBISO
        IDPL=INDX(ISO)
        IF(IDPL.EQ.0) GO TO 160
        WRITE(HNAMIS,'(3A4)') (ISONAM(I0,ISO),I0=1,3)
        CALL LCMSIX(IPLIB,HNAMIS,1)
*----
*  SET RADIOACTIVE DECAY CONSTANT
*----
        DECAY(ISO)=RRD(IDPL)
*----
*  COMPUTE CONDENSED FISSION RATES.
*----
        IF(MOD(KDRI(2,IDPL),100).EQ.4) THEN
          IFI=KDRI(2,IDPL)/100
          CALL LCMGET(IPLIB,'NUSIGF',SIG)
          DO 90 IREG=1,NREGIO
            IMR=IMERGE(IREG)
            IF((IMR.GT.0).AND.(MATCOD(IREG).EQ.MIX(ISO))) THEN
              PIFI(IFI,IMR)=ISO
              IGRFIN=0
              DO 80 IGC=1,NGCOND
                IGRDEB=IGRFIN+1
                IGRFIN=IGCOND(IGC)
                GAR=0.0D0
                DO 60 IGR=IGRDEB,IGRFIN
                  GAR=GAR+FLUXES(IREG,IGR)*DEN(ISO)*VOLUME(IREG)*
     >            SIG(IGR)
  60            CONTINUE
                DO 70 JSO=1,NBISO
                  JDPL=INDX(JSO)
                  IF(JDPL.EQ.0) GO TO 70
                  IF(MOD(KDRI(2,JDPL),100).EQ.5) THEN
                    ISOFP=KDRI(2,JDPL)/100
                    DELTA=GAR*FIYI(IFI,ISOFP)
                    YIELD(1,JSO,IMR)=YIELD(1,JSO,IMR)+DELTA
                    YIELD(IGC+1,JSO,IMR)=YIELD(IGC+1,JSO,IMR)+DELTA
                    PYIELD(IFI,JSO,IMR)=FIYI(IFI,ISOFP)
                  ENDIF
  70            CONTINUE
                FIRA(1,IMR)=FIRA(1,IMR)+GAR
                FIRA(IGC+1,IMR)=FIRA(IGC+1,IMR)+GAR
                PFIRA(IFI,IMR)=PFIRA(IFI,IMR)+GAR
  80          CONTINUE
            ENDIF
  90      CONTINUE
        ENDIF
*----
*  COMPUTE FISSION ENERGY
*----
        EMEVF2(ISO)=0.0
        CALL LCMLEN(IPLIB,'NFTOT',ILLCM,ITLCM)
        IF(ILLCM.EQ.NGROUP) THEN
          EMEVF2(ISO)=RER(2,IDPL)
          IZFISS=IZFISS+1
          CALL LCMGET(IPLIB,'NFTOT',SIG)
          DO 120 IREG=1,NREGIO
            IMR=IMERGE(IREG)
            IF((IMR.GT.0).AND.(MATCOD(IREG).EQ.MIX(ISO))) THEN
              IGRFIN=0
              DO 110 IGC=1,NGCOND
                IGRDEB=IGRFIN+1
                IGRFIN=IGCOND(IGC)
                GAR=0.0D0
                DO 100 IGR=IGRDEB,IGRFIN
                  GAR=GAR+FLUXES(IREG,IGR)*DEN(ISO)*VOLUME(IREG)*
     >            SIG(IGR)
 100            CONTINUE
                WORK(IMR,IGC)=WORK(IMR,IGC)+GAR*RER(2,IDPL)
 110          CONTINUE
            ENDIF
 120      CONTINUE
        ENDIF
*----
*  COMPUTE CAPTURE ENERGY
*----
        EMEVG2(ISO)=0.0
        CALL LCMLEN(IPLIB,'NG',ILLCM,ITLCM)
        IF(ILLCM.EQ.NGROUP) THEN
          EMEVG2(ISO)=RER(3,IDPL)
          IZFISS=IZFISS+1
          CALL LCMGET(IPLIB,'NG',SIG)
          DO 150 IREG=1,NREGIO
            IMR=IMERGE(IREG)
            IF((IMR.GT.0).AND.(MATCOD(IREG).EQ.MIX(ISO))) THEN
              IGRFIN=0
              DO 140 IGC=1,NGCOND
                IGRDEB=IGRFIN+1
                IGRFIN=IGCOND(IGC)
                GAR=0.0D0
                DO 130 IGR=IGRDEB,IGRFIN
                  GAR=GAR+FLUXES(IREG,IGR)*DEN(ISO)*VOLUME(IREG)*
     >            SIG(IGR)
 130            CONTINUE
                WORK(IMR,IGC)=WORK(IMR,IGC)+GAR*RER(3,IDPL)
 140          CONTINUE
            ENDIF
 150      CONTINUE
        ENDIF
*
        CALL LCMSIX(IPLIB,' ',2)
 160  CONTINUE
*----
*  COMPUTE THE HOMOGENIZED/CONDENSED FLUX
*----
      IF(IZFISS.NE.0) THEN
        DO 190 IREG=1,NREGIO
          IMR=IMERGE(IREG)
          IF(IMR.GT.0) THEN
            IGRFIN=0
            DO 180 IGC=1,NGCOND
              IGRDEB=IGRFIN+1
              IGRFIN=IGCOND(IGC)
              GAR=0.0D0
              DO 170 IGR=IGRDEB,IGRFIN
                GAR=GAR+FLUXES(IREG,IGR)*VOLUME(IREG)
 170          CONTINUE
              FLXMER(IMR,IGC)=FLXMER(IMR,IGC)+GAR
 180        CONTINUE
          ENDIF
 190    CONTINUE
        DO 210 IGC=1,NGCOND
          DO 200 IMR=1,NMERGE
            IF(FLXMER(IMR,IGC).GT.0.0) THEN
              WORK(IMR,IGC)=WORK(IMR,IGC)*SPH(IMR,IGC)/FLXMER(IMR,IGC)
            ENDIF
 200      CONTINUE
 210    CONTINUE
      ENDIF
*----
*  SAVE ON LCM
*----
      CALL LCMSIX(IPEDIT,CURNAM,1)
      CALL LCMSIX(IPEDIT,'MACROLIB',1)
      JPEDIT=LCMLID(IPEDIT,'GROUP',NGCOND)
      DO 230 IGC=1,NGCOND
        DO 220 IMR=1,NMERGE
        HFACT(IMR)=WORK(IMR,IGC)
 220    CONTINUE
        KPEDIT=LCMDIL(JPEDIT,IGC)
        CALL LCMPUT(KPEDIT,'H-FACTOR',NMERGE,2,HFACT)
 230  CONTINUE
      CALL LCMSIX(IPEDIT,' ',2)
      CALL LCMSIX(IPEDIT,' ',2)
*
      DO 280 IMR=1,NMERGE
        DO 250 IGC=1,NGCOND+1
          IF(FIRA(IGC,IMR).NE.0.0) THEN
            DO 240 ISO=1,NBISO
 240        YIELD(IGC,ISO,IMR)=YIELD(IGC,ISO,IMR)/FIRA(IGC,IMR)
          ENDIF
 250    CONTINUE
 280  CONTINUE
      RETURN
      END
