*DECK EVO
      SUBROUTINE EVO(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for an isotopic depletion calculation.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): A. Hebert
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): creation or modification type(L_BURNUP);
*         HENTRY(2): modification type(L_LIBRARY);
*         HENTRY(3): read-only type(L_TRACK);
*         HENTRY(4): optional read-only type(L_FLUX)
*         or
*         HENTRY(1): creation or modification type(L_BURNUP);
*         HENTRY(2): creation type(L_LIBRARY);
*         HENTRY(3): read-only type(L_LIBRARY);
*         HENTRY(4): read-only type(L_TRACK);
*         HENTRY(5): optional read-only type(L_FLUX).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY                  	
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER HSMG*131,TEXT12*12,HSIGN*12,TEXT4*4
      INTEGER IPAR(NSTATE),KCHAR(3),IGP(NSTATE),IBASE(1)
      REAL RPAR(5),XT(2)
      DOUBLE PRECISION DFLOTT
      LOGICAL LOG,LMACRO
      COMMON RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.LE.2) CALL XABORT('EVO: THREE PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('EVO: LCM O'
     1 //'BJECT EXPECTED AT FIRST LHS.')
      IF(JENTRY(1).EQ.2) CALL XABORT('EVO: BURNUP HISTORY STORAGE IN CR'
     1 //'EATION OR MODIFICATION MODE EXPECTED.')
      IPDEPL=KENTRY(1)
      IF((IENTRY(2).NE.1).AND.(IENTRY(2).NE.2)) CALL XABORT('EVO: LCM O'
     1 //'BJECT EXPECTED AT SECOND LHS.')
      IF(JENTRY(2).EQ.0) THEN
*        INTERNAL LIBRARY CREATION. COPY THE FIRST RHS ON THIS LHS.
         IF(NENTRY.LE.3) CALL XABORT('EVO: FOUR PARAMETERS EXPECTED.')
         IF((JENTRY(3).NE.2).OR.((IENTRY(3).NE.1).AND.
     1   (IENTRY(3).NE.2)))  CALL XABORT('EVO: LCM OBJECT IN READ-ONLY '
     2   //'MODE EXPECTED AT FIRST RHS.')
         CALL LCMEQU(KENTRY(3),KENTRY(2))
         IENT=4
      ELSE IF(JENTRY(2).EQ.1) THEN
*        INTERNAL LIBRARY MODIFICATION.
         IENT=3
      ELSE
         CALL XABORT('EVO: INTERNAL LIBRARY IN CREATE OR MODIFICATION M'
     1   //'ODE EXPECTED.')
      ENDIF
      IPLIB=KENTRY(2)
      TEXT12=HENTRY(2)
      READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPDEPL,'LINK.LIB',3,3,KCHAR)
*----
*  RECOVER IPTRK AND IPFLUX POINTERS
*----
      IPFLUX=0
      IPTRK=0
      DO 10 I=IENT,NENTRY
      IF((JENTRY(I).EQ.2).AND.((IENTRY(I).EQ.1).OR.(IENTRY(I).EQ.2)))
     1 THEN
         CALL LCMGET(KENTRY(I),'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.EQ.'L_FLUX') THEN
            IPFLUX=KENTRY(I)
         ELSE IF(HSIGN.EQ.'L_TRACK') THEN
            IPTRK=KENTRY(I)
         ELSE
            WRITE(HSMG,'(35HEVO: UNEXPECTED SIGNATURE AT RHS ('',A12,
     1      3H'').)') HSIGN
            CALL XABORT(HSMG)
         ENDIF
      ELSE
         CALL XABORT('EVO: LCM OBJECT IN READ-ONLY MODE EXPECTED AT '
     1   //'RHS.')
      ENDIF
   10 CONTINUE
      IF(IPTRK.EQ.0) CALL XABORT('EVO: UNABLE TO FIND A POINTER TO A TR'
     1 //'ACKING LCM OBJECT.')
*----
*  RECOVER GENERAL TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IGP)
      NREG=IGP(1)
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(RBASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',RBASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
*----
*  RECOVER GENERAL INTERNAL LIBRARY INFORMATION
*----
      CALL LCMGET(IPLIB,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
      IF(HSIGN.NE.'L_LIBRARY') THEN
         TEXT12=HENTRY(2)
         CALL XABORT('EVO: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1   '. L_LIBRARY EXPECTED.')
      ENDIF
      CALL LCMGET(IPLIB,'STATE-VECTOR',IPAR)
      NBMIX=IPAR(1)
      NBISO=IPAR(2)
      NGRO=IPAR(3)
      NDEPL=IPAR(11)
      NCOMB=IPAR(12)
      LMACRO=(IPAR(17).GE.0).AND.(IPAR(18).EQ.1)
      IF(IGP(4).GT.NBMIX) THEN
         WRITE(HSMG,'(43HEVO: THE NUMBER OF MIXTURES IN THE TRACKING,
     1   2H (,I5,51H) IS GREATER THAN THE NUMBER OF MIXTURES IN THE INT,
     2   15HERNAL LIBRARY (,I5,2H).)') IGP(4),NBMIX
         CALL XABORT(HSMG)
      ENDIF
      CALL LCMSIX(IPLIB,'DEPL-CHAIN',1)
         CALL LCMGET(IPLIB,'STATE-VECTOR',IPAR)
         IF(IPAR(1).NE.NDEPL) CALL XABORT('EVO: INVALID NUMBER OF D'
     1   //'EPLETING ISOTOPES.')
         NSUPS=IPAR(7)
         NREAC=IPAR(8)
      CALL LCMSIX(IPLIB,' ',2)
      CALL SETARA(IBASE,NBISO,JJMIX)
      CALL SETARA(RBASE,NBISO,JJDEN)
      CALL SETARA(RBASE,NBISO,JIEVOL)
      CALL LCMGET(IPLIB,'ISOTOPESMIX',IBASE(JJMIX))
      CALL LCMGET(IPLIB,'ISOTOPESDENS',RBASE(JJDEN))
      CALL LCMGET(IPLIB,'ISOTOPESTODO',IBASE(JIEVOL))
*----
*  RECOVER LOCAL ISOTOPES NAMES
*----
      CALL SETARA(IBASE,3*NBISO,IISONA)
      CALL SETARA(IBASE,3*NBISO,IISONR)
      CALL LCMGET(IPLIB,'ISOTOPESUSED',IBASE(IISONA))
      CALL LCMGET(IPLIB,'ISOTOPERNAME',IBASE(IISONR))
*----
*  RECOVER THE AVERAGE NEUTRON FLUXES IN THE L_FLUX SPECIFICATION
*----
      CALL SETARA(RBASE,NREG*NGRO,JFLUXE)
      IF(IPFLUX.NE.0) THEN
         JPFLUX=LCMGID(IPFLUX,'FLUX')
         CALL LCMLEL(JPFLUX,1,ILONG,ITYLCM)
         CALL SETARA(RBASE,ILONG,IFUNKN)
         DO 40 IGR=1,NGRO
         CALL LCMGDL(JPFLUX,IGR,RBASE(IFUNKN))
         DO 40 IREG=1,NREG
         KEYFLX=IBASE(IIDL+IREG-1)
   40    RBASE(JFLUXE+(IGR-1)*NREG+IREG-1)=RBASE(IFUNKN+KEYFLX-1)
         CALL RLSARA(RBASE(IFUNKN))
      ELSE
         CALL XDRSET(RBASE(JFLUXE),NREG*NGRO,0.0)
      ENDIF
*----
*  READ THE INPUT DATA
*----
*     DEFAULT OPTIONS:
      IMPX=1
      ISAVE=0
      IDEPL=0
      ISET=0
      INR=-1
      IF(JENTRY(1).EQ.0) THEN
         INDREC=1
         EPS1=1.0E-5
         EPS2=1.0E-4
         EXPMAX=80.0
         H1=1.0E-4
         ITYPE=2
         IEXTR=1
         IGLOB=0
         ISAT=0
         IDIRAC=0
         HSIGN='L_BURNUP'
         READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(IPDEPL,'SIGNATURE',3,3,KCHAR)
         CALL LCMPUT(IPDEPL,'ISOTOPESMIX',NBISO,1,IBASE(JJMIX))
         CALL LCMPUT(IPDEPL,'ISOTOPESUSED',3*NBISO,3,IBASE(IISONA))
      ELSE IF(JENTRY(1).EQ.1) THEN
         CALL LCMGET(IPDEPL,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.NE.'L_BURNUP') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('LIB: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_BURNUP EXPECTED.')
         ENDIF
         INDREC=2
         CALL LCMGET(IPDEPL,'STATE-VECTOR',IPAR)
         ITYPE=IPAR(1)
         IEXTR=IPAR(9)
         IGLOB=IPAR(10)
         ISAT=IPAR(11)
         IDIRAC=IPAR(12)
         IF(IPAR(4).NE.NBISO) CALL XABORT('EVO: INVALID NUMBER OF ISOTO'
     1   //'PES.')
         IF(IPAR(5).NE.NCOMB) CALL XABORT('EVO: INVALID NUMBER OF DEPLE'
     1   //'TING MIXTURES.')
         IF(IPAR(6).NE.NREAC) CALL XABORT('EVO: INVALID NUMBER OF DEPLE'
     1   //'TING REACTIONS.')
         IF(IPAR(7).NE.NDEPL-NSUPS) CALL XABORT('EVO: INVALID NUMBER OF'
     1   //' DEPLETING ISOTOPES.')
         IF(IPAR(8).NE.NBMIX) CALL XABORT('EVO: INVALID NUMBER OF MIXTU'
     1   //'RES.')
         CALL LCMGET(IPDEPL,'EVOLUTION-R',RPAR)
         EPS1=RPAR(1)
         EPS2=RPAR(2)
         EXPMAX=RPAR(3)
         H1=RPAR(4)
         FIT=RPAR(5)
      ENDIF
*----
*  READ LIBRARY DATA
*----
   50 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.EQ.10) GO TO 100
      IF(INDIC.NE.3) CALL XABORT('EVO: CHARACTER DATA EXPECTED.')
      IF(TEXT4.EQ.'EDIT') THEN
*        READ THE PRINT INDEX.
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('EVO: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'EPS1') THEN
         CALL REDGET(INDIC,NITMA,EPS1,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'EPS2') THEN
         CALL REDGET(INDIC,NITMA,EPS2,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'EXPM') THEN
         CALL REDGET(INDIC,NITMA,EXPMAX,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'H1') THEN
         CALL REDGET(INDIC,NITMA,H1,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
      ELSE IF(TEXT4.EQ.'RUNG') THEN
         ITYPE=1
      ELSE IF(TEXT4.EQ.'KAPS') THEN
         ITYPE=2
      ELSE IF(TEXT4.EQ.'NOEX') THEN
         IEXTR=0
      ELSE IF(TEXT4.EQ.'EXTR') THEN
         IEXTR=1
      ELSE IF(TEXT4.EQ.'NOGL') THEN
         IGLOB=0
      ELSE IF(TEXT4.EQ.'GLOB') THEN
         IGLOB=1
      ELSE IF(TEXT4.EQ.'SAT') THEN
         ISAT=1
      ELSE IF(TEXT4.EQ.'NSAT') THEN
         ISAT=0
      ELSE IF(TEXT4.EQ.'DIRA') THEN
         IDIRAC=1
      ELSE IF(TEXT4.EQ.'NODI') THEN
         IDIRAC=0
      ELSE IF(TEXT4.EQ.'NOSA') THEN
         ISAVE=-1
      ELSE IF(TEXT4.EQ.'SAVE') THEN
         ISAVE=1
*        SAVE THE LAST FLUX CALCULATION IN THE DEPLETION TABLE.
         CALL REDGET(INDIC,NITMA,XTI,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('EVO: CHARACTER DATA EXPECTED.')
         IF(TEXT4.EQ.'S') THEN
           XTI=XTI*1.0E-8
         ELSE IF(TEXT4.EQ.'DAY') THEN
           XTI=XTI*8.64E-4
         ELSE IF(TEXT4.EQ.'YEAR') THEN
           XTI=XTI*3.1536E-1
         ELSE
           CALL XABORT('EVO: S, DAY OR YEAR EXPECTED.')
         ENDIF
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('EVO: CHARACTER DATA EXPECTED.')
         IF(TEXT4.EQ.'FLUX') THEN
           INR=1
           CALL REDGET(INDIC,NITMA,FIT,TEXT4,DFLOTT)
           IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
           IF(FIT.LT.1.E5) CALL XABORT('EVO: STRANGE FIT FACTOR.')
         ELSE IF(TEXT4.EQ.'POWR') THEN
           INR=2
           CALL REDGET(INDIC,NITMA,FIT,TEXT4,DFLOTT)
           IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
           IF(FIT.GE.1.E5) CALL XABORT('EVO: STRANGE FIT FACTOR.')
         ELSE IF(TEXT4.EQ.'W/CC') THEN
           INR=3
           CALL REDGET(INDIC,NITMA,FIT,TEXT4,DFLOTT)
           IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
           IF(FIT.GE.1.E5) CALL XABORT('EVO: STRANGE FIT FACTOR.')
         ELSE
           CALL XABORT('EVO: FLUX, POWR OR W/CC EXPECTED.')
         ENDIF
      ELSE IF(TEXT4.EQ.'DEPL') THEN
         IDEPL=1
         CALL REDGET(INDIC,NITMA,XT(1),TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
         CALL REDGET(INDIC,NITMA,XT(2),TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('EVO: CHARACTER DATA EXPECTED.')
         IF(TEXT4.EQ.'S') THEN
           XT(1)=XT(1)*1.0E-8
           XT(2)=XT(2)*1.0E-8
         ELSE IF(TEXT4.EQ.'DAY') THEN
           XT(1)=XT(1)*8.64E-4
           XT(2)=XT(2)*8.64E-4
         ELSE IF(TEXT4.EQ.'YEAR') THEN
           XT(1)=XT(1)*3.1536E-1
           XT(2)=XT(2)*3.1536E-1
         ELSE
           CALL XABORT('EVO: S, DAY OR YEAR EXPECTED.')
         ENDIF
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('EVO: CHARACTER DATA EXPECTED.')
         IF(TEXT4.EQ.'COOL') THEN
           INR=0
           FIT=0.0
         ELSE IF(TEXT4.EQ.'FLUX') THEN
           INR=1
           CALL REDGET(INDIC,NITMA,FIT,TEXT4,DFLOTT)
           IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
           IF(FIT.EQ.0.0) THEN
             INR=0
           ELSE IF(FIT.LT.1.E5) THEN
             CALL XABORT('EVO: STRANGE FIT FACTOR.')
           ENDIF
         ELSE IF(TEXT4.EQ.'POWR') THEN
           INR=2
           CALL REDGET(INDIC,NITMA,FIT,TEXT4,DFLOTT)
           IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
           IF(FIT.EQ.0.0) THEN
             INR=0
           ELSE IF(FIT.GE.1.E5) THEN
             CALL XABORT('EVO: STRANGE FIT FACTOR.')
           ENDIF
         ELSE IF(TEXT4.EQ.'W/CC') THEN
           INR=3
           CALL REDGET(INDIC,NITMA,FIT,TEXT4,DFLOTT)
           IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
           IF(FIT.EQ.0.0) THEN
             INR=0
           ELSE IF(FIT.GE.1.E5) THEN
             CALL XABORT('EVO: STRANGE FIT FACTOR.')
           ENDIF
         ELSE
           CALL XABORT('EVO: COOL, FLUX, POWR, W/CC OR COOL EXPECTED.')
         ENDIF
         IF(XT(2).LE.XT(1)) CALL XABORT('EVO: FINAL TIME IS LESS OR EQU'
     1   //'AL TO THE INITIAL TIME.')
      ELSE IF(TEXT4.EQ.'SET') THEN
         ISET=1
         CALL REDGET(INDIC,NITMA,XTF,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('EVO: REAL DATA EXPECTED.')
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('EVO: CHARACTER DATA EXPECTED.')
         IF(TEXT4.EQ.'S') THEN
           XTF=XTF*1.0E-8
         ELSE IF(TEXT4.EQ.'DAY') THEN
           XTF=XTF*8.64E-4
         ELSE IF(TEXT4.EQ.'YEAR') THEN
           XTF=XTF*3.1536E-1
         ELSE
           CALL XABORT('EVO: S, DAY OR YEAR EXPECTED.')
         ENDIF
      ELSE IF(TEXT4.EQ.';') THEN
         GO TO 100
      ELSE
         CALL XABORT('EVO: '//TEXT4//' IS AN INVALID KEY-WORD.')
      ENDIF
      GO TO 50
*
  100 IF((ISET.EQ.0).AND.(IDEPL.EQ.0)) ISET=-1
      IF(ISAVE.EQ.0) XTI=XT(1)
      IF(ISET.EQ.0) XTF=XT(2)
      IF(NBMIX.EQ.0) CALL XABORT('EVO: NBMIX NOT YET DEFINED.')
      IF(NREG.EQ.0) CALL XABORT('EVO: NREG = 0')
      IF(NDEPL.EQ.0) CALL XABORT('EVO: NO DEPLETING ISOTOPES.')
      IF(NCOMB.EQ.0) CALL XABORT('EVO: NO DEPLETING MIXTURES.')
      LOG=(ISAVE.GE.0).OR.(IDEPL.GT.0)
      IF(LOG.AND.(INR.EQ.-1)) CALL XABORT('EVO: TYPE OF DEPLETION NO'//
     1 'T DEFINED.')
      CALL SETARA(IBASE,NBMIX,LMASKM)
      CALL SETARA(IBASE,NGRO,LMASKG)
      CALL SETARA(IBASE,NDEPL*NBMIX,IJM)
      CALL SETARA(IBASE,NCOMB,IMILVO)
      CALL SETARA(RBASE,(NDEPL-NSUPS+1)*NCOMB*2,IYDPL)
      CALL SETARA(RBASE,NBMIX,IVX)
      CALL SETARA(IBASE,NBISO,ISOCMB)
      CALL SETARA(IBASE,3*NDEPL,IINADP)
      CALL SETARA(RBASE,3*(NDEPL-NSUPS+1)*NBMIX*(NREAC+1),ISIG)
      CALL SETARA(RBASE,3*NBMIX,IVPHV)
      CALL SETARA(RBASE,NBMIX,IENERG)
      CALL SETARA(RBASE,NDEPL,IAWR)
      CALL EVODRV(IPDEPL,IPLIB,INDREC,IMPX,NBISO,NGRO,NBMIX,NREG,
     1 IBASE(IISONA),IBASE(IISONR),IBASE(JJMIX),RBASE(JJDEN),
     2 IBASE(JIEVOL),IBASE(IMAT),RBASE(IVOL),NDEPL,NSUPS,NREAC,NCOMB,
     3 EPS1,EPS2,EXPMAX,H1,ITYPE,INR,IEXTR,IGLOB,ISAT,IDIRAC,FIT,ISAVE,
     4 ISET,IDEPL,XTI,XTF,XT,LMACRO,RBASE(JFLUXE),IBASE(LMASKM),
     5 IBASE(LMASKG),IBASE(IJM),IBASE(IMILVO),RBASE(IYDPL),RBASE(IVX),
     6 IBASE(ISOCMB),IBASE(IINADP),RBASE(ISIG),RBASE(IVPHV),
     7 RBASE(IENERG),RBASE(IAWR))
      CALL RLSARA(RBASE(IAWR))
      CALL RLSARA(RBASE(IENERG))
      CALL RLSARA(RBASE(IVPHV))
      CALL RLSARA(RBASE(ISIG))
      CALL RLSARA(IBASE(IINADP))
      CALL RLSARA(IBASE(ISOCMB))
      CALL RLSARA(RBASE(IVX))
      CALL RLSARA(RBASE(IYDPL))
      CALL RLSARA(IBASE(IMILVO))
      CALL RLSARA(IBASE(IJM))
      CALL RLSARA(IBASE(LMASKG))
      CALL RLSARA(IBASE(LMASKM))
      CALL RLSARA(RBASE(JFLUXE))
      CALL LCMPUT(IPLIB,'ISOTOPESDENS',NBISO,2,RBASE(JJDEN))
*----
*  RELEASE GENERAL TRACKING INFORMATION
*----
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(RBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
*----
*  RELEASE GENERAL INTERNAL LIBRARY INFORMATION
*----
      CALL RLSARA(IBASE(IISONR))
      CALL RLSARA(IBASE(IISONA))
      CALL RLSARA(IBASE(JIEVOL))
      CALL RLSARA(RBASE(JJDEN))
      CALL RLSARA(IBASE(JJMIX))
      RETURN
      END
