*DECK EVOSIG
      SUBROUTINE EVOSIG(IPLIB,IMPX,INR,IGLOB,NGROUP,NREGIO,NBMIX,NBISO,
     1 NCOMB,ISONAM,DEN,MAT,VOL,FLUXES,VX,MILVO,JM,NVAR,NSUPS,NREAC,
     2 NFISS,HREAC,IDR,RER,RRD,FIT,FUELDN,PFACT,SIG,VPHV,SIGGAR,KFISS,
     3 FLX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute and normalize the microscopic depletion reaction rates.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature).
* IMPX    print flag (equal to zero for no print).
* INR     type of flux normalization:
*         =0: out-of-core depletion;
*         =1: constant flux depletion;
*         =2: constant fuel power depletion;
*         =3: constant assembly power depletion.
* IGLOB   out-of-fuel power in flux normalization:
*         =0: compute the burnup using the power released in the fuel;
*         =1: compute the burnup using the power released in the global
*         geometry.
* NGROUP  number of energy groups.
* NREGIO  number of volumes.
* NBMIX   number of mixtures.
* NBISO   number of isotopes/materials including non-depleting ones.
* NCOMB   number of depleting mixtures.
* ISONAM  alias name of isotopes.
* DEN     density of each isotope.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* FLUXES  neutron fluxes.
* VX      volumes of the depleting mixtures.
* MILVO   mixture index corresponding to each depleting mixture.
* JM      position in isotope list of each nuclide of the depletion
*         chain. A negative value indicates a non-depleting isotope
*         producing energy.
* NVAR    number of depleting nuclides.
* NSUPS   number of non-depleting isotopes producing energy.
* NREAC   maximum number of depletion reactions.
* NFISS   number of fissile isotopes producing fission products.
* HREAC   names of used depletion reactions:
*         HREAC(1)='DECAY'; HREAC(2)='NFTOT';
*         HREAC(3)='NG'   ; HREAC(4)='N2N';  etc.
* IDR     identifier for each depleting reaction.
* RER     energy (Mev) per reaction. If RER(3,J)=0., the fission energy
*         is including radiative capture energy. Neutrino energy is
*         never included.
* RRD     sum of radioactive decay constants in 10**-8/s.
* FIT     flux normalization factor:
*         n/cm**2/s if INR=1;
*         MW/tonne of initial heavy elements if INR=2;
*         W/cc of assembly volume if INR=3.
* FUELDN  fuel initial density and mass.
*
*Parameters: output
* PFACT   form factor for out-of-fuel power production.
* SIG     microscopic reaction rates for nuclide I in mixture IBM:
*         SIG(I,1,IBM) fission reaction rate;
*         SIG(I,2,IBM) gamma reaction rate;
*         SIG(I,3,IBM) N2N reaction rate;
*         cont...;
*         SIG(I,NREAC,IBM) neutron-induced energy released;
*         SIG(I,NREAC+1,IBM) decay energy released (10**-8 MeV/s).
* VPHV    integrated fluxes in mixtures.
*
*Parameters: scratch
* SIGGAR
* KFISS
* FLX
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,IMPX,INR,IGLOB,NGROUP,NREGIO,NBMIX,NBISO,NCOMB,
     1 ISONAM(3,NBISO),MAT(NREGIO),MILVO(NCOMB),JM(NBMIX,NVAR+NSUPS),
     2 NVAR,NSUPS,NREAC,NFISS,HREAC(2,NREAC),IDR(NREAC,NVAR+NSUPS),
     3 KFISS(NFISS)
      REAL DEN(NBISO),VOL(NREGIO),FLUXES(NREGIO,NGROUP),VX(NBMIX),
     1 RER(NREAC,NVAR+NSUPS),RRD(NVAR+NSUPS),FIT,FUELDN(3),PFACT,
     2 SIG(NVAR+1,NREAC+1,NBMIX),VPHV(NBMIX),SIGGAR(NGROUP),
     3 FLX(NGROUP,NBMIX)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      CHARACTER TEXT8*8,HNAMIS*12,HSMG*131
      LOGICAL LFISS
      DOUBLE PRECISION GAR,GAR1,GAR2,GARD,XDRCST,EVJ
      COMMON BASE(1)
*
      DO 30 IBM=1,NBMIX
      VPHV(IBM)=0.0
      DO 30 IU=1,NGROUP
      FLX(IU,IBM)=0.0
      IF(VX(IBM).NE.0.0) THEN
         DO 20 I=1,NREGIO
         IF(MAT(I).EQ.IBM) FLX(IU,IBM)=FLX(IU,IBM)+FLUXES(I,IU)*VOL(I)
   20    CONTINUE
         FLX(IU,IBM)=FLX(IU,IBM)/VX(IBM)
      ENDIF
   30 CONTINUE
*
      EVJ=XDRCST('eV','J')*1.0E22
      CALL XDISET(KFISS,NFISS,0)
      DO 50 IS=1,NVAR
      KDRI=IDR(2,IS)/100
      IF((KDRI.GT.0).AND.(MOD(IDR(2,IS),100).EQ.4)) THEN
         IF(KDRI.GT.NFISS) CALL XABORT('EVOSIG: INVALID NFISS.')
         KFISS(KDRI)=IS
      ENDIF
   50 CONTINUE
      VPH=0.0
      DO 70 IU=1,NGROUP
      DO 60 IBM=1,NBMIX
   60 VPHV(IBM)=VPHV(IBM)+VX(IBM)*FLX(IU,IBM)
      DO 70 ICMB=1,NCOMB
      IBM=MILVO(ICMB)
   70 VPH=VPH+VX(IBM)*FLX(IU,IBM)
      DO 80 IBM=1,NBMIX
      DO 80 IQ=1,NREAC+1
      DO 80 IS=1,NVAR+1
   80 SIG(IS,IQ,IBM)=0.0
      DO 220 IBM=1,NBMIX
      DO 220 IST=1,NVAR+NSUPS
      LFISS=.FALSE.
      DO 110 IFIS=1,NFISS
  110 LFISS=LFISS.OR.(KFISS(IFIS).EQ.IST)
      K=JM(IBM,IST)
      IF(K.EQ.0) THEN
         GO TO 220
      ELSE IF(K.GT.0) THEN
*        DEPLETING ISOTOPE.
         IS=IST
         FACT=1.0
      ELSE
*        STABLE ISOTOPE PRODUCING ENERGY.
         K=-K
         IS=NVAR+1
         FACT=DEN(K)*VX(IBM)
      ENDIF
      SIG(IS,NREAC+1,IBM)=SIG(IS,NREAC+1,IBM)+FACT*RER(1,IST)*RRD(IST)
      IF(INR.EQ.0) GO TO 220
      WRITE(HNAMIS,'(3A4)') (ISONAM(I0,K),I0=1,3)
      CALL LCMSIX(IPLIB,HNAMIS,1)
      CALL LCMLEN(IPLIB,'NFTOT',LENGT,ITYLCM)
      IF(LENGT.EQ.NGROUP) THEN
         CALL LCMGET(IPLIB,'NFTOT',SIGGAR)
         DO 120 IU=1,NGROUP
         SIG(IS,1,IBM)=SIG(IS,1,IBM)+1.0E-3*FACT*SIGGAR(IU)*FLX(IU,IBM)
         SIG(IS,NREAC,IBM)=SIG(IS,NREAC,IBM)+1.0E-3*FACT*RER(2,IST)*
     1   SIGGAR(IU)*FLX(IU,IBM)
  120    CONTINUE
      ELSE IF(LFISS.AND.(LENGT.NE.NGROUP)) THEN
         CALL LCMLIB(IPLIB)
         CALL XABORT('EVOSIG: INVALID NFTOT DATA FOR '//HNAMIS//'.')
      ENDIF
*
      CALL LCMLEN(IPLIB,'NG',LENGT,ITYLCM)
      IF(LENGT.EQ.NGROUP) THEN
         CALL LCMGET(IPLIB,'NG',SIGGAR)
      ELSE
*        NG CROSS SECTIONS ARE NOT AVAILABLE. TRY TO COMPUTE THEM.
         CALL SETARA(BASE,NGROUP,ISIGAR)
         CALL LCMLEN(IPLIB,'NTOT0',LENGT,ITYLCM)
         IF(LENGT.NE.NGROUP) THEN
            CALL LCMLIB(IPLIB)
            CALL XABORT('EVOSIG: NO TOTAL X-S FOR ISOTOPE '//HNAMIS
     1      //'.')
         ENDIF
         CALL LCMGET(IPLIB,'NTOT0',SIGGAR)
         CALL LCMLEN(IPLIB,'SIGS00',LENGT,ITYLCM)
         IF(LENGT.EQ.NGROUP) THEN
            CALL LCMGET(IPLIB,'SIGS00',BASE(ISIGAR))
            DO 130 IU=1,NGROUP
  130       SIGGAR(IU)=SIGGAR(IU)-BASE(ISIGAR+IU-1)
         ENDIF
         CALL LCMLEN(IPLIB,'NFTOT',LENGT,ITYLCM)
         IF(LENGT.EQ.NGROUP) THEN
            CALL LCMGET(IPLIB,'NFTOT',BASE(ISIGAR))
            DO 140 IU=1,NGROUP
  140       SIGGAR(IU)=SIGGAR(IU)-BASE(ISIGAR+IU-1)
         ENDIF
         CALL LCMLEN(IPLIB,'N2N',LENGT,ITYLCM)
         IF(LENGT.EQ.NGROUP) THEN
            CALL LCMGET(IPLIB,'N2N',BASE(ISIGAR))
            DO 150 IU=1,NGROUP
  150       SIGGAR(IU)=SIGGAR(IU)+BASE(ISIGAR+IU-1)
         ENDIF
         CALL LCMLEN(IPLIB,'N3N',LENGT,ITYLCM)
         IF(LENGT.EQ.NGROUP) THEN
            CALL LCMGET(IPLIB,'N3N',BASE(ISIGAR))
            DO 160 IU=1,NGROUP
  160       SIGGAR(IU)=SIGGAR(IU)+2.0*BASE(ISIGAR+IU-1)
         ENDIF
         CALL LCMLEN(IPLIB,'N4N',LENGT,ITYLCM)
         IF(LENGT.EQ.NGROUP) THEN
            CALL LCMGET(IPLIB,'N4N',BASE(ISIGAR))
            DO 170 IU=1,NGROUP
  170       SIGGAR(IU)=SIGGAR(IU)+3.0*BASE(ISIGAR+IU-1)
         ENDIF
         DO 185 IREAC=7,NREAC
         WRITE(TEXT8,'(2A4)') HREAC(1,IREAC),HREAC(2,IREAC)
         CALL LCMLEN(IPLIB,TEXT8,LENGT,ITYLCM)
         IF(LENGT.EQ.NGROUP) THEN
            CALL LCMGET(IPLIB,TEXT8,BASE(ISIGAR))
            DO 180 IU=1,NGROUP
  180       SIGGAR(IU)=SIGGAR(IU)-BASE(ISIGAR+IU-1)
         ENDIF
  185    CONTINUE
         CALL RLSARA(BASE(ISIGAR))
      ENDIF
      DO 190 IU=1,NGROUP
      SIG(IS,2,IBM)=SIG(IS,2,IBM)+1.0E-3*FACT*SIGGAR(IU)*FLX(IU,IBM)
  190 SIG(IS,NREAC,IBM)=SIG(IS,NREAC,IBM)+1.0E-3*RER(3,IST)*FACT*
     1 SIGGAR(IU)*FLX(IU,IBM)
*
      DO 210 IREAC=4,NREAC
      WRITE(TEXT8,'(2A4)') HREAC(1,IREAC),HREAC(2,IREAC)
      CALL LCMLEN(IPLIB,TEXT8,LENGT,ITYLCM)
      IF((LENGT.NE.NGROUP).AND.(IDR(IREAC,IST).GT.0)) THEN
         IF(IMPX.GT.90) CALL LCMLIB(IPLIB)
         IF(IMPX.GT.3) THEN
            WRITE(HSMG,'(17HEVOSIG: REACTION ,A8,18H IS MISSING FOR IS,
     1      7HOTOPE '',A12,2H''.)') TEXT8,HNAMIS
            WRITE(IOUT,'(1X,A)') HSMG
         ENDIF
      ELSE IF(LENGT.EQ.NGROUP) THEN
         CALL LCMGET(IPLIB,TEXT8,SIGGAR)
         DO 200 IU=1,NGROUP
         SIG(IS,IREAC-1,IBM)=SIG(IS,IREAC-1,IBM)+1.0E-3*FACT*SIGGAR(IU)*
     1   FLX(IU,IBM)
         SIG(IS,NREAC,IBM)=SIG(IS,NREAC,IBM)+1.0E-3*FACT*RER(IREAC,IST)*
     1   SIGGAR(IU)*FLX(IU,IBM)
  200    CONTINUE
      ENDIF
  210 CONTINUE
      CALL LCMSIX(IPLIB,' ',2)
  220 CONTINUE
*----
*  CONSTANT FLUX OR CONSTANT POWER NORMALIZATION
*----
      PFACT=1.0
      VTOT=0.0
      DO 230 ICMB=1,NCOMB
  230 VTOT=VTOT+VX(MILVO(ICMB))
      IF(INR.EQ.1) THEN
         PHI=FIT*1.E-13
      ELSE IF(INR.GE.2) THEN
         GAR=0.0D0
         GARD=0.0D0
         DO 240 ICMB=1,NCOMB
         IBM=MILVO(ICMB)
         DO 240 IS=1,NVAR
         IF(JM(IBM,IS).GT.0) THEN
            GAR=GAR+VX(IBM)*DEN(JM(IBM,IS))*SIG(IS,NREAC,IBM)
            GARD=GARD+VX(IBM)*DEN(JM(IBM,IS))*SIG(IS,NREAC+1,IBM)
         ENDIF
  240    CONTINUE
         GAR1=GAR
         DO 250 ICMB=1,NCOMB
  250    GAR1=GAR1+SIG(NVAR+1,NREAC,MILVO(ICMB))
         GAR2=GAR
         DO 260 IBM=1,NBMIX
  260    GAR2=GAR2+SIG(NVAR+1,NREAC,IBM)
         PFACT=GAR2/GAR1
         IF((IGLOB.EQ.1).OR.(INR.EQ.3)) THEN
            GAR=GAR2
         ELSE IF(IGLOB.EQ.0) THEN
            GAR=GAR1
         ENDIF
         IF(GAR.EQ.0.0D0) CALL XABORT('EVOSIG: UNABLE TO NORMALIZE.')
         IF(INR.EQ.2) THEN
*           FITD IS THE DECAY POWER IN WATT PER GRAM.
            FITD=(EVJ*GARD)/(FUELDN(1)*VTOT)
            IF(FITD.GT.FIT) THEN
               WRITE(HSMG,'(35HEVOSIG: NEGATIVE FIT(1) FIT(DECAY)=,1P,
     1         E11.4,12H FIT(INPUT)=,E11.4,1H.)') FITD,FIT
               CALL XABORT(HSMG)
            ENDIF
            PHI=(FIT-FITD)*FUELDN(1)*VPH/(EVJ*GAR)
         ELSE IF(INR.EQ.3) THEN
*           FITD IS THE DECAY POWER IN WATT PER CUBIC CENTIMETER.
            FITD=(EVJ*GARD*FUELDN(3))/(FUELDN(1)*VTOT)
            IF(FITD.GT.FIT) THEN
               WRITE(HSMG,'(35HEVOSIG: NEGATIVE FIT(2) FIT(DECAY)=,1P,
     1         E11.4,12H FIT(INPUT)=,E11.4,1H.)') FITD,FIT
               CALL XABORT(HSMG)
            ENDIF
            PHI=(FIT-FITD)*FUELDN(1)*VPH/(EVJ*GAR*FUELDN(3))
         ENDIF
      ENDIF
      IF(INR.GT.0) THEN
         FNORM=PHI*VTOT/VPH
         DO 270 IBM=1,NBMIX
         VPHV(IBM)=VPHV(IBM)*FNORM
         DO 270 IQ=1,NREAC
         DO 270 IS=1,NVAR+1
  270    SIG(IS,IQ,IBM)=SIG(IS,IQ,IBM)*FNORM
      ENDIF
      RETURN
      END
