*DECK EXCELB
      SUBROUTINE EXCELB(  IPTRK, IFTRAK, IPRNTP,   NREG,  NBMIX,   NANI,
     >                   MATCOD, VOLUME, NRENOR, XSSIGT, XSSIGW, NELPIJ,
     >                    IPIJK,    PIJ, LEAKSW,   NSBG,  NPSYS,   NPST,
     >                   PROBKS, TITREC)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the collision probabilities for Excell. Interface
* subroutine used to implement double heterogeneity capabilities.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  unit of the sequential binary tracking file.
* IPRNTP  print flag (equal to zero for no print).
* NREG    total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures (NBMIX=max(MATCOD(i))).
* NANI    number of Legendre orders.
* MATCOD  index-number of the mixture type assigned to each volume.
* VOLUME  volumes.
* NRENOR  normalization scheme for PIJ matrices.
* XSSIGT  total macroscopic cross sections ordered by mixture.
* XSSIGW  P0 within-group scattering macroscopic cross sections
*         ordered by mixture.
* NELPIJ  number of elements in symmetrized pij matrix.
* IPIJK   pij option (=1 pij, =4 pijk).
* LEAKSW  leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* NSBG    number of energy groups.
* NPSYS   non-converged energy group indices.
* NPST    first dimension of matrix PROBKS.
* TITREC  title.
*
*Parameters: output
* PIJ     reduced and symmetrized collision probabilities.
* PROBKS  directional collision probabilities.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER TITREC*72
      LOGICAL LEAKSW
      INTEGER IPTRK,IFTRAK,IPRNTP,NREG,NBMIX,NANI,MATCOD(NREG),
     > NRENOR,NELPIJ,IPIJK,NSBG,NPSYS(NSBG),NPST
      REAL VOLUME(NREG),XSSIGT(0:NBMIX,NSBG),
     > XSSIGW(0:NBMIX,NANI,NSBG),PIJ(NELPIJ,IPIJK,NSBG),
     > PROBKS(NPST,NSBG)
*----
*  LOCAL VARIABLES
*----
      INTEGER IPAR(8)
      REAL BASE(1)
      COMMON IBASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*
      IF(IPIJK.NE.1) CALL XABORT('EXCELB: PIJK OPTION FORBIDDEN.')
      CALL LCMSIX(IPTRK,'BIHET',1)
      CALL LCMGET(IPTRK,'PARAM',IPAR)
      IR1=IPAR(1)
      IR2=IPAR(2)
      NREG2=IPAR(3)
      NG=IPAR(4)
      NSMAX=IPAR(5)
      MICRO=IPAR(7)
      IQUA10=IPAR(8)
      CALL SETARA(IBASE,NG,INS)
      CALL SETARA(IBASE,NREG2,IIBI)
      CALL SETARA(BASE,NG*(1+NSMAX),IRS)
      CALL SETARA(BASE,NG*IR2,IFRACT)
      CALL SETARA(BASE,NG*NSMAX,IVOLK)
      CALL SETARA(BASE,NREG2,IVOL2)
      CALL LCMGET(IPTRK,'NS',IBASE(INS))
      CALL LCMGET(IPTRK,'IBI',IBASE(IIBI))
      CALL LCMGET(IPTRK,'RS',BASE(IRS))
      CALL LCMGET(IPTRK,'FRACT',BASE(IFRACT))
      CALL LCMGET(IPTRK,'VOLK',BASE(IVOLK))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL2))
      CALL LCMSIX(IPTRK,' ',2)
*
      CALL SETARA(BASE,NREG*NREG*NSBG,IPIJ2)
      CALL SETARA(BASE,NREG,ISIGT)
      CALL SETARA(BASE,NREG,ISIGW)
      CALL SETARA(BASE,NREG2*NSBG,IRRR)
      CALL SETARA(BASE,NREG2*NSBG,ISTILD)
      CALL SETARA(BASE,NG*NSMAX*NREG2*NSBG,IQKD)
*
      NELPIJ2=NREG2*(NREG2+1)/2
      CALL SETARA(BASE,NSMAX*NSMAX,IQKN)
      CALL SETARA(BASE,NELPIJ2*NSBG,IPIL)
      CALL SETARA(BASE,(IR2+1)*NSBG,ISIGT2)
      CALL SETARA(BASE,(IR2+1)*NANI*NSBG,ISIGW2)
      CALL XDRSET(BASE(ISIGT2),(IR2+1)*NSBG,0.0)
      CALL XDRSET(BASE(ISIGW2),(IR2+1)*NANI*NSBG,0.0)
*
      DO 20 IGRP=1,NSBG
      DO 10 I=1,NREG
   10 BASE(ISIGT+I-1)=XSSIGT(MATCOD(I),IGRP)
   
      CALL SYBHT1(IR1,IR2,NREG,BASE(ISIGT),NREG2,NG,NSMAX,MICRO,
     > IQUA10,IBASE(INS),IBASE(IIBI),BASE(IRS),BASE(IFRACT),
     > BASE(IVOLK),BASE(IPIJ2+NREG*NREG*(IGRP-1)),
     > BASE(IRRR+NREG2*(IGRP-1)),BASE(ISTILD+NREG2*(IGRP-1)),
     > BASE(IQKD+NSMAX*NSMAX*(IGRP-1)),BASE(IQKN))
      DO 20 I=1,NREG2
      IBM=IBASE(IIBI+I-1)
   20 BASE(ISIGT2+(IR2+1)*(IGRP-1)+IBM)=BASE(ISTILD+I-1)
      CALL RLSARA(BASE(IQKN))
      CALL RLSARA(BASE(IRS))
*
      CALL EXCELP(IPTRK,IFTRAK,IPRNTP,NREG2,IR2,NANI,IBASE(IIBI),
     > BASE(IVOL2),NRENOR,BASE(ISIGT2),BASE(ISIGW2),NELPIJ2,IPIJK,
     > BASE(IPIL),LEAKSW,NSBG,NPSYS,NPST,PROBKS,TITREC)
      CALL RLSARA(BASE(ISIGW2))
      CALL RLSARA(BASE(ISIGT2))
*
      CALL SETARA(BASE,NREG2*NREG2,IPILSQ)
      DO 50 IGRP=1,NSBG
      IC=0
      DO 30 I=1,NREG2
      DO 30 J=1,I
      IC=IC+1
      PIL=BASE(IPIL+NELPIJ2*(IGRP-1)+IC-1)
      BASE(IPILSQ+NREG2*(J-1)+I-1)=PIL/BASE(IVOL2+I-1)
   30 BASE(IPILSQ+NREG2*(I-1)+J-1)=PIL/BASE(IVOL2+J-1)
      CALL SYBHT2(IR1,IR2,NREG,NREG2,NG,NSMAX,IBASE(INS),IBASE(IIBI),
     > BASE(IFRACT),BASE(IVOLK),BASE(IPIJ2+NREG*NREG*(IGRP-1)),
     > BASE(IRRR+NREG2*(IGRP-1)),BASE(ISTILD+NREG2*(IGRP-1)),
     > BASE(IQKD+NSMAX*NSMAX*(IGRP-1)),BASE(IPILSQ))
      DO 40 I=1,NREG
      BASE(ISIGT+I-1)=XSSIGT(MATCOD(I),IGRP)
   40 BASE(ISIGW+I-1)=XSSIGW(MATCOD(I),1,IGRP)
      CALL SYBWIJ(NREG,NREG,BASE(ISIGT),BASE(ISIGW),
     >            BASE(IPIJ2+NREG*NREG*(IGRP-1)))
      IC=0
      DO 50 I=1,NREG
      DO 50 J=1,I
      IC=IC+1
      PIJSQ=BASE(IPIJ2+NREG*NREG*(IGRP-1)+NREG*(I-1)+J-1)
   50 PIJ(IC,1,IGRP)=PIJSQ*VOLUME(J)
      CALL RLSARA(BASE(IPILSQ))
      CALL RLSARA(BASE(IPIL))
      CALL RLSARA(BASE(IQKD))
      CALL RLSARA(BASE(ISTILD))
      CALL RLSARA(BASE(IRRR))
      CALL RLSARA(BASE(ISIGW))
      CALL RLSARA(BASE(ISIGT))
      CALL RLSARA(BASE(IPIJ2))
      CALL RLSARA(BASE(IVOL2))
      CALL RLSARA(BASE(IVOLK))
      CALL RLSARA(BASE(IFRACT))
      CALL RLSARA(IBASE(IIBI))
      CALL RLSARA(IBASE(INS))
      RETURN
      END
