*DECK FLUDRV
      SUBROUTINE FLUDRV(IPRT,IPFLUX,IPTRK,IPMACR,IFTRAK,IPSYS,IPHASE,
     1 ITPIJ,CXDOOR,ITRANC,TITLE,B2,INITFL,LEAKSW,IREBAL,NGRP,NMAT,
     2 NIFIS,NANIS,NLF,NFUNL,OPTION,NUN,MAXINR,EPSINR,MAXOUT,EPSUNK,
     3 EPSOUT,IFRITR,IACITR,ITYPEC,ILEAK,NREG,NSOUT,MATCOD,KEYFLX,VOL,
     4 REFKEF,FLUXO,XSTOT,XSTRC,XSDIA,XSCHI,XSNUF,XSTK,VSCAT)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver for Boltzmann equation solvers.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Roy
*
*Parameters: input
* IPRT    print flag.
* IPFLUX  pointer to the flux LCM object.
* IPTRK   pointer to the tracking LCM object.
* IPMACR  pointer to the macrolib LCM object.
* IFTRAK  tracking file unit number.
* IPSYS   pointer to the system LCM object (=0 for the method of
*         characteristics).
* IPHASE  1 for asm 2 for pij.
* ITPIJ   type of collision probability information available:
*         =1: scattering modified pij (wij);
*         =2: standard pij;
*         =3: scattering modified pij+pijk (wij,wijk);
*         =4: standard pij+pijk.
* CXDOOR  name of the flux solution door.
* ITRANC  type of transport correction (>0 to perform a correction).
* TITLE   title.
* B2      initial or imposed directional bucklings.
* INITFL  flux initialization flag (=0: uniform flux).
* LEAKSW  leakage flag (=.true. if leakage is present on the outer
*         surface).
* IREBAL  flux rebalancing flag (=1: perform rebalancing).
* NGRP    number of energy groups.
* NMAT    number of mixtures.
* NIFIS   number of fissile isotopes.
* NANIS   maximum cross section Legendre order.
* NLF     number of Legendre orders for the flux.
* NFUNL   second dimension of matrix KEYFLX.
* OPTION  type of leakage coefficients; can be 'LKRD' (imposed leakage
*         coefficients), 'B0' (B-0), 'P0' (P-0), 'B1' (B-1), 'P1' (P-1)
*         or 'B0TR' (B-0 with transport correction).
* NUN     number of unknowns per energy group including spherical.
*         harmonic terms, interface currents and fundamental
*         currents.
* MAXINR  maximum number of thermal iterations.
* EPSINR  thermal iterations epsilon.
* MAXOUT  maximum number of outer iterations.
* EPSUNK  outer iterations eigenvector epsilon.
* EPSOUT  outer iterations eigenvalue epsilon.
* IFRITR  number of free iterations in an acceleration cycle.
* IACITR  number of accelerated iterations in an acceleration cycle.
* ITYPEC  type of flux evaluation:
*         =-1: skip the flux calculation;
*         =0: fixed sources;
*         =1: fission sources/K-eff convergence;
*         =2: fission sources/K-eff convergence/db2 buckling evaluation;
*         =3: fission sources/db2 buckling convergence;
*         =4: b2 sources/db2 buckling convergence.
* ILEAK   method used to include DB2 effect:
*         =1: the scattering modified cp matrix is multiplied by PNLR;
*         =2: the reduced cp matrix is multiplied by PNL;
*         =3: sigs0-db2 approximation;
*         =4: albedo approximation;
*         =5: Ecco-type isotropic streaming model;
*         >5: Tibere-type anisotropic streaming model.
* NREG    number of regions.
* NSOUT   number of outer surfaces.
* MATCOD  mixture indices.
* KEYFLX  index of L-th order flux components in unknown vector.
* VOL     volumes.
* REFKEF  target effective multiplication factor (K-eff).
*
*Parameters: scratch
* FLUXO
* XSTOT
* XSTRC
* XSDIA
* XSCHI
* XSNUF
* XSTK
* VSCAT
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER   CXDOOR*12,TITLE*72,OPTION*4
      INTEGER     IPRT,IPFLUX,IPTRK,IPMACR,IFTRAK,IPSYS,IPHASE,ITPIJ,
     >            ITRANC,INITFL,IREBAL,NGRP,NMAT,NIFIS,NANIS,NLF,NFUNL,
     >            NUN,MAXINR,MAXOUT,IFRITR,IACITR,ITYPEC,ILEAK,NREG,
     >            NSOUT,MATCOD(NREG),KEYFLX(NREG,NFUNL)
      REAL        EPSUNK,EPSINR,B2(4),VOL(NREG),FLUXO(NUN,NGRP),
     >            XSTOT(0:NMAT,NGRP),XSTRC(0:NMAT,NGRP),
     >            XSDIA(0:NMAT,0:NANIS,NGRP),XSCHI(0:NMAT,NIFIS,NGRP),
     >            XSNUF(0:NMAT,NIFIS,NGRP),XSTK(NMAT,NIFIS),VSCAT(NMAT)
      LOGICAL     LEAKSW
      DOUBLE PRECISION REFKEF
*----
*  LOCAL VARIABLES
*----
      PARAMETER ( IUNOUT=6, NSTATE=40, NDIMO=2 )
      LOGICAL     LREBAL,LTRANC,LTP1
      INTEGER     ISTATE(NSTATE)
      REAL        EPSCON(3)
      CHARACTER   CAN(0:9)*2
*----
*  SETARA POINTERS
*----
      INTEGER DIFF,FLUX,IJJ,NJJ,IPOS,XSCAT,XCSOU,GAMMA,INPSYS
*
      REAL        A
      COMMON      A(1)
      INTEGER     I(1)
      EQUIVALENCE (A(1),I(1))
      SAVE CAN
      DATA CAN /'00','01','02','03','04','05','06','07','08','09'/
*
      IF(IPRT.GE.3) THEN
        WRITE(IUNOUT,6000)
        WRITE(IUNOUT,6001) (IREGIO,VOL(IREGIO),MATCOD(IREGIO),
     >                      IREGIO=1,NREG)
      ENDIF
*----
*  FLUX INITIALIZATION
*----
      IF(INITFL.EQ.0) THEN
        JPFLUX=LCMLID(IPFLUX,'FLUX',NGRP)
        DO 100 IGR=1,NGRP
          CALL XDRSET(FLUXO(1,IGR),NUN,0.0)
          IF(ITYPEC.GT.0) THEN
            DO 90 IREGIO=1,NREG
            IND=KEYFLX(IREGIO,1)
            IF(IND.GT.0) FLUXO(IND,IGR)=1.0
   90       CONTINUE
          ENDIF
          CALL LCMPDL(JPFLUX,IGR,NUN,2,FLUXO(1,IGR))
  100   CONTINUE
      ENDIF
      IF(ITYPEC.GE.1) THEN
        EIGENK=REAL(REFKEF)
        CALL LCMPUT(IPFLUX,'K-EFFECTIVE',1,2,EIGENK)
      ENDIF
      IF(ITYPEC.GE.2) THEN
        CALL LCMPUT(IPFLUX,'B2  B1HOM',1,2,B2(4))
        IF(ILEAK.GE.6) CALL LCMPUT(IPFLUX,'B2  HETE',3,2,B2)
      ENDIF
      IF(ITYPEC.EQ.-1) GO TO 1001
*
      IF(ILEAK.GE.6.AND.ITYPEC.GE.2) THEN
        IF(ITRANC.EQ.0) THEN
          IF(OPTION.EQ.'B0TR'.OR.OPTION.EQ.'LKRD')
     >      CALL XABORT('FLUDRV: ILLEGAL OPTION = '//OPTION//
     >         ' FOR HETEROGENEOUS LEAKAGE CALCULATION'//
     >         ' WITHOUT TRANSPORT CORRECTED CROSS SECTIONS')
        ELSE
          IF(OPTION.NE.'B0TR')
     >      CALL XABORT('FLUDRV: ILLEGAL OPTION = '//OPTION//
     >         ' FOR HETEROGENEOUS LEAKAGE CALCULATION'//
     >         ' WITH TRANSPORT CORRECTED CROSS SECTIONS')
        ENDIF
      ENDIF
*----
*  RECOVER CROSS SECTIONS.
*----
      JPMACR=LCMGID(IPMACR,'GROUP')
      DO 150 IGR=1,NGRP
        KPMACR=LCMGIL(JPMACR,IGR)
        CALL LCMLEN(KPMACR,'NTOT0',ILNLCM,ITYLCM)
        IF(ILNLCM.GT.0) THEN
          CALL LCMGET(KPMACR,'NTOT0',XSTRC(1,IGR))
          XSTRC(0,IGR)=0.0
          DO 111 IR= 0, NMAT
 111      XSTOT(IR,IGR)=XSTRC(IR,IGR)
        ELSE
          CALL LCMLIB(KPMACR)
          CALL XABORT('FLUDRV: READ ERROR ON LCM RECORD= TOTAL')
        ENDIF
        DO 110 IANI=0,NANIS
        CALL LCMLEN(KPMACR,'SIGW'//CAN(IANI),ILNLCM,ITYLCM)
        IF( ILNLCM.GT.0 )THEN
          CALL LCMGET(KPMACR,'SIGW'//CAN(IANI),XSDIA(1,IANI,IGR))
          XSDIA(0,IANI,IGR)=0.0
        ELSE
          CALL XDRSET(XSDIA(0,IANI,IGR),NMAT+1,0.0)
        ENDIF
 110    CONTINUE
        CALL LCMLEN(KPMACR,'TRANC',ILCMLN,ITYLCM)
	LTRANC=ILCMLN.GT.0
        CALL LCMLEN(KPMACR,'NTOT1',ILCMLN,ITYLCM)
        LTP1=ILCMLN.GT.0
        IF( (ITRANC.NE.0).AND.LTRANC )THEN
*         TRANSPORT CORRECTION (INCLUDE THE LEAKAGE CORRECTION).
          CALL LCMGET(KPMACR,'TRANC',VSCAT)
	ELSE IF( LTP1 )THEN
*         PERFORM A LEAKAGE CORRECTION.
          CALL LCMGET(KPMACR,'NTOT1',VSCAT)
          DO 120 IMAT=1,NMAT
 120      VSCAT(IMAT)=XSTRC(IMAT,IGR)-VSCAT(IMAT)
        ELSE
          CALL XDRSET(VSCAT,NMAT,0.0)
	ENDIF
        DO 130 IMAT=1,NMAT
          XSTRC(IMAT,IGR)=XSTRC(IMAT,IGR)-VSCAT(IMAT)
          XSDIA(IMAT,0,IGR)=XSDIA(IMAT,0,IGR)-VSCAT(IMAT)
 130    CONTINUE
        IF(IPRT.GE.3) THEN
          WRITE(IUNOUT,6002)  IGR
          WRITE(IUNOUT,6003) (IMAT,XSTRC(IMAT,IGR),
     >                             XSDIA(IMAT,0,IGR),IMAT=1,NMAT)
        ENDIF
        IF( ITYPEC.NE.0 )THEN
           CALL LCMLEN(KPMACR,'CHI',ILONG,ITYLCM)
           IF( ILONG.EQ.0 )THEN
              IF( ITYPEC.NE.4 )CALL XABORT('FLUDRV: TYPE L REQUESTED')
              CALL XDRSET(XSCHI,NGRP*(NMAT+1)*NIFIS,0.0)
              CALL XDRSET(XSNUF,NGRP*(NMAT+1)*NIFIS,0.0)
           ELSE
              CALL LCMGET(KPMACR,'CHI',XSTK)
              DO 132 IFIS= 1, NIFIS
                 XSCHI(0,IFIS,IGR)= 0.0
                 DO 131 IMAT= 1, NMAT
                    XSCHI(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  131            CONTINUE
  132         CONTINUE
              CALL LCMGET(KPMACR,'NUSIGF',XSTK)
              DO 134 IFIS= 1, NIFIS
                 XSNUF(0,IFIS,IGR)= 0.0
                 DO 133 IMAT= 1, NMAT
                    XSNUF(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  133            CONTINUE
  134         CONTINUE
           ENDIF
           IF( IPRT.GT.3 )THEN
              WRITE(IUNOUT,6004) (IMAT,XSNUF(IMAT,1,IGR),
     >                                 XSCHI(IMAT,1,IGR),IMAT=1,NMAT)
           ENDIF
        ELSE
           IF( NIFIS.EQ.0 )THEN
              CALL XABORT('FLUDRV: FOR FIXED SOURCE PROBLEM, NIFIS > 0')
           ENDIF
           XSNUF(0,1,IGR)=0.0
           CALL LCMGET(KPMACR,'FIXE',XSNUF(1,1,IGR))
           DO 135 IFIS=2,NIFIS
             CALL XDRSET(XSNUF(0,IFIS,IGR),NMAT+1,0.0)
  135      CONTINUE
        ENDIF
        DO 50 IANI=0,NANIS
          CALL LCMLEN(KPMACR,'NJJS'//CAN(IANI),ILNLCM,ITYLCM)
        IF(ILNLCM.NE.NMAT) THEN
          CALL LCMLIB(KPMACR)
          CALL XABORT('FLUDRV: FLUX CALCULATION ERROR, '//
     >    'SCATTERING MATRIX OF ORDER ANIS ='//CAN(IANI)//' NOT ON LCM')
        ENDIF
   50   CONTINUE
 150  CONTINUE
*
      IF (CXDOOR.EQ.'MCCG') THEN
         CALL LCMLEN(IPTRK,'KEYCUR$MCCG',ICREB,ITYLCM)
         IF (ICREB.GT.0) THEN
            LREBAL=(IREBAL.EQ.1)
         ELSE
            LREBAL=(IREBAL.EQ.1).AND.(.NOT.LEAKSW)
         ENDIF
      ELSE
         LREBAL=(IREBAL.EQ.1).AND.(.NOT.LEAKSW)
      ENDIF
      CALL SETARA( A, NGRP, DIFF)
      CALL SETARA( A, 8*NGRP*NUN, FLUX)
      CALL SETARA( A, 1+NMAT, IJJ)
      CALL SETARA( A, 1+NMAT, NJJ)
      CALL SETARA( A, 1+NMAT, IPOS)
      CALL SETARA( A, 1+NGRP*NMAT, XSCAT)
      CALL SETARA( A, 2*NGRP, XCSOU)
      CALL SETARA( A, NGRP, GAMMA)
      CALL SETARA( A, NGRP, INPSYS)
      CALL FLU2DR(IPRT,IPMACR,IPFLUX,IPSYS,IPTRK,IFTRAK,CXDOOR,TITLE,
     1 NUN,NREG,NSOUT,NANIS,NLF,NFUNL,NGRP,NMAT,NIFIS,LEAKSW,MAXINR,
     2 EPSINR,MAXOUT,EPSUNK,EPSOUT,IFRITR,IACITR,ITYPEC,IPHASE,ITPIJ,
     3 ILEAK,OPTION,REFKEF,MATCOD,KEYFLX,VOL,XSTOT,XSTRC,XSDIA,XSNUF,
     4 XSCHI,LREBAL,A(DIFF),A(FLUX),A(IJJ),A(NJJ),A(IPOS),A(XSCAT),
     5 A(XCSOU),A(GAMMA),A(INPSYS))
      CALL RLSARA( A(INPSYS ) )
      CALL RLSARA( A(GAMMA  ) )
      CALL RLSARA( A(XCSOU  ) )
      CALL RLSARA( A(XSCAT  ) )
      CALL RLSARA( A(IPOS   ) )
      CALL RLSARA( A(NJJ    ) )
      CALL RLSARA( A(IJJ    ) )
      CALL RLSARA( A(FLUX   ) )
      CALL RLSARA( A(DIFF   ) )
*
 1001 CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NGRP
      ISTATE(2)=NUN
      ISTATE(3)=1
      ISTATE(4)=0
      ISTATE(5)=0
      ISTATE(6)=ITYPEC
      ISTATE(7)=ILEAK
      ISTATE(8)=IFRITR
      ISTATE(9)=IACITR
      ISTATE(10)=IREBAL
      ISTATE(11)=MAXINR
      ISTATE(12)=MAXOUT
      CALL LCMPUT(IPFLUX,'STATE-VECTOR',NSTATE,1,ISTATE)
      EPSCON(1)=EPSINR
      EPSCON(2)=EPSUNK
      EPSCON(3)=EPSOUT
      CALL LCMPUT(IPFLUX,'EPS-CONVERGE',3,2,EPSCON)
      RETURN
*----
*  FORMATS
*----
 6000 FORMAT(//30X,' EDITION REGION/VOL/MIXTURE '//
     >3(5X,'REGION',5X,'VOL  ',5X,'MIXTURE')/)
 6001 FORMAT(1P,3(5X,I4,4X,E12.5,4X,I4,4X))
 6002 FORMAT(//30X,' G R O U P : ',I5//
     >30X,' TOTAL MACROSCOPIC CROSS SECTIONS PER MIXTURE '/)
 6003 FORMAT(3(1X,'MIXTURE',4X,'NTOT0',11X,'SIGW',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
 6004 FORMAT(3(1X,'MIXTURE',4X,'NUSIGF',11X,'CHI ',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
      END
