*DECK LIBBAS
      SUBROUTINE LIBBAS(NISO,AT,AKT,AMT,T,IX,V,DV,NDTE,P,XS,X,E)
*
*-----------------------------------------------------------------------
*
*Purpose:
* scattering kernel based on the free gas model of Brown and St. John
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NISO   number of terms in the model:
*        NISO=1 : pure free gas model;
*        NISO=2 : Brown and St. John model
* AT     potential microscopic cross section
* AKT    exponential constant in the model. Equal to zero for the
*        pure free gas model
* AMT    isotope mass divided by neutron mass
* T      absolute temperature divided by 293.6K
* IX     number of thermal groups
* V      neutron velocities
* DV     used to transform velocity to energy
* NDTE   first dimension of matrix P
*
*Parameters: output
* P      scattering kernel. The first index is for secondary neutrons
* XS     scattering microscopic cross section
*
*Parameters: scratch
* X
* E
*
*-----------------------------------------------------------------------
*  REFERENCE:
*   H. C. HONECK, 'THE DISTRIBUTION OF THERMAL NEUTRONS IN SPACE AND
*   ENERGY IN REACTOR LATTICES. PART 1: THEORY', NUCL. SCI. ENG., 8,
*   193 (1960).
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NISO,IX,NDTE
      REAL AT(NISO),AKT(NISO),AMT(NISO),T,V(IX),DV(IX),P(NDTE,IX),
     1 XS(IX),X(IX),E(IX)
*
      DO 15 I=1,IX
      XS(I)=0.0
      DO 15 J=1,IX
   15 P(I,J)=0.0
  100 DO 500 N=1,NISO
      IF(AT(N))110,500,110
  110 AM=AMT(N)
      BETS=AM/T
      BET=SQRT(BETS)
      DO 120 I=1,IX
      X(I)=BET*V(I)
  120 E(I)=X(I)*X(I)
      TAUS=BETS/(BETS+AKT(N))
      TAU=SQRT(TAUS)
      ALPHA=AKT(N)*TAUS/BETS
      THETA=(AM+1.0)/(2.0*AM*TAU)
      ZETA=TAU-THETA
      OMEG=TAUS*(BETS+(AM+1.0)*AKT(N))/BETS
      CONST=(AT(N)*TAU*TAUS*(AM+1.0)*(AM+1.0))/(4.0*AM*OMEG)
  200 DO 400 I=1,IX
      DO 300 J=I,IX
      CONST1=CONST*X(I)/X(J)
      WA=ALPHA*E(J)
      TEM=0.0
      IF(WA-50.0) 245,250,250
  245 EA=AFERF((THETA*X(I))+(ZETA*X(J)))+AFERF((THETA*X(I))-(ZETA*X(J)))
      TEM=CONST1*EA*EXP(-WA)
  250 WB=(OMEG*E(I)-E(J))/AM
      IF(WB-50.0) 255,260,260
  255 EB=AFERF((THETA*X(J))-(ZETA*X(I)))-AFERF((ZETA*X(I))+(THETA*X(J)))
      TEM=TEM+CONST1*EB*EXP(-WB)
  260 IF(TEM-1.E-15) 350,350,270
  270 P(I,J)=TEM
  300 CONTINUE
  350 EA=TAU*X(I)
      WA=ALPHA*E(I)
      IF(WA-50.0)352,351,351
  351 WA=0.0
      GOTO 353
  352 WA=EXP(-WA)
  353 EB=WA*AFERF(EA)*(EA+(0.5/EA))
      IF(E(I)-50.0)355,354,354
  354 WB=0.0
      GOTO 356
  355 WB=EXP(-E(I))
  356 XS(I)=XS(I)+(AT(N)*TAUS*TAUS/BET)*(EB+0.5641896*WB)
  400 CONTINUE
  500 CONTINUE
      DO 610 I=1,IX
      E(I)=0.0
      WA=(V(I)*V(I))/T
      IF(WA-50.0)600,610,610
  600 E(I)=V(I)*V(I)*EXP(-WA)
  610 CONTINUE
      DO 620 I=1,IX
      DO 620 J=I,IX
      IF(E(I)-1.E-20)620,620,621
  621 P(J,I)=P(I,J)*E(J)/E(I)
  620 CONTINUE
      DO 650 J=1,IX
      TEM=0.0
      DO 640 I=1,IX
  640 TEM=TEM+P(I,J)*DV(I)
  650 P(J,J)=((XS(J)-TEM)/DV(J))+P(J,J)
      DO 680 I=1,IX
      XS(I)=XS(I)/V(I)
      DO 680 J=1,IX
  680 P(I,J)=P(I,J)*DV(J)/V(I)
      RETURN
      END
