*DECK LIBFQD
      SUBROUTINE LIBFQD(MAXNOR,LPART,MAXTRA,HNAMIS,IPLIB,NGRO,NL,NED,
     1 NDEL,NDIL,IGRMIN,IGRMAX,LBIN,NFS,IMPX,LSCAT,LSIGF,LADD,DILUT,
     2 FLUX,TOTAL,SIGF,SIGS,SCAT,SADD,ZDEL,DELTG,EBIN,SIGTF,SIGSF,AWR,
     3 ISMIN,ISMAX,GOLD,IPRECI,NOR,PRTSIG,WSLD,DELTA,UUU,STIS,SIGAF,
     4 SIGPF,PHIMT,MATRIX,PRI,PHI,PRTPH,WORK,CC,PRTSIW,PRTABS,ISM)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute NOR-point Calendf-type probability tables;
* compute physical and/or slowing-down correlated probability tables;
* component of the Ribox method.
*
*Copyright:
* Copyright (C) 2003 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* MAXNOR  first dimension of matrix PRTSIG. Equal to the maximum order
*         of a probability table.
* LPART   maximum scattering bandwidth for the isotope
* MAXTRA  maximum number of energy bins of size DELI.
* HNAMIS  name of the isotope.
* IPLIB   pointer to the internal library (L_LIBRARY signature).
* NGRO    number of energy groups.
* NL      number of Legendre orders required in the calculation
*         (NL=1 or higher).
* NED     number of extra vector edits.
* NDEL    number of delayed neutron precursor groups.
* NDIL    number of finite dilutions.
* IGRMIN  first self-shielded group with BIN data.
* IGRMAX  most thermal self-shielded group with BIN data.
* LBIN    number of fine (bin) energy groups.
* NFS     number of fine energy groups in each group.
* IMPX    print flag (equal to zero for no print).
* LSCAT   anisotropy flag (=.true. if a given Legendre order of the
*         scattering cross section exists).
* LSIGF   fission flag (=.true. if the isotope can fission).
* LADD    additional cross section flag (=.true. if a given additional
*         cross section exists).
* DILUT   dilutions.
* FLUX    weighting flux.
* TOTAL   total cross sections.
* SIGF    nu*fission cross sections.
* SIGS    scattering cross sections.
* SCAT    scattering transfer matrices (sec,prim,Legendre,dilution).
* SADD    additional cross sections.
* ZDEL    delayed nu-sigf cross sections.
* DELTG   lethargy widths.
* EBIN    fine group energy limits in EV.
* SIGTF   microscopic total x-sections in the fine groups.
* SIGSF   microscopic P0 scattering x-sections in the fine groups.
* AWR     mass ratio for current isotope.
* ISMIN   minimum secondary group corresponding to each primary group
* ISMAX   maximum secondary group corresponding to each primary group
* GOLD    method flag: =-999.0 to use the Ribox approach; =1.0 to use
*         the ST model.
* IPRECI  accuracy index for probability tables in CALENDF.
*
*Parameters: output
* NOR     number of subgroups in each group.
*
*Parameters: scratch
* PRTSIG
* WSLD
* DELTA
* UUU
* STIS
* SIGAF
* SIGPF
* PHIMT
* MATRIX
* PRI
* PHI
* PRTPH
* WORK
* CC
* PRTSIW
* PRTABS
* ISM
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER MAXNOR,LPART,MAXTRA,IPLIB,NGRO,NL,NED,NDEL,NDIL,IGRMIN,
     1 IGRMAX,LBIN,NFS(NGRO),IMPX,ISMIN(NL,NGRO),ISMAX(NL,NGRO),IPRECI,
     2 NOR(NGRO),ISM(2,NL)
      REAL DILUT(NDIL+1),FLUX(NGRO,NDIL+1),TOTAL(NGRO,NDIL+1),
     1 SIGF(NGRO,NDIL+1),SIGS(NGRO,NL,NDIL+1),SCAT(NGRO,NGRO,NL,NDIL+1),
     2 SADD(NGRO,NED,NDIL+1),ZDEL(NGRO,NDEL,NDIL+1),DELTG(NGRO),
     3 EBIN(LBIN+1),SIGTF(LBIN),SIGSF(LBIN),AWR,GOLD(NGRO),
     4 PRTSIG(MAXNOR,3+NL+NL*LPART+NED+NDEL),WSLD(MAXNOR**2),
     5 DELTA(LBIN),UUU(LBIN),STIS(LBIN),SIGAF(LBIN),
     6 SIGPF(LBIN,LPART*NL),PRI(MAXTRA,NL),PRTPH(MAXNOR,NDIL,NL),
     7 PRTSIW(MAXNOR),PRTABS(MAXNOR)
      LOGICAL LSCAT(NL),LSIGF,LADD(NED)
      CHARACTER HNAMIS*12
      DOUBLE PRECISION PHIMT(MAXNOR),MATRIX(MAXNOR,MAXNOR+1),
     1 PHI(LBIN,NDIL,NL),WORK(NDIL+1,MAXNOR),CC(MAXNOR)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (IALTER=0,MAXDIL=45)
      CHARACTER HSMG*131,TAG*2
      LOGICAL LNORAJ,LPHYS,LDIL(MAXDIL+1)
      INTEGER IPERD(MAXDIL+1)
      REAL XSREF(MAXDIL),TEST(6),DILUT2(MAXDIL+1)
      DOUBLE PRECISION SIGTI2,SIGAI2,SIGTIN,SIGAIN,DELMAC,T,T0,
     1 ACCUM1,ACCUM2,ACCUM3,ACCUM4,ERROR1,ERROR2,BB(MAXDIL+1)
      COMMON BASE(1)
*
      IF(NDIL.GT.MAXDIL) CALL XABORT('LIBFQD: MAXDIL OVERFLOW.')
*----
*  NORMALIZE THE BIN-TYPE DATA AND COMPUTE DELTA AND SIGAF.
*----
      IBIN=0
      DELMIN=1.0E10
      DO 40 IGRP=IGRMIN,IGRMAX
      SIGTMI=1.0E10
      SIGAMI=1.0E10
      SIGTIN=0.0D0
      SIGAIN=0.0D0
      SIGTI2=0.0D0
      SIGAI2=0.0D0
      DO 20 IGF=1,NFS(IGRP)
      DELM=LOG(EBIN(IBIN+IGF)/EBIN(IBIN+IGF+1))
      DELMIN=MIN(DELMIN,DELM)
      SIGTIN=SIGTIN+SIGTF(IBIN+IGF)*DELM
      SIGAIN=SIGAIN+(SIGTF(IBIN+IGF)-SIGSF(IBIN+IGF))*DELM
      SIGTF(IBIN+IGF)=MAX(0.002,SIGTF(IBIN+IGF))
      SIGAF(IBIN+IGF)=SIGTF(IBIN+IGF)-SIGSF(IBIN+IGF)
      SIGTMI=MIN(SIGTMI,SIGTF(IBIN+IGF))
      SIGAMI=MIN(SIGAMI,SIGAF(IBIN+IGF))
      SIGTI2=SIGTI2+SIGTF(IBIN+IGF)*DELM
      SIGAI2=SIGAI2+SIGAF(IBIN+IGF)*DELM
      UUU(IBIN+IGF)=LOG(EBIN(1)/EBIN(IBIN+IGF+1))
   20 DELTA(IBIN+IGF)=DELM
      DO 30 IGF=1,NFS(IGRP)
      SIGTF(IBIN+IGF)=SIGTF(IBIN+IGF)*(SIGTIN/SIGTI2)
   30 SIGAF(IBIN+IGF)=SIGAF(IBIN+IGF)*(SIGAIN/SIGAI2)
   40 IBIN=IBIN+NFS(IGRP)
*----
*  ASSUME THAT THE ELEMENTARY LETHARGY WIDTH DELI IS A RATIONAL FRACTION
*  OF THE LETHARGY UNIT. CHECK THIS ASSUMPTION.
*----
      DELI=1.0/REAL(INT(1.00001/DELMIN))
      IBIN=0
      ERR=0.0
      DO 60 IGRP=IGRMIN,IGRMAX
      DO 50 IGF=1,NFS(IGRP)
      LARGH=INT(DELTA(IBIN+IGF)/DELI+0.1)
   50 ERR=MAX(ERR,ABS(DELTA(IBIN+IGF)/DELI-REAL(LARGH)))
   60 IBIN=IBIN+NFS(IGRP)
      IF((IMPX.GT.0).OR.(ERR.GT.0.05)) THEN
         WRITE(6,'(/47H LIBFQD: THE ELEMENTARY LETHARGY WIDTH OF ISOTO,
     1   4HPE '',A12,11H'' IS SET TO,1P,E11.4,6H. ERR=,E10.3)') HNAMIS,
     2   DELI,ERR
      ENDIF
      IF(ERR.GT.0.05) THEN
         WRITE(HSMG,'(45HLIBFQD: UNABLE TO SET THE ELEMENTARY LETHARGY,
     1   20H WIDTH FOR ISOTOPE '',A12,2H''.)') HNAMIS
         WRITE(6,'(A)') HSMG
      ENDIF
*----
*  COMPUTE THE PRI ARRAY FOR VARIOUS LEGENDRE ORDERS.
*----
      DO 70 IL=1,NL
      CALL LIBPRI(MAXTRA,DELI,AWR,IALTER,IL-1,N,PRI(1,IL))
   70 CONTINUE
*----
*  MAIN LOOP OVER THE COARSE ENERGY GROUPS.
*----
      CALL XDISET(NOR,NGRO,0)
      CALL LCMSIX(IPLIB,'PT-PHYS',1)
      IBIN=0
      JPLIB=LCMLID(IPLIB,'GROUP',NGRO)
*     ------------------
      DO 710 IGRP=1,NGRO
*----
*  REMOVE BADLY BEHAVED COLLOCATIONS POINTS.
*----
      MDIL=NDIL
      CALL XDLSET(LDIL,NDIL+1,.TRUE.)
      DO 90 IDIL=NDIL,1,-1
      IF(DILUT(IDIL).LT.1.0) THEN
         MDIL=MDIL-1
         LDIL(IDIL)=.FALSE.
      ELSE IF((DILUT(IDIL).GT.1.0E5).AND.(DILUT(IDIL).LT.1.0E10)) THEN
         MDIL=MDIL-1
         LDIL(IDIL)=.FALSE.
      ELSE IF(TOTAL(IGRP,IDIL).LE.0.0) THEN
         MDIL=MDIL-1
         LDIL(IDIL)=.FALSE.
      ENDIF
   90 CONTINUE
      IDD=0
      DO 100 IDIL=1,NDIL+1
      IF(LDIL(IDIL)) THEN
         IDD=IDD+1
         DILUT2(IDD)=DILUT(IDIL)
         IPERD(IDD)=IDIL
      ENDIF
  100 CONTINUE
      IF(IDD.NE.MDIL+1) CALL XABORT('LIBFQD: INTERNAL ERROR.')
*
      LPHYS=(NFS(IGRP).EQ.0).OR.(GOLD(IGRP).NE.-999.0)
      CALL XDRSET(PRTSIG,(3+NL+NL*LPART+NED+NDEL)*MAXNOR,0.0)
*
      IF(IMPX.GT.0) THEN
         WRITE(6,'(/25H LIBFQD: PROCESSING GROUP,I4,14H FOR ISOTOPE '',
     1   A12,2H''.)') IGRP,HNAMIS
      ENDIF
      DO 110 IDIL=1,MDIL
      JDIL=IPERD(IDIL)
  110 XSREF(IDIL)=TOTAL(IGRP,JDIL)-SIGS(IGRP,1,JDIL)
*----
*  COMPUTE THE RESONANT FLUX BY SOLVING A SLOWING-DOWN EQUATION. COMPUTE
*  STIS USING LIBECT AND COMPUTE THE FINE FLUX. NORMALIZE THE RESONANT
*  FLUX TO THE DILUTION-DEPENDENT NJOY COLLISION RATES.
*----
      IF(.NOT.LPHYS) THEN
         DO 150 IDIL=1,MDIL
         T0=0.0D0
         DELMAC=0.0D0
         DO 130 IGF=1,NFS(IGRP)
         DELM=DELTA(IBIN+IGF)
         DELMAC=DELMAC+DELM
         CALL LIBECT(MAXTRA,IBIN+IGF,PRI(1,1),UUU,DELI,DELTA,N+2,1,MML,
     1   STIS)
         PHI(IBIN+IGF,IDIL,1)=DILUT2(IDIL)
         DO 120 J=2,MML
         JGF=IBIN+IGF-J+1
  120    PHI(IBIN+IGF,IDIL,1)=PHI(IBIN+IGF,IDIL,1)+DBLE(STIS(J))*
     1   PHI(JGF,IDIL,1)*(SIGTF(JGF)-SIGAF(JGF))
         PHI(IBIN+IGF,IDIL,1)=PHI(IBIN+IGF,IDIL,1)/(SIGTF(IBIN+IGF)+
     1   DILUT2(IDIL)-DBLE(STIS(1))*(SIGTF(IBIN+IGF)-SIGAF(IBIN+IGF)))
         T0=T0+PHI(IBIN+IGF,IDIL,1)*SIGTF(IBIN+IGF)*DELM
  130    CONTINUE
         JDIL=IPERD(IDIL)
         FACTOR=FLUX(IGRP,JDIL)*TOTAL(IGRP,JDIL)*(DELMAC/T0)
         DO 140 IGF=1,NFS(IGRP)
  140    PHI(IBIN+IGF,IDIL,1)=PHI(IBIN+IGF,IDIL,1)*FACTOR
         DO 150 IL=2,NL
         DO 150 IGF=1,NFS(IGRP)
         BONDAR=SIGTF(IBIN+IGF)+DILUT2(IDIL)
  150    PHI(IBIN+IGF,IDIL,IL)=PHI(IBIN+IGF,IDIL,IL-1)/BONDAR
      ENDIF
*----
*  TEST FINE FLUX.
*----
      IF((IMPX.GT.5).AND.(NFS(IGRP).GT.0)) THEN
         WRITE(6,810) IGRP,HNAMIS
         DO 155 IDIL=1,MDIL
         DELMAC=0.0D0
         T0=0.0D0
         T1=0.0D0
         DO 154 IGF=1,NFS(IGRP)
         DELM=DELTA(IBIN+IGF)
         DELMAC=DELMAC+DELM
         T0=T0+PHI(IBIN+IGF,IDIL,1)*SIGTF(IBIN+IGF)*DELM
  154    T1=T1+PHI(IBIN+IGF,IDIL,1)*SIGAF(IBIN+IGF)*DELM
         JDIL=IPERD(IDIL)
         BTOT=TOTAL(IGRP,JDIL)*FLUX(IGRP,JDIL)
         BABS=(TOTAL(IGRP,JDIL)-SIGS(IGRP,1,JDIL))*FLUX(IGRP,JDIL)
         WRITE(6,'(1X,I5,1P,4E12.4)') IDIL,T0/DELMAC,BTOT,T1/DELMAC,BABS
  155    CONTINUE
         DELMAC=0.0D0
         T0=0.0D0
         T1=0.0D0
         DO 156 IGF=1,NFS(IGRP)
         DELM=DELTA(IBIN+IGF)
         DELMAC=DELMAC+DELM
         T0=T0+SIGTF(IBIN+IGF)*DELM
  156    T1=T1+SIGAF(IBIN+IGF)*DELM
         BTOT=TOTAL(IGRP,NDIL+1)
         BABS=TOTAL(IGRP,NDIL+1)-SIGS(IGRP,1,NDIL+1)
         WRITE(6,'(3X,3HINF,1P,4E12.4)') T0/DELMAC,BTOT,T1/DELMAC,BABS
      ENDIF
*----
*  PROCESS CLASSICAL PROBABILITY TABLE INFORMATION IN TOTAL XS.
*----
      LNORAJ=.TRUE.
      ERROR1=0.0
  160 NPAR=1
      IF(LPHYS) THEN
         NPART=3+NL+NED+NDEL
         DO 170 IL=1,NL
  170    NPART=NPART+MAX(ISMAX(IL,IGRP)-ISMIN(IL,IGRP)+1,0)
         IF(NPART.GT.3+NL+NL*LPART+NED+NDEL) CALL XABORT('LIBFQD: BUG.')
         CALL SETARA(BASE,NDIL+1,ILDIL)
         CALL SETARA(BASE,NDIL+1,IDIL2)
         CALL LIBTAB (IGRP,NGRO,NL,NDIL,NPART,NED,NDEL,HNAMIS,IMPX,
     1   LSCAT,LSIGF,LADD,DILUT,TOTAL,SIGF,SIGS,SCAT,SADD,ZDEL,1.0,
     2   ISMIN,ISMAX,NOR(IGRP),PRTSIG,BASE(ILDIL),BASE(IDIL2))
         CALL RLSARA(BASE(IDIL2))
         CALL RLSARA(BASE(ILDIL))
         DO 180 JNOR=1,NOR(IGRP)
  180    PRTABS(JNOR)=PRTSIG(JNOR,2)-PRTSIG(JNOR,4)
         GO TO 650
      ELSE
         CALL SETARA(BASE,4*MAXNOR,IMOMT)
         CALL SETARA(BASE,2*MAXNOR,IMOMP)
         CALL SETARA(BASE,(NPAR+2)*MDIL,ISEFR)
         CALL LIBMOM(NFS(IGRP),MDIL,NPAR,DELTA(IBIN+1),SIGTF(IBIN+1),
     1   SIGAF(IBIN+1),MAXNOR,DILUT2,BASE(IMOMT),BASE(IMOMP),
     2   BASE(ISEFR))
*
         CALL SETARA(BASE,MAXNOR*MAXNOR*(NPAR+2),IPROSC)
         CALL SETARA(BASE,(NPAR+2)*MDIL,ISEFF)
         CALL LIBCAT(MAXNOR,NPAR,MDIL,BASE(IMOMT),BASE(IMOMP),IPRECI,
     1   LNORAJ,DILUT2,BASE(ISEFR),NOR(IGRP),PRTSIG,BASE(IPROSC),
     2   BASE(ISEFF))
         CALL RLSARA(BASE(ISEFF))
         CALL RLSARA(BASE(IPROSC))
*
         CALL RLSARA(BASE(ISEFR))
         CALL RLSARA(BASE(IMOMP))
         CALL RLSARA(BASE(IMOMT))
      ENDIF
      IF(NOR(IGRP).EQ.0) THEN
         CALL XABORT('LIBFQD: NO SUBGROUPS.')
      ELSE IF(NOR(IGRP).GT.MDIL) THEN
         LNORAJ=.FALSE.
         NOR(IGRP)=MDIL
         GO TO 160
      ENDIF
*----
*  REMOVING SMALL PROBABILITIES.
*----
      INOR=0
  190 INOR=INOR+1
      IF(INOR.GT.NOR(IGRP)) GO TO 210
      IF(ABS(PRTSIG(INOR,1)).LT.1.0E-10) THEN
         DO 200 JNOR=INOR+1,NOR(IGRP)
         DO 200 J=1,NPAR+2
  200    PRTSIG(JNOR-1,J)=PRTSIG(JNOR,J)
         INOR=INOR-1
         NOR(IGRP)=NOR(IGRP)-1
      ENDIF
      GO TO 190
*
  210 IF(.NOT.LPHYS.AND.(IMPX.GT.3)) THEN
         WRITE(6,'(/7X,11HPROBABILITY,7X,5HTOTAL,2X,10HABSORPTION)')
         CALL XDRSET(TEST,3,0.0)
         DO 220 INOR=1,NOR(IGRP)
         TEST(1)=TEST(1)+PRTSIG(INOR,1)
         TEST(2)=TEST(2)+PRTSIG(INOR,1)*PRTSIG(INOR,2)
         TEST(3)=TEST(3)+PRTSIG(INOR,1)*PRTSIG(INOR,3)
         WRITE(6,'(1X,I5,1P,3E12.4)') INOR,(PRTSIG(INOR,J),J=1,3)
  220    CONTINUE
         WRITE(6,'(6H CHECK,1P,3E12.4)') (TEST(J),J=1,3)
         CALL XDRSET(TEST,3,0.0)
         DO 230 I=1,NFS(IGRP)
         TEST(1)=TEST(1)+DELTA(IBIN+I)
         TEST(2)=TEST(2)+SIGTF(IBIN+I)*DELTA(IBIN+I)
  230    TEST(3)=TEST(3)+SIGAF(IBIN+I)*DELTA(IBIN+I)
         DO 240 J=2,3
  240    TEST(J)=TEST(J)/TEST(1)
         TEST(1)=1.0
         WRITE(6,'(6H EXACT,1P,3E12.4)') (TEST(J),J=1,3)
      ENDIF
*----
*  COMPUTE THE REFERENCE SELF-SHIELDED CROSS SECTIONS AT SELECTED
*  VALUES OF THE DILUTION FOR AN HOMOGENEOUS MEDIA. SECOL-TYPE
*  APPROXIMATION.
*----
      IF(IBIN+NFS(IGRP).GT.LBIN) CALL XABORT('LIBFQD: PHI OVERFLOW.')
*
      DO 280 IDIL=1,MDIL
      DO 280 IL=1,NL
      IF(LPHYS) THEN
*        USE A BONDARENKO RESONANT FLUX.
         T0=0.0D0
         DO 250 INOR=1,NOR(IGRP)
         BONDAR=(DILUT2(IDIL)+PRTSIG(INOR,2))**IL
         PRTPH(INOR,IDIL,IL)=DILUT2(IDIL)/BONDAR
  250    T0=T0+PRTSIG(INOR,1)*PRTPH(INOR,IDIL,1)
         IF(IL.EQ.1) BB(IDIL)=FLUX(IGRP,IPERD(IDIL))/T0
         DO 255 INOR=1,NOR(IGRP)
  255    PRTPH(INOR,IDIL,IL)=PRTPH(INOR,IDIL,IL)*BB(IDIL)
      ELSE
*        COMPUTE THE BASE POINTS OF THE RESONANT FLUX.
         JINI=(1-NOR(IGRP))/2
         CALL XDDSET(PHIMT,NOR(IGRP),0.0D0)
         DELMAC=0.0D0
         DO 265 IGF=1,NFS(IGRP)
         DELM=DELTA(IBIN+IGF)
         SIGT=SIGTF(IBIN+IGF)
         DELMAC=DELMAC+DELM
         T0=PHI(IBIN+IGF,IDIL,IL)*DELM
         T=T0
         DO 260 INOR=1-JINI,NOR(IGRP)
         PHIMT(INOR)=PHIMT(INOR)+T
  260    T=T*SIGT
         T=T0/SIGT
         DO 265 INOR=-JINI,1,-1
         PHIMT(INOR)=PHIMT(INOR)+T
  265    T=T/SIGT
         DO 270 INOR=1,NOR(IGRP)
  270    PHIMT(INOR)=PHIMT(INOR)/DELMAC
         CALL LIBMPA(NOR(IGRP),JINI,PRTSIG(1,1),PRTSIG(1,2),PHIMT,
     1   PRTPH(1,IDIL,IL))
      ENDIF
  280 CONTINUE
*----
*  NORMALIZATION OF THE FLUX-RELATED BASE POINTS. THIS NORMALIZATION
*  PERMITS TO RE-OBTAIN THE BASE POINTS IN TOTAL X-SECTION IF THE RMS
*  APPROACH IS APPLIED TO THE PRTPH MATRIX.
*----
      DO 330 IDIL=1,MDIL
      T0=0.0D0
      DO 290 INOR=1,NOR(IGRP)
  290 T0=T0+PRTSIG(INOR,1)*PRTSIG(INOR,2)*PRTPH(INOR,IDIL,1)
      JDIL=IPERD(IDIL)
      FACTOR=FLUX(IGRP,JDIL)*TOTAL(IGRP,JDIL)/T0
      DO 300 IL=1,NL
      DO 300 INOR=1,NOR(IGRP)
  300 PRTPH(INOR,IDIL,IL)=PRTPH(INOR,IDIL,IL)*FACTOR
*
      IF(IMPX.GT.9) THEN
         WRITE(6,'(/7X,11HPROBABILITY,3X,9HFINE-FLUX,4X,9HDILUTION=,1P,
     1   E8.1,5H BARN)') DILUT2(IDIL)
         TEST(1)=0.0
         TEST(2)=0.0
         TEST(3)=0.0
         DO 310 INOR=1,NOR(IGRP)
         PGAR=PRTPH(INOR,IDIL,1)
         TEST(1)=TEST(1)+PRTSIG(INOR,1)
         TEST(2)=TEST(2)+PRTSIG(INOR,1)*PGAR
         TEST(3)=TEST(3)+PRTSIG(INOR,1)*PRTSIG(INOR,2)*PGAR
         WRITE(6,'(1X,I5,1P,2E12.4)') INOR,PRTSIG(INOR,1),PGAR
  310    CONTINUE
         TEST(3)=TEST(3)/TEST(2)
	 TEST(2)=TEST(2)/TEST(1)
         WRITE(6,'(6H CHECK,1P,3E12.4)') (TEST(J),J=1,3)
         IF(.NOT.LPHYS) THEN
            TEST(1)=0.0
            TEST(2)=0.0
            TEST(3)=0.0
            DO 320 IGF=1,NFS(IGRP)
            DELM=DELTA(IBIN+IGF)
            TEST(1)=TEST(1)+DELM
            TEST(2)=TEST(2)+PHI(IBIN+IGF,IDIL,1)*DELM
  320       TEST(3)=TEST(3)+PHI(IBIN+IGF,IDIL,1)*SIGTF(IBIN+IGF)*DELM
            TEST(3)=TEST(3)/TEST(2)
            TEST(2)=TEST(2)/TEST(1)
            TEST(1)=1.0
            WRITE(6,'(6H EXACT,1P,3E12.4)') (TEST(J),J=1,3)
         ENDIF
      ENDIF
  330 CONTINUE
*----
*  USE A ROOT MEAN SQUARE TECHNIQUE TO FIND BASE POINTS OF THE
*  SCATTERING XS VECTOR AND MATRIX CORRELATED TO THE TOTAL XS IN
*  GROUP IGRP. NOTE: PRTPH(INOR,IDIL,1) IS USED INSTEAD OF
*  PRTPH(INOR,IDIL,IL) ON LINE LABELED 340 IN ORDER TO BE CONSISTENT
*  WITH USSIT0 AND USSIT1. THIS MAY CHANGE IN FUTURE.
*----
      IOF1=4
      IOF2=NL+4
      DO 400 IL=1,NL
      IF(LSCAT(IL)) THEN
         DO 340 INOR=1,NOR(IGRP)
         WORK(MDIL+1,INOR)=1.0D0
         DO 340 IDIL=1,MDIL
  340    WORK(IDIL,INOR)=PRTPH(INOR,IDIL,1)
         CALL ALST2F(NDIL+1,MDIL+1,NOR(IGRP),WORK,PHIMT)
         DO 350 IDIL=1,MDIL+1
         JDIL=IPERD(IDIL)
  350    BB(IDIL)=SIGS(IGRP,IL,JDIL)*FLUX(IGRP,JDIL)
         CALL ALST2S(NDIL+1,MDIL+1,NOR(IGRP),WORK,PHIMT,BB,CC)
         DO 360 INOR=1,NOR(IGRP)
  360    PRTSIG(INOR,IOF1)=CC(INOR)/PRTSIG(INOR,1)
         DO 390 JGRP=ISMIN(IL,IGRP),ISMAX(IL,IGRP)
         DO 370 IDIL=1,MDIL+1
         JDIL=IPERD(IDIL)
  370    BB(IDIL)=SCAT(JGRP,IGRP,IL,JDIL)*FLUX(IGRP,JDIL)
         CALL ALST2S(NDIL+1,MDIL+1,NOR(IGRP),WORK,PHIMT,BB,CC)
         DO 380 INOR=1,NOR(IGRP)
  380    PRTSIG(INOR,IOF2)=CC(INOR)/PRTSIG(INOR,1)
  390    IOF2=IOF2+1
      ENDIF
  400 IOF1=IOF1+1
*----
*  COMPUTE THE ROOT MEAN SQUARE COEFFICIENT MATRIX FOR P0 FLUX.
*----
      DO 410 INOR=1,NOR(IGRP)
      WORK(MDIL+1,INOR)=1.0D0
      DO 410 IDIL=1,MDIL
  410 WORK(IDIL,INOR)=PRTPH(INOR,IDIL,1)
      CALL ALST2F(NDIL+1,MDIL+1,NOR(IGRP),WORK,PHIMT)
*----
*  USE A ROOT MEAN SQUARE TECHNIQUE TO FIND BASE POINTS OF THE
*  ABSORPTION XS CORRELATED TO THE TOTAL XS IN GROUP IGRP.
*----
      DO 420 IDIL=1,MDIL+1
      JDIL=IPERD(IDIL)
  420 BB(IDIL)=(TOTAL(IGRP,JDIL)-SIGS(IGRP,1,JDIL))*FLUX(IGRP,JDIL)
      CALL ALST2S(NDIL+1,MDIL+1,NOR(IGRP),WORK,PHIMT,BB,CC)
      DO 430 INOR=1,NOR(IGRP)
  430 PRTABS(INOR)=CC(INOR)/PRTSIG(INOR,1)
*----
*  USE A ROOT MEAN SQUARE TECHNIQUE TO FIND BASE POINTS OF THE NU*SIGF
*  XS CORRELATED TO THE TOTAL XS IN GROUP IGRP.
*----
      IF(LSIGF) THEN
         DO 440 IDIL=1,MDIL+1
         JDIL=IPERD(IDIL)
  440    BB(IDIL)=SIGF(IGRP,JDIL)*FLUX(IGRP,JDIL)
         CALL ALST2S(NDIL+1,MDIL+1,NOR(IGRP),WORK,PHIMT,BB,CC)
         DO 450 INOR=1,NOR(IGRP)
  450    PRTSIG(INOR,3)=CC(INOR)/PRTSIG(INOR,1)
      ENDIF
*----
*  USE A ROOT MEAN SQUARE TECHNIQUE TO FIND BASE POINTS OF THE
*  ADDITIONAL XS CORRELATED TO THE TOTAL XS IN GROUP IGRP.
*----
      DO 510 IED=1,NED
      IF(LADD(IED)) THEN
         DO 490 IDIL=1,MDIL+1
         JDIL=IPERD(IDIL)
  490    BB(IDIL)=SADD(IGRP,IED,JDIL)*FLUX(IGRP,JDIL)
         CALL ALST2S(NDIL+1,MDIL+1,NOR(IGRP),WORK,PHIMT,BB,CC)
         DO 500 INOR=1,NOR(IGRP)
  500    PRTSIG(INOR,IOF2)=CC(INOR)/PRTSIG(INOR,1)
      ENDIF
  510 IOF2=IOF2+1
*----
*  USE A ROOT MEAN SQUARE TECHNIQUE TO FIND BASE POINTS OF THE DELAYED
*  NU*SIGF XS CORRELATED TO THE TOTAL XS IN GROUP IGRP.
*----
      DO 540 IDEL=1,NDEL
      DO 520 IDIL=1,MDIL+1
      JDIL=IPERD(IDIL)
  520 BB(IDIL)=ZDEL(IGRP,IDEL,JDIL)*FLUX(IGRP,JDIL)
      CALL ALST2S(NDIL+1,MDIL+1,NOR(IGRP),WORK,PHIMT,BB,CC)
      DO 530 INOR=1,NOR(IGRP)
  530 PRTSIG(INOR,IOF2)=CC(INOR)/PRTSIG(INOR,1)
  540 IOF2=IOF2+1
*----
*  USE A ROOT MEAN SQUARE TECHNIQUE TO FIND THE ELEMENTS OF THE
*  SLOWING-DOWN RELATED CORRELATED WEIGHT MATRIX AND SECONDARY
*  SCATTERING XS IN GROUP IGRP.
*----
      IF(LPHYS) THEN
        DO 545 INOR=1,NOR(IGRP)
        PRTSIW(INOR)=PRTSIG(INOR,4)
        DO 545 JNOR=1,NOR(IGRP)
  545   WSLD((INOR-1)*NOR(IGRP)+JNOR)=PRTSIG(INOR,1)*PRTSIG(JNOR,1)
      ELSE
        DO 550 INOR=1,NOR(IGRP)
        DO 550 IDIL=1,MDIL
  550   WORK(IDIL,INOR)=PRTPH(INOR,IDIL,1)
        CALL ALST2F(NDIL+1,MDIL,NOR(IGRP),WORK,PHIMT)
*
        DO 590 INOR=1,NOR(IGRP)
        DO 580 IDIL=1,MDIL
        BB(IDIL)=PRTSIG(INOR,1)*(PRTPH(INOR,IDIL,1)*(PRTSIG(INOR,2)+
     1  DILUT2(IDIL))-DILUT2(IDIL))
  580   CONTINUE
        CALL ALST2S(NDIL+1,MDIL,NOR(IGRP),WORK,PHIMT,BB,CC)
        DO 590 I=1,NOR(IGRP)
  590   WSLD((I-1)*NOR(IGRP)+INOR)=CC(I)
*
        DO 610 J=1,NOR(IGRP)
        T0=0.0D0
        DO 595 I=1,NOR(IGRP)
  595   T0=T0+WSLD((J-1)*NOR(IGRP)+I)
        DO 600 I=1,NOR(IGRP)
  600   WSLD((J-1)*NOR(IGRP)+I)=WSLD((J-1)*NOR(IGRP)+I)*(PRTSIG(J,1)/T0)
  610   PRTSIW(J)=T0/PRTSIG(J,1)
      ENDIF
*----
*  SOLVE SUBGROUP FORM OF THE SLOWING-DOWN EQUATION FOR AN HOMOGENEOUS
*  MIXTURE AT SELECTED DILUTIONS.
*----
      ERROR1=-9999.0
      ERROR2=-9999.0
      IDMAX=0
      DO 640 IDIL=1,MDIL
      DO 620 I=1,NOR(IGRP)
      MATRIX(I,NOR(IGRP)+1)=PRTSIG(I,1)*DILUT2(IDIL)
      DO 615 J=1,NOR(IGRP)
  615 MATRIX(I,J)=-WSLD((J-1)*NOR(IGRP)+I)*PRTSIW(J)
  620 MATRIX(I,I)=MATRIX(I,I)+(PRTSIG(I,2)+DILUT2(IDIL))*PRTSIG(I,1)
      CALL ALSBD(NOR(IGRP),1,MATRIX,IER,MAXNOR)
      IF(IER.NE.0) CALL XABORT('LIBFQD: SINGULAR MATRIX(2).')
*----
*  TEST THE ACCURACY OF THE PROBABILITY TABLES FOR THIS ENERGY GROUP.
*----
      ACCUM1=0.0D0
      ACCUM2=0.0D0
      ACCUM3=0.0D0
      ACCUM4=0.0D0
      DO 630 I=1,NOR(IGRP)
      ACCUM1=ACCUM1+PRTSIG(I,1)*PRTABS(I)*MATRIX(I,NOR(IGRP)+1)
      ACCUM2=ACCUM2+PRTSIG(I,1)*MATRIX(I,NOR(IGRP)+1)
      ACCUM3=ACCUM3+PRTSIG(I,1)*PRTABS(I)*PRTPH(I,IDIL,1)
  630 ACCUM4=ACCUM4+PRTSIG(I,1)*PRTPH(I,IDIL,1)
      ACCUM1=ACCUM1/ACCUM2
      ACCUM3=ACCUM3/ACCUM4
      IF(ABS(ACCUM1-XSREF(IDIL))/ABS(XSREF(IDIL)).GT.ERROR1) THEN
         EROLD1=ERROR1
         EROLD2=ERROR2
         IDOMAX=IDMAX
         ERROR1=ABS(ACCUM1-XSREF(IDIL))/ABS(XSREF(IDIL))
         ERROR2=ABS(ACCUM3-XSREF(IDIL))/ABS(XSREF(IDIL))
         IDMAX=IDIL
      ELSE IF(ABS(ACCUM1-XSREF(IDIL))/ABS(XSREF(IDIL)).GT.EROLD1) THEN
         EROLD1=ABS(ACCUM1-XSREF(IDIL))/ABS(XSREF(IDIL))
         EROLD2=ABS(ACCUM3-XSREF(IDIL))/ABS(XSREF(IDIL))
         IDOMAX=IDIL
      ENDIF
  640 CONTINUE
      IF(IMPX.GT.0) THEN
         TAG='=>'
         IF(LPHYS) TAG='--'
         WRITE(6,800) TAG,IGRP,NOR(IGRP),ERROR1*100.0,ERROR2*100.0,
     1   DILUT2(IDMAX),EROLD1*100.0,EROLD2*100.0,DILUT2(IDOMAX)
      ENDIF
      IF(ERROR1.GT.0.01) THEN
         WRITE(HSMG,'(42HLIBFQD: UNABLE TO COMPUTE THE PROBABILITY ,
     1   15HTABLES IN GROUP,I4,17H. TABLE ACCURACY=,1P,E9.2,2H %,
     2   10H ISOTOPE='',A12,2H''.)') IGRP,ERROR1*100.0,HNAMIS
         WRITE(6,'(1X,A)') HSMG
      ENDIF
*
  650 IF((IMPX.GT.2).AND.(NOR(IGRP).GT.1)) THEN
         WRITE(6,'(/7H GROUP=,I4,16H TABLE ACCURACY=,1P,E9.2,2H %)')
     1   IGRP,ERROR1*100.0
         WRITE(6,'(/7X,11HPROBABILITY,7X,5HTOTAL,2X,10HABSORPTION,2X,
     1   10HNU-FISSION,2X,10HSCATTERING,12(1H.))')
         CALL XDRSET(TEST,6,0.0)
         IOF=NL+IGRP-ISMIN(1,IGRP)
         DO 670 JNOR=1,NOR(IGRP)
         TEST(1)=TEST(1)+PRTSIG(JNOR,1)
         TEST(2)=TEST(2)+PRTSIG(JNOR,1)*PRTSIG(JNOR,2)
         TEST(3)=TEST(3)+PRTSIG(JNOR,1)*PRTABS(JNOR)
         TEST(4)=TEST(4)+PRTSIG(JNOR,1)*PRTSIG(JNOR,3)
         TEST(5)=TEST(5)+PRTSIG(JNOR,1)*PRTSIG(JNOR,IOF+4)
         TEST(6)=TEST(6)+PRTSIG(JNOR,1)*PRTSIG(JNOR,IOF+5)
         WRITE(6,'(1X,I5,1P,6E12.4)') JNOR,(PRTSIG(JNOR,J),J=1,2),
     1   PRTABS(JNOR),PRTSIG(JNOR,3),(PRTSIG(JNOR,NL+J),J=4,5)
  670    CONTINUE
         WRITE(6,'(6H CHECK,1P,6E12.4)') (TEST(J),J=1,6)
         TEST(1)=1.0
         TEST(2)=TOTAL(IGRP,NDIL+1)
         TEST(3)=TOTAL(IGRP,NDIL+1)-SIGS(IGRP,1,NDIL+1)
         TEST(4)=SIGF(IGRP,NDIL+1)
         TEST(5)=SCAT(IGRP,IGRP,1,NDIL+1)
         TEST(6)=SCAT(IGRP+1,IGRP,1,NDIL+1)
         WRITE(6,'(6H EXACT,1P,6E12.4)') (TEST(I),I=1,6)
      ENDIF
*
      IF(NOR(IGRP).GT.1) THEN
*        SAVE THE PROBABILITY TABLE INTO IPLIB.
         KPLIB=LCMDIL(JPLIB,IGRP)
         NPART=3+NL+NED+NDEL
         DO 700 IL=1,NL
         ISM(1,IL)=ISMIN(IL,IGRP)
         ISM(2,IL)=ISMAX(IL,IGRP)
  700    NPART=NPART+MAX(0,(ISMAX(IL,IGRP)-ISMIN(IL,IGRP)+1))
         CALL LCMPUT(KPLIB,'PROB-TABLE',NPART*MAXNOR,2,PRTSIG)
         IF(.NOT.LPHYS) THEN
            CALL LCMPUT(KPLIB,'SIGQT-SIGS',NOR(IGRP),2,PRTSIW)
            CALL LCMPUT(KPLIB,'SIGQT-SLOW',NOR(IGRP)**2,2,WSLD)
         ENDIF
         CALL LCMPUT(KPLIB,'ISM-LIMITS',2*NL,1,ISM)
      ENDIF
  710 IBIN=IBIN+NFS(IGRP)
      CALL LCMPUT(IPLIB,'NOR',NGRO,1,NOR)
      CALL LCMSIX(IPLIB,' ',2)
      RETURN
*
  800 FORMAT(/9H LIBFQD: ,A2,6HGROUP=,I3,7H ORDER=,I2,7H ERROR=,1P,
     1 E9.2,4H % (,E9.2,15H %) AT DILUTION,E10.3,5H BARN/29X,
     2 7H ERROR=,1P,E9.2,4H % (,E9.2,15H %) AT DILUTION,E10.3,6H BARN.)
  810 FORMAT(/32H LIBFQD: TEST FINE FLUX IN GROUP,I5,14H FOR ISOTOPE ',
     1 A12,2H':/9H DILUTION,16X,5HTOTAL,14X,10HABSORPTION/11X,7HAUTOLIB,
     2 8X,4HNJOY,5X,7HAUTOLIB,8X,4HNJOY)
      END
