*DECK LIBMIX
      SUBROUTINE LIBMIX(IPLIB,NBMIX,NGROUP,NBISO,ISONAM,MIX,DEN,MASK,
     1 MASKL)
*
*-----------------------------------------------------------------------
*
*Purpose:
* transformation of the isotope ordered microscopic cross sections to
* group ordered macroscopic cross sections (part 1).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   pointer to the lattice microscopic cross section library
*         (L_LIBRARY signature).
* NBMIX   number of material mixtures.
* NGROUP  number of energy groups.
* NBISO   number of isotopes present in the calculation domain.
* ISONAM  alias name of isotopes.
* MIX     mixture number of each isotope (can be zero).
* DEN     density of each isotope.
* MASK    mixture mask (=.true. if a mixture is to be made).
* MASKL   group mask (=.true. if an energy group is to be treated).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,NBMIX,NGROUP,NBISO,ISONAM(3,NBISO),MIX(NBISO)
      REAL DEN(NBISO)
      LOGICAL MASK(NBMIX),MASKL(NGROUP)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NBLK=50,NSTATE=40)
      LOGICAL LSAME
      INTEGER ISTATE(NSTATE)
      COMMON BASE(1)
*----
*  RECOVER SOME LIBRARY PARAMETERS.
*----
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      NL=ISTATE(4)
      ITRANC=ISTATE(5)
      IPROB=ISTATE(6)
      NED=ISTATE(13)
      NDEL=ISTATE(19)
      CALL SETARA(BASE,2*NED,IJNED)
      CALL LCMGET(IPLIB,'ADDXSNAME-P0',BASE(IJNED))
*----
*  TRANSPOSE THE MICROSCOPIC CROSS SECTIONS TO ADJOINT ORDERING.
*----
      IF(IPROB.EQ.1) THEN
         CALL SETARA (BASE,NGROUP*2,IGA1)
         CALL SETARA (BASE,NGROUP*NGROUP,IGA2)
         CALL SETARA (BASE,NGROUP*NL,ISIGS)
         CALL SETARA (BASE,NGROUP*NGROUP*NL,ISCAT)
         CALL SETARA (BASE,NL,IITYPR)
         CALL LIBADJ (IPLIB,NGROUP,NBISO,NL,NDEL,ISONAM,NED,BASE(IJNED),
     1   BASE(IGA1),BASE(IGA2),BASE(ISIGS),BASE(ISCAT),BASE(IITYPR))
         CALL RLSARA (BASE(IITYPR))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(ISIGS))
         CALL RLSARA (BASE(IGA2))
         CALL RLSARA (BASE(IGA1))
      ENDIF
*----
*  COMPUTE THE MAXIMUM NUMBER OF FISSIONABLE ISOTOPES IN A MIXTURE.
*----
      DO 10 I=1,NBISO
      IF(MIX(I).GT.NBMIX) CALL XABORT('LIBMIX: NBMIX OVERFLOW.')
   10 CONTINUE
      CALL SETARA (BASE,NBMIX*NBISO,IINDFI)
      CALL LIBNFI (IPLIB,NGROUP,NBISO,NBMIX,NDEL,ISONAM,MIX,MAXNFI,
     1 LSAME,BASE(IINDFI))
      CALL RLSARA (BASE(IINDFI))
*----
*  BUILD THE MACROSCOPIC CROSS SECTIONS.
*----
      CALL SETARA (BASE,NGROUP+1,IGA1)
      CALL SETARA (BASE,NGROUP*NGROUP,IGA2)
      CALL SETARA (BASE,NDEL*MAXNFI,IGA3)
      CALL SETARA (BASE,NBMIX*(NBLK+1),IGAR)
      CALL SETARA (BASE,NBMIX*NGROUP*NBLK,IGAF)
      CALL SETARA (BASE,NGROUP*NBMIX,ISCAT)
      CALL SETARA (BASE,NBMIX*NBLK,INJJM)
      CALL SETARA (BASE,NBMIX*NBLK,IIJJM)
      CALL SETARA (BASE,NBMIX,IIPOS)
      CALL SETARA (BASE,NBMIX*NGROUP*NL,ICHECK)
      CALL SETARA (BASE,NBMIX*MAXNFI,IINDFI)
      CALL SETARA (BASE,NBMIX*MAXNFI*NGROUP*(NDEL+1),IZNUS)
      CALL SETARA (BASE,NBMIX*MAXNFI*NGROUP*(NDEL+1),IZCHI)
      CALL SETARA (BASE,NGROUP,IIJJ)
      CALL SETARA (BASE,NGROUP,INJJ)
      CALL SETARA (BASE,NBISO,ILMADE)
      CALL SETARA (BASE,NGROUP,IIPGRP)
      CALL LIBDEN (IPLIB,NGROUP,NBISO,NBMIX,NL,NDEL,ISONAM,MIX,DEN,
     1 MASK,MASKL,NED,BASE(IJNED),ITRANC,MAXNFI,LSAME,BASE(IGA1),
     2 BASE(IGA2),BASE(IGA3),BASE(IGAR),BASE(IGAF),BASE(ISCAT),
     3 BASE(INJJM),BASE(IIJJM),BASE(IIPOS),BASE(ICHECK),BASE(IINDFI),
     4 BASE(IZNUS),BASE(IZCHI),BASE(IIJJ),BASE(INJJ),BASE(ILMADE),
     5 BASE(IIPGRP))
      CALL RLSARA (BASE(IIPGRP))
      CALL RLSARA (BASE(ILMADE))
      CALL RLSARA (BASE(INJJ))
      CALL RLSARA (BASE(IIJJ))
      CALL RLSARA (BASE(IZCHI))
      CALL RLSARA (BASE(IZNUS))
      CALL RLSARA (BASE(IINDFI))
      CALL RLSARA (BASE(ICHECK))
      CALL RLSARA (BASE(IIPOS))
      CALL RLSARA (BASE(IIJJM))
      CALL RLSARA (BASE(INJJM))
      CALL RLSARA (BASE(ISCAT))
      CALL RLSARA (BASE(IGAF))
      CALL RLSARA (BASE(IGAR))
      CALL RLSARA (BASE(IGA3))
      CALL RLSARA (BASE(IGA2))
      CALL RLSARA (BASE(IGA1))
*----
*  TRANSPOSE THE MICROSCOPIC CROSS SECTIONS BACK TO FORWARD ORDERING.
*----
      IF(IPROB.EQ.1) THEN
         CALL SETARA (BASE,NGROUP*2,IGA1)
         CALL SETARA (BASE,NGROUP*NGROUP,IGA2)
         CALL SETARA (BASE,NGROUP*NL,ISIGS)
         CALL SETARA (BASE,NGROUP*NGROUP*NL,ISCAT)
         CALL SETARA (BASE,NL,IITYPR)
         CALL LIBADJ (IPLIB,NGROUP,NBISO,NL,NDEL,ISONAM,NED,BASE(IJNED),
     1   BASE(IGA1),BASE(IGA2),BASE(ISIGS),BASE(ISCAT),BASE(IITYPR))
         CALL RLSARA (BASE(IITYPR))
         CALL RLSARA (BASE(ISCAT))
         CALL RLSARA (BASE(ISIGS))
         CALL RLSARA (BASE(IGA2))
         CALL RLSARA (BASE(IGA1))
      ENDIF
      CALL RLSARA(BASE(IJNED))
      RETURN
      END
