*DECK MAC
      SUBROUTINE MAC(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* macroscopic cross sections and diffusion coefficients input module.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert and G. Marleau
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1) : create or modification type(L_MACROLIB).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER      NENTRY                  	
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(NSTATE=40,IOUT=6)
      CHARACTER TEXT12*12,HSIGN*12
      INTEGER IDATA(NSTATE),KCHAR(3)
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.EQ.0) CALL XABORT('MAC: PARAMETER EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2)) CALL XABORT('MAC: LCM '
     1 //'OBJECT EXPECTED AT LHS.')
      IF((JENTRY(1).NE.0).AND.(JENTRY(1).NE.1)) CALL XABORT('MAC: ENTR'
     1 //'Y IN CREATE OR MODIFICATION MODE EXPECTED.')
      ITYPE=JENTRY(1)
      IPLIST=KENTRY(1)
*----
*  READ THE INPUT DATA
*----
*     DEFAULT OPTIONS:
      IMPX=1
      IF(ITYPE.EQ.0) THEN
         INDREC=1
         NANISO=1
         NGROUP=0
         NBMIX=0
         NIFISS=0
         NEDMAC=0
         ITRANC=0
         NDELG=0
         NSTEP=0
      ELSE IF(ITYPE.EQ.1) THEN
         CALL LCMGET(IPLIST,'SIGNATURE',KCHAR)
         WRITE(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         IF(HSIGN.EQ.'L_LIBRARY') THEN
            CALL LCMSIX(IPLIST,'MACROLIB',1)
         ELSE IF(HSIGN.NE.'L_MACROLIB') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('MAC: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_MACROLIB EXPECTED.')
         ENDIF
         INDREC=2
         CALL LCMGET(IPLIST,'STATE-VECTOR',IDATA)
         NGROUP=IDATA(1)
         NBMIX=IDATA(2)
         NANISO=IDATA(3)
         NIFISS=IDATA(4)
         NEDMAC=IDATA(5)
         ITRANC=IDATA(6)
         NDELG=IDATA(7)
         NSTEP=IDATA(11)
      ENDIF
*----
*  PROCESS THE MAC: INPUT DATA
*----
      CALL MACDRV(IPLIST,INDREC,IMPX,NBMIX,NGROUP,NANISO,NIFISS,NEDMAC,
     1 ITRANC,NDELG,NSTEP)
*
      IF(ITYPE.EQ.0) THEN
         HSIGN='L_MACROLIB'
         READ(HSIGN,'(3A4)') (KCHAR(I1),I1=1,3)
         CALL LCMPUT(IPLIST,'SIGNATURE',3,3,KCHAR)
         CALL XDISET(IDATA,NSTATE,0)
         IDATA(1)=NGROUP
         IDATA(2)=NBMIX
         IDATA(3)=NANISO
         IDATA(4)=NIFISS
         IDATA(5)=NEDMAC
         IDATA(6)=ITRANC
         IDATA(7)=NDELG
         IDATA(11)=NSTEP
         IF(ITRANC.NE.0) IDATA(6)=2
         JPLIST=LCMGID(IPLIST,'GROUP')
         KPLIST=LCMGIL(JPLIST,1)
         CALL LCMLEN(KPLIST,'DIFF',ILONG,ITYLCM)
         IF(ILONG.GT.0) IDATA(9)=1
         CALL LCMLEN(KPLIST,'DIFFX',ILONG,ITYLCM)
         IF(ILONG.GT.0) IDATA(9)=2
         CALL LCMPUT(IPLIST,'STATE-VECTOR',NSTATE,1,IDATA)
      ENDIF
      IF(IMPX.GT.1) CALL LCMLIB(IPLIST)
      IF(IMPX.GT.0) WRITE(IOUT,100) IMPX,(IDATA(I),I=1,7),IDATA(9),
     1 IDATA(11)
      CALL LCMSIX(IPLIST,' ',0)
      RETURN
*
  100 FORMAT(/8H OPTIONS/8H -------/
     1 7H IMPX  ,I6,30H   (0=NO PRINT/1=SHORT/2=MORE)/
     2 7H NGROUP,I6,28H   (NUMBER OF ENERGY GROUPS)/
     3 7H NBMIX ,I6,39H   (NUMBER OF MIXTURES IN THE MACROLIB)/
     4 7H NANISO,I6,34H   (MAXIMUM SCATTERING ANISOTROPY)/
     5 7H NIFISS,I6,45H   (MAXIMUM NUMBER OF FISSILE ISOTOPES IN A M,
     6 7HIXTURE)/
     7 7H NEDMAC,I6,34H   (NUMBER OF CROSS SECTION EDITS)/
     8 7H ITRANC,I6,45H   (0=NO TRANSPORT CORRECTION/1=APOLLO TYPE/2,
     9 43H=RECOVER FROM LIBRARY/4=LEAKAGE CORRECTION)/
     1 7H NLG   ,I6,39H   (NUMBER OF DELAYED PRECURSOR GROUPS)/
     2 7H ILEAK ,I6,40H   (1=DIFF AVAILABLE; 2=DIFFX AVAILABLE)/
     3 7H NSTEP ,I6,39H   (NUMBER OF PERTURBATION DIRECTORIES))
      END
