*DECK MCCGF
      SUBROUTINE MCCGF(IPSYS,NPSYS,IPTRK,IFTRAK,IPMACR,IMPX,NGRP,IDIR,
     1                 NBREG,NUNKNO,NLF,LEXAC,MAT,VOL,KEYFLX,FUNKNO,
     2                 SUNKNO,TITR,REBFLG)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solve one group transport equation for fluxes using the method of
* characteristics. Vectorial version.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert and R. Le Tellier
*
*Parameters: input/output
* IPSYS  pointer to the assembly LCM object (L_PIJ signature). IPSYS is
*        a list of directories.
* NPSYS  index array pointing to the IPSYS list component corresponding
*        to each energy group. Set to zero if a group is not to be
*        processed. Usually, NPSYS(I)=I.
* IPTRK  pointer to the tracking (L_TRACK signature).
* IPMACR pointer to the macrolib LCM object.
* IFTRAK tracking file unit number.
* IMPX   print flag (equal to zero for no print).
* NGRP   number of energy groups.
* IDIR   =0 only for MCCG3D.
* NBREG  total number of volumes for which specific values of the
*        neutron flux and reactions rates are required.
* NUNKNO total number of unknowns in vectors SUNKNO and FUNKNO.
* MAT    index-number of the mixture type assigned to each volume.
* NLF     number of Legendre orders for the flux.
* VOL    volumes.
* KEYFLX position of flux elements in FUNKNO vector.
* FUNKNO unknown vector.
* SUNKNO input source vector.
* TITR   title.
* LEXAC   type of exponential function calculation (=.false. to compute
*         exponential functions using tables).
* REBFLG  ACA or SCR rebalancing flag.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPSYS,NGRP,NPSYS(NGRP),IPTRK,IFTRAK,IMPX,IPMACR,IDIR,
     1 NBREG,NUNKNO,MAT(NBREG),NLF,KEYFLX(NBREG,NLF*(NLF+1)/2)
      REAL VOL(NBREG),FUNKNO(NUNKNO,NGRP),SUNKNO(NUNKNO,NGRP)
      CHARACTER TITR*72
      LOGICAL LEXAC,REBFLG
*----
*  LOCAL VARIABLES
*----
      PARAMETER  (IUNOUT=6,NSTATE=40,NALB=6,MXNMU=64)
      CHARACTER   TEXT4*4,TOPT*72
      INTEGER     JPAR(NSTATE),PACA,STIS,TRTY,MATALB
      REAL        ZREAL(2),ALBEDO(NALB),HDD
      LOGICAL     CYCLIC,LVOID,LEXF
      EXTERNAL    MOCFFI,MOCFFA,MOCFFIS,MOCFFAS,MOCFFIT,MOCFFAT,
     1            MCGFFI,MCGFFA,MCGFFIS,MCGFFAS,MCGFFIT,MCGFFAT
      EXTERNAL    MOCSCA,MOCDDF,MOCSCE,MOCSCAS,MOCDDFS,MOCSCES,MOCSCAT,
     1            MOCDDFT,MOCSCET,
     2            MCGSCA,MCGDDF,MCGSCE,MCGSCAS,MCGDDFS,MCGSCES,MCGSCAT,
     3            MCGDDFT,MCGSCET
*
      INTEGER     IBASE(1)
      LOGICAL     LBASE(1)
      COMMON      BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
      EQUIVALENCE (BASE(1),LBASE(1))
*
      IF(IDIR.NE.0) CALL XABORT('MCCGF: PIJK FORBIDDEN HERE')
*---
* DETERMINE THE NUMBER OF GROUPS TO BE PROCESSED
* RECOVER FLUXES FROM A PREVIOUS SELF-SHIELDING CALCULATION IF AVAILABLE
*---
      NGEFF=0
      DO IG=1,NGRP
         IOFSET=NPSYS(IG)
         IF(IOFSET.NE.0) THEN
            NGEFF=NGEFF+1
            JPSYS=LCMGIL(IPSYS,IOFSET)
            CALL LCMLEN(JPSYS,'FUNKNO$USS',ILENG,ITYLCM)
            IF(ILENG.EQ.NUNKNO) THEN
               CALL LCMGET(JPSYS,'FUNKNO$USS',FUNKNO(1,IG))
            ENDIF
         ENDIF
      ENDDO
*---
* RECOVER POINTERS TO EACH GROUP PROPERTIES
* CREATE AN INDEX FOR THE GROUPS TO BE PROCESSED
*---
      CALL SETARA(IBASE,NGEFF,INGIND)
      CALL SETARA(IBASE,NGEFF,KPSYS)
      II=1
      DO IG=1,NGRP
         IOFSET=NPSYS(IG)
         IF(IOFSET.NE.0) THEN
            IBASE(INGIND-1+II)=IG
            IBASE(KPSYS-1+II)=LCMGIL(IPSYS,IOFSET)
            II=II+1
         ENDIF
      ENDDO
*----
*  RECOVER MCCG SPECIFIC PARAMETERS
*----
*     check for cross-sections in SYS object
      JPSYS=IBASE(KPSYS)
      CALL LCMLEN(JPSYS,'DRAGON-TXSC',ILENG,ITYLCM)
      NBMIX=ILENG-1
      CALL LCMGET(IPTRK,'STATE-VECTOR',JPAR)
      IF(JPAR(4).GT.NBMIX) CALL XABORT('MCCGF: MIXTURE OVERFLOW.')
*     check for a tracking binary file
      IF(IFTRAK.LE.0) CALL XABORT('MCCGF: INVALID TRACKING FILE.')
*     recover state-vector information
      NREG=JPAR(1)
      NSOUT=JPAR(5)
      NFI=NREG+NSOUT
      IF(JPAR(2).GT.NUNKNO) 
     1 CALL XABORT('MCCGF: UNKNOWN VECTOR OVERFLOW.')
      NANI=JPAR(6)
      TRTY=JPAR(9)
      IF(TRTY.EQ.1) THEN
         CYCLIC=.TRUE.
         NLONG=NREG
      ELSE
         CYCLIC=.FALSE.
         NLONG=NFI
      ENDIF
      NMU=JPAR(27)
      IF (NMU.GT.MXNMU)
     1 CALL XABORT('MCCGF: POLAR ANGLE QUADRATURE OVERFLOW') 
      NMAX=JPAR(25)
*     
      MAXI=JPAR(33)
      STIS=JPAR(35)
      IAAC=JPAR(22)
      LC=JPAR(26)
      KRYL=JPAR(23)
      IDIFC=JPAR(24)
      ISCR=JPAR(28)
      LPS=JPAR(29)
      PACA=JPAR(30)
      LEXF=(JPAR(32).EQ.1)
*     to be coherent with the exponential function used for the Pjj calculation
      IF ((LEXAC).AND.(.NOT.LEXF).AND.(STIS.EQ.1)) STIS=0 
      NPJJM=JPAR(36)
*     recover real parameters
      CALL LCMGET(IPTRK,'REAL-PARAM',ZREAL)
      EPSI=ZREAL(1)
!!! temporary
      HDD=ZREAL(2)
      IF (HDD.GT.0.0) THEN
         ISCH=0
      ELSEIF (LEXF) THEN
         ISCH=-1
      ELSE
         ISCH=1
      ENDIF
!!!
*----
* RECOVER TRACKING FILE INFORMATION
*----
      CALL SETARA(IBASE,NFI+1,MATALB)
      REWIND IFTRAK
      READ(IFTRAK) TEXT4,NCOMNT,NBTR
      DO ICOM=1,NCOMNT
         READ(IFTRAK)
      ENDDO
      READ(IFTRAK) NDIM,ISPEC,NREG2,NSOUT2,NALBG,NCOR,NANGL,MXSEG
      IF (NCOR.NE.1) 
     1 CALL XABORT('MCCGF: INVALID TRACKING FILE: NCOR.NE.1')
      READ(IFTRAK)
      READ(IFTRAK) (IBASE(MATALB+JJ),JJ=0,NFI)
      READ(IFTRAK)
      READ(IFTRAK)
      CALL SETARA(BASE,NANGL,ICAZ)
      IF (NDIM.EQ.2) THEN
         CALL SETARA(BASE,NMU,ICPO)
         CALL LCMGET(IPTRK,'XMU$MCCG',BASE(ICPO))
         READ(IFTRAK) (BASE(ICAZ-1+JJ),DUMMY,JJ=1,NANGL)
      ELSE ! NDIM.EQ.3
         CALL SETARA(BASE,NANGL,ICPO)
         READ(IFTRAK) (BASE(ICPO-1+JJ),BASE(ICAZ-1+JJ),DUMMY,
     1                 JJ=1,NANGL)
         DO JJ=1,NANGL
            BASE(ICAZ-1+JJ)=BASE(ICAZ-1+JJ)/
     1                      SQRT(1.0-BASE(ICPO-1+JJ)*BASE(ICPO-1+JJ))
         ENDDO
      ENDIF
*----
* RECOVER TRACKING TABLE INFORMATION
*----
*     recover albedo
      CALL LCMLEN(IPTRK,'ALBEDO',ILNLCM,ITYLCM)
      IF(ILNLCM.NE.0) THEN
         IF(ILNLCM.NE.NALB) THEN
            CALL XABORT('MCCGF: INCOMPATIBLE NUMBER OF ALBEDOS.')
         ENDIF
         CALL LCMGET(IPTRK,'ALBEDO',ALBEDO)
      ELSE
         CALL XDRSET(ALBEDO,NALB,1.0)
      ENDIF
*     recover polar quadrature
      CALL LCMIOF(IPTRK,'WZMU$MCCG',IWZMU)
      CALL LCMIOF(IPTRK,'ZMU$MCCG',IZMU)
*     recover modified MATALB and VOLSUR
      IF (CYCLIC) THEN
         CALL LCMIOF(IPTRK,'VOLUME',IV)
         CALL LCMIOF(IPTRK,'MATCOD',INZON)
      ELSE
         CALL LCMIOF(IPTRK,'V$MCCG',IV)
         CALL LCMIOF(IPTRK,'NZON$MCCG',INZON)
      ENDIF
*     recover index for the currents in FUNKNO (non-cyclic case)
      IF (.NOT.CYCLIC) CALL LCMIOF(IPTRK,'KEYCUR$MCCG',KEYCUR)
*     calculate the number of anisotropic modes
      IF (NDIM.EQ.2) THEN
         NFUNL=NANI*(NANI+1)/2
         NMOD=4
      ELSE ! NDIM.EQ.3
         NFUNL=NANI*NANI
         NMOD=8
      ENDIF
      CALL LCMIOF(IPTRK,'ISGNR$MCCG',ISGNR)
*----
*  CONSTRUCT TOTAL CROSS SECTIONS ARRAY AND CHECK FOR ZERO CROSS SECTION
*----
      CALL SETARA(BASE,(NBMIX+7)*NGEFF,ISIGAL)
      CALL MCGSIG(NBMIX,NGEFF,IBASE(KPSYS),ALBEDO,BASE(ISIGAL),LVOID)
      IF ((LVOID).AND.(STIS.EQ.-1)) THEN
         IF (IMPX.GT.0) 
     1       WRITE(6,*) 'VOID EXISTS -> STIS SET TO 1 INSTEAD OF -1'
         STIS=1
      ENDIF
      ISCH=ISCH+10*STIS
!      write(*,*) 'ISCH=',ISCH,' CYCLIC=',CYCLIC,' NDIM=',NDIM,
!     1           ' NLONG=',NLONG,' NBMIX=',NBMIX
*----
*  PERFORM INNER ITERATIONS TO COMPUTE THE NEUTRON FLUX IN THE DIFFERENT GROUPS
*----
      CALL SETARA(BASE,(MAXI*NGEFF),IREPS)
      CALL SETARA(BASE,NGEFF,IEPS)
      CALL SETARA(IBASE,NGEFF,ITST)
      CALL SETARA(LBASE,NGEFF,INCONV)
      DO II=1,NGEFF
         LBASE(INCONV-1+II)=.TRUE.
      ENDDO
      LNCONV=NGEFF
*
      IF (IDIFC.EQ.1) THEN
*     ------------------------------------
*     ACA-Simplified Transport Calculation
*     ------------------------------------
         TOPT='ACA-SIMPLIFIED TRANSPORT OPERATOR'
         CALL MCGFLS(IMPX,IPTRK,IPMACR,NUNKNO,NFI,NBREG,NLONG,NBMIX,
     1        NGRP,NGEFF,LC,PACA,IBASE(INZON),KEYFLX,IBASE(KEYCUR),
     2        IBASE(INGIND),IBASE(KPSYS),LBASE(INCONV),EPSI,MAXI,FUNKNO,
     3        SUNKNO)
*     ------------------------------------
      ELSE
      CALL SETARA(BASE,(2*NUNKNO*NGEFF),IPHI)
      CALL SETARA(BASE,(2*NUNKNO*NGEFF),IS)
      CALL SETARA(BASE,NMAX,IH)
      CALL SETARA(IBASE,NMAX,NOM)
      IF (CYCLIC) THEN
*     --------------------------------
*     Method of Cyclic Characteristics
*     --------------------------------
*********'Source Term Isolation' Strategy turned off 
         IF (ISCH.EQ.1) THEN
*          Step-Characteristics Scheme with Tabulated Exponentials
           TOPT='CYCLIC - STIS 0 - SC SCHEME - TABULATED EXP'
           CALL MCGFLX(MOCFFIS,MOCFFAS,MOCSCAS,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.0) THEN
*          Diamond-Differencing Scheme
           TOPT='CYCLIC - STIS 0 - DD SCHEME'
           CALL MCGFLX(MOCFFIS,MOCFFAS,MOCDDFS,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.-1) THEN
*          Step-Characteristics Scheme with Exact Exponentials
           TOPT='CYCLIC - STIS 0 - SC SCHEME - EXACT EXP'
           CALL MCGFLX(MOCFFIS,MOCFFAS,MOCSCES,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
*********'Source Term Isolation' Strategy turned on
         ELSEIF (ISCH.EQ.11) THEN
*          Step-Characteristics Scheme with Tabulated Exponentials
           TOPT='CYCLIC - STIS 1 - SC SCHEME - TABULATED EXP'
           CALL MCGFLX(MOCFFI,MOCFFA,MOCSCA,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.10) THEN
*          Diamond-Differencing Scheme
           TOPT='CYCLIC - STIS 1 - DD SCHEME'
           CALL MCGFLX(MOCFFI,MOCFFA,MOCDDF,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.9) THEN
*          Step-Characteristics Scheme with Exact Exponentials
           TOPT='CYCLIC - STIS 1 - SC SCHEME - EXACT EXP'
           CALL MCGFLX(MOCFFI,MOCFFA,MOCSCE,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
*********'MOCC/MCI' Iterative Strategy
         ELSEIF (ISCH.EQ.-9) THEN
*          Step-Characteristics Scheme with Tabulated Exponentials
           TOPT='CYCLIC - STIS -1 - SC SCHEME - TABULATED EXP'
           CALL MCGFLX(MOCFFIT,MOCFFAT,MOCSCAT,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.-10) THEN
*          Diamond-Differencing Scheme
           TOPT='CYCLIC - STIS -1 - DD SCHEME'
           CALL MCGFLX(MOCFFIT,MOCFFAT,MOCDDFT,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))             
         ELSEIF (ISCH.EQ.-11) THEN
*          Step-Characteristics Scheme with Exact Exponentials
           TOPT='CYCLIC - STIS -1 - SC SCHEME - EXACT EXP'
           CALL MCGFLX(MOCFFIT,MOCFFAT,MOCSCET,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))          
         ENDIF   
*     --------------------------------
      ELSE
*     ------------------------------------
*     Method of Non-Cyclic Characteristics
*     ------------------------------------
*********'Source Term Isolation' Strategy turned off 
         IF (ISCH.EQ.1) THEN
*          Step-Characteristics Scheme with Tabulated Exponentials
           TOPT='NON CYCLIC - STIS 0 - SC SCHEME - TABULATED EXP'
           CALL MCGFLX(MCGFFIS,MCGFFAS,MCGSCAS,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.0) THEN
*          Diamond-Differencing Scheme
           TOPT='NON CYCLIC - STIS 0 - DD SCHEME'
           CALL MCGFLX(MCGFFIS,MCGFFAS,MCGDDFS,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.-1) THEN
*          Step-Characteristics Scheme with Exact Exponentials
           TOPT='NON CYCLIC - STIS 0 - SC SCHEME - EXACT EXP'
           CALL MCGFLX(MCGFFIS,MCGFFAS,MCGSCES,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
*********'Source Term Isolation' Strategy turned on
         ELSEIF (ISCH.EQ.11) THEN
*          Step-Characteristics Scheme with Tabulated Exponentials
           TOPT='NON CYCLIC - STIS 1 - SC SCHEME - TABULATED EXP'
           CALL MCGFLX(MCGFFI,MCGFFA,MCGSCA,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.10) THEN
*          Diamond-Differencing Scheme
           TOPT='NON CYCLIC - STIS 1 - DD SCHEME'
           CALL MCGFLX(MCGFFI,MCGFFA,MCGDDF,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.9) THEN
*          Step-Characteristics Scheme with Exact Exponentials
           TOPT='NON CYCLIC - STIS 1 - SC SCHEME - EXACT EXP'
           CALL MCGFLX(MCGFFI,MCGFFA,MCGSCE,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
*********'MOCC/MCI' Iterative Strategy
         ELSEIF (ISCH.EQ.-9) THEN
*          Step-Characteristics Scheme with Tabulated Exponentials
           TOPT='NON CYCLIC - STIS -1 - SC SCHEME - TABULATED EXP'
           CALL MCGFLX(MCGFFIT,MCGFFAT,MCGSCAT,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))
         ELSEIF (ISCH.EQ.-10) THEN
*          Diamond-Differencing Scheme
           TOPT='NON CYCLIC - STIS -1 - DD SCHEME'
           CALL MCGFLX(MCGFFIT,MCGFFAT,MCGDDFT,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))              
         ELSEIF (ISCH.EQ.-11) THEN
*          Step-Characteristics Scheme with Exact Exponentials
           TOPT='NON CYCLIC - STIS -1 - SC SCHEME - EXACT EXP'
           CALL MCGFLX(MCGFFIT,MCGFFAT,MCGSCET,CYCLIC,IBASE(KPSYS),IMPX,
     1          IPTRK,IFTRAK,IPMACR,NDIM,NFI,NUNKNO,NLONG,NBREG,NSOUT,
     2          NGRP,NGEFF,IBASE(INGIND),IBASE(INZON),IBASE(MATALB),
     3          BASE(IV),FUNKNO,SUNKNO,NBMIX,NANI,MAXI,IAAC,KRYL,ISCR,
     4          NMU,NANGL,NMAX,LC,EPSI,BASE(ICAZ),BASE(ICPO),
     5          BASE(IZMU),BASE(IWZMU),PACA,NFUNL,NMOD,IBASE(ISGNR),
     6          KEYFLX,IBASE(KEYCUR),BASE(ISIGAL),LPS,BASE(IREPS),
     7          BASE(IEPS),IBASE(ITST),LBASE(INCONV),LNCONV,REBFLG,
     8          STIS,NPJJM,IBASE(NOM),BASE(IH),BASE(IS),BASE(IPHI))               
         ENDIF   
*     ------------------------------------
      ENDIF
      CALL RLSARA(IBASE(NOM))
      CALL RLSARA(BASE(IH))
      CALL RLSARA(BASE(IS))
      CALL RLSARA(BASE(IPHI))
      ENDIF
*---
* PRINT RESULTS
*---
      IF (IMPX.GT.0) WRITE(IUNOUT,50) TOPT
      IF ((IDIFC.EQ.0).AND.(MAXI.GT.1).AND.(IMPX.GT.1)) THEN     
      DO II=1,NGEFF
         IG=IBASE(INGIND-1+II)
         WRITE(IUNOUT,100) IG
         IF(IMPX.GT.3) 
     1      WRITE(IUNOUT,200) (SUNKNO(KEYFLX(I,1),IG),I=1,NBREG)
         ITEMP=IBASE(ITST-1+II)
         TEMP=BASE(IEPS-1+II)
         IF((ITEMP.EQ.MAXI).AND.(TEMP.GT.EPSI)) WRITE(IUNOUT,60) 
         WRITE(IUNOUT,70) ITEMP,TEMP
         IF(IMPX.GT.2) CALL PRINAM('EPS   ',BASE(IREPS+MAXI*(II-1)),
     1                                      MIN(ITEMP,100))
         IF(IMPX.GT.4) THEN
            WRITE(IUNOUT,400) (FUNKNO(I,IG),I=1,NUNKNO)
         ELSEIF(IMPX.GT.3) THEN
            WRITE(IUNOUT,300) (FUNKNO(KEYFLX(I,1),IG),I=1,NBREG)
         ENDIF
      ENDDO
!!!!      CUROUT=0.0
!!!!      DO I=1,NSOUT
!!!!         CUROUT=CUROUT+FUNKNO(IBASE(KEYCUR-1+I),IG)
!!!!     1        *BASE(IV-1+NBREG+I)
!!!!      ENDDO
!!!!      WRITE(IUNOUT,*) 'TOTAL OUTGOING CURRENT ',CUROUT
      ENDIF
*
      CALL RLSARA(LBASE(INCONV))
      CALL RLSARA(IBASE(ITST))
      CALL RLSARA(BASE(IEPS))
      CALL RLSARA(BASE(IREPS))
      CALL RLSARA(BASE(ISIGAL))
      CALL RLSARA(BASE(ICPO))
      CALL RLSARA(BASE(ICAZ))
      CALL RLSARA(IBASE(MATALB))
      CALL RLSARA(IBASE(KPSYS))
      CALL RLSARA(IBASE(INGIND))
*
      RETURN
*
 50   FORMAT(9X,18H M O C PARAMETERS:,2X,A72)
 60   FORMAT(49H *** WARNING *** MAXIMUM NUMBER OF MCCG ITERATION,
     1       10HS REACHED.)
 70   FORMAT(35H MCGFLX: NUMBER OF MCCG ITERATIONS=,I4,
     1       11H  ACCURACY=,1P,E11.4,1H.)
 100  FORMAT(9X,8H GROUP (,I4,4H) : )
 200  FORMAT(33H N E U T R O N    S O U R C E S :/(1P,6(5X,E15.7)))
 300  FORMAT(31H N E U T R O N    F L U X E S :/(1P,6(5X,E15.7)))
 400  FORMAT(31H U N K N O W N    V E C T O R :/(1P,6(5X,E15.7)))
      END
