*DECK MCCGT
      SUBROUTINE MCCGT(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
*Purpose:
* adapt EXCELL tracking to MCCG requirements.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert and R. Le Tellier
*
*Parameters: input/output
* NENTRY  number of LCM objects or files used by the operator.
* HENTRY  name of each LCM object or file:
*         HENTRY(1): modification type(L_TRACK);
*         HENTRY(2): sequential binary tracking file.
*         HENTRY(3): read-only type(L_GEOM).
* IENTRY  type of each LCM object or file:
*         =1 LCM memory object; =2 XSM file; =3 sequential binary file;
*         =4 sequential ascii file.
* JENTRY  access of each LCM object or file:
*         =0 the LCM object or file is created;
*         =1 the LCM object or file is open for modifications;
*         =2 the LCM object or file is open in read-only mode.
* KENTRY  LCM object address or file unit number.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER    NENTRY                  
      CHARACTER  HENTRY(NENTRY)*12
      INTEGER    IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,MXGAUS=64,IOUT=6,IBCV=-7)
      CHARACTER TEXT4*4,TEXT12*12,TITLE*72,HSIGN*12,CFTRAK*12,
     1 COMNT(10)*80
      DOUBLE PRECISION DFLOTT,WEIGHT
      INTEGER DIMKEYF,TYPOR1,TYPOR2,LTMT,STIS
      INTEGER IGP(NSTATE),ISTATE(NSTATE),KCHAR(3),LMXMCU,TRTY,PACA
      REAL ZREAL(2), ZMU(MXGAUS), WZMU(MXGAUS), XMU(MXGAUS), ALBEDO(6)
      LOGICAL ACFLAG,LACA,LSCR,CYCLIC
      INTEGER IV,NZON,NZONA,ITEMP,SEGLEN,NRSEG,MCUW,MCUI,KM,MCU,
     1 IM,LEV,LEVPT,KMROR,MCUROR,JU,ISGNR,KEYANI,KEYFLX,KEYCUR,IVA,PI,
     2 INVPI,IS,JS,IM0,MCU0,H
      INTEGER IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  PARAMETER VALIDATION
*----
      IF(NENTRY.LE.2) CALL XABORT('MCCGT: THREE PARAMETERS EXPECTED.')
      IF((IENTRY(1).NE.1).AND.(IENTRY(1).NE.2))
     1  CALL XABORT('MCCGT: LINKED LIST EXPECTED AT LHS.')
      IF(JENTRY(1).NE.1) 
     1  CALL XABORT('MCCGT: ENTRY IN MODIFICATION MODE EXPECTED.')
      IF(IENTRY(2).NE.3) 
     1  CALL XABORT('MCCGT: SEQ BINARY FILE EXPECTED AT RHS.')
      IF(JENTRY(2).NE.2) 
     1  CALL XABORT('MCCGT: ENTRY IN READ-ONLY MODE EXPECTED.')
      IF((IENTRY(3).NE.1).AND.(IENTRY(3).NE.2))
     1  CALL XABORT('MCCGT: LINKED LIST EXPECTED AT RHS.')
      IF(JENTRY(3).NE.2) 
     1  CALL XABORT('MCCGT: ENTRY IN READ-ONLY MODE EXPECTED.')
      CALL LCMGET(KENTRY(1),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(J),J=1,3)
      IF(HSIGN.NE.'L_TRACK') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('MCCGT: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. L_TRACK EXPECTED.')
      ENDIF
      CALL LCMGET(KENTRY(1),'TRACK-TYPE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(J),J=1,3)
      IF(HSIGN.NE.'EXCELL') THEN
            TEXT12=HENTRY(1)
            CALL XABORT('MCCGT: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1      '. EXCELL EXPECTED.')
      ENDIF
*----
*  RECOVER GEOMETRY
*----
      CALL LCMGET(KENTRY(3),'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(J),J=1,3)
      IF(HSIGN.NE.'L_GEOM') THEN
         TEXT12=HENTRY(3)
         CALL XABORT('MCCGT: SIGNATURE OF '//TEXT12//' IS '//HSIGN//
     1        '. L_GEOM EXPECTED.')
      ENDIF
      IPGEOM=KENTRY(3)
      CALL LCMGET(IPGEOM,'STATE-VECTOR',ISTATE)
      TEXT12=HENTRY(3)
      READ(TEXT12,'(3A4)') (KCHAR(I1),I1=1,3)
      CALL LCMPUT(KENTRY(1),'LINK.GEOM',3,3,KCHAR)
*----
*  RECOVER SEQUENTIAL BINARY TRACKING FILE CHARACTERISTICS
*----
      CFTRAK=HENTRY(2)
      IFTRAK=KENTRY(2)
      READ(CFTRAK,'(3A4)') (KCHAR(I1),I1=1,3)
      CALL LCMPUT(KENTRY(1),'LINK.FTRACK',3,3,KCHAR)
      REWIND IFTRAK
      READ(IFTRAK) TEXT4,NCOMNT,NBTR
      DO ICOM=1,NCOMNT
         READ(IFTRAK) COMNT(ICOM)
      ENDDO
      READ(IFTRAK) NDIM,ISPEC,NREG,NSOUT,NALBG,NCOR,NANGL,MXSEG
      IF((NDIM.NE.2).AND.(NDIM.NE.3)) 
     & CALL XABORT('2D OR 3D EXCELT TRACKING EXPECTED')
*----
*  RECOVER TRACKING STATE-VECTOR INFORMATION
*----
      CALL LCMGET(KENTRY(1),'STATE-VECTOR',IGP)
      IF(NDIM.EQ.2) THEN
         LMXMCU=8*(IGP(1)+IGP(5))
      ELSE
         LMXMCU=12*(IGP(1)+IGP(5))         
      ENDIF
      NANIS=IGP(6)
      TRTY=IGP(9)
      IF(TRTY.EQ.1) THEN
         CYCLIC=.TRUE.
      ELSE
         CYCLIC=.FALSE.
      ENDIF
      NFI=IGP(1)+IGP(5)
      NANGL=IGP(11)
      ISYMM=IGP(12)
*
      IMPX=1
      LCACT=3
      NMU=0
      MAXI=20
      IAAC=1
      ISCR=0
      KRYL=10
      IDIFC=0
      EPSI=1.0E-4
      HDD=0.0
      PACA=3
      ILEXA=0
      LTMT=0
      ILEXF=0
      STIS=0
      IF(NANIS.LE.4) STIS=1
*
   50 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
   55 IF(INDIC.EQ.10) GO TO 60
      IF(INDIC.NE.3) CALL XABORT('MCCGT: CHARACTER DATA EXPECTED.')
      IF(TEXT4.EQ.'EDIT') THEN
         CALL REDGET(INDIC,IMPX,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'GAUS') THEN
         LCACT=0
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'CACA') THEN
         LCACT=1
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'CACB') THEN
         LCACT=2
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'LCMD') THEN
         LCACT=3
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) GO TO 55
         NMU=NITMA
      ELSE IF(TEXT4.EQ.'EPSI') THEN
*        CONVERGENCE CRITERION FOR INNER ITERATIONS.
         CALL REDGET(INDIC,NITMA,EPSI,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('MCCGT: REAL DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'SCR') THEN
*        SCR ACCELERATION FLAG.
         CALL REDGET(INDIC,ISCR,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(2).')
      ELSE IF(TEXT4.EQ.'KRYL') THEN
*        GMRES FLAG.
         CALL REDGET(INDIC,KRYL,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(3).')
      ELSE IF(TEXT4.EQ.'AAC') THEN
*        ACA ACCELERATION FLAG.
         CALL REDGET(INDIC,IAAC,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(4).')
      ELSE IF(TEXT4.EQ.'BICG') THEN
*        ACA SYSTEM RESOLUTION TYPE (obsolete because it is the only option!)
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: BICG ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'ILU0') THEN
*        ILU0 PRECONDITIONER FOR SOLVING ACA SYSTEM
         PACA=3
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: ILU0 ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'DIAG') THEN
*        DIAGONAL PRECONDITIONER FOR SOLVING ACA SYSTEM
         PACA=1
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: DIAG ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'NONE') THEN
*        NO PRECONDITIONER FOR SOLVING ACA SYSTEM
         PACA=0
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: NONE ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'TMT') THEN
*        TO USE A TRACK MERGING TECHNIQUE IN ACA CALCULATION
         LTMT=1
         IF(IAAC.EQ.0) CALL XABORT('MCCGT: LTMT ONLY IF ACA IS ON.')
      ELSE IF(TEXT4.EQ.'LEXA') THEN
*        TO FORCE EXACT EXPONENTIALS IN PRECONDITIONER CALCULATIONS
         ILEXA=1
      ELSE IF(TEXT4.EQ.'DIFC') THEN
*        TRANSPORT/DIFFUSION SOLUTION FLAG.
         IDIFC=1
         IAAC=1
      ELSE IF(TEXT4.EQ.'MCU') THEN
*        MAXIMUM DIMENSION OF MCU FOR MEMORY ALLOCATION.
         CALL REDGET(INDIC,LMXMCU,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(5).')
      ELSE IF(TEXT4.EQ.'HDD') THEN
*        SELECTION OD STEP CHARACTERISTICS METHOD.
         CALL REDGET(INDIC,NITMA,HDD,TEXT4,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('MCCGT: REAL DATA EXPECTED(2).')
      ELSE IF(TEXT4.EQ.'STIS') THEN
*        'SOURCE TERM ISOLATION' FLAG
         CALL REDGET(INDIC,STIS,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(6).')
         IF(ABS(STIS).GT.1) THEN
            CALL XABORT('MCCGT: STIS MUST BE SET TO -1, 0 OR 1.')
         ENDIF
      ELSE IF(TEXT4.EQ.'LEXF') THEN
*        TO FORCE EXACT EXPONENTIALS IN FLUX CALCULATIONS
         ILEXF=1
      ELSE IF(TEXT4.EQ.'MAXI') THEN
*        MAXIMUM NUMBER OF INNER ITERATIONS.
         CALL REDGET(INDIC,MAXI,FLOTT,TEXT4,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('MCCGT: INTEGER DATA EXPECTED(7).')
      ELSE IF(TEXT4.EQ.';') THEN
         GO TO 60
      ELSE 
         CALL XABORT('MCCGT: '//TEXT4//' IS AN INVALID KEY WORD.')
      ENDIF
      GO TO 50
*
 60   IF(NMU.EQ.0) THEN
         IF(ISYMM.LE.1) THEN
            NMU=(NANGL+1)/2
         ELSE IF(ISYMM.GE.2) THEN
            NMU=NANGL
         ENDIF
      ENDIF
      IGP(21)=LCACT
      IGP(22)=IAAC
      IF(IAAC.GT.0) THEN
         LACA=.TRUE.
      ELSE
         LACA=.FALSE.
      ENDIF
      IGP(23)=KRYL
      IGP(24)=IDIFC
      IGP(28)=ISCR
      IF((ISCR.GT.0).AND.(.NOT.CYCLIC)) THEN
         LSCR=.TRUE.
      ELSE
         LSCR=.FALSE.
      ENDIF
      IGP(31)=ILEXA
      IGP(32)=ILEXF
      IGP(33)=MAXI
      IGP(34)=LTMT
      ZREAL(1)=EPSI
      ZREAL(2)=HDD
      IF((LSCR).OR.(LACA)) THEN
         ACFLAG=.TRUE.
      ELSE
         ACFLAG=.FALSE.
      ENDIF
*
      IF(IMPX.GT.1) THEN
         CALL LCMGET(KENTRY(1),'TITLE',KCHAR)
         WRITE(TITLE,'(3A4)') (KCHAR(J),J=1,3)
         WRITE(IOUT,100) TITLE
      ENDIF
*---
* CALCULATE POLAR QUADRATURE (IF NDIM.EQ.2)
*--- 
      IF(NDIM.EQ.2) THEN
         IF(LCACT.EQ.0) THEN   
            CALL ALGPT ( NMU, 0.0, 1.0, XMU, WZMU)
         ELSE
            IF(LCACT.EQ.3) THEN
               IF(NMU.GT.4) NMU=4
               IF(NMU.LT.2) NMU=2
            ENDIF
            CALL ALCACT( LCACT, NMU, XMU, WZMU)
         ENDIF
         DO IMU=1,NMU
            DUM=SQRT(1.-XMU(IMU)*XMU(IMU))
            ZMU(IMU)=1./DUM
            WZMU(IMU)=WZMU(IMU)*DUM
         ENDDO
         IF(IMPX.GT.3) THEN
            CALL PRINAM('XMU   ',XMU,NMU)
            CALL PRINAM('ZMU   ',ZMU,NMU)
            CALL PRINAM('WZMU  ',WZMU,NMU)
         ENDIF
      ELSE ! NDIM.EQ.3
         NMU=1
         WZMU(1)=1.0
         XMU(1)=0.0
         ZMU(1)=1.0
      ENDIF
      CALL LCMPUT(KENTRY(1),'ZMU$MCCG',NMU,2,ZMU)
      CALL LCMPUT(KENTRY(1),'XMU$MCCG',NMU,2,XMU)
      CALL LCMPUT(KENTRY(1),'WZMU$MCCG',NMU,2,WZMU)
*---
* REORDER VOLUME AND MATALB ARRAYS (SURFACE -j BECOMES NV+j)
*     [S(-NS) S(-NS+1) ... S(-1) 0 V(1) ... V(NV-1) V(NV)]
*     ->[V(1)... V(NV-1) V(NV) S(-1) ... S(-NS+1) S(-NS)]
* CALCULATE NUMERICAL SURFACES FOR NON CYCLIC TRACKING
* IF AAC OR SCR USED, CALCULATE CONNECTION MATRICES
*---
      CALL SETARA(BASE,NSOUT+NREG+1,IV)
      CALL SETARA(IBASE,NSOUT+NREG+1,NZON)
      IF(ACFLAG) CALL SETARA(IBASE,NSOUT+NREG+1,NZONA)
      CALL SETARA(IBASE,NSOUT,ITEMP)
      CALL SETARA(BASE,NSOUT,IRTEMP)
*
      CALL LCMGET(KENTRY(1),'ALBEDO',ALBEDO)
*
      READ(IFTRAK) (BASE(IV+NSOUT+II),II=-NSOUT,NREG)
      READ(IFTRAK) (IBASE(NZON+NSOUT+II),II=-NSOUT,NREG)
      DO I=0,NSOUT-1
         IBASE(ITEMP+I)=IBASE(NZON+NSOUT-1-I)
         IBASE(IRTEMP+I)=IBASE(IV+NSOUT-1-I)
      ENDDO
      DO I=0,NREG-1
         BASE(IV+I)=BASE(IV+NSOUT+I+1)
         !BASE(IV+I)=0.
         IBASE(NZON+I)=IBASE(NZON+NSOUT+I+1)
         IF(ACFLAG) IBASE(NZONA+I)=IBASE(NZON+I)
      ENDDO
      DO I=0,NSOUT-1
         IBASE(NZON+NREG+I)=IBASE(ITEMP+I)
         IF(ACFLAG) THEN
            IF(ALBEDO(-IBASE(ITEMP+I)).GT.(0.0)) THEN
               IBASE(NZONA+NREG+I)=IBASE(ITEMP+I)
            ELSE
               IBASE(NZONA+NREG+I)=IBCV
            ENDIF
         ENDIF
         !BASE(IV+NREG+I)=BASE(IRTEMP+I)
         BASE(IV+NREG+I)=0.0
      ENDDO
*
      CALL RLSARA(BASE(IRTEMP))
      CALL RLSARA(IBASE(ITEMP))
*
      READ(IFTRAK)
      READ(IFTRAK)
      READ(IFTRAK)
      READ(IFTRAK)
*
      IF(ACFLAG) THEN
         CALL SETARA(IBASE,LMXMCU,MCUW)
         CALL SETARA(IBASE,LMXMCU,MCUI)
         CALL XDISET(IBASE(MCUW),LMXMCU,0)
         CALL XDISET(IBASE(MCUI),LMXMCU,0)
      ENDIF
      CALL SETARA(BASE,MXSEG,SEGLEN)
      CALL SETARA(IBASE,MXSEG,NRSEG)
      IF(.NOT.CYCLIC) THEN
         CALL SETARA(BASE,2*NSOUT,IDTEMP)
         CALL XDDSET(BASE(IDTEMP),NSOUT,0.D0)
      ENDIF
      LMCU=NFI
      DO ILINE=1,NBTR
         READ(IFTRAK) IANGL,NSEG,WEI2D,
     1        (IBASE(NRSEG-1+II),II=1,NSEG),
     2        (BASE(SEGLEN-1+II),II=1,NSEG)
         IF(NSEG.GT.0) THEN
            DO II=1,NSEG
               IF(IBASE(NRSEG-1+II).LT.0) THEN
                  IBASE(NRSEG-1+II)=NREG-IBASE(NRSEG-1+II)
               ENDIF
            ENDDO
            IF(.NOT.CYCLIC) THEN
*           non cyclic tracking: calculate numerical surfaces
               DO I=1,NSEG,NSEG-1
               NOMCEL=IBASE(NRSEG-1+I)
               IF(NOMCEL.GT.NREG) THEN
                  IF (NDIM.EQ.2) THEN
                  DO IMU=1,NMU
                     WEIGHT=DBLE(WEI2D*WZMU(IMU))
                     CALL ADDBLE(NSOUT,NOMCEL-NREG,BASE(IDTEMP),WEIGHT)
                     !BASE(IV-1+NOMCEL)=WEIGHT+BASE(IV-1+NOMCEL)
                  ENDDO
                  ELSE
                     WEIGHT=DBLE(WEI2D)
                     CALL ADDBLE(NSOUT,NOMCEL-NREG,BASE(IDTEMP),WEIGHT)
                     !BASE(IV-1+NOMCEL)=WEI2D+BASE(IV-1+NOMCEL)
                  ENDIF
!!!!               ELSE
!!!!                  IF (NDIM.EQ.2) THEN
!!!!                  DO IMU=1,NMU
!!!!                     WEIGHT=WEI2D*WZMU(IMU)
!!!!                     BASE(IV-1+NOMCEL)=BASE(IV-1+NOMCEL)
!!!!     1                 + BASE(SEGLEN-1+I)*2.0*WEIGHT*ZMU(IMU)
!!!!                  ENDDO
!!!!                  ELSE
!!!!                     BASE(IV-1+NOMCEL)=BASE(IV-1+NOMCEL)
!!!!     1                 + BASE(SEGLEN-1+I)*2.0*=WEI2D
!!!!                  ENDIF
               ENDIF
               ENDDO
            ELSE IF((CYCLIC).AND.(ACFLAG)) THEN
*           cyclic tracking: "unfold" the tracking line
               CALL MCGTRK(NFI,IBASE(NZONA),NSEG,IBASE(NRSEG),
     1              BASE(SEGLEN))
            ENDIF
            IF(ACFLAG) THEN
               IF(.NOT.CYCLIC) THEN
*              non-cyclic tracking: calculate connection matrices
                  CALL MCGCAL(NSEG,IBASE(NRSEG),NFI,NREG,IBASE(MCUW),
     1                 IBASE(MCUI),LMCU,LMXMCU)
                  DO II=1,NSEG/2
                     ITEMP=IBASE(NRSEG-1+II)
                     IBASE(NRSEG-1+II)=IBASE(NRSEG+NSEG-II)
                     IBASE(NRSEG+NSEG-II)=ITEMP
                  ENDDO
                  CALL MCGCAL(NSEG,IBASE(NRSEG),NFI,NREG,IBASE(MCUW),
     1                 IBASE(MCUI),LMCU,LMXMCU)
               ELSE
*              cycling tracking: calculate connection matrices
                  CALL MOCCAL(NSEG,IBASE(NRSEG),NFI,NREG,IBASE(MCUW),
     1                 IBASE(MCUI),LMCU,LMXMCU)
                  DO II=1,NSEG/2
                     ITEMP=IBASE(NRSEG-1+II)
                     IBASE(NRSEG-1+II)=IBASE(NRSEG+NSEG-II)
                     IBASE(NRSEG+NSEG-II)=ITEMP
                  ENDDO
                  CALL MOCCAL(NSEG,IBASE(NRSEG),NFI,NREG,IBASE(MCUW),
     1                 IBASE(MCUI),LMCU,LMXMCU)
               ENDIF
            ENDIF
         ENDIF
      ENDDO
      IF(.NOT.CYCLIC) THEN
         CALL XDRSDB(NSOUT,BASE(IV+NREG),BASE(IDTEMP),1)
         CALL RLSARA(BASE(IDTEMP))
      ENDIF
      CALL RLSARA(IBASE(NRSEG))
      CALL RLSARA(BASE(SEGLEN))
*
      LMCU0=0
      IF(ACFLAG) THEN
*---
* CREATE CONNECTION MATRICES IN KM/MCU FORMAT
*---
*        KM(i) is the number of non-diagonal element on row i
*        MCU gives the column indexes.
         CALL SETARA(IBASE,NFI,KM)
         CALL SETARA(IBASE,LMXMCU,MCU)
         CALL MCGREC(NFI,IBASE(KM),IBASE(MCUW),IBASE(MCUI),
     1        IBASE(MCU),LMCU,LMXMCU,0)
         CALL RLSARA(IBASE(MCUI))
         CALL RLSARA(IBASE(MCUW))
         NLONG=NFI
         IF(CYCLIC) THEN
*        if cyclic tracking, only the volume related data are stored
            LMCU=0
            DO I=1,NREG
               LMCU=LMCU+IBASE(KM-1+I)
            ENDDO
            NLONG=NREG
         ENDIF
         CALL SETARA(IBASE,NLONG+1,IM)
*        construct IM
*        containing number of sparse matrix elements in rows before I
*        so location of J-th non-zero in row # I is IM(I)+J i.e.
*        IM(K+1)=sum_i=1^K KM(i) where K in [1,NLONG]
         IBASE(IM)=0
         DO I=1,NLONG
            IBASE(IM+I)=IBASE(IM+I-1)+IBASE(KM-1+I)
         ENDDO
         IF(LSCR) THEN
*---
* SCR ACCELERATION : CREATE INDEX FOR THE SURFACES NEIGHBORS
*---
            LPS=IBASE(IM+NFI)-IBASE(IM+NREG)
            CALL SETARA(IBASE,(NSOUT+1),IS)
            CALL SETARA(IBASE,LPS,JS)
            LPS=0
            DO I=NREG+1,NFI
               IBASE(IS-1+I-NREG)=LPS
               DO J=IBASE(IM-1+I)+1,IBASE(IM+I)
                  IF(IBASE(MCU-1+J).GT.0) THEN
                     LPS=LPS+1
                     IBASE(JS-1+LPS)=IBASE(MCU-1+J)
                  ENDIF
               ENDDO
            ENDDO
            IBASE(IS+NSOUT)=LPS
         ENDIF
         IF(LACA) THEN
*---
* ACA ACCELERATION : 
*---
            CALL SETARA(IBASE,NFI,PI)
            CALL SETARA(IBASE,NFI,INVPI)
            IF(PACA.GE.2) THEN
               LMCU0=LMCU
               CALL SETARA(IBASE,NLONG,LEV)
               CALL SETARA(IBASE,NLONG+1,LEVPT)
               CALL SETARA(IBASE,NLONG,KMROR)
               CALL SETARA(IBASE,LMCU,MCUROR)
               CALL SETARA(IBASE,NLONG+1,IMROR)
               CALL SETARA(IBASE,NLONG,JU)
               CALL SETARA(BASE,NFI,IVA)
               IF(PACA.EQ.3) THEN
                  CALL SETARA(IBASE,NFI,IWORK)
                  CALL SETARA(IBASE,(NFI+1),IM0)
                  CALL SETARA(IBASE,LMCU0,MCU0)
               ENDIF
*              construct PI permutation : old_index=PI(new_index) or F_new=F_old(PI)
*              reordering of the unknowns of the corrective system for ilu0 preconditioner.
               NFIRST=1
               TYPOR1=0
               TYPOR2=0
               CALL RENUM(NLONG,LMCU,NFIRST,IBASE(IM),IBASE(MCU),TYPOR1,
     1             TYPOR2,NLEV,IBASE(LEV),IBASE(LEVPT),IBASE(PI))
               IF(CYCLIC) THEN
                  DO I=NLONG+1,NFI
                     IBASE(PI-1+I)=I
                  ENDDO
               ENDIF
*              reorder everything according to PI
*              construct INVPI permutation : new_index=INVPI(old_index) or F_old=F_new(INVPI)
               DO I=1,NFI
                  J=IBASE(PI-1+I)
                  IBASE(INVPI-1+J)=I
               ENDDO
               DO I=1,NLONG
                  J=IBASE(PI-1+I)               
                  IBASE(KMROR-1+I)=IBASE(KM-1+J)
               ENDDO
               IBASE(IMROR)=0
               DO I=1,NLONG
                  IBASE(IMROR+I)=IBASE(IMROR-1+I)+IBASE(KMROR-1+I)
               ENDDO
               DO I=1,NLONG
                  J=IBASE(PI-1+I)
                  IK=IBASE(IMROR-1+I)
                  DO JK=IBASE(IM-1+J)+1,IBASE(IM+J)
                     IK=IK+1
                     IF(IBASE(MCU-1+JK).GT.0) THEN
                       IBASE(MCUROR-1+IK)=IBASE(INVPI-1+IBASE(MCU-1+JK))
                     ELSE
                        IBASE(MCUROR-1+IK)=IBASE(MCU-1+JK)
                     ENDIF
                  ENDDO
               ENDDO
*              sort each line by increasing column index
               DO I=1,NLONG
                  K=IBASE(IMROR-1+I)+1
                  CALL SORTIN(IBASE(KMROR-1+I),IBASE(MCUROR-1+K))
               ENDDO
               DO I=1,NFI
                  J=IBASE(PI-1+I)
                  IBASE(NZONA-1+I)=IBASE(NZON-1+J)
                  BASE(IVA-1+I)=BASE(IV-1+J)
                  IF(J.GT.NREG) THEN
                     IF(ALBEDO(-IBASE(NZON-1+J)).EQ.(0.0)) THEN
                        IBASE(NZONA-1+I)=IBCV
                     ENDIF
                  ENDIF
               ENDDO
               CALL XDISET(IBASE(JU),NLONG,0)
               IF(PACA.EQ.3) THEN
                  CALL XDISET(IBASE(IM0),(NLONG+1),LMCU)
                  CALL XDISET(IBASE(IWORK),NLONG,0)
                  ILAST=0
                  LMCU0=0
               ENDIF
               DO 30 I=1,NLONG
*              construct JU (and IM0/MCU0 for optimized storage)
*              MCUROR(JU(i):IMROR(i+1)) corresponds to the upper triangular part of line i.
*              MCUROR(IMROR(i)+1:JU(i)-1) correspond to the lower triangular part of line i.
                  DO IH=IBASE(IMROR-1+I)+1,IBASE(IMROR+I)
                     H=IBASE(MCUROR-1+IH)
                     IF(H.GT.0) THEN
                        IF((H.GT.I).AND.(IBASE(JU-1+I).EQ.0)) THEN
                           IBASE(JU-1+I)=IH
                        ENDIF
                        IF(PACA.EQ.3) IBASE(IWORK-1+H)=IH
                     ENDIF
                  ENDDO
                  IF(IBASE(JU-1+I).EQ.0) THEN
                     IBASE(JU-1+I)=IBASE(IMROR+I)+1
                  ENDIF
                  IF(PACA.EQ.3) THEN
                  DO IK=IBASE(IMROR-1+I)+1,IBASE(JU-1+I)-1
                     K=IBASE(MCUROR-1+IK)
                     IF(K.GT.0) THEN
                     DO KJ=IBASE(JU-1+K),IBASE(IMROR+K)
                        J=IBASE(MCUROR-1+KJ)
                        IF(IBASE(IWORK-1+J).GT.0) THEN
                           IPOS=0
                           IJEND=MIN(IBASE(IM0+I),LMCU0)
                           DO IJ=IBASE(IM0-1+I)+1,IJEND
                           IF(IBASE(MCU0-1+IJ).EQ.J) THEN
                              IPOS=IJ
                              GOTO 40
                           ENDIF
                           ENDDO
 40                        CONTINUE
                           IF(IPOS.EQ.0) THEN
                           IF(ILAST.NE.I) THEN
                           CALL XDISET(IBASE(IM0+ILAST),(I-ILAST),LMCU0)
                           ILAST=I
                           ENDIF
                           LMCU0=LMCU0+1
                           IBASE(MCU0-1+LMCU0)=J
                           ENDIF
                        ENDIF
                     ENDDO
                     ENDIF
                  ENDDO
                  DO IH=IBASE(IMROR-1+I)+1,IBASE(IMROR+I)
                     H=IBASE(MCUROR-1+IH)
                     IF(H.GT.0) THEN
                        IBASE(IWORK-1+H)=0
                     ENDIF
                  ENDDO
                  ENDIF
 30            CONTINUE
               IF(PACA.EQ.3) THEN
               IF(LMCU0.EQ.0) THEN
                  PACA=4 ! SPECIAL CASE WHEN THERE IS NO EXTRA-STORAGE FOR ILU0-ACA
               ELSE
                  CALL XDISET(IBASE(IM0+ILAST),(NLONG+1-ILAST),LMCU0)
               ENDIF
               ENDIF
            ELSE
               DO I=1,NFI
                  IBASE(PI-1+I)=I
                  IBASE(INVPI-1+I)=I
               ENDDO
            ENDIF
         ENDIF
      ENDIF
      IF(IMPX.GT.3) THEN
         CALL PRINIM('MATALB',IBASE(NZON),NFI)
         CALL PRINAM('VOLSUR',BASE(IV),NFI)
         IF(ACFLAG) THEN
            CALL PRINIM('MATALA',IBASE(NZONA),NFI)
            WRITE(IOUT,11) LMCU
 11         FORMAT(1X,'MCGREC : LMCU =',I6)
            CALL PRINIM('KM    ',IBASE(KM),NLONG)
            CALL PRINIM('MCU   ',IBASE(MCU),LMCU)
            IF((LACA).AND.(PACA.GE.2)) THEN
               CALL PRINIM('IPERM ',IBASE(INVPI),NFI)
               CALL PRINIM('KMROR ',IBASE(KMROR),NLONG)
               CALL PRINIM('MCUROR',IBASE(MCUROR),LMCU)
               CALL PRINIM('JU    ',IBASE(JU),NLONG)
               IF(PACA.GE.3) THEN
                  WRITE(IOUT,12) LMCU0
 12               FORMAT(1X,'MCCGT : LMCU0 =',I6)
                  IF(LMCU0.GT.0) THEN
                     CALL PRINIM('IM0    ',IBASE(IM0),NLONG+1)
                     CALL PRINIM('MCU0   ',IBASE(MCU0),LMCU0)
                  ENDIF
               ENDIF
            ENDIF
            IF(LSCR) THEN
               CALL PRINIM('IS    ',IBASE(IS),NSOUT+1)
               CALL PRINIM('JS    ',IBASE(JS),LPS)
            ENDIF
         ENDIF
      ENDIF
*     
      CALL LCMPUT(KENTRY(1),'NZON$MCCG',NFI,1,IBASE(NZON))
      CALL LCMPUT(KENTRY(1),'V$MCCG',NFI,2,BASE(IV))
      IF(ACFLAG) THEN
         IF(LACA) THEN
            CALL LCMPUT(KENTRY(1),'NZONA$MCCG',NFI,1,IBASE(NZONA))
            IF(PACA.GE.2) THEN
               CALL LCMPUT(KENTRY(1),'VA$MCCG',NFI,2,BASE(IVA))
               CALL LCMPUT(KENTRY(1),'KM$MCCG',NLONG,1,IBASE(KMROR))
               CALL LCMPUT(KENTRY(1),'IM$MCCG',NLONG+1,1,IBASE(IMROR))
               CALL LCMPUT(KENTRY(1),'MCU$MCCG',LMCU,1,IBASE(MCUROR))
               CALL LCMPUT(KENTRY(1),'JU$MCCG',NLONG,1,IBASE(JU))
               IF(PACA.EQ.3) THEN
                  CALL LCMPUT(KENTRY(1),'IM0$MCCG',NLONG+1,1,IBASE(IM0))
                  CALL LCMPUT(KENTRY(1),'MCU0$MCCG',LMCU0,1,IBASE(MCU0))    
               ENDIF
               IF(PACA.GE.3) THEN
                  CALL RLSARA(IBASE(MCU0))
                  CALL RLSARA(IBASE(IM0))
                  CALL RLSARA(IBASE(IWORK))
               ENDIF
               CALL RLSARA(BASE(IVA))
               CALL RLSARA(IBASE(JU))
               CALL RLSARA(IBASE(IMROR))
               CALL RLSARA(IBASE(MCUROR))
               CALL RLSARA(IBASE(KMROR))
               CALL RLSARA(IBASE(LEVPT))
               CALL RLSARA(IBASE(LEV))
            ELSE
               CALL LCMPUT(KENTRY(1),'KM$MCCG',NLONG,1,IBASE(KM))
               CALL LCMPUT(KENTRY(1),'IM$MCCG',NLONG+1,1,IBASE(IM))
               CALL LCMPUT(KENTRY(1),'MCU$MCCG',LMCU,1,IBASE(MCU)) 
               CALL LCMPUT(KENTRY(1),'VA$MCCG',NLONG,2,BASE(IV))           
            ENDIF
            CALL LCMPUT(KENTRY(1),'INVPI$MCCG',NFI,1,IBASE(INVPI))
            CALL LCMPUT(KENTRY(1),'PI$MCCG',NLONG,1,IBASE(PI))
            CALL RLSARA(IBASE(INVPI))
            CALL RLSARA(IBASE(PI))
         ENDIF
         IF(LSCR) THEN
            CALL LCMPUT(KENTRY(1),'IS$MCCG',NSOUT+1,1,IBASE(IS))
            CALL LCMPUT(KENTRY(1),'JS$MCCG',LPS,1,IBASE(JS))
            CALL RLSARA(IBASE(JS))
            CALL RLSARA(IBASE(IS))
         ENDIF
         CALL RLSARA(IBASE(IM))
         CALL RLSARA(IBASE(MCU))
         CALL RLSARA(IBASE(KM))
         CALL RLSARA(IBASE(NZONA))
      ENDIF
      CALL RLSARA(IBASE(NZON))
      CALL RLSARA(BASE(IV))  
      IF(.NOT.LACA) LMCU=0
      IF(.NOT.LSCR) LPS=0
*---
* MODIFY KEYFLX FOR ANISOTROPIC SCATTERING
* CREATE KEYCUR
*---
      IF(NDIM.EQ.2) THEN
         NFUNL=NANIS*(NANIS+1)/2
      ELSE ! NDIM.EQ.3
         NFUNL=NANIS*NANIS
      ENDIF
      DIMKEYF=NREG*NFUNL

      IGP(2)=DIMKEYF
      TEXT12='MCCG'
      READ(TEXT12,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(KENTRY(1),'TRACK-TYPE',3,3,KCHAR)
      IF(.NOT.CYCLIC) THEN
*     non-cyclic tracking -> MCCG used (else MOCC)
         IGP(2)=IGP(2)+IGP(5)
      ENDIF

      NUN=IGP(2)
      CALL SETARA(IBASE,DIMKEYF,KEYFLX)
      CALL LCMGET(KENTRY(1),'KEYFLX',IBASE(KEYFLX))
      DO 20 IA=1,NFUNL
         DO 20 IR=1,NREG
 20         IBASE(KEYFLX+(IA-1)*NREG+IR-1)=IR+(IA-1)
     1                                     *NUN/NFUNL
      IF(.NOT.CYCLIC) THEN
         CALL SETARA(IBASE,NSOUT,KEYCUR)
         IKEY=1
         ICUR=0
         DO I=1,NUN
            IF((IBASE(KEYFLX+IKEY-1).NE.I).OR.(IKEY.GT.DIMKEYF)) THEN
               ICUR=ICUR+1
               IF (ICUR.GT.NSOUT)
     1         CALL XABORT('MCCGT: INCORRECT NUMBER OF UNKNOWNS')
               IBASE(KEYCUR-1+ICUR)=I
            ELSE
               IKEY=IKEY+1
            ENDIF
         ENDDO
         CALL LCMPUT(KENTRY(1),'KEYCUR$MCCG',NSOUT,1,IBASE(KEYCUR))
         CALL RLSARA(IBASE(KEYCUR))      
      ENDIF
      CALL LCMPUT(KENTRY(1),'KEYFLX$MCCG',DIMKEYF,1,IBASE(KEYFLX))
      CALL RLSARA(IBASE(KEYFLX))
*---
* GENERATE ALL SIGNS FOR SPHERICAL HARMONICS
*---
      NRNL=NANIS*NANIS
      CALL SETARA(IBASE,NRNL,KEYANI)
      CALL SETARA(IBASE,8*NRNL,ISGNR)
      CALL XDISET(IBASE(KEYANI),NRNL,0)
      CALL MOCIK3(NDIM,NANIS-1,IBASE(ISGNR),IBASE(KEYANI)) 
      CALL LCMPUT(KENTRY(1),'ISGNR$MCCG',8*NRNL,1,IBASE(ISGNR))
      CALL LCMPUT(KENTRY(1),'KEYANI$MCCG',NRNL,1,IBASE(KEYANI))
      CALL RLSARA(IBASE(ISGNR))
*---
* GENERATE INDEX FOR PJJ(NU'->NU) STORAGE
* IF 'SOURCE TERM ISOLATION' OPTION IS ON
*---
      IF(STIS.NE.0) THEN
         CALL MCGPJJ(KENTRY(1),IMPX,NDIM,NANIS,NFUNL,NPJJM,
     1        IBASE(KEYANI))
      ELSE
         NPJJM=1
      ENDIF
      CALL RLSARA(IBASE(KEYANI))
*     
      IGP(25)=MXSEG
      IGP(26)=LMCU
      IGP(27)=NMU
      IGP(29)=LPS
      IGP(30)=PACA
      IGP(35)=STIS
      IGP(36)=NPJJM
      IGP(37)=LMCU0
      CALL LCMPUT(KENTRY(1),'STATE-VECTOR',NSTATE,1,IGP)
      CALL LCMPUT(KENTRY(1),'REAL-PARAM',2,2,ZREAL)
*
      IF(IMPX.GT.1) THEN
         CALL LCMGET(KENTRY(1),'STATE-VECTOR',IGP)
         WRITE(IOUT,120) (IGP(I),I=21,37)
         CALL LCMGET(KENTRY(1),'REAL-PARAM',ZREAL)
         WRITE(IOUT,130) (ZREAL(I),I=1,2)
      ENDIF
*
      IF(IMPX.GT.2) CALL LCMLIB(KENTRY(1))
*
  100 FORMAT(/
     1 44H MM      MM  CCCCC   CCCCC   GGGGG  TTTTTTTT/
     2 41H MMM    MMM CCCCCCC CCCCCCC GGGGGGG    TT/
     4 41H MMMM  MMMM CC   CC CC   CC GG         TT/
     5 41H MM  MM  MM CC      CC      GG  GGG    TT/
     6 41H MM      MM CC      CC      GG  GGG    TT/
     7 41H MM      MM CC   CC CC   CC GG   GG    TT/
     8 41H MM      MM CCCCCCC CCCCCCC GGGGGGG    TT/
     9 41H MM      MM  CCCCC   CCCCC   GGGGG     TT/
     1 18H TRACKING TITLE : ,A72/) 
  120 FORMAT(/
     1 55H STATE VECTOR RELATED TO THE METHOD OF CHARACTERISTICS:/
     2 7H LCACT ,I8,29H   (TYPE OF POLAR QUADRATURE)/
     3 7H IAAC  ,I8,48H   (0=NO ACCELERATION/1=CDD ACCELERATION OF INNE,
     4 13HR ITERATIONS)/
     5 7H KRYL  ,I8,48H   (<0 Bi-CGSTAB SCHEME USED /0=KRYLOV SCHEMES N,
     6 31H OT USED/ >0=GMRES SCHEME USED)/
     7 7H IDIFC ,I8,39H   (0=TRANSPORT/1=CDD SOLUTION OF FLUX)/
     8 7H NMAX  ,I8,42H   (MAXIMUM NUMBER OF ELEMENTS IN A TRACK)/
     9 7H LMCU  ,I8,42H   (DIMENSION OF MCU FOR ACA ACCELERATION)/
     1 7H NMU   ,I8,48H   (ORDER OF THE POLAR QUADRATURE IN 2D/1 IN 3D)/
     2 7H SCR   ,I8,48H   (0=NO ACCELERATION/1=SCR ACCELERATION OF INNE,
     3 13HR ITERATIONS)/
     4 7H LPS   ,I8,42H   (DIMENSION OF PSJ FOR SCR ACCELERATION)/
     5 7H PACA  ,I8,48H   (PRECONDITIONER FOR SOLVING THE ACA SYSTEM WI,
     6 37HTH BICGSTAB (>2=ILU0, 1=DIAG, 0=NONE)/
     7 7H LEXA  ,I8,48H   (TO FORCE EXACT EXPONENTIAL USAGE IN PRECONDI,
     8 19HTIONER CALCULATION)/
     9 7H LEXF  ,I8,48H   (TO FORCE EXACT EXPONENTIAL USAGE IN FLUX CAL,
     1 9HCULATION)/
     2 7H MAXI  ,I8,39H   (MAXIMUM NUMBER OF INNER ITERATIONS)/
     3 7H LTMT  ,I8,48H   (TO USE TRACK MERGING FOR ACA SYSTEM CALCULAT,
     4 4HION)/
     5 7H STIS  ,I8,48H   (1=SOURCE TERM ISOLATION FOR FLUX INTEGRATION,
     6 1H)/
     7 7H NPJJM ,I8,48H   (NUMBER OF PJJ MODES TO STORE FOR STIS OPTION,
     8 1H)/
     9 7H LMCU0 ,I8,48H   (DIMENSION OF MCU0 FOR ILU0-ACA ACCELERATION),
     1 /)
  130 FORMAT(/
     1 12H REAL PARAM:/
     2 7H EPSI  ,1P,E12.4,33H   (TOLERANCE ON INNER ITERATION)/
     3 7H HDD   ,1P,E12.4,41H   (0.0=STEP CHARACTERISTICS SOLUTION/>>0,
     4 33H.0=DIAMOND DIFFERENCING SOLUTION)/)
*
      RETURN
      END
