*DECK MCGASM
      SUBROUTINE MCGASM(SUBPJJ,SUBDS2,SUBDSP,SUBDSC,KPSYS,IPRINT,IFTRAK,
     1                  NANI,NG,NGEFF,NFI,NREG,NLONG,M,NMU,NANGL,NMAX,
     2                  LC,NDIM,NFUNL,NMOD,NGIND,NZON,NZONA,ISGNR,
     3                  KEYANI,V,VA,CAZ,CPO,ZMU,WZMU,KM,MCU,IM,JU,IPERM,
     4                  LC0,MCU0,IM0,IS,JS,PACA,LPS,LTMT,NPJJM,PJJIND,
     5                  LACA,LPJJ,LPJJAN,SIGAL)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Preconditioning matrices calculation based on the Algebraic Collapsing
* Acceleration developed by I. R. Suslov and R. Le Tellier 
* or Self-Collision Probabilities acceleration developed by G.J. Wu 
* and R. Roy.
*
*  Igor R. Suslov, "An Algebraic Collapsing Acceleration in Long 
*                   Characteristics Transport Theory"
*   Proc. of 11-th Symposium of AER,178\9-188,Csopak,September 2001.
*  Igor R. Suslov, "Solution of Transport Equation in 2- and 3-
*                   dimensional Irregular Geometry by the Method 
*                   of Characteristics"
*   Int. Conf. Mathematical. Methods and Supercomputing in Nuclear
*   Applications,Karlsruhe,1994.
*  G.J. Wu and R. Roy, "Acceleration Techniques for Trajectory-based
*                        Deterministic 3D Transport Solvers",
*   Ann. Nucl. Energy, 30, 567-583 (2003).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): I. Suslov and R. Le Tellier
*
*Parameters: input/output
* SUBPJJ  PJJ calculation subroutine.
* SUBDS2  ACA coefficients summation subroutine.
* SUBDSP  ACA coefficients position subroutine.
* SUBDSC  ACA coefficients calculation subroutine.
* KPSYS   pointer array for each group properties.
* IPRINT  print parameter (equal to zero for no print).
* IFTRAK  tracking file unit number if IOFSET=0.
* NANI    number of Legendre orders.
* NG      number of energy groups.
* NFI     total number of volumes and surfaces for which specific values
*         of the neutron flux and reactions rates are required.
* NREG    number of volumes for which specific values
*         of the neutron flux and reactions rates are required.
* NLONG   order of the corrective system.
* M       number of material mixtures.
* NMU     order of the polar quadrature in 2D / 1 in 3D.
* NANGL   number of tracking angles in the plan.
* NMAX    maximum number of elements in a track
* LC      dimension of vector MCU.
* NDIM    number of dimensions for the geometry.
* NFUNL   number of moments of the flux (in 2D : NFUNL=NANI*(NANI+1)/2).
* NMOD    second dimension of ISGNR.
* NGIND   index of the groups to process.
* NZON    index-number of the mixture type assigned to each volume.
* NZONA   index-number of the mixture type assigned to each volume for ACA.
* ISGNR   array of the spherical harmonics signs for the different reflections.
* KEYANI  index for anisotropy.
* V       volumes and surfaces.
* VA      volumes and surfaces reordered for ACA.
* CAZ     cosines of the different tracking azimuthal angles.
* CPO     cosines of the different tracking polar angles.
*         (polar quadrature in 2D / tracking angles in 3D).
* ZMU     polar quadrature set in 2D.
* WZMU    polar quadrature set in 2D.
* KM      used in ACA acceleration.
* MCU     used in ACA acceleration.
* IM      used in ACA acceleration.
* JU      used in ACA  acceleration for ilu0.
* IPERM   permutation array for the unknowns of the corrective system for ilu0.
* LC0     used in ILU0-ACA acceleration.
* IM0     used in ILU0-ACA acceleration.
* MCU0    used in ILU0-ACA acceleration.
* IS      arrays for surfaces neighbors
* JS      JS(IS(ISOUT)+1:IS(ISOUT+1)) give the neighboring regions to surface ISOUT.
* PACA    type of preconditioner to solve the ACA corrective system.
* LPS     dimension of JS.
* LTMT    tracking merging flag.
* NPJJM   number of pjj modes to store for STIS option.
* PJJIND  index of the modes for STIS option.
* LACA    ACA flag.
* LPJJ    PJJ flag.
* LPJJAN  anisotropic PJJ flag.
* SIGAL   albedos and total cross sections array.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER KPSYS(NGEFF),IPRINT,IFTRAK,NG,NGEFF,NFI,NREG,NLONG,
     1 M,NMU,NANGL,NMAX,LC,NDIM,NFUNL,NMOD,NGIND(NGEFF),NZON(NFI),
     2 NZONA(NFI),ISGNR(NFUNL,NMOD),KEYANI(NFUNL),KM(NFI),MCU(LC),
     3 IM(NLONG+1),JU(NLONG),IPERM(NFI),LC0,MCU0(*),IM0(*),
     4 IS(NFI-NREG+1),JS(LPS),PACA,LPS,NPJJM,PJJIND(NPJJM,2)
      REAL V(NFI),VA(NFI),CAZ(NANGL),CPO(*),ZMU(NMU),WZMU(NMU),
     1 SIGAL(-6:M,NGEFF)
      LOGICAL LTMT,LACA,LPJJ,LPJJAN
      EXTERNAL SUBPJJ,SUBDS2,SUBDSP,SUBDSC
*----
*  LOCAL VARIABLES
*----
      INTEGER IBASE(1)
      INTEGER NOM,IH,IHH,IZMUA,IWZMUA,DIAGF,CF,PREV,NEXT,CQ,DIAGQ,
     1 DIAGFR,CFR,ILUDF,ILUCF,PJJ,PJJD,PSJ,NOM0,IH0,IXSW,IWORK
      REAL T1,T2,T3
      CHARACTER TEXT4*4
      EXTERNAL MCGDS2,MOCDS2,MCGDSP,MOCDSP
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*----
*  MEMORY ALLOCATION
*---- 
      CALL SETARA(IBASE,NMAX,NOM)
      CALL SETARA(BASE,NMAX,IH)
      CALL SETARA(BASE,NMAX,IHH)
      CALL SETARA(BASE,NMU,IZMUA)
      CALL SETARA(BASE,NMU,IWZMUA)      
      IF (LPJJ) THEN
*     Self-Collision Probabilities
         IF (LPJJAN) THEN
            NRNL=NANI*NANI
            CALL SETARA(BASE,NMU*8*NRNL,ITRHAR)
            CALL SETARA(BASE,NMU*NRNL,IRHARM)
         ENDIF
         CALL SETARA(BASE,NREG*NPJJM*NGEFF,PJJ)
         IF (LPS.GT.0) THEN
            CALL SETARA(BASE,LPS*NGEFF,PSJ)
            CALL XDRSET(BASE(PSJ),LPS*NGEFF,0.0)
         ENDIF
         CALL SETARA(BASE,2*NREG*NPJJM*NGEFF,PJJD)
         CALL XDDSET(BASE(PJJD),NREG*NPJJM*NGEFF,0.D0)
      ENDIF
      IF (LACA) THEN
*     Algebraic Collapsing Acceleration
         CALL SETARA(BASE,(M+1)*NANI*NGEFF,IXSW)
         IF (LTMT) THEN
            CALL SETARA(IBASE,NMAX,NOM0)
            CALL SETARA(BASE,NMAX,IH0)
         ENDIF
         CALL SETARA(BASE,2*NLONG*NGEFF,DIAGF)
         CALL SETARA(BASE,2*LC*NGEFF,CF)
         CALL SETARA(IBASE,NMAX,PREV)
         CALL SETARA(IBASE,NMAX,NEXT)
         CALL SETARA(BASE,LC*NGEFF,CQ)
         CALL SETARA(BASE,NLONG*NGEFF,DIAGQ)
         CALL SETARA(BASE,NLONG,DIAGFR)
         CALL SETARA(BASE,LC,CFR)
         CALL SETARA(BASE,6*NMAX,IWORK)
         IF (PACA.GE.2) THEN
            CALL SETARA(BASE,NLONG,ILUDF)
            IF (LC0.GT.0) CALL SETARA(BASE,LC0,ILUCF)
         ENDIF
         DO II=1,NGEFF
            JPSYS=KPSYS(II)
            CALL LCMGET(JPSYS,'DRAGON-S0XSC',BASE(IXSW+(II-1)*(M+1)))
         ENDDO
         CALL XDRSET(BASE(DIAGQ),NLONG*NGEFF,0.0)
         CALL XDRSET(BASE(CQ),LC*NGEFF,0.0)
         CALL XDDSET(BASE(DIAGF),NLONG*NGEFF,0.D0)
         CALL XDDSET(BASE(CF),LC*NGEFF,0.D0)
      ENDIF 
*
      NMUA=NMU
      DO IE=1,NMUA
         BASE(IZMUA-1+IE)=ZMU(IE)
         BASE(IWZMUA-1+IE)=WZMU(IE)       
      ENDDO
      IF ((NDIM.EQ.2).AND.LTMT) THEN
         NMUA=1
         ZMUI=BASE(IZMUA)
         W=BASE(IWZMUA)               
         TEMP=ZMUI*W
         DO IE=2,NMU
            ZMUI=BASE(IZMUA-1+IE)
            WZMUI=BASE(IWZMUA-1+IE)
            W=W+WZMUI
            TEMP=TEMP+ZMUI*WZMUI
         ENDDO               
         BASE(IZMUA)=TEMP/W
         BASE(IWZMUA)=W
      ENDIF
*---
* SUMMATION UPON THE TRACKING
*---
      REWIND IFTRAK
      READ(IFTRAK) TEXT4,NCOMNT,NBTR  
      DO ICOM=1,NCOMNT+7
         READ(IFTRAK)
      ENDDO
      CALL KDRCPU(T1)
      IANGL0=0
      NMERG=0
      NBTRTMT=0
      NTS=0
      NTSTMT=0
*
      DO 25 ILINE=1,NBTR
*     loop over tracking file trajectories
         READ(IFTRAK) IANGL,NSEG,WEIGHT,
     1        (IBASE(NOM-1+II),II=1,NSEG),
     2        (BASE(IH-1+II),II=1,NSEG)
         IF (NSEG.GT.NMAX) CALL XABORT('MCGASM: TRACKING OVERFLOW.')
         DO II=1,NSEG
            IF (IBASE(NOM-1+II).LT.0) THEN
               IBASE(NOM-1+II)=NREG-IBASE(NOM-1+II)
            ENDIF
         ENDDO
         IF ((LPJJ).AND.(NSEG.GT.0)) THEN
*        ----------------------------
*        Self-Collision Probabilities
*        ----------------------------
            IF (LPJJAN) THEN
               IF (IANGL.NE.IANGL0) THEN
                  IANGL0=IANGL
                  IF (NDIM.EQ.2) THEN
                     CALL MOCCHR(NDIM,NANI-1,NMU,CPO,CAZ(IANGL),
     1                    BASE(IRHARM))
                  ELSE
                     CALL MOCCHR(NDIM,NANI-1,NMU,CPO(IANGL),CAZ(IANGL),
     1                    BASE(IRHARM))
                  ENDIF
                  DO JM=1,NMOD
                  DO JF=1,NFUNL
                  DO IE=1,NMU
                    IND2=IE+(JF-1)*NMU
                    IND1=IND2+(JM-1)*NFUNL*NMU
                    BASE(ITRHAR-1+IND1)=ISGNR(JF,JM)*BASE(IRHARM-1+IND2)
*                       (IE,JF,JM)     =     (JF,JM)     (IE,JF)
                  ENDDO
                  ENDDO
                  ENDDO
               ENDIF
            ENDIF
            CALL MCGDS4(SUBPJJ,NSEG,NMU,LPS,NANI,NFUNL,NMOD,NG,NGEFF,
     1           WEIGHT,BASE(ITRHAR),BASE(IH),ZMU,WZMU,IBASE(NOM),
     2           BASE(IHH),NZON,NFI,NREG,NDIM,M,IS,JS,BASE(PJJD),
     3           BASE(PSJ),LPJJAN,NPJJM,PJJIND,SIGAL)
         ENDIF
*        ----------------------------
         NTS=NTS+NSEG
         IF (LTMT) THEN
*        ----------------
*        Tracking Merging (angle by angle) for ACA
*        ----------------
            IF (NMERG.GT.0) THEN
               IF (NSEG.NE.NSEG0) GOTO 20
               DO II=1,NSEG
                  IF (IBASE(NOM-1+II).NE.IBASE(NOM0-1+II)) GOTO 20
               ENDDO
*              merge this track with the previous one
               DO II=1,NSEG
                  BASE(IH0-1+II)=BASE(IH0-1+II)+WEIGHT*BASE(IH-1+II)
               ENDDO
               WEIGHT0=WEIGHT0+WEIGHT
            ELSE
*              start
               DO II=1,NSEG
                  BASE(IH0-1+II)=WEIGHT*BASE(IH-1+II)
                  IBASE(NOM0-1+II)=IBASE(NOM-1+II)
               ENDDO
               WEIGHT0=WEIGHT
            ENDIF
            NMERG=NMERG+1
            NSEG0=NSEG
            GOTO 15
 20         CONTINUE 
*           finalize this "merged" track and start a new one
            NBTRTMT=NBTRTMT+1
            NTSTMT=NTSTMT+NSEG0
            DO II=1,MAX(NSEG0,NSEG)
               TEMP=BASE(IH-1+II)
               BASE(IH-1+II)=BASE(IH0-1+II)/WEIGHT0
               BASE(IH0-1+II)=WEIGHT*TEMP
               NTEMP=IBASE(NOM-1+II)
               IBASE(NOM-1+II)=IBASE(NOM0-1+II)
               IBASE(NOM0-1+II)=NTEMP
            ENDDO
            NMERG=1
            TEMP=WEIGHT
            WEIGHT=WEIGHT0
            WEIGHT0=TEMP
            NTEMP=NSEG
            NSEG=NSEG0
            NSEG0=NTEMP
         ENDIF
*        ----------------
         IF ((LACA).AND.(NSEG.GT.0)) THEN
*        ---------------------------------
*        Algebraic Collapsing Acceleration
*        ---------------------------------
            DO II=1,NSEG
               IBASE(NOM-1+II)=IPERM(IBASE(NOM-1+II))
            ENDDO
            CALL MCGDS1(SUBDS2,SUBDSP,SUBDSC,NSEG,NMUA,NANI,NG,NGEFF,
     1                  WEIGHT,BASE(IH),BASE(IZMUA),BASE(IWZMUA),
     2                  IBASE(NOM),BASE(IHH),NZONA,NLONG,NFI,NDIM,LC,M,
     3                  KM,IM,MCU,BASE(DIAGF),BASE(DIAGQ),BASE(CF),
     4                  BASE(CQ),IBASE(PREV),IBASE(NEXT),SIGAL,
     5                  BASE(IXSW),BASE(IWORK))
         ENDIF
*        ---------------------------------
 15      CONTINUE
 25   CONTINUE
*
      IF ((LTMT).AND.(IPRINT.GT.1)) THEN
         WRITE(*,*) 'TRACKING MERGING:',NBTR,' TRACKS ->',
     1              NBTRTMT,' EQUIVALENT TRACKS'
         WRITE(*,*) '                 ',
     1              NTS,' SEGMENTS ->',
     2              NTSTMT, ' SEGMENTS'
         WRITE(*,*) '                 ',
     1              NMU,' POLAR ANGLES ->',
     2              ' 1 EQUIVALENT POLAR ANGLE'
      ENDIF
      CALL KDRCPU(T2)
      IF (IPRINT.GT.0) THEN
         WRITE(6,100) 'SUMMATION UPON THE TRACKING FOR ACA/SCR ',(T2-T1)
      ENDIF
*---
*  FOR ACA: (IF PACA.GE.2)
*  CALCULATION OF ILU0 PRECONDITIONER FOR BICGSTAB ITERATIONS
*---
      IF (LACA) THEN
         DO II=1,NGEFF
            IG=NGIND(II)
            JPSYS=KPSYS(II)
            IF (IPRINT.GT.2) THEN
               WRITE(6,200) 'GROUP',IG
            ENDIF
            CALL MCGDS3(IPRINT,NLONG,PACA,M,SIGAL(0,II),
     1           BASE(IXSW+(II-1)*(M+1)),VA,NZONA,LC,KM,MCU,IM,JU,LC0,
     2           IM0,MCU0,BASE(DIAGF+2*(II-1)*NLONG),
     3           BASE(CF+2*(II-1)*LC),BASE(DIAGQ+(II-1)*NLONG),
     4           BASE(DIAGFR),BASE(CFR),BASE(ILUDF),BASE(ILUCF))
*     
            IF(IPRINT.GT.3) THEN
               CALL PRINAM('DIAGF ',BASE(DIAGFR),NLONG)
               CALL PRINAM('CF    ',BASE(CFR),LC)
            ENDIF
            CALL LCMPUT(JPSYS,'DIAGF$MCCG',NLONG,2,BASE(DIAGFR))
            CALL LCMPUT(JPSYS,'CF$MCCG',LC,2,BASE(CFR))
            IF (PACA.GE.2) THEN
               CALL LCMPUT(JPSYS,'ILUDF$MCCG',NLONG,2,BASE(ILUDF))
               IF (LC0.GT.0)
     1            CALL LCMPUT(JPSYS,'ILUCF$MCCG',LC0,2,BASE(ILUCF))
            ENDIF
            IF(IPRINT.GT.3) THEN
               CALL PRINAM('DIAGQ ',BASE(DIAGQ+(II-1)*NLONG),NLONG)
               CALL PRINAM('CQ    ',BASE(CQ+(II-1)*LC),LC)
            ENDIF
            CALL LCMPUT(JPSYS,'CQ$MCCG',LC,2,BASE(CQ+(II-1)*LC))
            CALL LCMPUT(JPSYS,'DIAGQ$MCCG',NLONG,2,
     1           BASE(DIAGQ+(II-1)*NLONG))
         ENDDO
         CALL KDRCPU(T3)
         IF (IPRINT.GT.0) THEN
            WRITE(6,100) 'CALCULATION OF ACA PRECONDITIONER       ',
     1                   (T3-T2)
         ENDIF
*
         IF (PACA.GE.2) THEN
            IF (LC0.GT.0) CALL RLSARA(BASE(ILUCF))
            CALL RLSARA(BASE(ILUDF))
         ENDIF
         CALL RLSARA(BASE(IWORK))
         CALL RLSARA(BASE(CFR))
         CALL RLSARA(BASE(DIAGFR))
         CALL RLSARA(BASE(DIAGQ))
         CALL RLSARA(BASE(CQ))
         CALL RLSARA(IBASE(NEXT))
         CALL RLSARA(IBASE(PREV))
         CALL RLSARA(BASE(CF))
         CALL RLSARA(BASE(DIAGF))
         IF (LTMT) THEN
            CALL RLSARA(BASE(IH0))
            CALL RLSARA(IBASE(NOM0))
         ENDIF
*         stop -> rien
         CALL RLSARA(BASE(IXSW))
*         stop -> plante
      ENDIF
*----
*  FOR SCR/STIS:
*  VECTORS NORMALIZATION
*----
      IF (LPJJ) THEN
         CALL MCGDS6(NGEFF,NPJJM,NREG,BASE(PJJD),V,BASE(PJJ))
         CALL RLSARA(BASE(PJJD))
*
         DO II=1,NGEFF
            JPSYS=KPSYS(II)
            IF(IPRINT.GT.3) THEN
               CALL PRINAM('PJJ    ',BASE(PJJ+(II-1)*NREG*NPJJM),
     1                               NREG*NPJJM)
               IF (LPS.GT.0) THEN
                  CALL PRINAM('PSJ    ',BASE(PSJ+(II-1)*LPS),LPS)
               ENDIF
            ENDIF
            CALL LCMPUT(JPSYS,'PJJ$MCCG',NREG*NPJJM,2,
     1                  BASE(PJJ+(II-1)*NREG*NPJJM))
            IF (LPS.GT.0) THEN
               CALL LCMPUT(JPSYS,'PSJ$MCCG',LPS,2,BASE(PSJ+(II-1)*LPS))
            ENDIF
         ENDDO
*
         IF (LPS.GT.0) CALL RLSARA(BASE(PSJ))
         CALL RLSARA(BASE(PJJ))
         IF (LPJJAN) THEN
            CALL RLSARA(BASE(IRHARM))
            CALL RLSARA(BASE(ITRHAR))
         ENDIF
      ENDIF
*
      CALL RLSARA(BASE(IWZMUA))
      CALL RLSARA(BASE(IZMUA))
      CALL RLSARA(BASE(IHH))
      CALL RLSARA(BASE(IH))
      CALL RLSARA(IBASE(NOM))
*
      RETURN
*
 100  FORMAT('   -->>TIME SPENT IN: ',A40,': ',F10.3,' s.')
 200  FORMAT(1X,A6,1X,I4)
      END
