*DECK MCGFCR
      SUBROUTINE MCGFCR(IPRINT,IG,II,NG,NGEFF,KPN,N,NREG,NANI,NFUNL,M,
     1                  LPERM,KEYFLX,KEYCUR,NZON,NGINDV,FORM,REBAL,FI,
     2                  FIOLD,AR,SC,NJJ,IJJ,IPOS,XSCAT,IPERM)
*
*-----------------------------------------------------------------------
*
*Purpose:
* Compute residual of a previous free iterations for ACA method.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): R. Le Tellier
*
*Parameters: input
* IPRINT  print parameter (equal to zero for no print).
* IG      index of group to process in "NG format".
* II      index of group to process in "NGEFF format".
* NG      number of groups.
* NGEFF   number of groups to process.
* KPN     total number of unknowns in vectors SUNKNO and FUNKNO.
* N       total number of volumes for which specific values
*         of the neutron flux and reactions rates are required.
* NREG    number of volumes.
* NANI    scattering anisotropy (=1 for isotropic scattering).
* NFUNL   number of moments of the flux (in 2D : NFUNL=NANI*(NANI+1)/2).
* M       number of material mixtures.
* LPERM   flag to know if a permutation array is provided.
* KEYFLX  position of flux elements in FI vector.
* KEYCUR  position of current elements in FI vector.
* NZON    index-number of the mixture type assigned to each volume.
* NGINDV  index to pass from "NGEFF format" to "NG format"
* FORM    input flux format flag (.TRUE. : same format as output flux ; .FALSE. same format as input source).
* REBAL   type of acceleration (.TRUE. : rebalancing ; .FALSE. : inner iterations acceleration).
* FI      zonal scalar flux.
* SC      macroscopic "in group" scattering cross section.
* IPERM   permutation array.
*Parameters: output
* AR      residual form previous free iteration.
*Parameters: scratch
* AR
* NJJ
* IJJ
* IPOS
* XSCAT
*-----------------------------------------------------------------------
*
*----
* SUBROUTINE ARGUMENTS
*----
      INTEGER IPRINT,IG,II,NG,NGEFF,KPN,N,NREG,NANI,NFUNL,M,
     1 KEYFLX(NREG,NFUNL),KEYCUR(*),IPERM(N),NZON(N),NGINDV(NG),
     2 NJJ(0:M),IJJ(0:M),IPOS(0:M)
      REAL FIOLD(KPN,NG),SC(0:M),XSCAT(0:M*NG)
      DOUBLE PRECISION AR(N),FI(KPN,NGEFF)
      LOGICAL REBAL,FORM,LPERM
*
      IGG=IG
      IF (FORM) IGG=II
      DO I=1,N
         IF (LPERM) THEN
            J=IPERM(I)
         ELSE
            J=I
         ENDIF
         IBM=NZON(J)
         IF(IBM.GE.0) THEN
            SIGC=SC(IBM)
            IND=KEYFLX(J,1)
         ELSE
            SIGC=0.5
            IND=KEYCUR(J-NREG)
         ENDIF
         AR(I)=(FI(IND,II)-FIOLD(IND,IGG))*SIGC
         IF ((REBAL).AND.(IBM.GT.0)) THEN
*        rebalancing option on : contribution from other groups.
            JG=IJJ(IBM)
            DO 10 JND=1,NJJ(IBM)
               IF(JG.NE.IG) THEN
               JJ=NGINDV(JG)
               IF (JJ.GT.0) THEN
                  JGG=JG
                  IF (FORM) JGG=JJ
                  AR(I)=AR(I)+XSCAT(IPOS(IBM)+JND-1)*
     1                 (FI(IND,JJ)-FIOLD(IND,JGG))
               ENDIF
               ENDIF
 10         JG=JG-1
         ENDIF
      ENDDO
*
      RETURN
      END
