*DECK MCGFLX
      SUBROUTINE MCGFLX(SUBFFI,SUBFFA,SUBSCH,CYCLIC,KPSYS,IPRINT,IPTRK,
     1           IFTRAK,IPMACR,NDIM,K,KPN,NLONG,NREG,NSOUT,NG,NGEFF,
     2           NGIND,NZON,MATALB,V,FIMEM,QFR,M,NANI,MAXI,IAAC,KRYL,
     3           ISCR,NMU,NANGL,NMAX,LC,EPSI,CAZ,CPO,ZMU,WZMU,PACA,
     4           NFUNL,NMOD,ISGNR,KEYFLX,KEYCUR,SIGAL,LPS,REPS,EPS,ITST,
     5           NCONV,LNCONV,REBFLG,STIS,NPJJM,NOM,H,SOUR,FLUX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* MOC solution of the transport equation in 2D,3D-irregular geometry.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert and R. Le Tellier
*
*Parameters: input/output
* SUBFFI  isotropic flux integration subroutine.
* SUBFFA  anisotropic flux integration subroutine.
* SUBSCH  track coefficients calculation subroutine.
* CYCLIC  cyclic tracking flag.
* KPSYS   pointer array for each group properties.
* IPRINT  print parameter (equal to zero for no print).
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  tracking file unit number.
* IPMACR  pointer to the macrolib LCM object.
* NDIM    number of dimensions for the geometry.
* K       total number of volumes-surfaces for which specific values
*         of the neutron flux and reactions rates are required.
* KPN     total number of unknowns in vectors SUNKNO and FUNKNO.
* NLONG   number of spatial unknowns.
* NREG    number of regions (volumes).
* NSOUT   number of outer surfaces.
* NG      number of groups.
* NGEFF   number of groups to process.
* NGIND   index of the groups to process.
* NZON    mixture-albedo index array in MCCG format.
* MATALB  albedo-mixture index array in MOCC format.
* V       volumes and surfaces.
* FIMEM   unknown vector.
* QFR     input source vector.
* M       number of material mixtures.
* NANI    scattering anisotropy (=1 for isotropic scattering).
* MAXI    maximum number of inner iterations.
* IAAC    no acceleration / CDD acceleration of inner iterations (0/1).
* KRYL    Bi-CGSTAB scheme used / GMRES scheme not used / GMRES scheme used 
*         (Krylov subspace dimension = KRYL) (<0 / 0 / >0)
* ISCR    no acceleration / SCR acceleration of inner iterations (0/1).
* NMU     order of the polar quadrature in 2D / 1 in 3D.
* NANGL   number of tracking angles in the plan.
* NMAX    maximum number of elements in a track.
* LC      dimension of MCU vector.
* EPSI    tolerance of inner iterations.
* CAZ     cosines of the different tracking azimuthal angles.
* CPO     cosines of the different tracking polar angles.
*         (polar quadrature in 2D / tracking angles in 3D).
* ZMU     polar quadrature set in 2D.
* WZMU    polar quadrature set in 2D.
* PACA    type of preconditioner to solve the ACA corrective system.
* NFUNL   number of moments of the flux (in 2D : NFUNL=NANI*(NANI+1)/2).
* NMOD    second dimension of ISGNR.
* ISGNR   array of the spherical harmonics signs. 
* KEYFLX  position of flux elements in FUNKNO vector.
* KEYCUR  position of current elements in FUNKNO vector.
* SIGAL   total cross-section and albedo array.
* LPS     dimension of PSJ vector for SCR acceleration.
* EPS     array of the precision reached for each group.
* REPS    array of the precision for each iteration and each group.
* ITST    array of the number of iterations for each group.
* NCONV   array of convergence flag for each group.
* LNCONV  number of unconverged groups.
* REBFLG  ACA or SCR rebalancing flag.
* STIS    'Source term isolation' option for flux integration.
* NPJJM   number of pjj modes to store for STIS option.
*
*Parameters: scratch
* NOM
* H
* SOUR
* FLUX
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NGEFF,KPSYS(NGEFF),IPRINT,IPTRK,IFTRAK,IPMACR,NDIM,K,KPN,
     1 NLONG,NREG,NSOUT,NG,NGIND(NGEFF),NZON(NLONG),MATALB(-NSOUT:NREG),
     2 M,NANI,MAXI,IAAC,KRYL,NMU,NANGL,NMAX,LC,PACA,ISCR,LPS,
     3 ITST(NGEFF),LNCONV,NFUNL,NMOD,ISGNR(NFUNL,NMOD),
     4 KEYFLX(NREG,NFUNL),KEYCUR(NLONG-NREG),STIS,NPJJM,NOM(NMAX)
      REAL V(NLONG),FIMEM(KPN,NG),QFR(KPN,NG),EPSI,CAZ(NANGL),
     1 CPO(*),ZMU(NMU),WZMU(NMU),SIGAL(-6:M,NGEFF),REPS(MAXI,NGEFF),
     1 EPS(NGEFF),H(NMAX)
      DOUBLE PRECISION SOUR(KPN,NGEFF),FLUX(KPN,NGEFF)
      LOGICAL CYCLIC,NCONV(NGEFF),REBFLG
      EXTERNAL SUBFFI,SUBFFA,SUBSCH
*----
*  LOCAL VARIABLES
*----
      PARAMETER (IUNOUT=6)
      INTEGER IBASE(1)
      REAL MAXDIF,MAXFL
      LOGICAL IACS,FORM
      COMMON BASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*---
      IF(IPRINT.GT.5) THEN
         DO II=1,NGEFF
            IG=NGIND(II)
            WRITE(6,*) 'GROUP(',IG,')'
            CALL PRINAM('FI-0  ',FIMEM(1,IG),KPN)
         ENDDO
      ENDIF
*----
*  INNER ITERATIONS FOR THE TRANSPORT SOLUTION
*----
      IF (KRYL.EQ.0) THEN
*     ---------------------------
*     Richardson Iterative Scheme
*     ---------------------------
         EPSINT=0.1*EPSI
         MAXINT=200
         ITER=0
         FORM=.FALSE.
         IACS=.FALSE.
         IF ((IAAC.GT.0).AND.(ISCR.GT.0)) THEN
*        alternate preconditioners
            IACS=.TRUE.
            IAAC2=IAAC
            ISCR2=ISCR
            ISCR=0
         ENDIF
         DO WHILE ((LNCONV.GT.0).AND.(ITER.LT.MAXI))
            ITER=ITER+1
            CALL MCGFL1(SUBFFI,SUBFFA,SUBSCH,CYCLIC,KPSYS,IPRINT,IPTRK,
     1           IFTRAK,IPMACR,NDIM,K,KPN,NLONG,FLUX,NZON,MATALB,M,NANI,
     2           NMU,NMAX,NANGL,NREG,NSOUT,NG,NGEFF,NGIND,H,NOM,SOUR,
     3           IAAC,ISCR,LC,PACA,EPSINT,MAXINT,NFUNL,NMOD,ISGNR,
     4           KEYFLX,KEYCUR,QFR,FIMEM,CAZ,CPO,ZMU,WZMU,V,SIGAL,LPS,
     5           NCONV,FORM,(MAXI.EQ.1),STIS,NPJJM,REBFLG)
            IF (IACS) THEN
*           alternate preconditioners
               IF (ISCR.GT.0) THEN
                  ISCR=0
                  IAAC=IAAC2
               ELSEIF(IAAC.GT.0) THEN
                  ISCR=ISCR2
                  IAAC=0
               ENDIF
            ENDIF
*           residual calculation and update NCONV
            DO II=1,NGEFF
               IF (NCONV(II)) THEN
                  IG=NGIND(II)
                  IF (MAXI.GT.1) THEN
                     MAXFL=0.0
                     MAXDIF=0.0
                     DO I=1,KPN
                        TEMP=REAL(ABS(FLUX(I,II)))
                        MAXFL=MAX(TEMP,MAXFL)
c                        MAXFL=MAX(ABS(FIMEM(I,IG)),MAXFL)
                        FIMEM(I,IG)=FIMEM(I,IG)-FLUX(I,II)
                        MAXDIF=MAX(ABS(FIMEM(I,IG)),MAXDIF)
                     ENDDO
                     IF (MAXFL.EQ.0.0) MAXFL=1.0
                     REPS(ITER,II)=MAXDIF/MAXFL
                     IF (ITER.GT.2) THEN
                     IF (REPS(ITER,II).GT.REPS(ITER-1,II)) THEN
*                    preconditioning cutoff
                        IAAC=0
                        ISCR=0
                     ENDIF
                     ENDIF
                     IF ((REPS(ITER,II).LT.EPSI).OR.(ITER.EQ.MAXI)) THEN
                        NCONV(II)=.FALSE.
                        ITST(II)=ITER
                        EPS(II)=REPS(ITER,II)
                        LNCONV=LNCONV-1
                     ENDIF
                  ENDIF
                  DO I=1,KPN
                     FIMEM(I,IG)=FLUX(I,II)
                  ENDDO
               ENDIF
            ENDDO
         ENDDO
      ELSEIF(KRYL.GT.0) THEN
*     ----------------------
*     GMRES Iterative Scheme
*     ----------------------
         CALL SETARA(BASE,(KPN*NGEFF),IRHS)
         CALL SETARA(BASE,(KPN*NGEFF),IPHI)
         CALL SETARA(BASE,(KPN*(KRYL+1)*NGEFF),IAUX)
         CALL SETARA(BASE,((KRYL+1)*KRYL*NGEFF),IHH)
         CALL SETARA(BASE,(KRYL*NGEFF),ICOS)
         CALL SETARA(BASE,(KRYL*NGEFF),ISIN)
         CALL SETARA(BASE,((KRYL+1)*NGEFF),IRS)
         CALL SETARA(BASE,((KRYL+1)*NGEFF),IRS2)
         CALL MCGMRE(SUBFFI,SUBFFA,SUBSCH,CYCLIC,KPSYS,IPRINT,IPTRK,
     1        IFTRAK,IPMACR,NDIM,K,KPN,NLONG,FLUX,NZON,MATALB,M,NANI,
     2        NMU,NMAX,NANGL,NREG,NSOUT,H,NOM,SOUR,IAAC,ISCR,LC,PACA,
     3        ITST,MAXI,QFR,FIMEM,CAZ,CPO,ZMU,WZMU,V,EPS,EPSI,REPS,KRYL,
     4        SIGAL,LPS,BASE(IRHS),BASE(IPHI),BASE(IAUX),BASE(IHH),
     5        BASE(ICOS),BASE(ISIN),BASE(IRS),BASE(IRS2),NG,NGEFF,NGIND,
     6        NCONV,LNCONV,NFUNL,NMOD,ISGNR,KEYFLX,KEYCUR,STIS,NPJJM,
     7        REBFLG)
         CALL RLSARA(BASE(IRS2))
         CALL RLSARA(BASE(IRS))
         CALL RLSARA(BASE(ISIN))
         CALL RLSARA(BASE(ICOS))
         CALL RLSARA(BASE(IHH))
         CALL RLSARA(BASE(IPHI))
         CALL RLSARA(BASE(IRHS))
         CALL RLSARA(BASE(IAUX))
      ELSE
*     --------------------------
*     Bi-CGSTAB Iterative Scheme
*     --------------------------
         CALL SETARA(BASE,(KPN*NGEFF),IRHS)
         CALL SETARA(BASE,(KPN*NGEFF),IPI)
         CALL SETARA(BASE,(KPN*NGEFF),IRI)
         CALL SETARA(BASE,(KPN*NGEFF),ISI)
         CALL SETARA(BASE,(KPN*NGEFF),IROT)
         CALL SETARA(BASE,(KPN*NGEFF),IAPI)
         CALL SETARA(BASE,(2*NGEFF),IAUX)
         CALL MCGBIC(SUBFFI,SUBFFA,SUBSCH,CYCLIC,KPSYS,IPRINT,IPTRK,
     1        IFTRAK,IPMACR,NDIM,K,KPN,NLONG,FLUX,NZON,MATALB,M,NANI,
     2        NMU,NMAX,NANGL,NREG,NSOUT,H,NOM,SOUR,IAAC,ISCR,LC,PACA,
     3        ITST,MAXI,QFR,FIMEM,CAZ,CPO,ZMU,WZMU,V,EPS,EPSI,REPS,
     4        SIGAL,LPS,BASE(IRHS),BASE(IPI),BASE(IRI),BASE(ISI),
     5        BASE(IROT),BASE(IAPI),BASE(IAUX),NG,NGEFF,NGIND,NCONV,
     6        LNCONV,NFUNL,NMOD,ISGNR,KEYFLX,KEYCUR,STIS,NPJJM,REBFLG)
         CALL RLSARA(BASE(IAUX))
         CALL RLSARA(BASE(IAPI))
         CALL RLSARA(BASE(IROT))
         CALL RLSARA(BASE(ISI))
         CALL RLSARA(BASE(IRI))
         CALL RLSARA(BASE(IPI))
         CALL RLSARA(BASE(IRHS))
      ENDIF
*
      RETURN
      END
