*DECK NCRRGR
      SUBROUTINE NCRRGR(IPLIB,IPCPO,IPMAP,IACCS,NMIX,NGRP,IMPX,NMIL,
     1 NCAL,KFUEL,NW,NZC,NCH,NK,ITER,MIXC,TERP,INDEX,BZC,ZONEB,WINT,
     2 WPAR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover and/or interpolate compo information and store them in a new
* or existing microlib. Use global and local parameters from a MAP
* object and optional user-defined values.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   address of the microlib LCM object.
* IPCPO   address of the multidimensional compo object.
* IPMAP   address of the MAP object.
* IACCS   =0 microlib is created; =2 ... is updated.
* NMIX    maximum number of material mixtures in the microlib.
* NGRP    number of energy groups.
* IMPX    print parameter (equal to zero for no print).
* NMIL    number of material mixtures in the compo.
* NCAL    number of elementary calculations in the compo.
* KFUEL   type of fuel property calculation.
* NW      number of different fuel burnup distributions.
* NZC     number of fuel combustion zones.
* NCH     number of fuel channels.
* NK      number of fuel bundles per channel.
*
*Parameters: output
* ITER    completion flag (=0: all over; =1: use another compo).
*
*Parameters: scratch
* MIXC
* TERP
* INDEX   fuel type number
* BZC     average exit burnup per combustion zones
* ZONEB   combustion zone index
* WINT    fuel burnup distributions.
* WPAR    other parameter distributions.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB,IPCPO,IPMAP,IACCS,NMIX,NGRP,IMPX,NMIL,NCAL,KFUEL,
     1 NW,NZC,NCH,NK,ITER,MIXC(NMIX),INDEX(NCH*NK),ZONEB(NCH)
      REAL TERP(NCAL,NMIX),BZC(NZC),WINT(NW,2),WPAR(NCH*NK,4)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (REPS=1.0E-4,IOUT=6,MAXPAR=50,MAXLIN=50,MAXVAL=100,
     1 NSTATE=40,MAXISD=250,NPARM=5)
      CHARACTER TEXT12*12,PARKEY(MAXPAR)*12,PARFMT(MAXPAR)*8,
     1 PARKEL(MAXPAR)*12,HSMG*131,COMMEN(MAXLIN)*80,VALH(MAXPAR)*12,
     2 RECNAM*12,VCHAR(MAXVAL)*12,HISO(MAXISD)*12,HPARM(NPARM)*12
      INTEGER ISTATE(NSTATE),VALI(MAXPAR),NVALUE(MAXPAR),VINTE(MAXVAL),
     1 MUPLET(2*MAXPAR),MUTYPE(2*MAXPAR)
      DOUBLE PRECISION DFLOTT
      REAL VALR(2*MAXPAR,2),VREAL(MAXVAL),CONC(MAXISD)
      LOGICAL LISO,LPARM(NPARM)
      COMMON BASE(1)
      DATA HPARM/'B-BURN','B-TCOOL','B-DCOOL','B-TFUEL','POWER-BUN'/
*----
*  RECOVER TABLE-OF-CONTENT INFORMATION FOR THE COMPO.
*----
      CALL LCMGET(IPCPO,'STATE-VECTOR',ISTATE)
      NPAR=ISTATE(5)
      NLOC=ISTATE(6)
      NCOMLI=ISTATE(10)
      CALL COMGTC(IPCPO,'COMMEN',80,NCOMLI,COMMEN)
      IF(NPAR.GT.0) THEN
         CALL LCMSIX(IPCPO,'GLOBAL',1)
         CALL COMGTC(IPCPO,'PARKEY',12,NPAR,PARKEY)
         CALL COMGTC(IPCPO,'PARFMT',8,NPAR,PARFMT)
         CALL LCMSIX(IPCPO,' ',2)
      ENDIF
      IF(NLOC.GT.0) THEN
         CALL LCMSIX(IPCPO,'LOCAL',1)
         CALL COMGTC(IPCPO,'PARKEY',12,NLOC,PARKEL)
         CALL LCMSIX(IPCPO,' ',2)
      ENDIF
      IF(IMPX.GT.0) WRITE(IOUT,'(1X,A)') (COMMEN(I),I=1,NCOMLI)
      CALL XDRSET(TERP,NCAL*NMIX,0.0)
      CALL XDISET(MIXC,NMIX,0)
*----
*  RECOVER INFORMATION FOR THE MAP OBJECT.
*----
      CALL LCMGET(IPMAP,'ZONE',ZONEB)
      CALL LCMGET(IPMAP,'INDEX',INDEX)
      CALL LCMGET(IPMAP,'BURN-ZONE',BZC)
      CALL XDRSET(WINT,2*NW,0.0)
      IF((KFUEL.EQ.4).OR.(KFUEL.EQ.5)) THEN
         CALL LCMGET(IPMAP,'BURN-DEB',WINT(1,1))
      ELSEIF((KFUEL.EQ.2).OR.(KFUEL.EQ.3)) THEN
         CALL LCMGET(IPMAP,'BURN-DEB',WINT(1,1))
         CALL LCMGET(IPMAP,'BURN-END',WINT(1,2))
      ENDIF
      LPARM(1)=.TRUE.
      DO 10 JPARM=2,NPARM
      CALL LCMLEN(IPMAP,'HPARM(JPARM)',LENGTH,ITYLCM)
      LPARM(JPARM)=(LENGTH.NE.0)
      IF(LPARM(JPARM)) CALL LCMGET(IPMAP,'HPARM(JPARM)',WPAR(1,JPARM-1))
   10 CONTINUE
*----
*  READ (INTERP_DATA) AND SET VALI, VALR AND VALH PARAMETERS
*  CORRESPONDING TO THE INTERPOLATION POINT. FILL MUPLET FOR
*  PARAMETERS SET WITHOUT INTERPOLATION.
*----
      LISO=.TRUE.
   20 CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
      IF(INDIC.NE.3) CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
   30 IF(TEXT12.EQ.'MIX') THEN
         CALL XDISET(MUPLET,NPAR+NLOC,0)
         CALL XDISET(MUTYPE,NPAR+NLOC,0)
         CALL XDISET(VALI,NPAR,0)
         CALL XDRSET(VALR(1,1),NPAR+NLOC,0.0)
         CALL XDRSET(VALR(1,2),NPAR+NLOC,0.0)
         DO 40 I=1,NPAR
   40    VALH(I)=' '
         CALL REDGET(INDIC,IBM,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.1) CALL XABORT('NCRRGR: INTEGER DATA EXPECTED.')
         IBMOLD=1
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
         IF(TEXT12.EQ.'FROM') THEN
            CALL REDGET(INDIC,IBMOLD,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.NE.1) CALL XABORT('NCRRGR: INTEGER DATA EXPECTED.')
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.NE.3) CALL XABORT('NCRRGR: CHARACTER DATA EXPECTE'
     1      //'D.')
         ENDIF
         MIXC(IBM)=IBMOLD
         DO 130 IRES=1,NCH*NK
         IF(INDEX(IRES).EQ.IBM) THEN
	    ITYPE=1
            DO 120 JPARM=1,NPARM
	    IF(.NOT.LPARM(JPARM)) GO TO 120
            IF(HPARM(JPARM).EQ.'B-BURN') THEN
               IP=(IRES-1)/NCH
               ICH=IRES-IP*NCH
               IF(KFUEL.EQ.1) THEN
                  BURN0=0.0
                  BURN1=BZC(ZONEB(ICH))
               ELSE IF((KFUEL.EQ.3).OR.(KFUEL.EQ.2)) THEN
                  BURN0=WINT(IRES,1)
                  BURN1=WINT(IRES,2)
               ELSE IF(KFUEL.EQ.5) THEN
                  IBUND=IRES-ICH*NK
                  IW=ZONEB(ICH)*IBUND
                  BURN0=WINT(IW,1)
                  BURN1=BURN0
               ELSE IF(KFUEL.EQ.4) THEN
                  BURN0=WINT(IRES,1)
                  BURN1=BURN0
               ENDIF
               VALR(I,1)=BURN0
               VALR(I,2)=BURN1
	       IF(BURN0.NE.BURN1) ITYPE=2
	    ELSE IF(HPARM(JPARM).EQ.'B-TCOOL') THEN
               VALR(I,1)=WPAR(IRES,1)
               VALR(I,2)=WPAR(IRES,1)
	    ELSE IF(HPARM(JPARM).EQ.'B-DCOOL') THEN
               VALR(I,1)=WPAR(IRES,2)
               VALR(I,2)=WPAR(IRES,2)
	    ELSE IF(HPARM(JPARM).EQ.'B-TFUEL') THEN
               VALR(I,1)=WPAR(IRES,3)
               VALR(I,2)=WPAR(IRES,3)
	    ELSE IF(HPARM(JPARM).EQ.'POWER-BUN') THEN
               VALR(I,1)=WPAR(IRES,4)
               VALR(I,2)=WPAR(IRES,4)
	    ELSE
	       CALL XABORT('NCRRGR: '//HPARM(JPARM)//' NOT KNOWN.')
	    ENDIF
            DO 50 I=1,NPAR
            IF(HPARM(JPARM).EQ.PARKEY(I)) THEN
               IPAR=I
               GO TO 60
            ENDIF
   50       CONTINUE
            GO TO 80
   60       LPCPO=LCMGID(IPCPO,'GLOBAL')
            CALL LCMGET(LPCPO,'NVALUE',NVALUE)
            IF(NVALUE(IPAR).GT.MAXVAL) CALL XABORT('NCRRGR: MAXVAL OVE'
     1      //'RFLOW.')
            WRITE(RECNAM,'(''pval'',I8)') IPAR
            CALL LCMLEN(LPCPO,RECNAM,LENGTH,ITYLCM)
            IF(LENGTH.EQ.0) THEN
              WRITE(HSMG,'(25HNCRRGR: GLOBAL PARAMETER ,A,9H NOT SET.)')
     1        PARKEY(IPAR)
              CALL XABORT(HSMG)
            ENDIF
            CALL LCMGET(LPCPO,RECNAM,VREAL)
            IF(IMPX.GT.0) THEN
               WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1         1P,6E12.4/(35X,6E12.4))') PARKEY(IPAR),(VREAL(I),I=1,
     2         NVALUE(IPAR))
            ENDIF
            IF(VALR(IPAR,1).EQ.VALR(IPAR,2)) THEN
               DO 70 J=1,NVALUE(IPAR)
               IF(ABS(VALR(IPAR,1)-VREAL(J)).LE.REPS*ABS(VREAL(J))) THEN
                  MUPLET(IPAR)=J
                  IF(ITYPE.NE.1) MUPLET(IPAR)=-1
                  MUTYPE(IPAR)=ITYPE
                  GO TO 120
               ENDIF
   70          CONTINUE
            ENDIF
            IF(VALR(IPAR,1).LT.VREAL(1)) THEN
               WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,10H WITH VALU,
     1         1HE,1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEY(IPAR),
     2         VALR(IPAR,1)
               CALL XABORT(HSMG)
            ELSE IF(VALR(IPAR,2).GT.VREAL(NVALUE(IPAR))) THEN
               WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,10H WITH VALU,
     1         1HE,1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEY(IPAR),
     2         VALR(IPAR,2)
               CALL XABORT(HSMG)
            ELSE IF(VALR(IPAR,1).GT.VALR(IPAR,2)) THEN
               WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,9H IS DEFIN,
     1         7HED WITH,1P,E12.4,2H >,E12.4,1H.)') PARKEY(IPAR),
     2         VALR(IPAR,1),VALR(IPAR,2)
               CALL XABORT(HSMG)
            ENDIF
            MUPLET(IPAR)=-1
            MUTYPE(IPAR)=ITYPE
	    GO TO 120
   80       DO 90 I=1,NLOC
            IF(HPARM(JPARM).EQ.PARKEL(I)) THEN
               IPAR=NPAR+I
               GO TO 100
            ENDIF
   90       CONTINUE
            CALL XABORT('NCRRGR: PARAMETER '//TEXT12//' NOT FOUND.')
  100       JPCPO=LCMGID(IPCPO,'MIXTURES')
            KPCPO=LCMGIL(JPCPO,IBMOLD)
            LPCPO=LCMGID(KPCPO,'TREE')
            CALL LCMGET(LPCPO,'NVALUE',NVALUE)
            WRITE(RECNAM,'(''pval'',I8)') IPAR-NPAR
            CALL LCMLEN(LPCPO,RECNAM,LENGTH,ITYLCM)
            IF(LENGTH.EQ.0) THEN
               WRITE(HSMG,'(24HNCRRGR: LOCAL PARAMETER ,A,9H NOT SET.)')
     1         PARKEL(IPAR-NPAR)
               CALL XABORT(HSMG)
            ENDIF
            CALL LCMGET(LPCPO,RECNAM,VREAL)
            IF(IMPX.GT.0) THEN
               WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1         1P,6E12.4/(35X,6E12.4))') PARKEL(IPAR-NPAR),(VREAL(I),I=1
     2         ,NVALUE(IPAR-NPAR))
            ENDIF
            IF(VALR(IPAR,1).EQ.VALR(IPAR,2)) THEN
               DO 110 J=1,NVALUE(IPAR-NPAR)
               IF(ABS(VALR(IPAR,1)-VREAL(J)).LE.REPS*ABS(VREAL(J))) THEN
                  MUPLET(IPAR)=J
                  IF(ITYPE.NE.1) MUPLET(IPAR)=-1
                  MUTYPE(IPAR)=ITYPE
                  GO TO 120
               ENDIF
  110          CONTINUE
            ENDIF
            IF(VALR(IPAR,1).LT.VREAL(1)) THEN
               WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,11H WITH VALUE,
     1         1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEL(IPAR-NPAR),
     2         VALR(IPAR,1)
               CALL XABORT(HSMG)
            ELSE IF(VALR(IPAR,2).GT.VREAL(NVALUE(IPAR-NPAR))) THEN
               WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,11H WITH VALUE,
     1         1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEL(IPAR-NPAR),
     2         VALR(IPAR,2)
               CALL XABORT(HSMG)
            ELSE IF(VALR(IPAR,1).GT.VALR(IPAR,2)) THEN
               WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,9H IS DEFIN,
     1         7HED WITH,1P,E12.4,2H >,E12.4,1H.)') PARKEL(IPAR-NPAR),
     2         VALR(IPAR,1),VALR(IPAR,2)
               CALL XABORT(HSMG)
            ENDIF
            MUPLET(IPAR)=-1
            MUTYPE(IPAR)=ITYPE
  120       CONTINUE
         ENDIF
  130    CONTINUE
         GO TO 30
      ELSE IF(TEXT12.EQ.'MICRO') THEN
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
         IF(TEXT12.EQ.'ALL') THEN
            LISO=.TRUE.
         ELSE IF(TEXT12.EQ.'ONLY') THEN
            LISO=.FALSE.
         ENDIF
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
  140    IF(TEXT12.EQ.'ENDMIX') THEN
            GO TO 30
         ELSE
            NISO=NISO+1
            IF(NISO.GT.MAXISD) CALL XABORT('NCRRGR: MAXISD OVERFLOW.')
            HISO(NISO)=TEXT12
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.EQ.2) THEN
               CONC(NISO)=FLOTT
            ELSE IF((INDIC.EQ.3).AND.(TEXT12.EQ.'*')) THEN
               CONC(NISO)=-99.99
            ELSE
               CALL XABORT('NCRRGR: INVALID HISO DATA.')
            ENDIF
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.NE.3) CALL XABORT('NCRRGR: CHARACTER DATA EXPECTE'
     1      //'D.')
            GO TO 140
         ENDIF
      ELSE IF((TEXT12.EQ.'SET').OR.(TEXT12.EQ.'DELTA')) THEN
         IF(TEXT12.EQ.'SET') THEN
            ITYPE=1
         ELSE IF(TEXT12.EQ.'DELTA') THEN
            ITYPE=2
         ENDIF
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.NE.3) CALL XABORT('NCRRGR: CHARACTER DATA EXPECTED.')
         DO 150 I=1,NPAR
         IF(TEXT12.EQ.PARKEY(I)) THEN
            IPAR=I
            GO TO 160
         ENDIF
  150    CONTINUE
         GO TO 200
  160    LPCPO=LCMGID(IPCPO,'GLOBAL')
         CALL LCMGET(LPCPO,'NVALUE',NVALUE)
         IF(NVALUE(IPAR).GT.MAXVAL) CALL XABORT('NCRRGR: MAXVAL OVERFL'
     1   //'OW.')
         WRITE(RECNAM,'(''pval'',I8)') IPAR
         CALL LCMLEN(LPCPO,RECNAM,LENGTH,ITYLCM)
         IF(LENGTH.EQ.0) THEN
            WRITE(HSMG,'(25HNCRRGR: GLOBAL PARAMETER ,A,9H NOT SET.)')
     1      PARKEY(IPAR)
            CALL XABORT(HSMG)
         ENDIF
         IF(PARFMT(IPAR).EQ.'INTEGER') THEN
            IF(ITYPE.NE.1) CALL XABORT('NCRRGR: SET MANDATORY WITH INT'
     1      //'EGER PARAMETERS.')
            CALL REDGET(INDIC,VALI(IPAR),FLOTT,TEXT12,DFLOTT)
            IF(INDIC.NE.1) CALL XABORT('NCRRGR: INTEGER DATA EXPECTED.')
            CALL LCMGET(LPCPO,RECNAM,VINTE)
            IF(IMPX.GT.0) THEN
               WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1         1P,6I12/(35X,6I12))') PARKEY(IPAR),(VINTE(I),I=1,
     2         NVALUE(IPAR))
            ENDIF
            DO 170 J=1,NVALUE(IPAR)
            IF(VALI(IPAR).EQ.VINTE(J)) THEN
               MUPLET(IPAR)=J
               MUTYPE(IPAR)=ITYPE
               GO TO 20
            ENDIF
  170       CONTINUE
            WRITE(HSMG,'(26HNCRRGR: INTEGER PARAMETER ,A,9H WITH VAL,
     1      2HUE,I5,29H NOT FOUND IN COMPO DATABASE.)') PARKEY(IPAR),
     2      VALI(IPAR)
            CALL XABORT(HSMG)
         ELSE IF(PARFMT(IPAR).EQ.'REAL') THEN
            CALL REDGET(INDIC,NITMA,VALR(IPAR,1),TEXT12,DFLOTT)
            IF(INDIC.NE.2) CALL XABORT('NCRRGR: REAL DATA EXPECTED.')
            VALR(IPAR,2)=VALR(IPAR,1)
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            IF(INDIC.EQ.2) THEN
               VALR(IPAR,2)=FLOTT
               CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
            ENDIF
            CALL LCMGET(LPCPO,RECNAM,VREAL)
            IF(IMPX.GT.0) THEN
               WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1         1P,6E12.4/(35X,6E12.4))') PARKEY(IPAR),(VREAL(I),I=1,
     2         NVALUE(IPAR))
            ENDIF
            IF(VALR(IPAR,1).EQ.VALR(IPAR,2)) THEN
               DO 180 J=1,NVALUE(IPAR)
               IF(ABS(VALR(IPAR,1)-VREAL(J)).LE.REPS*ABS(VREAL(J))) THEN
                  MUPLET(IPAR)=J
                  IF(ITYPE.NE.1) MUPLET(IPAR)=-1
                  MUTYPE(IPAR)=ITYPE
                  GO TO 30
               ENDIF
  180          CONTINUE
            ENDIF
            IF(VALR(IPAR,1).LT.VREAL(1)) THEN
               WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,10H WITH VALU,
     1         1HE,1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEY(IPAR),
     2         VALR(IPAR,1)
               CALL XABORT(HSMG)
            ELSE IF(VALR(IPAR,2).GT.VREAL(NVALUE(IPAR))) THEN
               WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,10H WITH VALU,
     1         1HE,1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEY(IPAR),
     2         VALR(IPAR,2)
               CALL XABORT(HSMG)
            ELSE IF(VALR(IPAR,1).GT.VALR(IPAR,2)) THEN
               WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,9H IS DEFIN,
     1         7HED WITH,1P,E12.4,2H >,E12.4,1H.)') PARKEY(IPAR),
     2         VALR(IPAR,1),VALR(IPAR,2)
               CALL XABORT(HSMG)
            ENDIF
            MUPLET(IPAR)=-1
            MUTYPE(IPAR)=ITYPE
            GO TO 30
         ELSE IF(PARFMT(IPAR).EQ.'STRING') THEN
            IF(ITYPE.NE.1) CALL XABORT('NCRRGR: SET MANDATORY WITH STR'
     1      //'ING PARAMETERS.')
            CALL REDGET(INDIC,NITMA,FLOTT,VALH(IPAR),DFLOTT)
            IF(INDIC.NE.3) CALL XABORT('NCRRGR: STRING DATA EXPECTED.')
            CALL COMGTC(LPCPO,RECNAM,12,NVALUE(IPAR),VCHAR)
            IF(IMPX.GT.0) THEN
               WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1         1P,6A12/(35X,6A12))') PARKEY(IPAR),(VCHAR(I),I=1,
     2         NVALUE(IPAR))
            ENDIF
            DO 190 J=1,NVALUE(IPAR)
            IF(VALH(IPAR).EQ.VCHAR(J)) THEN
               MUPLET(IPAR)=J
               MUTYPE(IPAR)=ITYPE
               GO TO 20
            ENDIF
  190       CONTINUE
            WRITE(HSMG,'(25HNCRRGR: STRING PARAMETER ,A,10H WITH VALU,
     1      1HE,A12,29H NOT FOUND IN COMPO DATABASE.)') PARKEY(IPAR),
     2      VALH(IPAR)
            CALL XABORT(HSMG)
         ENDIF
  200    DO 210 I=1,NLOC
         IF(TEXT12.EQ.PARKEL(I)) THEN
            IPAR=NPAR+I
            GO TO 220
         ENDIF
  210    CONTINUE
         CALL XABORT('NCRRGR: PARAMETER '//TEXT12//' NOT FOUND.')
  220    JPCPO=LCMGID(IPCPO,'MIXTURES')
         KPCPO=LCMGIL(JPCPO,IBMOLD)
         LPCPO=LCMGID(KPCPO,'TREE')
         CALL LCMGET(LPCPO,'NVALUE',NVALUE)
         CALL REDGET(INDIC,NITMA,VALR(IPAR,1),TEXT12,DFLOTT)
         IF(INDIC.NE.2) CALL XABORT('NCRRGR: REAL DATA EXPECTED.')
         VALR(IPAR,2)=VALR(IPAR,1)
         CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         IF(INDIC.EQ.2) THEN
            VALR(IPAR,2)=FLOTT
            CALL REDGET(INDIC,NITMA,FLOTT,TEXT12,DFLOTT)
         ENDIF
         WRITE(RECNAM,'(''pval'',I8)') IPAR-NPAR
         CALL LCMLEN(LPCPO,RECNAM,LENGTH,ITYLCM)
         IF(LENGTH.EQ.0) THEN
            WRITE(HSMG,'(24HNCRRGR: LOCAL PARAMETER ,A,9H NOT SET.)')
     1      PARKEL(IPAR-NPAR)
            CALL XABORT(HSMG)
         ENDIF
         CALL LCMGET(LPCPO,RECNAM,VREAL)
         IF(IMPX.GT.0) THEN
            WRITE(IOUT,'(13H NCRRGR: KEY=,A,18H TABULATED POINTS=,
     1      1P,6E12.4/(35X,6E12.4))') PARKEL(IPAR-NPAR),(VREAL(I),I=1,
     2      NVALUE(IPAR-NPAR))
         ENDIF
         IF(VALR(IPAR,1).EQ.VALR(IPAR,2)) THEN
            DO 230 J=1,NVALUE(IPAR-NPAR)
            IF(ABS(VALR(IPAR,1)-VREAL(J)).LE.REPS*ABS(VREAL(J))) THEN
               MUPLET(IPAR)=J
               IF(ITYPE.NE.1) MUPLET(IPAR)=-1
               MUTYPE(IPAR)=ITYPE
               GO TO 30
            ENDIF
  230       CONTINUE
         ENDIF
         IF(VALR(IPAR,1).LT.VREAL(1)) THEN
            WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,11H WITH VALUE,
     1      1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEL(IPAR-NPAR),
     2      VALR(IPAR,1)
            CALL XABORT(HSMG)
         ELSE IF(VALR(IPAR,2).GT.VREAL(NVALUE(IPAR-NPAR))) THEN
            WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,11H WITH VALUE,
     1      1P,E12.4,23H IS OUTSIDE THE DOMAIN.)') PARKEL(IPAR-NPAR),
     2      VALR(IPAR,2)
            CALL XABORT(HSMG)
         ELSE IF(VALR(IPAR,1).GT.VALR(IPAR,2)) THEN
            WRITE(HSMG,'(23HNCRRGR: REAL PARAMETER ,A,9H IS DEFIN,
     1      7HED WITH,1P,E12.4,2H >,E12.4,1H.)') PARKEL(IPAR-NPAR),
     2      VALR(IPAR,1),VALR(IPAR,2)
            CALL XABORT(HSMG)
         ENDIF
         MUPLET(IPAR)=-1
         MUTYPE(IPAR)=ITYPE
         GO TO 30
      ELSE IF(TEXT12.EQ.'ENDMIX') THEN
*----
*  COMPUTE THE TERP FACTORS USING TABLE-OF-CONTENT INFORMATION.
*----
         IF(IBMOLD.GT.NMIL) CALL XABORT('NCRRGR: MIX OVERFLOW (COMPO).')
         IF(IBM.GT.NMIX) CALL XABORT('NCRRGR: MIX OVERFLOW (MICROLIB).')
         CALL NCRTRP(IPCPO,IMPX,IBMOLD,NPAR,NLOC,NCAL,MUPLET,MUTYPE,
     1   VALR,TERP(1,IBM))
      ELSE IF((TEXT12.EQ.'COMPO').OR.(TEXT12.EQ.';')) THEN
*----
*  BUILD THE MICROLIB.
*----
         IF(TEXT12.EQ.';') ITER=0
         IF(TEXT12.EQ.'TABLE') ITER=1
         IF(IACCS.EQ.0) THEN
            MAXISO=MAXISD*NMIX
         ELSE
            CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
            MAXISO=MAX(MAXISD*NMIX,ISTATE(2))
         ENDIF
         CALL SETARA(BASE,3*MAXISO,IHUSE1)
         CALL SETARA(BASE,3*MAXISO,IHNAM1)
         CALL SETARA(BASE,MAXISO,IDENS1)
         CALL SETARA(BASE,MAXISO,IITYP1)
         CALL SETARA(BASE,MAXISO,IITOD1)
         CALL SETARA(BASE,MAXISO,ITEMP1)
         CALL SETARA(BASE,MAXISO,IVOL1)
         CALL SETARA(BASE,3*MAXISO,IHUSE2)
         CALL SETARA(BASE,3*MAXISO,IHNAM2)
         CALL SETARA(BASE,MAXISO,IDENS2)
         CALL SETARA(BASE,MAXISO,IIMIX2)
         CALL SETARA(BASE,MAXISO,IITYP2)
         CALL SETARA(BASE,MAXISO,IITOD2)
         CALL SETARA(BASE,MAXISO,ITEMP2)
         CALL SETARA(BASE,MAXISO,IVOL2)
         CALL SETARA(BASE,NGRP,IWORK)
         CALL SETARA(BASE,MAXISO,IPIFI)
         CALL SETARA(BASE,MAXISO,ISW)
         CALL SETARA(BASE,NGRP+1,IENER)
         CALL SETARA(BASE,NGRP,IDELT)
         CALL SETARA(BASE,NCAL*MAXISO,IFACT)
         CALL SETARA(BASE,NMIX,IMILVO)
         CALL NCRLIB(MAXISO,IPLIB,IPCPO,IACCS,NMIX,NGRP,IMPX,NCAL,
     1   TERP,NISO,LISO,HISO,CONC,MIXC,BASE(IHUSE1),BASE(IHNAM1),
     2   BASE(IDENS1),BASE(IITYP1),BASE(IITOD1),BASE(ITEMP1),
     3   BASE(IVOL1),BASE(IHUSE2),BASE(IHNAM2),BASE(IDENS2),
     4   BASE(IIMIX2),BASE(IITYP2),BASE(IITOD2),BASE(ITEMP2),
     5   BASE(IVOL2),BASE(IWORK),BASE(IPIFI),BASE(ISW),BASE(IENER),
     6   BASE(IDELT),BASE(IFACT),BASE(IMILVO))
         CALL RLSARA(BASE(IMILVO))
         CALL RLSARA(BASE(IFACT))
         CALL RLSARA(BASE(IDELT))
         CALL RLSARA(BASE(IENER))
         CALL RLSARA(BASE(ISW))
         CALL RLSARA(BASE(IPIFI))
         CALL RLSARA(BASE(IWORK))
         CALL RLSARA(BASE(IVOL2))
         CALL RLSARA(BASE(ITEMP2))
         CALL RLSARA(BASE(IITOD2))
         CALL RLSARA(BASE(IITYP2))
         CALL RLSARA(BASE(IIMIX2))
         CALL RLSARA(BASE(IDENS2))
         CALL RLSARA(BASE(IHNAM2))
         CALL RLSARA(BASE(IHUSE2))
         CALL RLSARA(BASE(IVOL1))
         CALL RLSARA(BASE(ITEMP1))
         CALL RLSARA(BASE(IITOD1))
         CALL RLSARA(BASE(IITYP1))
         CALL RLSARA(BASE(IDENS1))
         CALL RLSARA(BASE(IHNAM1))
         CALL RLSARA(BASE(IHUSE1))
         RETURN
      ELSE
         CALL XABORT('NCRRGR: '//TEXT12//' IS AN INVALID KEYWORD.')
      ENDIF
      GO TO 20
      END
