*DECK NCRTRP
      SUBROUTINE NCRTRP(IPCPO,IMPX,IBMOLD,NPAR,NLOC,NCAL,MUPLET,MUTYPE,
     1 VALR,TERP)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the TERP interpolation/derivation/integration factors using
* table-of-content information of the Compo for mixture IBMOLD.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
*
*Author(s): A. Hebert
*
*Parameters: input
* IPCPO   address of the multidimensional compo object.
* IMPX    print parameter (equal to zero for no print).
* IBMOLD  material mixture index in the COMPO.
* NPAR    number of global parameters.
* NLOC    numberof local parameters.
* NCAL    number of elementary calculations in the compo.
* MUPLET  tuple used to identify an elementary calculation.
* MUTYPE  type of interpolation (=1: interpolation; =2: delta-sigma).
* VALR    real values of the interpolated point.
*
*Parameters: output
* TERP    interpolation factors.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      PARAMETER (MAXPAR=50)
      INTEGER IPCPO,IMPX,IBMOLD,NPAR,NLOC,NCAL,MUPLET(NPAR+NLOC),
     1 MUTYPE(NPAR+NLOC)
      REAL VALR(2*MAXPAR,2),TERP(NCAL)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MAXVAL=100,IOUT=6,MAXDIM=4)
      INTEGER IPAR(MAXDIM),NVALUE(MAXPAR),NVAL(MAXDIM)
      REAL VREAL(MAXVAL),T1D(MAXVAL,MAXDIM),WORK(MAXVAL)
      CHARACTER HSMG*131,TEXT2*2,RECNAM*12
      LOGICAL LSINGL
      COMMON BASE(1)
*
      CALL XDRSET(TERP,NCAL,0.0)
      CALL XDISET(IPAR,MAXDIM,0)
      NDIM=0
      NDELTA=0
      DO 10 I=1,NPAR+NLOC
      IF(MUPLET(I).EQ.-1) THEN
         NDIM=NDIM+1
	 IF(MUTYPE(I).NE.1) NDELTA=NDELTA+1
	 IF(NDIM.GT.MAXDIM) THEN
            WRITE(HSMG,'(7HNCRTRP:,I4,29H-DIMENSIONAL INTERPOLATION NO,
     1      14HT IMPLEMENTED.)') NDIM
            CALL XABORT(HSMG)
	 ENDIF
	 IPAR(NDIM)=I
      ENDIF
   10 CONTINUE
      IF(NDELTA.GT.1) THEN
         WRITE(HSMG,'(45HNCRTRP: MORE THAN ONE DSET OR DELTA PER MIX I,
     1   24HS NOT SUPPORTED (MIXTURE,I4,2H).)') IBMOLD
         CALL XABORT(HSMG)
      ENDIF
      IF(IMPX.GT.0) THEN
         WRITE(IOUT,'(8H NCRTRP:,I4,31H-DIMENSIONAL INTERPOLATION IN C,
     1   12HOMPO MIXTURE,I5,1H.)') NDIM,IBMOLD
      ENDIF
      IF(NDIM.EQ.0) THEN
         JPCPO=LCMGID(IPCPO,'MIXTURES')
         KPCPO=LCMGIL(JPCPO,IBMOLD)
         LPCPO=LCMGID(KPCPO,'TREE')
         ICAL=NCRCAL(LPCPO,NPAR+NLOC,MUPLET)
	 IF(ICAL.EQ.0) GO TO 200
	 IF(ICAL.EQ.-1) GO TO 210
	 TERP(ICAL)=1.0
      ELSE
         DO 40 ID=1,NDIM
         IF(IPAR(ID).LE.NPAR) THEN
            LPCPO=LCMGID(IPCPO,'GLOBAL')
            WRITE(RECNAM,'(''pval'',I8)') IPAR(ID)
            CALL LCMGET(LPCPO,'NVALUE',NVALUE)
	    NVAL(ID)=NVALUE(IPAR(ID))
	 ELSE
            JPCPO=LCMGID(IPCPO,'MIXTURES')
            KPCPO=LCMGIL(JPCPO,IBMOLD)
            LPCPO=LCMGID(KPCPO,'TREE')
            WRITE(RECNAM,'(''pval'',I8)') IPAR(ID)-NPAR
            CALL LCMGET(LPCPO,'NVALUE',NVALUE)
	    NVAL(ID)=NVALUE(IPAR(ID)-NPAR)
	 ENDIF
         CALL LCMGET(LPCPO,RECNAM,VREAL)
	 LSINGL=(VALR(IPAR(ID),1).EQ.VALR(IPAR(ID),2))
	 CALL SETARA(BASE,3*NVAL(ID),IWK)
	 IF((MUTYPE(IPAR(ID)).EQ.1).AND.LSINGL) THEN
            CALL ALTERP(NVAL(ID),VREAL,VALR(IPAR(ID),1),.FALSE.,
     1      T1D(1,ID),BASE(IWK))
	 ELSE IF(MUTYPE(IPAR(ID)).EQ.1) THEN
            CALL ALTERI(NVAL(ID),VREAL,VALR(IPAR(ID),1),
     1      VALR(IPAR(ID),2),T1D(1,ID),BASE(IWK))
            DO 20 I=1,NVAL(ID)
   20       T1D(I,ID)=T1D(I,ID)/(VALR(IPAR(ID),2)-VALR(IPAR(ID),1))
	 ELSE IF((MUTYPE(IPAR(ID)).EQ.2).AND.(.NOT.LSINGL)) THEN
            CALL ALTERP(NVAL(ID),VREAL,VALR(IPAR(ID),1),.FALSE.,
     1      WORK,BASE(IWK))
            CALL ALTERP(NVAL(ID),VREAL,VALR(IPAR(ID),2),.FALSE.,
     1      T1D(1,ID),BASE(IWK))
	    DO 30 I=1,NVAL(ID)
   30       T1D(I,ID)=T1D(I,ID)-WORK(I)
         ELSE
	    CALL XABORT('NCRTRP: INVALID OPTION.')
	 ENDIF
	 CALL RLSARA(BASE(IWK))
   40    CONTINUE
         JPCPO=LCMGID(IPCPO,'MIXTURES')
         KPCPO=LCMGIL(JPCPO,IBMOLD)
         LPCPO=LCMGID(KPCPO,'TREE')
         IF(NDIM.EQ.1) THEN
	    DO 50 I=1,NVAL(1)
	    MUPLET(IPAR(1))=I
            ICAL=NCRCAL(LPCPO,NPAR+NLOC,MUPLET)
	    IF(ICAL.EQ.0) GO TO 200
	    IF(ICAL.EQ.-1) GO TO 210
   50       TERP(ICAL)=TERP(ICAL)+T1D(I,1)
         ELSE IF(NDIM.EQ.2) THEN
	    DO 60 I1=1,NVAL(1)
	    MUPLET(IPAR(1))=I1
	    DO 60 I2=1,NVAL(2)
	    MUPLET(IPAR(2))=I2
            ICAL=NCRCAL(LPCPO,NPAR+NLOC,MUPLET)
	    IF(ICAL.EQ.0) GO TO 200
	    IF(ICAL.EQ.-1) GO TO 210
   60       TERP(ICAL)=TERP(ICAL)+T1D(I,1)*T1D(I,2)
         ELSE IF(NDIM.EQ.3) THEN
	    DO 70 I1=1,NVAL(1)
	    MUPLET(IPAR(1))=I1
	    DO 70 I2=1,NVAL(2)
	    MUPLET(IPAR(2))=I2
	    DO 70 I3=1,NVAL(3)
	    MUPLET(IPAR(3))=I3
            ICAL=NCRCAL(LPCPO,NPAR+NLOC,MUPLET)
	    IF(ICAL.EQ.0) GO TO 200
	    IF(ICAL.EQ.-1) GO TO 210
   70       TERP(ICAL)=TERP(ICAL)+T1D(I,1)*T1D(I,2)*T1D(I,3)
         ELSE IF(NDIM.EQ.4) THEN
	    DO 80 I1=1,NVAL(1)
	    MUPLET(IPAR(1))=I1
	    DO 80 I2=1,NVAL(2)
	    MUPLET(IPAR(2))=I2
	    DO 80 I3=1,NVAL(3)
	    MUPLET(IPAR(3))=I3
	    DO 80 I4=1,NVAL(4)
	    MUPLET(IPAR(4))=I4
            ICAL=NCRCAL(LPCPO,NPAR+NLOC,MUPLET)
	    IF(ICAL.EQ.0) GO TO 200
	    IF(ICAL.EQ.-1) GO TO 210
   80       TERP(ICAL)=TERP(ICAL)+T1D(I,1)*T1D(I,2)*T1D(I,3)*T1D(I,4)
         ENDIF
      ENDIF
      IF(IMPX.GT.0) THEN
         WRITE(IOUT,'(35H NCRTRP: TERP PARAMETERS IN MIXTURE,I4,1H:/(1X,
     1   1P,10E12.4))') IBMOLD,(TERP(I),I=1,NCAL)
         SUM=0.0
	 DO 90 I=1,NCAL
   90    SUM=SUM+TERP(I)
         WRITE(IOUT,'(19H NCRTRP: SUM(TERP)=,1P,E12.4))') SUM
      ENDIF
      RETURN
*----
*  MISSING ELEMENTARY CALCULATION EXCEPTION.
*----
  200 WRITE(TEXT2,'(I2)') NPAR+NLOC
      WRITE(HSMG,'(42HNCRTRP: ELEMENTARY CALCULATION AT MUPLET (,'//
     1 TEXT2//'I3,14H ) IS MISSING.)') (MUPLET(I),I=1,NPAR+NLOC)
      CALL XABORT(HSMG)
  210 WRITE(TEXT2,'(I2)') NPAR+NLOC
      WRITE(HSMG,'(42HNCRTRP: ELEMENTARY CALCULATION AT MUPLET (,'//
     1 TEXT2//'I3,17H ) IS DEGENERATE.)') (MUPLET(I),I=1,NPAR+NLOC)
      CALL XABORT(HSMG)
      END
