*DECK NXTMCD
      SUBROUTINE NXTMCD(IPGEO ,IPTRK ,IPRINT,NDIM  ,ITYPBC,
     >                  IDIAG ,ISAXIS,ILCELL,NBOCEL,NBUCEL,NOCELL,
     >                  NUCELL,MAXMSH,MAXREG,MAXPIN,NBTCLS,
     >                  ITSYM ,IDFEX ,IUNFLD,DCMESH,
     >                  NAGGEO,ITURN ,MERGE ,ITPIN ,
     >                  NFSUR ,NFREG ,MXGSUR,MXGREG,ISPLT ,MIX   ,
     >                  DAMESH,DRAPIN,RMESH ,RPIN  ,APIN  )
*
*----------
*
*Purpose:
* To create a multicell description for the geometry and verify
* if intrinsic cell symmetry required by boundary conditions 
* are satisfied.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau.
*
*Update(s): 
* 2005/09/29: Validated with DRAGON and documented in
*             Report IGE-260.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*
*Parameters: input
* IPGEO   pointer to the GEOMETRY data structure.
* IPTRK   pointer to the TRACKING data structure.
* IPRINT  print level.
* NDIM    problem dimensions.
* ITYPBC  type of boundary conditions where
*         \begin{itemize} 
*         \item \moc{ITYPBC}=0 for Cartesian
*         boundary conditions; 
*         \item \moc{ITYPBC}=1 for mixted annular/Cartesian (3D)
*         boundary conditions. 
*         \end{itemize}
* IDIAG   the diagonal symmetry flag where
*         \begin{itemize} 
*         \item \moc{IDIAG}=-1 indicates X- Y+ \moc{DIAG} symmetry; 
*         \item \moc{IDIAG}= 1 indicates X+ Y- \moc{DIAG} symmetry; 
*         \item \moc{IDIAG}=0  indicates no \moc{DIAG} symmetry.
*         \end{itemize}
* ISAXIS  symmetry vector for each direction.
* ILCELL  cell level. 
* NBOCEL  number of cells in original geometry.
* NBUCEL  number of cells in unfolded geometry.
* NOCELL  number of cell before unfolding in 
*         $X$, $Y$ and $Z$ directions.
* NUCELL  number of cell after unfolding in 
*         $X$, $Y$ and $Z$ directions.
* MAXMSH  maximum number of elements in MESH array.
* MAXREG  maximum number of elements in MIX array.
* MAXPIN  maximum number of pins in clusters.
* NBTCLS  total number of cluster geometry.
* ITSYM   array to identify the symmetry to test for each original 
*         cell where
*         \begin{itemize} 
*         \item \moc{ITSYM(1,*)} identify $X$ symmetry;
*         \item \moc{ITSYM(2,*)} identify $Y$ symmetry;
*         \item \moc{ITSYM(3,*)} identify $Z$ symmetry;
*         \item \moc{ITSYM(4,*)} identify $X-Y$ symmetry.
*         \end{itemize}
*         A value of 0 indicate that the geometry does not need
*         to be verified while a value of 1 implies a verification
*         of the geometry.
* IDFEX   identify faces associated with external boundary.
* IUNFLD  array to identify the generating cell (\moc{IUNFLD(1,*)})
*         and the rotation associated with this region in space.
* DCMESH  global mesh for each cell.
* NAGGEO  names of generating geometries.
* ITURN   geometry turns.
* MERGE   geometry merge.
*
*Parameters: output
* ITPIN   pin information array.
* NFSUR   final number of surfaces.
* NFREG   final number of regions.
* MXGSUR  maximum number of surfaces for any geometry.
* MXGREG  maximum number of region for any geometry.
*
*Parameters: temporary storage
* ISPLT   temporary array for reading cell mesh split.
* MIX     temporary array for reading cell mixture.
* DAMESH  temporary vector for storing global mesh array.
* DRAPIN  temporary vector for storing global pin positions.
* RMESH   temporary vector for reading cell mesh array.
* RPIN    temporary vector for reading pin positions.
* APIN    temporary vector for reading pin angles.
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPGEO,IPTRK
      INTEGER          IPRINT,NDIM,ITYPBC,IDIAG,ISAXIS(3),
     >                 ILCELL,NBOCEL,NBUCEL,NOCELL(3),NUCELL(3),
     >                 MAXMSH,MAXREG,MAXPIN,NBTCLS,
     >                 ITSYM(4,NBOCEL),IDFEX(7,NBOCEL),
     >                 IUNFLD(2,NBUCEL)
      DOUBLE PRECISION DCMESH(3,2,NBOCEL)
      INTEGER          NAGGEO(3,NBOCEL),ITURN(NBOCEL),MERGE(NBOCEL),
     >                 ITPIN(3,MAXPIN),
     >                 NFSUR,NFREG,MXGSUR,MXGREG
      INTEGER          ISPLT(MAXMSH,4),MIX(MAXREG)
      DOUBLE PRECISION DAMESH(-1:MAXMSH,4),DRAPIN(-1:4,MAXPIN)     
      REAL             RMESH(0:MAXMSH),RPIN(MAXPIN),APIN(MAXPIN)
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTMCD')
      INTEGER          NSTATE
      PARAMETER       (NSTATE=40)
      DOUBLE PRECISION DCUTOF,DZERO,DONE,DTWO
      PARAMETER       (DCUTOF=1.0D-10,DZERO=0.0D0,DONE=1.0D0,DTWO=2.0D0)
*----
*  Functions
*----
      DOUBLE PRECISION XDRCST,PI
*----
*  Memory allocation variables
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          MIXC,NAGCLS,ISPLTR,DAMESR,IDREG,IDSUR,IREN,
     >                 INDXSR,SURVOL,SVT,INREN,MIXS,MIXCS
*----
*  Local variables
*----
      INTEGER          IOFNAC,ICEL,ITC,ITYPG,ITRN,ITST,MAXMSS,
     >                 NBGCLS,ICLS,ICN,ITYPC,IGCLS,ILEV,
     >                 NMIX,NMIXS,NMIXC,NMIXCS,
     >                 NREG,NSUR,NREGS,NSURS,NREGC,NSURC,NREGCS,NSURCS,
     >                 NREGN,NSURN,NREGCN,NSURCN
      INTEGER          ISTATG(NSTATE),ISTATC(NSTATE),
     >                 IEDIMX(NSTATE),IEDIMP(NSTATE)
      INTEGER          NM(4),NMC(4),NMS(4),NMCS(4),
     >                 ITSYMC(4)
      CHARACTER        NAMGG*12,NAMCL*12,NAMREC*12 
      INTEGER          ILCMLN,ILCMTY
      INTEGER          NAREG,NASUR,NEREN
      INTEGER          NPIN,IP,IPINT,ICPIN,NPIR,IDGPP,NBPIN
      DOUBLE PRECISION DELTA,TWOPI,DROUT,DHPIN
*----
*  Data
*----
      CHARACTER        CDIR(1:4)*1,CLEV(2)*1
      SAVE             CDIR,CLEV
      DATA             CDIR /'X','Y','Z','R'/
      DATA             CLEV /'C','P'/
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      PI=XDRCST('Pi',' ')
      TWOPI=DTWO*PI
      CALL SETARA(IBASE,2*MAXREG,MIXC)
      CALL SETARA(IBASE,3*NBTCLS,NAGCLS)
      CALL SETARA(IBASE,(MAXMSH)*4*2,ISPLTR)
      MXGSUR=0
      MXGREG=0
      IOFNAC=NAGCLS
      IGCLS=0
      ICPIN=0
      NAREG=0
      NASUR=0
      DO ICEL=1,NBOCEL
        IF(ILCELL .EQ. 1) THEN
          WRITE(NAMGG,'(3A4)') (NAGGEO(ITC,ICEL),ITC=1,3)
          CALL LCMSIX(IPGEO,NAMGG,1)
        ELSE
          NAMGG='/           '
        ENDIF
        IF(IPRINT .GE. 10) THEN
          WRITE(IOUT,6010) ICEL,NAMGG
        ENDIF
        CALL XDISET(ISTATG,NSTATE,0)
        CALL LCMGET(IPGEO,'STATE-VECTOR',ISTATG)
        ITYPG=ISTATG(1)
        IF(ITYPG .EQ. 20) THEN
          IF(ISTATG(2) .EQ. 0) ITYPG=5
        ELSE IF(ITYPG .EQ. 21 .OR. ITYPG .EQ. 22 .OR.
     >          ITYPG .EQ. 23) THEN
          IF(ISTATG(2) .EQ. 0) ITYPG=7
        ENDIF
        NM(4)=ISTATG(2)
        NM(1)=ISTATG(3)
        NM(2)=ISTATG(4)
        NM(3)=ISTATG(5)
        NMIX=ISTATG(6)
        ITRN=ITURN(ICEL)
        NBGCLS=ISTATG(13)
        ILEV=1
        CALL XDISET(ITPIN,3*MAXPIN,0)
        ICPIN=0
        IF(NBGCLS .NE. 0) THEN
*----
*  Get pin cluster geometry information
*----
          CALL XDDSET(DRAPIN,6*MAXPIN,DZERO)
          ILEV=2
          CALL XDISET(ITSYMC,4,0)
          IF(ABS(ITSYM(1,ICEL)) .EQ. 1 .OR. 
     >       ABS(ITSYM(2,ICEL)) .EQ. 1 .OR. 
     >       ABS(ITSYM(3,ICEL)) .EQ. 1) ITSYMC(1)=1
          IF(ABS(ITSYM(4,ICEL)) .EQ. 1) ITSYMC(4)=1
          CALL LCMGET(IPGEO,'CLUSTER',IBASE(IOFNAC))
          DO ICLS=1,NBGCLS
            IGCLS=IGCLS+1
            ICN=IOFNAC+3*(ICLS-1)
            WRITE(NAMCL,'(3A4)') (IBASE(ICN+ITC),ITC=0,2)
            CALL LCMSIX(IPGEO,NAMCL,1)
            IF(IPRINT .GE. 10) THEN
              WRITE(IOUT,6011) ICLS,NAMCL
            ENDIF
            CALL XDISET(ISTATC,NSTATE,0)
            CALL LCMGET(IPGEO,'STATE-VECTOR',ISTATC)
            ITYPC=ISTATC(1)
            IF(ITYPC .EQ. 20) THEN
              IF(ISTATC(2) .EQ. 0) ITYPC=5
            ELSE IF(ITYPC .EQ. 21 .OR. ITYPC .EQ. 22 .OR.
     >              ITYPC .EQ. 23) THEN
              IF(ISTATC(2) .EQ. 0) ITYPC=7
            ENDIF
*----
*  Cartesian or Annular pin
*----
            IPINT=0
            IF(ITYPC .EQ.  5 .OR. ITYPC .EQ.  7 .OR. 
     >         ITYPC .EQ. 20 .OR. ITYPC .EQ. 23      ) THEN
              IPINT=-3
            ELSE IF(ITYPC .EQ.  3 .OR. ITYPC .EQ.  6 ) THEN
              IPINT=3
            ELSE IF(ITYPC .EQ. 21) THEN
              IPINT=-1
            ELSE IF(ITYPC .EQ. 10) THEN
              IPINT=1
            ELSE IF(ITYPC .EQ. 22) THEN
              IPINT=-2
            ELSE IF(ITYPC .EQ. 11 ) THEN
              IPINT=2
            ENDIF
            NMC(4)=ISTATC(2)
            NMC(1)=ISTATC(3)
            NMC(2)=ISTATC(4)
            NMC(3)=ISTATC(5)
            NMIXC=ISTATC(6)
            CALL XDDSET(DAMESH,(MAXMSH+2)*4,DZERO)
            CALL XDISET(ISPLT,4*MAXMSH,1)
            CALL XDISET(MIX,NMIXC,0)
            CALL NXTEGI(IPGEO ,IPRINT,ITYPC ,MAXMSH,NMIXC ,NMC   ,
     >                  MAXMSS,NMCS  ,NREGC ,NREGCS,NSURC ,NSURCS,
     >                  MIX   ,ISPLT ,DAMESH,
     >                  RMESH ,IBASE(MIXC)  )
            CALL SETARA(IBASE,NSURCS,IDSUR)
            CALL SETARA(IBASE,NREGCS,IDREG)
            NEREN=NREGCS+NSURCS
            CALL SETARA(IBASE,NEREN,IREN)
            CALL SETARA(RBASE,2*(MAXMSS+2)*4*2,DAMESR)
            CALL XDDSET(RBASE(DAMESR),(MAXMSS+2)*4*2,DZERO)
            NMIXCS=NREGCS
            DROUT=DAMESH(NMC(4),4)
*----
*  Test if pin cells satisfy symmetry options
*----
            IF(ABS(ITSYMC(1)) .EQ. 1 .OR. ABS(ITSYMC(4)) .EQ. 1) THEN
*----
*  Rotate pin cells and test for intrinsic symmetry
*----
              ITST=1
              CALL NXTRIS(IPRINT,ITYPC ,MAXMSH,NREGC ,ITRN  ,ITST  ,
     >                    ITSYMC,NMC   ,MIX   ,ISPLT ,DAMESH,
     >                    NMCS  ,IBASE(MIXC)  ,IBASE(ISPLTR),
     >                    RBASE(DAMESR))
            ELSE
*----
*  Rotate pin cells 
*----
              ITST=0
              CALL NXTRIS(IPRINT,ITYPC ,MAXMSH,NREGC ,ITRN  ,ITST  ,
     >                    ITSYMC,NMC   ,MIX   ,ISPLT ,DAMESH,
     >                    NMCS  ,IBASE(MIXC)  ,IBASE(ISPLTR),
     >                    RBASE(DAMESR))
            ENDIF
*----
*  Final mesh after unfolding
*----
            IF(MOD(ITRN,2) .EQ. 0) THEN
*----
*  First turn geometry type if required
*----
              IF(ITYPC .EQ. 10) THEN
                ITYPC=11
              ELSE IF(ITYPC .EQ.11) THEN
                ITYPC=10
              ELSE IF(ITYPC .EQ.21) THEN
                ITYPC=22
              ELSE IF(ITYPC .EQ.22) THEN
                ITYPC=21
              ENDIF
            ENDIF
*----
*  Then create new mesh
*----
            CALL SETARA(IBASE,NMIXCS,MIXCS)
            CALL NXTSGI(IPTRK ,IPRINT,MAXMSH,ITYPC ,IGCLS ,ILEV  ,
     >                  MAXMSS,NMIXC ,NMC   ,MIX   ,DAMESH,ISPLT ,
     >                  NMIXCS,NMCS  ,IBASE(MIXCS) ,RBASE(DAMESR),
     >                  ITSYMC,
     >                  NREGCS,NSURCS,NREGCN,NSURCN,NEREN ,
     >                  IBASE(IDREG) ,IBASE(IDSUR) ,IBASE(IREN)  )
            CALL RLSARA(IBASE(MIXCS))
            CALL SETARA(IBASE,4*(NREGCS+NSURCS+1),INDXSR)
            CALL SETARA(RBASE,2*(NREGCS+NSURCS+1),SURVOL)
            CALL SETARA(RBASE,2*(NREGCN+NSURCN+1),SVT)
            CALL SETARA(IBASE,(NREGCN+NSURCN+1),INREN)
            NBPIN=0
            MXGSUR=MAX(MXGSUR,NSURCS)
            MXGREG=MAX(MXGREG,NREGCS)
            CALL XDISET(IEDIMP,NSTATE,0)
            IEDIMP(1)=ITYPC
            IEDIMP(2)=NMCS(4)
            IEDIMP(3)=NMCS(1)
            IEDIMP(4)=NMCS(2)
            IEDIMP(5)=NMCS(3)
            IEDIMP(6)=NMIXCS
            IEDIMP(7)=ILEV
            IEDIMP(8)=NREGCS
            IEDIMP(9)=NSURCS
            IEDIMP(10)=NREGCN
            IEDIMP(11)=NSURCN
            IEDIMP(12)=NAREG+1
            CALL NXTVOL(IPTRK ,IPRINT,MAXMSS,ITYPC ,IGCLS ,
     >                  ILEV  ,NMCS  ,NREGCS,NSURCS,NREGCN,NSURCN,
     >                  MAXPIN,NBPIN ,ITPIN ,DRAPIN,
     >                  IBASE(IDREG) ,IBASE(IDSUR) ,RBASE(DAMESR),
     >                  IBASE(INDXSR),RBASE(SURVOL),RBASE(SVT)   ,
     >                  IBASE(INREN) ,NAREG )
            IEDIMP(13)=NAREG
            CALL RLSARA(IBASE(INREN))
            CALL RLSARA(RBASE(SVT))
            CALL RLSARA(RBASE(SURVOL))
            CALL RLSARA(RBASE(DAMESR))
            CALL RLSARA(IBASE(IREN))
            CALL RLSARA(IBASE(IDREG))
*----
*  Read pin locations
*----
            CALL LCMGET(IPGEO,'NPIN',NPIN) 
            CALL LCMLEN(IPGEO,'RPIN',ILCMLN,ILCMTY)
            IF(ILCMLN .EQ. 0) THEN
               CALL XABORT(NAMSBR//
     >        ': RPIN is absent for pin cluster')
            ELSE IF(ILCMLN .EQ. 1) THEN
              CALL LCMGET(IPGEO,'RPIN',RPIN)
              DO IP=1,NPIN
                DRAPIN(0,ICPIN+IP)=DBLE(RPIN(1))
              ENDDO 
            ELSE IF(ILCMLN .EQ. NPIN) THEN
              CALL LCMGET(IPGEO,'RPIN',RPIN)
              DO IP=1,NPIN
                DRAPIN(0,ICPIN+IP)=DBLE(RPIN(IP))
              ENDDO 
            ELSE
              CALL XABORT(NAMSBR//
     >        ': Length of RPIN vector is invalid')
            ENDIF
            CALL LCMLEN(IPGEO,'APIN',ILCMLN,ILCMTY)
            IF(ILCMLN .EQ. 0) THEN
              DRAPIN(-1,ICPIN+1)=DZERO
              DELTA=(DTWO*PI)/DBLE(NPIN)
              DO IP=2,NPIN
                DRAPIN(-1,ICPIN+IP)=DRAPIN(-1,ICPIN+IP-1)+DELTA
              ENDDO 
            ELSE IF(ILCMLN .EQ. 1) THEN
              CALL LCMGET(IPGEO,'APIN',APIN)
              DRAPIN(-1,ICPIN+1)=DBLE(APIN(1))
              DELTA=(DTWO*PI)/DBLE(NPIN)
              DO IP=2,NPIN
                DRAPIN(-1,ICPIN+IP)=DRAPIN(-1,ICPIN+IP-1)+DELTA
              ENDDO 
            ELSE IF(ILCMLN .EQ. NPIN) THEN
              CALL LCMGET(IPGEO,'APIN',APIN)
              DO IP=1,NPIN
                DRAPIN(-1,ICPIN+IP)=DBLE(APIN(IP))
              ENDDO 
            ELSE
              CALL XABORT(NAMSBR//
     >        ': Length of APIN vector is invalid')
            ENDIF
*----
*  Find pin height
*----
            IF(ITYPC .EQ. 10  .OR. ITYPC .EQ.21) THEN
              IDGPP=1
              DHPIN=DAMESH(NMC(1),1)-DAMESH(0,1)
            ELSE IF(ITYPC .EQ. 11 .OR. ITYPC .EQ. 22) THEN
              IDGPP=2
              DHPIN=DAMESH(NMC(2),2)-DAMESH(0,2)
            ELSE IF(ITYPC .EQ.  6 .OR. ITYPC .EQ. 23) THEN
              IDGPP=3
              DHPIN=DAMESH(NMC(3),3)-DAMESH(0,3)
            ELSE
              IDGPP=3
              DHPIN=DONE
            ENDIF
*----
*  Replace angles in range 0 to 2*Pi
*----
            DO IP=1,NPIN
              IF(DRAPIN(-1,ICPIN+IP) .GE. DZERO) THEN
                NPIR=INT((DRAPIN(-1,ICPIN+IP)+DCUTOF)/TWOPI)
                DRAPIN(-1,ICPIN+IP)=DRAPIN(-1,ICPIN+IP)
     >                             -DBLE(NPIR)*TWOPI
              ELSE
                NPIR=INT((DRAPIN(-1,ICPIN+IP)-DCUTOF)/TWOPI)
                DRAPIN(-1,ICPIN+IP)=DRAPIN(-1,ICPIN+IP)
     >                             -DBLE(NPIR-1)*TWOPI
              ENDIF
              DRAPIN(IDGPP,ICPIN+IP)=DHPIN
              DRAPIN(4,ICPIN+IP)=DROUT
              ITPIN(1,ICPIN+IP)=ICEL
              ITPIN(2,ICPIN+IP)=IGCLS
              ITPIN(3,ICPIN+IP)=IPINT
            ENDDO
            IF(ABS(ITSYM(1,ICEL)) .EQ. 1 .OR. 
     >         ABS(ITSYM(2,ICEL)) .EQ. 1 .OR. 
     >         ABS(ITSYM(3,ICEL)) .EQ. 1 .OR. 
     >         ABS(ITSYM(4,ICEL)) .EQ. 1) THEN
*----
*  Test pin in cluster for symmetry
*----
              CALL NXTTPS(IPRINT,NPIN  ,IDGPP ,ITSYM(1,ICEL),
     >                    DRAPIN(-1,ICPIN+1))
            ENDIF
*----
*  Renumber pin surfaces
*----
            CALL SETARA(IBASE,NSURCN,INREN)
            IEDIMP(14)=NASUR+1
            CALL NXTRPS(IPTRK ,IPRINT,ITYPC ,IGCLS ,ILEV  ,
     >                  NREGCS,NSURCS,NREGCN,NSURCN,IDFEX(1,ICEL),
     >                  IBASE(INDXSR),DHPIN ,DCMESH(1,1,ICEL),
     >                  NASUR ,IBASE(IDSUR) ,IBASE(INREN) )
            CALL RLSARA(IBASE(INREN))
            CALL RLSARA(IBASE(INDXSR))
            CALL RLSARA(IBASE(IDSUR))
            IEDIMP(15)=NASUR
            IEDIMP(16)=ICPIN+1
            IEDIMP(17)=NPIN
            WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),IGCLS,'DIM'
            CALL LCMPUT(IPTRK,NAMREC,NSTATE,1,IEDIMP)
            ICPIN=ICPIN+NPIN
            CALL LCMSIX(IPGEO,NAMCL,2)
          ENDDO
*----
*  Save cluster information for this cell
*----
          ILEV=1
          IOFNAC=IOFNAC+3*NBGCLS
          WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),ICEL,'PIN'
          CALL LCMPUT(IPTRK,NAMREC,2*6*ICPIN,4,DRAPIN)
          WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),ICEL,'PNT'
          CALL LCMPUT(IPTRK,NAMREC,3*ICPIN,1,ITPIN)
        ENDIF
        CALL XDDSET(DAMESH,(MAXMSH+2)*4,DZERO)
        CALL XDISET(ISPLT,4*MAXMSH,1)
        CALL XDISET(MIX,NMIX,0)
*----
*  Get main geometry information
*----
        CALL NXTEGI(IPGEO ,IPRINT,ITYPG ,MAXMSH,NMIX  ,NM    ,
     >              MAXMSS,NMS   ,NREG  ,NREGS ,NSUR  ,NSURS ,
     >              MIX   ,ISPLT ,DAMESH,
     >              RMESH ,IBASE(MIXC)  )
        NMIXS=NREGS
        CALL SETARA(IBASE,NREGS,IDREG)
        CALL SETARA(IBASE,NSURS,IDSUR)
        NEREN=NREGS+NSURS
        CALL SETARA(IBASE,NEREN,IREN)
        CALL SETARA(RBASE,2*(MAXMSS+2)*4*2,DAMESR)
        CALL XDDSET(RBASE(DAMESR),(MAXMSS+2)*4*2,DZERO)
        IF(ABS(ITSYM(1,ICEL)) .EQ. 1 .OR. 
     >     ABS(ITSYM(2,ICEL)) .EQ. 1 .OR. 
     >     ABS(ITSYM(3,ICEL)) .EQ. 1 .OR. 
     >     ABS(ITSYM(4,ICEL)) .EQ. 1) THEN
*----
*  Rotate global geometry and test for intrinsic symmetry
*----
          ITST=1
          CALL NXTRIS(IPRINT,ITYPG ,MAXMSH,NREG  ,ITRN  ,ITST,
     >                ITSYM(1,ICEL),NM    ,MIX   ,ISPLT ,DAMESH,
     >                NMS   ,IBASE(MIXC)  ,IBASE(ISPLTR),
     >                RBASE(DAMESR))
        ELSE
*----
*  Rotate global geometry 
*----
          ITST=0
          CALL NXTRIS(IPRINT,ITYPG ,MAXMSH,NREG  ,ITRN  ,ITST,
     >                ITSYM(1,ICEL),NM    ,MIX   ,ISPLT ,DAMESH,
     >                NMS   ,IBASE(MIXC)  ,IBASE(ISPLTR),
     >                RBASE(DAMESR))
        ENDIF
*----
*  Final mesh after unfolding
*----
        IF(MOD(ITRN,2) .EQ. 0) THEN
*----
*  First turn geometry type if required
*----
          IF(ITYPG .EQ. 10) THEN
            ITYPG=11
          ELSE IF(ITYPG .EQ.11) THEN
            ITYPG=10
          ELSE IF(ITYPG .EQ.21) THEN
            ITYPG=22
          ELSE IF(ITYPG .EQ.22) THEN
            ITYPG=21
          ENDIF
        ENDIF
*----
*  Then create new mesh
*----
        CALL SETARA(IBASE,NMIXS,MIXS)
        CALL NXTSGI(IPTRK ,IPRINT,MAXMSH,ITYPG ,ICEL  ,ILEV  ,
     >              MAXMSS,NMIX  ,NM    ,MIX   ,DAMESH,ISPLT ,
     >              NMIXS ,NMS   ,IBASE(MIXS)  ,RBASE(DAMESR),
     >              ITSYM(1,ICEL),
     >              NREGS ,NSURS ,NREGN ,NSURN ,NEREN ,
     >              IBASE(IDREG) ,IBASE(IDSUR) ,IBASE(IREN)  )
        CALL RLSARA(IBASE(MIXS))
        CALL SETARA(IBASE,4*(NREGS+NSURS+1),INDXSR)
        CALL SETARA(RBASE,2*(NREGS+NSURS+1),SURVOL)
        CALL SETARA(RBASE,2*(NREGN+NSURN+1),SVT)
        CALL SETARA(IBASE,(NREGN+NSURN+1),INREN)
        NBPIN=ICPIN
        MXGSUR=MAX(MXGSUR,NSURS)
        MXGREG=MAX(MXGREG,NREGS)
        CALL XDISET(IEDIMX,NSTATE,0)
        IEDIMX(1)=ITYPG
        IEDIMX(2)=NMS(4)
        IEDIMX(3)=NMS(1)
        IEDIMX(4)=NMS(2)
        IEDIMX(5)=NMS(3)
        IEDIMX(6)=NMIXS
        IEDIMX(7)=ILEV
        IEDIMX(8)=NREGS
        IEDIMX(9)=NSURS
        IEDIMX(10)=NREGN
        IEDIMX(11)=NSURN
        IEDIMX(12)=NAREG+1
        CALL NXTVOL(IPTRK ,IPRINT,MAXMSS,ITYPG ,ICEL  ,
     >              ILEV  ,NMS   ,NREGS ,NSURS ,NREGN ,NSURN ,
     >              MAXPIN,NBPIN ,ITPIN ,DRAPIN,
     >              IBASE(IDREG) ,IBASE(IDSUR) ,RBASE(DAMESR),
     >              IBASE(INDXSR),RBASE(SURVOL),RBASE(SVT)   ,
     >              IBASE(INREN) ,NAREG )
*----
*  Renumber cell surfaces
*----
        IEDIMX(13)=NAREG
        CALL SETARA(IBASE,NSURN,INREN)
        IEDIMX(14)=NASUR+1
        CALL NXTRCS(IPTRK ,IPRINT,ITYPG ,ICEL ,ILEV  ,
     >              NREGS ,NSURS ,NREGN ,NSURN ,IDFEX(1,ICEL),
     >              IBASE(INDXSR),
     >              NASUR ,IBASE(IDSUR) ,IBASE(INREN) )
        CALL RLSARA(IBASE(INREN))
        IEDIMX(15)=NASUR
        IEDIMX(16)=NBGCLS
        IEDIMX(17)=IGCLS-NBGCLS+1
        IEDIMX(18)=NBPIN
        IEDIMX(19)=IDFEX(7,ICEL)
        CALL RLSARA(RBASE(SVT))
        CALL RLSARA(RBASE(SURVOL))
        CALL RLSARA(IBASE(INDXSR))
        WRITE(NAMREC,'(A1,I8.8,A3)') CLEV(ILEV),ICEL,'DIM'
        CALL LCMPUT(IPTRK,NAMREC,NSTATE,1,IEDIMX)
        CALL RLSARA(RBASE(DAMESR))
        CALL RLSARA(IBASE(IREN))
        CALL RLSARA(IBASE(IDSUR))
        CALL RLSARA(IBASE(IDREG))
        IF(ILCELL .EQ. 1) THEN
          CALL LCMSIX(IPGEO,NAMGG,2)
        ENDIF
      ENDDO
*----
*  Processing finished:
*  print routine closing output header if required
*  and return
*----
      CALL RLSARA(IBASE(ISPLTR))
      CALL RLSARA(IBASE(NAGCLS))
      CALL RLSARA(IBASE(MIXC))
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6001) NAMSBR
      ENDIF
      NFSUR=NASUR
      NFREG=NAREG
      RETURN
*----
*  Output formats
*----
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6010 FORMAT(' Processing CELL    = ',I5.4,5X,A12)
 6011 FORMAT(' Processing CLUSTER = ',I5.4,5X,A12)
      END
