*DECK NXTTLC
      SUBROUTINE NXTTLC(IPTRK ,IFTEMP,IPRINT,IGTRK ,NDIM  ,MAXMSH,
     >                  NFSUR ,NFREG ,NUCELL,NBUCEL,NBANGL,
     >                  LINMAX,MXGSUR,MXGREG,RENO  ,
     >                  MAXPIN,AZMQUA,IPER  ,
     >                  IUNFLD,SURVOL,DGMESH,
     >                  DANGLT,DDENWT,DNSANG,NBSANG,DEPART,
     >                  MAXSGL,NTLINE,DVNOR ,DSNOR ,
     >                  NUMERO,LENGTH,DLENGT,ICINT ,DCINT )
*
*-----------------------------------------------------------------------
*
*Purpose:
* To generate the cyclic tracking lines (specular tracking)
* for a geometry.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal.
*
*Author(s): 
* G. Marleau
*
*Update(s): 
* None.
*
*Reference: 
*  G. Marleau, 
*  \textsl{New Geometries Processing in DRAGON: The NXT: Module},
*  Report IGE-260, \'{E}cole Polytechnique de Montr\'{e}al, 
*  Montr\'{e}al, 2005.
*  Based on the XELTI2 and XELTI3 routines.
*
*Parameters: input
* IPTRK   pointer to the TRACKING data structure in 
*         update or creation mode.
* IFTEMP  pointer to a temporary TRACKING data structure in 
*         update or creation mode.
* IPRINT  print level.
* IGTRK   flag to generate the tracking file. In the case where 
*         IGTRK=1, the tracking is performed and
*         used to evaluate the track normalisation factor and the 
*         tracking file is generated. When IGTRK=0, the tracking is
*         still performed and used to evaluate the 
*         track normalisation factor but the tracking file is not 
*         generated. 
* NDIM    problem dimensions.
* MAXMSH  maximum number of elements in MESH array.
* NFSUR   number of surfaces.
* NFREG   number of regions.
* NUCELL  number of cell after unfolding in 
*         $X$, $Y$ and $Z$ directions.
* NBUCEL  number of cells in unfolded geometry.
* NBANGL  number of angles.
* LINMAX  maximum number of segments in a track.
* MXGSUR  maximum number of surfaces for any geometry.
* MXGREG  maximum number of region for any geometry.
* RENO    track normalisation option. A value RENO=-1 implies 
*         a direction dependent normalization of the tracks
*         for the volume while a value RENO=0, implies
*         a global normalisation.  
* MAXPIN  maximum number of pins in a cell.
* AZMQUA  tracking type.
* IPER    cell periodicity factor in each direction.
* IUNFLD  description of unfolded geometry.
* SURVOL  global surface volume vector.
* DGMESH  meshing vector for global geometry.
* DANGLT  angles.
* DDENWT  angular density for each angle.
* DNSANG  spatial density required.
* NBSANG  number of segments for each angles.
* DEPART  track starting point.
*
*Parameters: output
* MAXSGL  maximum number of segments in a line.
* NTLINE  total number of lines generated.
* DVNOR   ratio of analytic to tracked volume.
* DSNOR   ratio of analytic to tracked surface area.
*
*Parameters: temporary storage
* NUMERO  region/surface identification number
*         for segment.
* LENGTH  segment length.
* DLENGT  spatial location of each line segment.
* ICINT   identification of spatial position for each
*         line segment in cell description of geometry.
* DCINT   position of each intersection point for each
*         line segment in cell description of geometry.
*----------
*
      IMPLICIT         NONE
*----
*  Subroutine arguments
*----
      INTEGER          IPTRK,IFTEMP
      INTEGER          IPRINT,IGTRK,NDIM,MAXMSH,NFSUR,NFREG,
     >                 NUCELL(3),NBUCEL,NBANGL,LINMAX,MXGSUR,MXGREG,
     >                 RENO,MAXPIN,AZMQUA,IPER(3),MAXSGL,NTLINE
      INTEGER          IUNFLD(2,NBUCEL)
      DOUBLE PRECISION SURVOL(-NFSUR:NFREG),DGMESH(-1:MAXMSH,4),
     >                 DANGLT(NDIM,NBANGL),DDENWT(NBANGL),
     >                 DNSANG(NBANGL)
      INTEGER          NBSANG(5,NBANGL)
      DOUBLE PRECISION DEPART(NDIM,2,NBANGL)
      DOUBLE PRECISION DVNOR(NFREG),
     >                 DSNOR(NFSUR)
      INTEGER          NUMERO(LINMAX)
      REAL             LENGTH(LINMAX)
      DOUBLE PRECISION DLENGT(LINMAX)
      INTEGER          ICINT(0:4,2*(NBUCEL+4))
      DOUBLE PRECISION DCINT(2*(NBUCEL+4))
*----
*  Local parameters
*----
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='NXTTLC')
      DOUBLE PRECISION DCUTOF,DZERO,DONE,DTWO
      PARAMETER       (DCUTOF=1.0D-8,DZERO=0.0D0,DONE=1.0D0,DTWO=2.0D0)
*----
*  Functions
*----
      DOUBLE PRECISION XDRCST,PI
      INTEGER          NXTLCA,IRLCA
      INTEGER          KDROPN,ILOUT,KDRCLS,ICLS
*----
*  Memory allocation variables
*----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          IDSUR,IDREG,DCMESH,ICSINT,DCSINT,
     >                 ITPIN,DRAPIN
*----
*  Local variables
*----
      INTEGER          LMAXT,ITST,IANGL,NGLINE,IDIR,ITXY(3),
     >                 NSCAN,NUMANG,NPOINT,NBPTS,ISCAN,
     >                 ISTART,ISUM,NBCOR(2)
      INTEGER          NBSINT,ISDIR,IBLIN,IELIN,ICI,IX,IY,IZ,
     >                 IOX,IOY,IOZ,IOC,ICEL,ITRN,ISURF
      DOUBLE PRECISION FACVOL,FACSUR,DWGT,PMAX,PMIN,
     >                 DSTART,RADT2,ANGT2
      DOUBLE PRECISION ANGLED(3),ANGLES(3),ANGLEN(3),
     >                 TRKORI(3),TRKOR2(3),TRKORD(3),TRKORN(3),
     >                 TRKLIM(2),CELLPO(3,2)
      DOUBLE PRECISION DERR,DCERR,DSVERR,DMVERR,DAVERR,DSSERR,
     >                 DMSERR,DASERR
      REAL             WGT,WGTFAC
      INTEGER          ISEG,JSEG,IREG,ILREG,ISUR,NBVERR,NBSERR,
     >                 NBV0,NBS0,IPRINL
      INTEGER          ISD,NSDEB,ISF,NSFIN,NBREG,ISBL,NTSEG
      INTEGER          ISINT,NBSEG,IPT,JSDIR,ISDIRX,IPTN
      DOUBLE PRECISION DHALF
*----
*  Data
*----
      CHARACTER        CDIR(4)*1
      SAVE             CDIR
      DATA             CDIR /'X','Y','Z','R'/
*----
*  Processing starts:
*  print routine openning output header if required
*  and initialize various parameters.
*----
      IF(IPRINT .GE. 2) THEN
        WRITE(IOUT,6000) NAMSBR
      ENDIF
      IF(IPRINT .LE. -1000) THEN
        ILOUT=KDROPN('NXTPLines',0,3,0,0)
        WRITE(ILOUT,8000) NFREG,NFSUR,NDIM,NBANGL
        WRITE(ILOUT,8002) (DGMESH(0,IDIR),DGMESH(NUCELL(IDIR),IDIR),
     >                     IDIR=1,NDIM)
        WRITE(ILOUT,8004) 
      ENDIF
      PI=XDRCST('Pi',' ')
      DHALF=DONE/DTWO
      MAXSGL=0
      NTLINE=0
      LMAXT=2*(NBUCEL+4)
      ITST=1
      CALL XDDSET(DVNOR,NFREG,DZERO)
      CALL XDDSET(DSNOR,NFSUR,DZERO)
      FACVOL=DTWO
      IF(NDIM .EQ. 2) THEN
        FACSUR=DTWO*PI
      ELSE
        FACSUR=DTWO*DTWO
      ENDIF
      CALL SETARA(IBASE,MXGSUR,IDSUR)
      CALL SETARA(IBASE,MXGREG,IDREG)
      CALL SETARA(RBASE,2*4*(MAXMSH+2),DCMESH)
      CALL SETARA(IBASE,5*4*(MXGREG+4)*3,ICSINT)
      CALL SETARA(RBASE,2*4*(MXGREG+4)*3,DCSINT)
      CALL SETARA(IBASE,3*(MAXPIN+1),ITPIN)
      CALL SETARA(RBASE,6*2*(MAXPIN+1),DRAPIN)
*----
*  Loop over directions
*----
      NGLINE=0
      DO IANGL=1,NBANGL
        DO IDIR=1,NDIM
          ITXY(IDIR)=NBSANG(IDIR,IANGL)
        ENDDO
        ITXY(3)=ITXY(1)*ITXY(2)
        NSCAN=NBSANG(3,IANGL)
        NUMANG=NBSANG(4,IANGL)
        NPOINT=NBSANG(5,IANGL)
*----
*  Loop over quadrant if both quadrant not scanned simultaneously
*  If NSCAN = 2  0 TO PI/2 AND PI/2 TO PI are scanned simultaneously
*  If NSCAN = 1  0 TO PI/2 AND PI/2 TO PI are scanned independently
*----
        NBPTS=NSCAN*NPOINT
        DO ISCAN=2,NSCAN,-1
*----
*  Initialize weight
*  Store track direction in ANGLES 
*  and track starting point in TRKOR2
*----
          DWGT=DNSANG(IANGL)
          WGT=DWGT
          DO IDIR=1,NDIM
            ANGLED(IDIR)=DANGLT(IDIR,IANGL)
            TRKOR2(IDIR)=DEPART(IDIR,1,IANGL)
          ENDDO
          IF(ISCAN .EQ. 1) THEN
            IF(ITXY(1) .EQ. 0) THEN
              ANGLED(2)=-ANGLED(2)
            ELSE IF(ITXY(2) .EQ. 0) THEN
              ANGLED(1)=-ANGLED(1)
            ENDIF
          ENDIF
          DO IDIR=1,NDIM
            ANGLES(IDIR)=ANGLED(IDIR)
            ANGLEN(IDIR)=ANGLED(IDIR)
            TRKORD(IDIR)=TRKOR2(IDIR)
            TRKORI(IDIR)=TRKORD(IDIR)
            TRKORN(IDIR)=TRKORD(IDIR)
          ENDDO
*----
*  Loop over points
*----
          IF(IPRINT .GE. 10) THEN
            WRITE(IOUT,6050) IANGL,ISCAN,ITXY,DWGT,
     >      (DANGLT(IDIR,IANGL),IDIR=1,NDIM),
     >      (DEPART(IDIR,1,IANGL),IDIR=1,NDIM)
          ENDIF
          ISTART=0
          ISUM=ISTART
          DO IPT=1,NBPTS
*----
*  Find if this track crosses geometry
*----
            ISINT=1
            DCINT(ISINT)=DZERO
            IRLCA=NXTLCA(IPRINL,ITST  ,NDIM  ,MAXMSH,LMAXT,
     >                   NUCELL,TRKORI,ANGLES,DGMESH,
     >                   NBCOR ,NBSINT,ICINT ,DCINT)
*----
*  When no external face crossed go to next line
*----
            IF(IRLCA .EQ. 0) GO TO 105
            ISDIRX=ABS(ICINT(0,1))
            ISDIR=ABS(ICINT(0,NBSINT))
            JSDIR=MOD(ISDIR,2)+1
            IF(ISTART .EQ. 0) THEN
*----
*  Define position of cyclic starting point
*----
              NGLINE=NGLINE+1 
              DO IDIR=1,NDIM
                TRKORD(IDIR)=TRKORD(IDIR)+DCINT(ISINT)*ANGLES(IDIR)
                TRKORI(IDIR)=TRKORD(IDIR)
              ENDDO
              DCINT(ISINT)=DZERO
              IF(ISCAN .EQ. 1 .AND. ITXY(3) .NE. 0) THEN
                ANGLED(ISDIRX)=-ANGLED(ISDIRX)
                DO IDIR=1,NDIM
                  ANGLES(IDIR)=ANGLED(IDIR)
                ENDDO
                DCINT(ISINT)=DZERO
                IRLCA=NXTLCA(IPRINL,ITST  ,NDIM  ,MAXMSH,LMAXT,
     >                       NUCELL,TRKORI,ANGLES,DGMESH,
     >                       NBCOR ,NBSINT,ICINT ,DCINT)
                IF(IRLCA .EQ. 0) GO TO 105
                DO IDIR=1,NDIM
                  TRKORD(IDIR)=TRKORD(IDIR)+DCINT(ISINT)*ANGLES(IDIR)
                  TRKORI(IDIR)=TRKORD(IDIR)
                ENDDO
                DCINT(ISINT)=DZERO
                ISDIR=ABS(ICINT(0,NBSINT))
                JSDIR=MOD(ISDIR,2)+1
              ENDIF
              DO IDIR=1,NDIM
                ANGLES(IDIR)=ANGLED(IDIR)
                ANGLEN(IDIR)=ANGLED(IDIR)
              ENDDO
            ENDIF
*----
*  For track crossing geometry, end find surface direction
*  and end of track location
*----
            DO ISINT=1,NBSINT
              DO IDIR=1,NDIM
                TRKORN(IDIR)=TRKORI(IDIR)+DCINT(ISINT)*ANGLES(IDIR)
              ENDDO
            ENDDO
*----
*  Find location with respect end surfaces positions
*----
            PMAX=ABS(TRKORN(ISDIR)-DGMESH(NUCELL(ISDIR),ISDIR))
            PMIN=ABS(TRKORN(ISDIR)-DGMESH(0,ISDIR))
            IF(IPER(ISDIR) .EQ. 1) THEN
*----
*  Translate next initial starting point if possible
*----
              IF(PMAX .LT. DCUTOF) THEN
                TRKORN(ISDIR)=DGMESH(0,ISDIR)
              ELSE IF(PMIN .LT. DCUTOF) THEN
                TRKORN(ISDIR)=DGMESH(NUCELL(ISDIR),ISDIR)
              ELSE
                CALL XABORT(NAMSBR//
     >          ': Translation before cell limit attempted')
              ENDIF
            ELSE
*----
*  Change track direction upon reflexion if possible
*----
              IF(PMAX .GT. DCUTOF .AND. PMIN .GT. DCUTOF) THEN
                CALL XABORT(NAMSBR//
     >          ': Reflexion before cell limit attempted')
              ENDIF
              ANGLEN(ISDIR)=-ANGLEN(ISDIR)
            ENDIF
*----
*  Track seems ok, locate track segments for geometry
*----
            IF(ISTART .EQ. 0) THEN
*----
*  For first series of line segments initialize track information
*----
              IBLIN=1
              IELIN=0
            ELSE
*----
*  Next series of line segments, add at the end of
*  current line
*----
              IBLIN=IELIN+1
              DLENGT(IBLIN)=DZERO
            ENDIF
            ISTART=ISTART+1
            ISUM=ISUM+1
            NUMERO(IBLIN)=0
            DLENGT(IBLIN)=DCINT(1)
            DSTART=DCINT(1)
            IBLIN=IBLIN+1
*----
*  Find line segments for this track series
*----
            DO ICI=NBCOR(1)+1,NBSINT-NBCOR(2)+1
              IX=ICINT(1,ICI)
              IY=ICINT(2,ICI)
              IOX=IX
              CELLPO(1,2)=DGMESH(IX,1)
              CELLPO(1,1)=DGMESH(IX-1,1)
              CELLPO(2,2)=DGMESH(IY,2)
              CELLPO(2,1)=DGMESH(IY-1,2)
              IOY=(IY-1)*NUCELL(1)
              IOZ=0
              IF(NDIM .EQ. 3) THEN
                IZ=ICINT(3,ICI)
                IOZ=(IZ-1)*NUCELL(1)*NUCELL(2)
                CELLPO(3,2)=DGMESH(IZ,3)
                CELLPO(3,1)=DGMESH(IZ-1,3)
              ENDIF
              IOC=IOX+IOY+IOZ
              ICEL=IUNFLD(1,IOC)
              ITRN=IUNFLD(2,IOC)
              TRKLIM(1)=DSTART
              DSTART=DSTART+DCINT(ICI)
              TRKLIM(2)=DSTART
              IF(ICI .EQ. NBCOR(1)+1) THEN
*----
*  initial surfaces (at TRKLIM(1)) considered
*----
                ISURF=-1
              ELSE IF(ICI .EQ. NBSINT-NBCOR(2)) THEN
*----
*  final surfaces (at TRKLIM(2)) considered
*----
                ISURF=1
              ELSE
*----
*  no surface considered
*----
                ISURF=0
              ENDIF
*----
*  Track turned Cell
*----       
              CALL NXTTCR(IPTRK ,IPRINL,ICEL  ,ITRN  ,ISURF ,
     >                    NDIM  ,MAXMSH,LINMAX,MXGSUR,MXGREG,
     >                    MAXPIN,CELLPO,TRKLIM,TRKORI,ANGLES,
     >                    IBLIN ,IELIN ,NUMERO,DLENGT,
     >                    IBASE(IDSUR) ,IBASE(IDREG) ,
     >                    IBASE(ITPIN) ,RBASE(DCMESH),
     >                    RBASE(DRAPIN),IBASE(ICSINT),
     >                    RBASE(DCSINT))
              DERR=MAX(ABS(TRKLIM(1)),ABS(TRKLIM(2)))
              DERR=(DLENGT(IELIN)-DSTART)/DERR
              IF(DERR .GT. DCUTOF) THEN
                WRITE(IOUT,9100) NAMSBR,NGLINE,
     >          DLENGT(IELIN),DSTART,DERR
                CALL XABORT(NAMSBR//
     >': End of track does not coincide with end of cell')
              ENDIF
              IBLIN=IELIN+1
*----
*  End of tracking for this line segment
*----
            ENDDO
*----
*  This series of line segments completed
*  Test if the cycle is back to starting point
*----
            RADT2=DZERO
            ANGT2=DZERO
            DO IDIR=1,NDIM
              RADT2=RADT2+(TRKORD(IDIR)-TRKORN(IDIR))**2  
              ANGT2=ANGT2+(ANGLED(IDIR)-ANGLEN(IDIR))**2  
            ENDDO
            RADT2=SQRT(RADT2)
            ANGT2=SQRT(ANGT2)
            IF(RADT2 .GT. DCUTOF .OR. ANGT2 .GT. DCUTOF) THEN
*----
*  Cycle is incomplete
*  Check if period is higher than perdicted
*  reset starting point and direction
*----
              IF(ISTART .GE. NUMANG) CALL XABORT(NAMSBR//
     >        ': Cyclic period is too long')
              DO IDIR=1,NDIM
                TRKORI(IDIR)=TRKORN(IDIR)
                ANGLES(IDIR)=ANGLEN(IDIR)
              ENDDO
            ELSE
*----
*  Cycle is complete
*  Check if period is ok
*----
              NTLINE=NTLINE+1
*              IF(ANGT2 .GT. DCUTOF) CALL XABORT(NAMSBR//
*     >        ': Invalid cycling direction')
*----
*  Process tracking line and save
*----
              NBSEG=IELIN
*----
*  Compress tracking vector for region with DLENGT=0.0 and
*  for successive segments in the same region.
*----
              JSEG=0
              ILREG=-1
              DO ISEG=1,NBSEG
                IREG=NUMERO(ISEG)
                IF(DLENGT(ISEG) .GT. DZERO) THEN
                  IF(IREG .GT. 0) THEN
                    IF(IREG .EQ. ILREG) THEN
                      DLENGT(JSEG)=DLENGT(JSEG)+DLENGT(ISEG)
                    ELSE
                      JSEG=JSEG+1
                      DLENGT(JSEG)=DLENGT(ISEG)
                      NUMERO(JSEG)=IREG
                      ILREG=IREG
                    ENDIF
                  ELSE
                    JSEG=JSEG+1
                    DLENGT(JSEG)=DLENGT(ISEG)*DHALF
                    NUMERO(JSEG)=IREG
                    ILREG=-1
                  ENDIF
                ENDIF
              ENDDO
              NBSEG=JSEG
*----
*  Add contribution of track to volume integration for this angle
*  in this quadrant and compress tracking line by removing
*  segments with NUMERO=0.
*----
              JSEG=0
              NSDEB=0
              NBREG=0
              DO ISEG=1,NBSEG
                IREG=NUMERO(ISEG)
                IF(DLENGT(ISEG) .GT. DZERO) THEN
                  IF(IREG .GT. 0) THEN
                    NBREG=NBREG+1
                    DVNOR(IREG)=DVNOR(IREG)
     >              +DLENGT(ISEG)*DWGT*FACVOL
                    JSEG=JSEG+1
                    LENGTH(JSEG)=DLENGT(ISEG)
                    NUMERO(JSEG)=IREG
                  ELSE IF(IREG .LT. 0) THEN
                    IF(NBREG .EQ. 0) THEN
                      NSDEB=NSDEB+1
                    ENDIF
                    DSNOR(-IREG)=DSNOR(-IREG)
     >              +DLENGT(ISEG)*DWGT*FACSUR
                    JSEG=JSEG+1
                    LENGTH(JSEG)=DLENGT(ISEG)
                    NUMERO(JSEG)=IREG
                  ENDIF
                ENDIF
              ENDDO
              NBSEG=JSEG
              NSFIN=0
              DO ISEG=NBSEG,1,-1
                IREG=NUMERO(ISEG)
                IF(DLENGT(ISEG) .GT. DZERO) THEN
                  IF(IREG .GT. 0) THEN
                    GO TO 115
                  ELSE IF(IREG .LT. 0) THEN
                    NSFIN=NSFIN+1
                  ENDIF
                ENDIF
              ENDDO
 115          CONTINUE
              IF(NSDEB .GT. 1 .OR. NSFIN .GT. 1) THEN
                IF(IPRINL .GE. 500) THEN
                  WRITE(IOUT,6025) NGLINE,IANGL,NBSEG,
     >                             NSDEB,NSFIN,NBREG,
     >                             DLENGT(1),DLENGT(NBSEG)
                  WRITE(IOUT,6023) 'LineReg',NGLINE
                  WRITE(IOUT,6021) (NUMERO(ISEG),ISEG=1,NBSEG)
                  WRITE(IOUT,6024)
                  WRITE(IOUT,6023) 'LinePos',NGLINE
                  WRITE(IOUT,6022) 
     >            (LENGTH(ISEG),ISEG=1,NBSEG)
                  WRITE(IOUT,6024)
                ENDIF
              ENDIF
*----
*  Store line on temporary tracking file if required
*----
              MAXSGL=MAX(MAXSGL,NBSEG)
              IF(IPRINL .GE. 500) THEN
                WRITE(IOUT,6020) NTLINE,IANGL,NBSEG,WGT*WGTFAC,
     >                           DLENGT(1),DLENGT(NBSEG)
                WRITE(IOUT,6023) 'LineReg',NGLINE
                WRITE(IOUT,6021) (NUMERO(ISEG),ISEG=1,NBSEG)
                WRITE(IOUT,6024)
                WRITE(IOUT,6023) 'LinePos',NTLINE
                WRITE(IOUT,6022) 
     >          (LENGTH(ISEG),ISEG=1,NBSEG)
                WRITE(IOUT,6024)
              ENDIF
              IF(IPRINT .LE. -1000) THEN
                WRITE(ILOUT,8003) NBSEG,
     >         (NUMERO(ISEG),LENGTH(ISEG),ISEG=1,NBSEG)
              ENDIF
              NTSEG=NBSEG-NSDEB-NSFIN+2
              ISBL=0
              WGTFAC=1.0/REAL(NSDEB*NSFIN)
              DO ISD=1,NSDEB
                DO ISF=0,NSFIN-1
                  ISBL=ISBL+1
                  IF(NSDEB*NSFIN .GT. 1 .AND. IPRINL .GE. 500) 
     >            WRITE(IOUT,6026) NTLINE-1+ISBL,NTLINE,ISBL
                  IF(IGTRK .EQ. 1) THEN
                     WRITE(IFTEMP) IANGL,NTSEG,WGT*WGTFAC,
     >               NUMERO(ISD),
     >               (NUMERO(ISEG),ISEG=NSDEB+1,NBSEG-NSFIN),
     >               NUMERO(NBSEG-ISF),
     >               LENGTH(ISD),
     >               (LENGTH(ISEG),ISEG=NSDEB+1,NBSEG-NSFIN),
     >               LENGTH(NBSEG-ISF)
*              IF( IPRT.GT.1000 ) THEN
*                WRITE(IOUT,1010) NTLINE-1+ISBL,IANGL,NBSEG
*                WRITE(IOUT,1011) (LENGTH(ISEG),NUMERO(ISEG),
*     >                            ISEG=1,NBSEG)
*              ENDIF
                  ENDIF
                ENDDO
              ENDDO
              IF(ISBL .EQ. 0) THEN
                WRITE(IOUT,6027) NTLINE,IANGL,NBSEG,
     >                           NSDEB,NSFIN,NBREG,
     >                           NUMERO(1),DLENGT(1),
     >                           NUMERO(NBSEG),DLENGT(NBSEG)
                ISBL=1
                WRITE(IFTEMP) IANGL,NTSEG,WGT*WGTFAC,
     >          NUMERO(1),
     >          (NUMERO(ISEG),ISEG=NSDEB+1,NBSEG-NSFIN),
     >          NUMERO(NBSEG),
     >          LENGTH(1),
     >          (LENGTH(ISEG),ISEG=NSDEB+1,NBSEG-NSFIN),
     >          LENGTH(NBSEG)
              ENDIF
              NTLINE=NTLINE-1+ISBL
              IF(ISTART .EQ. NUMANG) THEN
*----
*  Displace line to next starting point
*----
                DO IDIR=1,NDIM
                  ANGLES(IDIR)=ANGLED(IDIR)
                  ANGLEN(IDIR)=ANGLED(IDIR)
                  TRKOR2(IDIR)=TRKOR2(IDIR)+DEPART(IDIR,2,IANGL)
                  TRKORD(IDIR)=TRKOR2(IDIR)
                  TRKORI(IDIR)=TRKORD(IDIR)
                  TRKORN(IDIR)=TRKORD(IDIR)
                ENDDO
              ELSE
                IF(NSCAN .EQ. 2) THEN
                  IF(IPER(ISDIR) .EQ. 1) THEN
                    ANGLED(ISDIR)=-ANGLED(ISDIR)
                  ELSE IF(IPER(JSDIR) .EQ. 1) THEN
                    DO IPTN=1,NBPTS
                      DO IDIR=1,NDIM
                        TRKORI(IDIR)=TRKORN(IDIR)
                        ANGLES(IDIR)=ANGLEN(IDIR)
                      ENDDO
*----
*  Find next surface intersection track crosses geometry
*----  
                      IRLCA=NXTLCA(IPRINL,ITST  ,NDIM  ,MAXMSH,LMAXT,
     >                             NUCELL,TRKORI,ANGLES,DGMESH,
     >                             NBCOR ,NBSINT,ICINT ,DCINT)
                      
*----
*  Define position of cyclic starting point
*----
                      DO ISINT=1,NBSINT
                        DO IDIR=1,NDIM
                          TRKORN(IDIR)=TRKORI(IDIR)
     >                                +DCINT(ISINT)*ANGLES(IDIR)
                        ENDDO
                      ENDDO
                      ISDIRX=ABS(ICINT(0,NBSINT))
                      IF(ISDIRX .EQ. JSDIR) THEN 
                        DO IDIR=1,NDIM
                          ANGLED(IDIR)=ANGLES(IDIR)
                        ENDDO
                        ANGLED(JSDIR)=-ANGLED(JSDIR)
                        GO TO 125
                      ENDIF
                      IF(IPER(ISDIRX) .EQ. 1) THEN
                        ANGLEN(ISDIRX)=-ANGLEN(ISDIRX)
                      ENDIF
                    ENDDO
 125                CONTINUE
                  ENDIF                 
                ENDIF
                DO IDIR=1,NDIM
                  ANGLES(IDIR)=ANGLED(IDIR)
                  ANGLEN(IDIR)=ANGLED(IDIR)
                  TRKORD(IDIR)=TRKORN(IDIR)
                  TRKORI(IDIR)=TRKORD(IDIR)
                  TRKORN(IDIR)=TRKORD(IDIR)
                ENDDO
              ENDIF
              ISTART=0
            ENDIF
*----
*  End of tracking for this direction
*----
 105        CONTINUE
          ENDDO
*----
*  End of tracking for this set of angles
*----
        ENDDO
*----
*  End of tracking
*----
      ENDDO
*----
*  Tracking complete
*----
      CALL RLSARA(RBASE(DRAPIN))
      CALL RLSARA(IBASE(ITPIN))
      CALL RLSARA(RBASE(DCSINT))
      CALL RLSARA(IBASE(ICSINT))
      CALL RLSARA(RBASE(DCMESH))
      CALL RLSARA(IBASE(IDREG))
      CALL RLSARA(IBASE(IDSUR))
*----
*  Compute DVNOR and DSNOR by comparing ratio of analytical
*  to numerically integrated volume or surfaces.
*----
      NBVERR=0
      DSVERR=DZERO
      DMVERR=DZERO
      DAVERR=DZERO
      NBV0=0
      DO IREG=1,NFREG
        IF(IPRINT .GE. 20 ) THEN
          WRITE(IOUT,6030) IREG,
     >    SURVOL(IREG),DVNOR(IREG)
        ENDIF
        IF(DVNOR(IREG) .EQ. DZERO) THEN
          IF(IPRINT .GE. 10) THEN
            WRITE(IOUT,9000) NAMSBR,IREG,IANGL
          ENDIF
          DVNOR(IREG)=DONE
          NBV0=NBV0+1
        ELSE
          DVNOR(IREG)=SURVOL(IREG)
     >                     /DVNOR(IREG)
          NBVERR=NBVERR+1
        ENDIF
        DCERR=100.0D0*(DONE-DVNOR(IREG))
        DMVERR=MAX(DMVERR,ABS(DCERR))
        DSVERR=DSVERR+DCERR*DCERR
        DAVERR=DAVERR+DCERR
      ENDDO
      IF(NBV0 .GT. 0) THEN
        WRITE(IOUT,9002) NAMSBR,NBV0,IANGL
      ENDIF
      DSVERR=SQRT(DSVERR/DBLE(NBVERR))
      DAVERR=DAVERR/DBLE(NBVERR)
      NBSERR=0
      DSSERR=DZERO
      DMSERR=DZERO
      DASERR=DZERO
      NBS0=0
      DO ISUR=1,NFSUR
        IF(IPRINT .GE. 20 ) THEN
          WRITE(IOUT,6031) ISUR,
     >    SURVOL(-ISUR),DSNOR(ISUR)
        ENDIF
        IF(DSNOR(ISUR) .EQ. DZERO) THEN
          IF(IPRINT .GE. 10) THEN
            WRITE(IOUT,9001) NAMSBR,-ISUR
          ENDIF
          NBS0=NBS0+1
          DSNOR(ISUR)=DONE
        ELSE
          DSNOR(ISUR)=SURVOL(-ISUR)
     >                /DSNOR(ISUR)
          NBSERR=NBSERR+1
        ENDIF
        DCERR=100.0D0*(DONE-DSNOR(ISUR))
        DMSERR=MAX(DMSERR,ABS(DCERR))
        DSSERR=DSSERR+DCERR*DCERR
        DASERR=DASERR+DCERR
      ENDDO
      IF(NBS0 .GT. 0) THEN
        WRITE(IOUT,9003) NAMSBR,NBS0
      ENDIF
      DSSERR=SQRT(DSSERR/DBLE(NBSERR))
      DASERR=DASERR/DBLE(NBSERR)
*----
*  Processing finished:
*  print track normalization vector.
*  and routine closing output header if required
*  and return
*----
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6005) DSVERR,DMVERR,DAVERR
        IF(IPRINT .GE. 10) THEN
          DO IREG=1,NFREG
            WRITE(IOUT,6010) IREG,SURVOL(IREG)
            WRITE(IOUT,6012) DVNOR(IREG),
     >      100.0D0*(DONE-DVNOR(IREG))
          ENDDO
        ENDIF
        WRITE(IOUT,6006) DSSERR,DMSERR,DASERR
        IF(IPRINT .GE. 10) THEN
          DO ISUR=1,NFSUR
            WRITE(IOUT,6011) -ISUR,SURVOL(-ISUR)
            WRITE(IOUT,6012) DSNOR(ISUR),
     >      100.0D0*(DONE-DSNOR(ISUR))
          ENDDO
        ENDIF
        WRITE(IOUT,6001) NAMSBR
      ENDIF
*----
*  Save track normalisation vector
*----
      IF(IPRINT .LT. 0) THEN
        ICLS=KDRCLS(ILOUT,1)
      ENDIF
      IF(MAXSGL .EQ. 0) THEN
        WRITE(IOUT,9004) NAMSBR
        MAXSGL=LINMAX
      ENDIF
      RETURN 
*----
*  Output formats
*----
 1010 FORMAT(1X,'#',I10,' IANG=',I10,' LEN=',I10)
 1011 FORMAT(1P,(1X,E15.3,1X,I10))
 6000 FORMAT('(* Output from --',A6,'-- follows ')
 6001 FORMAT('   Output from --',A6,'-- completed *)')
 6005 FORMAT(' Global RMS, maximum and average errors (%) ',
     >       'on region volumes :',3(2X,F10.5))
 6006 FORMAT(' Global RMS, maximum and average errors (%) ',
     >       'on surface areas  :',3(2X,F10.5))
 6010 FORMAT(' Normalisation factors and relative errors (%) ',
     >       'for region ',I8,' with volume ',F19.10)
 6011 FORMAT(' Normalisation factors and relative error (%) ',
     >       'for surface ',I8,' with area   ',F19.10)
 6012 FORMAT((2X,F15.10,2X,F10.5))
 6020 FORMAT('Line',I10.10,'={',2(I10,','),
     >       F18.10,',',F18.10,',',F18.10,'};')
 6021 FORMAT(6(I10,:,',',9X))
 6022 FORMAT(6(F18.10,:,','))
 6023 FORMAT(A7,I10.10,'={')
 6024 FORMAT(18X,'};')
 6025 FORMAT('Line',I10.10,'={',5(I10,','),
     >       F18.10,',',F18.10,'};')
 6026 FORMAT('Line',I10.10,'={',I10,',',I10,'};')
 6027 FORMAT('Problem with Line',I10.10,'={',5(I10,','),
     >       I10,',',F18.10,',',I10,',',F18.10,'};')
 6030 FORMAT(' Normalization volumes  =',I10,1P,2D20.10)
 6031 FORMAT(' Normalization surfaces =',I10,1P,2D20.10)
 6039 FORMAT(1X)
 6040 FORMAT('Track',I10.10,'={',4(I10,','),
     >F15.8,',',F15.8,'};')
 6041 FORMAT('Torig',I10.10,'={',2(F15.8,','),F15.8,'};')
 6042 FORMAT('Tdire',I10.10,'={',8(F15.8,','),F15.8,'};')
 6043 FORMAT('Tstrt',I10.10,'={',F15.8,'};')
 6050 FORMAT(' Direction = ',I5,2X,'Scan =',4I5,2x,'Weight =',F20.15/
     >' Directions and starting point =',6F20.15)
 8000 FORMAT('% Dimensions pour fichier NXTPLines'/
     >'Nb regions     =',I10/'Nb surfaces    =',I10/ 
     >'Nb dimensions  =',I10/'Nb directions  =',I10)
 8001 FORMAT(6I10)
 8002 FORMAT(7F18.10)
 8003 FORMAT(I10/6(I10,F18.10))
 8004 FORMAT('% Contenu des 3 enregistrements pour chaque ligne:'/
     >'% 1- Ligne (I), direction (I), plan (I), U (I), V (I)'/
     >'% 2- Origine (R(NDIM)), direction (R(NDIM)),', 
     >' distance du premier point (R))'/
     >'% 3- Nombre de segments, numero, longueur')  
 9000 FORMAT(1X,'***** Warning in ',A6,'*****'/
     >       7X,'For region ',I8,
     >       1X,'no crossing by angle ',I8)
 9001 FORMAT(1X,'***** Warning in ',A6,'*****'/
     >       7X,'For surface ',I8,
     >       1X,'no crossing by any angle ')
 9002 FORMAT(1X,'***** Warning in ',A6,'*****'/
     >       7X,I8,' regions not tracked for direction  ',I8)
 9003 FORMAT(1X,' ***** Warning in ',A6,'*****'/
     >       7X,I8,' surfaces not tracked for direction ',I8)
 9004 FORMAT(1X,' ***** Warning in ',A6,'*****'/
     >       7X,' no line segments detected in tracking ')
 9100 FORMAT(1X,' ***** Error in ',A6,'***** for line ',I8/
     >       7X,'Positions (current and reference ) =',1P,2D20.14/
     >       7X,'Relative error = ',D20.14)
      END 
