*DECK PIJXLB
      SUBROUTINE PIJXLB(IPTRK,IPRT,NGRP,NREG,NANI,NBMIX,NPSYS,NRENOR,
     > LNORM,LEAKSW,MATCOD,VOLUME,XSSIGT,XSSIGW,NELPIJ,PIJ)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the collision probabilities in EXCELL without producing
* a tracking file.  Interface subroutine used to implement double
* heterogeneity capabilities.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   pointer to the tracking (L_TRACK signature).
* IPRT    print flag (equal to zero for no print).
* NGRP    number of energy groups.
* NREG    total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NANI    number of Legendre orders (usually equal to one).
* NBMIX   number of mixtures (NBMIX=max(MATCOD(i))).
* NPSYS   index set to zero if a group is not to be processed. Usually,
*         NPSYS(I)=I.
* NRENOR  normalization scheme for PIJ matrices.
* LNORM   logical switch for removing leakage from collision
*         probabilities and keeping the PIS information.
* LEAKSW  leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* MATCOD  index-number of the mixture type assigned to each volume.
* VOLUME  volumes.
* XSSIGT  total macroscopic cross sections ordered by mixture.
* XSSIGW  P0 within-group scattering macroscopic cross sections
*         ordered by mixture.
* NELPIJ  number of elements in symmetrized pij matrix.
*
*Parameters: output
* PIJ     reduced and symmetrized collision probabilities.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      LOGICAL         LEAKSW,LNORM
      INTEGER         IPTRK,IPRT,NGRP,NREG,NANI,NBMIX,NPSYS(NGRP),
     >                NRENOR,MATCOD(NREG),NELPIJ
      REAL            VOLUME(NREG),XSSIGT(0:NBMIX,NGRP),
     >                XSSIGW(0:NBMIX,NANI,NGRP),PIJ(NELPIJ,NGRP)
*----
*  LOCAL VARIABLES
*----
      INTEGER IPAR(8)
      REAL BASE(1)
      COMMON IBASE(1)
      EQUIVALENCE(BASE(1),IBASE(1))
*
      CALL LCMSIX(IPTRK,'BIHET',1)
      CALL LCMGET(IPTRK,'PARAM',IPAR)
      IR1=IPAR(1)
      IR2=IPAR(2)
      NREG2=IPAR(3)
      NG=IPAR(4)
      NSMAX=IPAR(5)
      MICRO=IPAR(7)
      IQUA10=IPAR(8)
      CALL SETARA(IBASE,NG,INS)
      CALL SETARA(IBASE,NREG2,IIBI)
      CALL SETARA(BASE,NG*(1+NSMAX),IRS)
      CALL SETARA(BASE,NG*IR2,IFRACT)
      CALL SETARA(BASE,NG*NSMAX,IVOLK)
      CALL SETARA(BASE,NREG2,IVOL2)
      CALL LCMGET(IPTRK,'NS',IBASE(INS))
      CALL LCMGET(IPTRK,'IBI',IBASE(IIBI))
      CALL LCMGET(IPTRK,'RS',BASE(IRS))
      CALL LCMGET(IPTRK,'FRACT',BASE(IFRACT))
      CALL LCMGET(IPTRK,'VOLK',BASE(IVOLK))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL2))
      CALL LCMSIX(IPTRK,' ',2)
*
      CALL SETARA(BASE,NREG*NREG*NGRP,IPIJ2)
      CALL SETARA(BASE,NREG,ISIGT)
      CALL SETARA(BASE,NREG,ISIGW)
      CALL SETARA(BASE,NREG2*NGRP,IRRR)
      CALL SETARA(BASE,NREG2*NGRP,ISTILD)
      CALL SETARA(BASE,NG*NSMAX*NREG2*NGRP,IQKD)
*
      NELPIJ2=NREG2*(NREG2+1)/2
      CALL SETARA(BASE,NSMAX*NSMAX,IQKN)
      CALL SETARA(BASE,NELPIJ2*NGRP,IPIL)
      CALL SETARA(BASE,(IR2+1)*NGRP,ISIGT2)
      CALL SETARA(BASE,(IR2+1)*NANI*NGRP,ISIGW2)
      CALL XDRSET(BASE(ISIGT2),(IR2+1)*NGRP,0.0)
      CALL XDRSET(BASE(ISIGW2),(IR2+1)*NANI*NGRP,0.0)
      DO 20 IGRP=1,NGRP
      DO 10 I=1,NREG
   10 BASE(ISIGT+I-1)=XSSIGT(MATCOD(I),IGRP)
*
      CALL SYBHT1(IR1,IR2,NREG,BASE(ISIGT),NREG2,NG,NSMAX,MICRO,
     > IQUA10,IBASE(INS),IBASE(IIBI),BASE(IRS),BASE(IFRACT),
     > BASE(IVOLK),BASE(IPIJ2+NREG*NREG*(IGRP-1)),
     > BASE(IRRR+NREG2*(IGRP-1)),BASE(ISTILD+NREG2*(IGRP-1)),
     > BASE(IQKD+NSMAX*NSMAX*(IGRP-1)),BASE(IQKN))
      DO 20 I=1,NREG2
      IBM=IBASE(IIBI+I-1)
   20 BASE(ISIGT2+(IR2+1)*(IGRP-1)+IBM)=BASE(ISTILD+I-1)
      CALL RLSARA(BASE(IQKN))
      CALL RLSARA(BASE(IRS))
*
      CALL PIJXL3(IPTRK,IPRT,NGRP,NANI,IR2,NPSYS,NRENOR,LNORM,LEAKSW,
     > BASE(ISIGT2),BASE(ISIGW2),NELPIJ2,BASE(IPIL))
      CALL RLSARA(BASE(ISIGW2))
      CALL RLSARA(BASE(ISIGT2))
*
      CALL SETARA(BASE,NREG2*NREG2,IPILSQ)
      DO 50 IGRP=1,NGRP
      IC=0
      DO 30 I=1,NREG2
      DO 30 J=1,I
      IC=IC+1
      PIL=BASE(IPIL+NELPIJ2*(IGRP-1)+IC-1)
      BASE(IPILSQ+NREG2*(J-1)+I-1)=PIL/BASE(IVOL2+I-1)
   30 BASE(IPILSQ+NREG2*(I-1)+J-1)=PIL/BASE(IVOL2+J-1)
      CALL SYBHT2(IR1,IR2,NREG,NREG2,NG,NSMAX,IBASE(INS),IBASE(IIBI),
     > BASE(IFRACT),BASE(IVOLK),BASE(IPIJ2+NREG*NREG*(IGRP-1)),
     > BASE(IRRR+NREG2*(IGRP-1)),BASE(ISTILD+NREG2*(IGRP-1)),
     > BASE(IQKD+NSMAX*NSMAX*(IGRP-1)),BASE(IPILSQ))
      DO 40 I=1,NREG
      BASE(ISIGT+I-1)=XSSIGT(MATCOD(I),IGRP)
   40 BASE(ISIGW+I-1)=XSSIGW(MATCOD(I),1,IGRP)
      CALL SYBWIJ(NREG,NREG,BASE(ISIGT),BASE(ISIGW),
     >            BASE(IPIJ2+NREG*NREG*(IGRP-1)))
      IC=0
      DO 50 I=1,NREG
      DO 50 J=1,I
      IC=IC+1
      PIJSQ=BASE(IPIJ2+NREG*NREG*(IGRP-1)+NREG*(I-1)+J-1)
   50 PIJ(IC,IGRP)=PIJSQ*VOLUME(J)
      CALL RLSARA(BASE(IPILSQ))
      CALL RLSARA(BASE(IPIL))
      CALL RLSARA(BASE(IQKD))
      CALL RLSARA(BASE(ISTILD))
      CALL RLSARA(BASE(IRRR))
      CALL RLSARA(BASE(ISIGW))
      CALL RLSARA(BASE(ISIGT))
      CALL RLSARA(BASE(IPIJ2))
      CALL RLSARA(BASE(IVOL2))
      CALL RLSARA(BASE(IVOLK))
      CALL RLSARA(BASE(IFRACT))
      CALL RLSARA(IBASE(IIBI))
      CALL RLSARA(IBASE(INS))
      RETURN
      END
