*DECK SHIPIJ
      SUBROUTINE SHIPIJ (IPSYS,NPSYS,IPTRK,IFTRAK,CDOOR,IMPX,NBM,NREG,
     1 NGRO,MAT,VOL,LEAKSW,NRES,IRES,SIG0,SIG1,SIG2,DIAG,IMODE,TITR,
     2 PSTAR,PSGAR,PIJ,VST,SIGT,SIGW,PIJSC)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the escape matrix (or reduced pij matrix) using
* standard collision probabilities.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSYS   pointer to the pij (L_PIJ signature).
* NPSYS   index array pointing to the IPSYS list component corresponding
*         to each energy group. Set to zero if a group is not to be
*         processed. Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking. (L_TRACK signature).
* IFTRAK  unit number of the sequential binary tracking file.
* CDOOR   name of the geometry/solution door.
* IMPX    print flag (equal to zero for no print).
* NBM     number of mixtures.
* NREG    total number of regions.
* NGRO    number of energy groups.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* LEAKSW  leakage flag (=.true. if leakage is present on the outer
*         surface).
* NRES    number of resonant mixtures.
* IRES    resonant mixture number assigned to each mixture.
* SIG0    total macroscopic cross sections of the resonant materials
*         in each mixture.
* SIG1    total macroscopic cross sections of the light materials in
*         each mixture.
* SIG2    transport correction in each mixture.
* DIAG    diagonal approximation flag (if DIAG=.true., compute a
*         diagonal approximation of PSTAR).
* IMODE   type of calculation: =1: compute escape matrix; =2: compute
*         reduced pij matrix.
* TITR    title.
*
*Parameters: output
* PSTAR   escape matrix (imode=1) or reduced pij matrix (imode=2).
*
*Parameters: scratch
* PSGAR
* PIJ
* VST
* SIGT
* SIGW
* PIJSC
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      LOGICAL LEAKSW,DIAG
      CHARACTER CDOOR*12,TITR*72
      INTEGER IPSYS,NPSYS(NGRO),IPTRK,IFTRAK,IMPX,NBM,NREG,NGRO,
     > MAT(NREG),NRES,IRES(NBM),IMODE
      REAL VOL(NREG),SIG0(NBM,NGRO),SIG1(NBM,NGRO),SIG2(NBM,NGRO),
     > PSTAR(NRES,NRES,NGRO),PSGAR(NRES,2*NRES),PIJ(NREG,NREG),
     > VST(NRES),SIGT(0:NBM),SIGW(0:NBM),PIJSC(NREG,2*NREG)
*----
*  LOCAL VARIABLES
*----
      LOGICAL LNORM
*----
*  SELECT THE MACROSCOPIC CROSS SECTIONS.
*----
      JPSYS=LCMLID(IPSYS,'GROUP',NGRO)
      SIGT(0)=0.0
      SIGW(0)=0.0
      DO 20 LLL=1,NGRO
      IF(NPSYS(LLL).NE.0) THEN
         DO 10 IBM=1,NBM
         SIGT(IBM)=SIG0(IBM,LLL)+SIG1(IBM,LLL)-SIG2(IBM,LLL)
   10    SIGW(IBM)=-SIG2(IBM,LLL)
         KPSYS=LCMDIL(JPSYS,LLL)
         CALL LCMPUT(KPSYS,'DRAGON-TXSC',NBM+1,2,SIGT(0))
         CALL LCMPUT(KPSYS,'DRAGON-S0XSC',NBM+1,2,SIGW(0))
      ENDIF
   20 CONTINUE
*----
*  COMPUTE THE COLLISION PROBABILITIES.
*----
      NANI=1
      IPIJK=1
      KNORM=1
      LNORM=.FALSE.
      CALL DOORPV (CDOOR,JPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRO,NREG,NBM,
     > NANI,MAT,VOL,KNORM,IPIJK,LEAKSW,LNORM,TITR)
*----
*  COMPUTE THE GROUPWISE PSTAR MATRICES.
*----
      DO 50 I=1,NRES
   50 VST(I)=0.0
      DO 60 I=1,NREG
      IF(MAT(I).EQ.0) GO TO 60
      IND=IRES(MAT(I))
      IF(IND.EQ.0) GO TO 60
      VST(IND)=VST(IND)+VOL(I)
   60 CONTINUE
      DO 180 LLL=1,NGRO
      IF((NPSYS(LLL).NE.0).AND.(IMODE.EQ.1)) THEN
*        COMPUTE THE ESCAPE MATRIX.
         KPSYS=LCMGIL(JPSYS,LLL)
         CALL LCMGET(KPSYS,'DRAGON-PCSCT',PIJ)
         DO 80 I=1,NRES
         DO 70 J=1,2*NRES
   70    PSGAR(I,J)=0.0
   80    PSGAR(I,NRES+I)=1.0
         ILIMIT=0
         DO 90 IBM=1,NBM
         IF(SIG0(IBM,LLL).GE.1.0E10) THEN
            ILIMIT=ILIMIT+1
            IND=IBM
         ENDIF
   90    CONTINUE
         IF(ILIMIT.EQ.0) THEN
            DO 110 I=1,NREG
            IF(MAT(I).EQ.0) GO TO 110
            IND=IRES(MAT(I))
            IF(IND.EQ.0) GO TO 110
            DO 100 J=1,NREG
            JBM=MAT(J)
            IF(JBM.EQ.0) GO TO 100
            JND=IRES(JBM)
            IF(JND.EQ.0) GO TO 100
            IF(DIAG.AND.(JND.NE.IND)) GO TO 100
            PSGAR(IND,JND)=PSGAR(IND,JND)+PIJ(I,J)*VOL(I)/VST(IND)
            PSGAR(IND,JND+NRES)=PSGAR(IND,JND+NRES)-PIJ(I,J)*
     1      SIG0(JBM,LLL)*VOL(I)/VST(IND)
  100       CONTINUE
  110       CONTINUE
            IF(NRES.EQ.1) THEN
               PSTAR(1,1,LLL)=PSGAR(1,2)/PSGAR(1,1)
            ELSE
               CALL ALSB(NRES,NRES,PSGAR,IRR,NRES)
               IF(IRR.NE.0) CALL XABORT('SHIPIJ: UNABLE TO SOLVE THE '
     >         //'LINEAR SYSTEM.')
               DO 120 I=1,NRES
               DO 120 J=1,NRES
  120          PSTAR(I,J,LLL)=PSGAR(I,NRES+J)
            ENDIF
         ELSE IF(ILIMIT.EQ.1) THEN
*           COMPUTE PSTAR DIRECTLY FROM THE FUEL TO NON-FUEL CP.
            IF(NRES.GT.1) CALL XABORT('SHIPIJ: THE NUMBER OF RESONANT'
     >      //' REGIONS IS GREATER THAN ONE.')
            SUM=0.0
            DO 140 I=1,NREG
            IBM=MAT(I)
            IF(IBM.EQ.0) GO TO 140
            IF(IRES(IBM).NE.0) GO TO 140
            DO 130 J=1,NREG
            IF(MAT(J).EQ.0) GO TO 130
            IF(IRES(MAT(J)).EQ.0) GO TO 130
            SUM=SUM+(SIG1(IBM,LLL)-SIG2(IBM,LLL))*PIJ(I,J)*VOL(I)
  130       CONTINUE
  140       CONTINUE
            PSTAR(1,1,LLL)=SUM*SIGT(IND)/VST(1)
         ELSE
            CALL XABORT('SHIPIJ: MORE THAN ONE MIXTURE HAVE AN INFINI'
     >      //'TE TOTAL CROSS SECTION.')
         ENDIF
      ELSE IF((NPSYS(LLL).NE.0).AND.(IMODE.EQ.2)) THEN
*        COMPUTE THE REDUCED PIJ MATRIX.
         KPSYS=LCMDIL(JPSYS,LLL)
         CALL LCMGET(KPSYS,'DRAGON-PCSCT',PIJ)
         DO 150 I=1,NRES
         DO 150 J=1,NRES
  150    PSTAR(I,J,LLL)=0.0
         DO 170 I=1,NREG
         IF(MAT(I).EQ.0) GO TO 170
         IND=IRES(MAT(I))
         IF(IND.EQ.0) GO TO 170
         DO 160 J=1,NREG
         JBM=MAT(J)
         IF(JBM.EQ.0) GO TO 160
         JND=IRES(JBM)
         IF(JND.EQ.0) GO TO 160
         IF(DIAG.AND.(JND.NE.IND)) GO TO 160
         PSTAR(IND,JND,LLL)=PSTAR(IND,JND,LLL)+PIJ(I,J)*VOL(I)/VST(IND)
  160    CONTINUE
  170    CONTINUE
      ENDIF
  180 CONTINUE
      RETURN
      END
