*DECK SHIPST
      SUBROUTINE SHIPST (IPSYS,NPSYS,IPTRK,IFTRAK,CDOOR,IMPX,NBM,NREG,
     1 NGRO,MAT,VOL,LEAKSW,NRES,IRES,SIG0,SIG1,SIG2,DIAG,IMODE,TITR,
     2 PSTAR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the escape matrix (or reduced pij matrix).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSYS   pointer to the pij (L_PIJ signature).
* NPSYS   index array pointing to the IPSYS list component corresponding
*         to each energy group. Set to zero if a group is not to be
*         processed. Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking. (L_TRACK signature).
* IFTRAK  unit number of the sequential binary tracking file.
* CDOOR   name of the geometry/solution module.
* IMPX    print flag (equal to zero for no print).
* NBM     number of mixtures.
* NREG    total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NGRO    number of energy groups.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* LEAKSW  leakage flag (=.true. if leakage is present on the outer
*         surface).
* NRES    number of resonant mixtures.
* IRES    resonant mixture number assigned to each mixture.
* SIG0    total macroscopic cross sections of the resonant materials
*         in each mixture.
* SIG1    total macroscopic cross sections of the light materials in
*         each mixture.
* SIG2    transport correction in each mixture.
* DIAG    diagonal approximation flag (if DIAG=.true., compute a
*         diagonal approximation of PSTAR).
* IMODE   type of calculation: =1: compute escape matrix; =2: compute
*         reduced pij matrix.
* TITR    title.
*
*Parameters: output
* PSTAR   escape matrix (imode=1) or reduced pij matrix (imode=2).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER CDOOR*12,TITR*72
      LOGICAL LEAKSW,DIAG
      INTEGER IPSYS,NPSYS(NGRO),IPTRK,IFTRAK,IMPX,NBM,NREG,NGRO,
     1 MAT(NREG),NRES,IRES(NBM),IMODE
      REAL VOL(NREG),SIG0(NBM,NGRO),SIG1(NBM,NGRO),SIG2(NBM,NGRO),
     1 PSTAR(NRES,NRES,NGRO)
*----
*  LOCAL VARIABLES
*----
      COMMON BASE(1)
*
      IMPX2=MAX(0,IMPX-4)
      CALL SETARA (BASE,2*NRES*NRES,IPSGAR)
      CALL SETARA (BASE,NREG*NREG,IPIJ)
      CALL SETARA (BASE,NRES,IVST)
      CALL SETARA (BASE,NBM+1,ISIGT)
      CALL SETARA (BASE,NBM+1,ISIGW)
      CALL SETARA (BASE,2*NREG*NREG,IPIJSC)
      CALL SHIPIJ (IPSYS,NPSYS,IPTRK,IFTRAK,CDOOR,IMPX2,NBM,NREG,NGRO,
     1 MAT,VOL,LEAKSW,NRES,IRES,SIG0,SIG1,SIG2,DIAG,IMODE,TITR,PSTAR,
     2 BASE(IPSGAR),BASE(IPIJ),BASE(IVST),BASE(ISIGT),BASE(ISIGW),
     3 BASE(IPIJSC))
      CALL RLSARA (BASE(IPIJSC))
      CALL RLSARA (BASE(ISIGW))
      CALL RLSARA (BASE(ISIGT))
      CALL RLSARA (BASE(IVST))
      CALL RLSARA (BASE(IPIJ))
      CALL RLSARA (BASE(IPSGAR))
      IF(IMPX.GE.7) THEN
         DO 20 LLL=1,NGRO
         IF(NPSYS(LLL).NE.0) THEN
            IF(IMODE.EQ.1) THEN
               WRITE (6,'(31H SHIPST: ESCAPE MATRIX IN GROUP,I5,1H:/)')
     1         LLL
            ELSE IF(IMODE.EQ.2) THEN
               WRITE (6,'(36H SHIPST: REDUCED PIJ MATRIX IN GROUP,I5,
     1         1H:/)') LLL
            ENDIF
            WRITE (6,30) (J,J=1,NRES)
            DO 10 I=1,NRES
            WRITE (6,40) I,(PSTAR(I,J,LLL),J=1,NRES)
   10       CONTINUE
            WRITE (6,'(//)')
         ENDIF
   20    CONTINUE
      ENDIF
      RETURN
   30 FORMAT (11X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,
     1 2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,5X,2HJ=,I4,:,
     2 5X,2HJ=,I4,:,5X,2HJ=,I4)
   40 FORMAT (3H I=,I4,2H: ,1P,11E11.3/(9X,11E11.3))
      END
