*DECK SNF
      SUBROUTINE SNF (KPSYS,IPTRK,IFTRAK,IMPX,IDIR,NREG,NBMIX,NUN,MAT,
     1 VOL,KEYFLX,FUNKNO,SUNKNO,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solve one-group transport equation for fluxes using the discrete
* ordinates (SN) method.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* KPSYS   pointer to the assembly matrices.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  not used.
* IMPX    print flag (equal to zero for no print).
* IDIR    not used.
* NREG    total number of regions for which specific values of the
*         neutron flux and reactions rates are required.
* NBMIX   number of mixtures.
* NUN     total number of unknowns in vectors SUNKNO and FUNKNO.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KEYFLX  position of averaged flux elements in FUNKNO vector.
* SUNKNO  input source vector.
* TITR    title.
*
*Parameters: input/output
* FUNKNO  unknown vector.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER   TITR*72
      INTEGER     KPSYS,IPTRK,IFTRAK,IMPX,IDIR,NREG,NBMIX,NUN,MAT(NREG),
     >            KEYFLX(NREG)
      REAL        VOL(NREG),FUNKNO(NUN),SUNKNO(NUN)
*----
*  LOCAL VARIABLES
*----
      PARAMETER  (IUNOUT=6,NSTATE=40,MAXIT=30,EPSINR=1.0E-5,ICL1=3,
     1            ICL2=3)
      INTEGER     IPAR(NSTATE),NCODE(6)
      REAL        ZCODE(6)
      LOGICAL     LFIXUP,LDSA
      DOUBLE PRECISION F1,F2,R1,R2,DMU
      COMMON      BASE(1)
*----
*  RECOVER SN SPECIFIC PARAMETERS.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IPAR)
      IF(NREG.NE.IPAR(1)) CALL XABORT('SNF: INVALID NREG ON LCM.')
      IF(NUN.NE.IPAR(2)) CALL XABORT('SNF: INVALID NUN ON LCM.')
      ITYPE=IPAR(6)
      IELEM=IPAR(8)
      L4=IPAR(11)
      LX=IPAR(12)
      LY=IPAR(13)
      NLF=IPAR(15)
      ISCAT=IPAR(16)
      LFIXUP=IPAR(18).EQ.1
      LDSA=IPAR(19).EQ.1
      IGAV=IPAR(30)
      IF(IMPX.GT.2) THEN
         WRITE(IUNOUT,'(//33H N E U T R O N    S O U R C E S :)')
         WRITE(IUNOUT,'(1P,6(5X,E15.7))') (SUNKNO(KEYFLX(I)),I=1,NREG)
      ENDIF
      CALL SETARA(BASE,NUN,IQEXT)
*----
*  WITHIN-GROUP SCATTERING REMOVAL FROM TOTAL CROSS SECTION.
*----
      CALL LCMLEN(KPSYS,'DRAGON-TXSC',ILONG,ITYLCM)
      IF(ILONG.NE.NBMIX+1) CALL XABORT('SNF: INVALID TXSC LENGTH.')
      CALL SETARA(BASE,ILONG,ISGAR)
      CALL LCMLEN(KPSYS,'DRAGON-S0XSC',ILONG,ITYLCM)
      NANI=ILONG/(NBMIX+1)
      CALL SETARA(BASE,ILONG,ISGAS)
      CALL LCMGET(KPSYS,'DRAGON-TXSC',BASE(ISGAR))
      CALL LCMGET(KPSYS,'DRAGON-S0XSC',BASE(ISGAS))
*----
*  INNER ITERATION LOOP FOR ONE-GROUP TRANSPORT EQUATION.
*----
      CALL SETARA(BASE,NUN,IOLD1)
      CALL SETARA(BASE,NUN,IOLD2)
      CALL XDRSET(BASE(IOLD2),NUN,0.0)
      ITER=0
   10 ITER=ITER+1
      IF(ITER.GT.MAXIT) THEN
         WRITE(IUNOUT,'(43H SNF: MAXIMUM NUMBER OF ONE-SPEED ITERATION,
     1   9H REACHED.)')
         GO TO 200
      ENDIF
*----
*  UPDATE THE FIXED SOURCE AND COMPUTE THE FLUX.
*----
      DO 20 I=1,NUN
      BASE(IQEXT+I-1)=SUNKNO(I)
      BASE(IOLD1+I-1)=BASE(IOLD2+I-1)
   20 BASE(IOLD2+I-1)=FUNKNO(I)
      IF(ITYPE.EQ.2) THEN
         NSCT=ISCAT
         IF(IELEM*NSCT*NREG.GT.NUN) CALL XABORT('SNF: QEXT OVERFLOW(1)')
         DO 40 IR=1,NREG
         IBM=MAT(IR)
         IF(IBM.EQ.0) GO TO 40
         DO 30 IL=0,MIN(ISCAT,NANI)-1
         FACT=REAL(2*IL+1)
         DO 30 IEL=1,IELEM
         IND=(IR-1)*NSCT*IELEM+IELEM*IL+IEL
   30    BASE(IQEXT+IND-1)=BASE(IQEXT+IND-1)+
     1                     FACT*BASE(ISGAS+IL*(NBMIX+1)+IBM)*FUNKNO(IND)
   40    CONTINUE
         CALL LCMIOF(IPTRK,'U',IU)
         CALL LCMIOF(IPTRK,'W',IW)
         CALL LCMIOF(IPTRK,'PL',IPL)
         CALL LCMGET(IPTRK,'NCODE',NCODE)
         CALL LCMGET(IPTRK,'ZCODE',ZCODE)
         CALL SNFT1P(NREG,NBMIX,IELEM,NLF,NSCT,BASE(IU),BASE(IW),
     1   BASE(IPL),MAT,VOL,BASE(ISGAR),NCODE,ZCODE,BASE(IQEXT),LFIXUP,
     2   FUNKNO)
      ELSE IF(ITYPE.EQ.3) THEN
         NSCT=(ISCAT/2)*(ISCAT/2+1)+(ISCAT+1)*MOD(ISCAT,2)/2
         IF(NSCT*NREG.GT.NUN) CALL XABORT('SNF: QEXT OVERFLOW(2)')
         DO 60 IR=1,NREG
         IBM=MAT(IR)
         IF(IBM.EQ.0) GO TO 60
         IOF=0
         DO 50 IL=0,MIN(ISCAT,NANI)-1
         FACT=REAL(2*IL+1)
         DO 50 IM=0,IL
         IF(MOD(IL+IM,2).EQ.1) GO TO 50
         IOF=IOF+1
         IND=(IR-1)*NSCT+IOF
         BASE(IQEXT+IND-1)=BASE(IQEXT+IND-1)+
     1                     FACT*BASE(ISGAS+IL*(NBMIX+1)+IBM)*FUNKNO(IND)
   50    CONTINUE
   60    CONTINUE
         M2=NLF/2
         CALL LCMLEN(IPTRK,'UPQ',NPQ,ITYLCM)
         CALL LCMIOF(IPTRK,'JOP',IJOP)
         CALL LCMIOF(IPTRK,'U',IU)
         CALL LCMIOF(IPTRK,'UPQ',IUPQ)
         CALL LCMIOF(IPTRK,'WPQ',IWPQ)
         CALL LCMIOF(IPTRK,'ALPHA',IALPHA)
         CALL LCMIOF(IPTRK,'PLZ',IPLZ)
         CALL LCMIOF(IPTRK,'PL',IPL)
         CALL LCMIOF(IPTRK,'SURF',ISURF)
         CALL LCMGET(IPTRK,'ZCODE',ZCODE)
*
         CALL SETARA(BASE,M2,IFLXB)
         CALL SETARA(BASE,NREG,IAFGL)
         CURR=0.0
         CALL SNFT1C(NREG,NBMIX,M2,NPQ,ISCAT,NSCT,BASE(IJOP),BASE(IU),
     1   BASE(IUPQ),BASE(IWPQ),BASE(IALPHA),BASE(IPLZ),BASE(IPL),MAT,
     2   VOL,BASE(ISURF),BASE(ISGAR),IGAV,BASE(IQEXT),LFIXUP,CURR,
     3   FUNKNO,BASE(IFLXB),BASE(IAFGL))
         IF(ZCODE(2).NE.0.0) THEN
           CA=CURR
           CB=1.0
           CALL SNFT1C(NREG,NBMIX,M2,NPQ,ISCAT,NSCT,BASE(IJOP),BASE(IU),
     1     BASE(IUPQ),BASE(IWPQ),BASE(IALPHA),BASE(IPLZ),BASE(IPL),MAT,
     2     VOL,BASE(ISURF),BASE(ISGAR),IGAV,BASE(IQEXT),LFIXUP,CB,
     3     FUNKNO,BASE(IFLXB),BASE(IAFGL))
           CURR=ZCODE(2)*CA/(1.0+ZCODE(2)*(CA-CB))
           CALL SNFT1C(NREG,NBMIX,M2,NPQ,ISCAT,NSCT,BASE(IJOP),BASE(IU),
     1     BASE(IUPQ),BASE(IWPQ),BASE(IALPHA),BASE(IPLZ),BASE(IPL),MAT,
     2     VOL,BASE(ISURF),BASE(ISGAR),IGAV,BASE(IQEXT),LFIXUP,CURR,
     3     FUNKNO,BASE(IFLXB),BASE(IAFGL))
         ENDIF
         CALL RLSARA(BASE(IAFGL))
         CALL RLSARA(BASE(IFLXB))
      ELSE IF(ITYPE.EQ.4) THEN
         NSCT=ISCAT
         IF(NSCT*NREG.GT.NUN) CALL XABORT('SNF: QEXT OVERFLOW(3)')
         DO 80 IR=1,NREG
         IBM=MAT(IR)
         IF(IBM.EQ.0) GO TO 80
         DO 70 IL=0,MIN(ISCAT,NANI)-1
         FACT=REAL(2*IL+1)
         IND=(IR-1)*NSCT+IL+1
   70    BASE(IQEXT+IND-1)=BASE(IQEXT+IND-1)+
     1                     FACT*BASE(ISGAS+IL*(NBMIX+1)+IBM)*FUNKNO(IND)
   80    CONTINUE
         CALL LCMIOF(IPTRK,'U',IU)
         CALL LCMIOF(IPTRK,'W',IW)
         CALL LCMIOF(IPTRK,'ALPHA',IALPHA)
         CALL LCMIOF(IPTRK,'PLZ',IPLZ)
         CALL LCMIOF(IPTRK,'PL',IPL)
         CALL LCMIOF(IPTRK,'SURF',ISURF)
         CALL LCMIOF(IPTRK,'XXX',IXXX)
         CALL LCMGET(IPTRK,'ZCODE',ZCODE)
         CURR=0.0
         CALL SETARA(BASE,NREG,IAFGL)
         CALL SETARA(BASE,NLF/2,IFLXB)
         CALL SNFT1S(NREG,NBMIX,NLF,NSCT,BASE(IU),BASE(IW),BASE(IALPHA),
     1   BASE(IPLZ),BASE(IPL),MAT,VOL,BASE(ISURF),BASE(IXXX),
     2   BASE(ISGAR),IGAV,BASE(IQEXT),LFIXUP,CURR,FUNKNO,BASE(IAFGL),
     3   BASE(IFLXB))
         IF(ZCODE(2).NE.0.0) THEN
            CA=CURR
            CB=1.0
            CALL SNFT1S(NREG,NBMIX,NLF,NSCT,BASE(IU),BASE(IW),
     1      BASE(IALPHA),BASE(IPLZ),BASE(IPL),MAT,VOL,BASE(ISURF),
     2      BASE(IXXX),BASE(ISGAR),IGAV,BASE(IQEXT),LFIXUP,CB,FUNKNO,
     3      BASE(IAFGL),BASE(IFLXB))
            CURR=ZCODE(2)*CA/(1.0+ZCODE(2)*(CA-CB))
            CALL SNFT1S(NREG,NBMIX,NLF,NSCT,BASE(IU),BASE(IW),
     1      BASE(IALPHA),BASE(IPLZ),BASE(IPL),MAT,VOL,BASE(ISURF),
     2      BASE(IXXX),BASE(ISGAR),IGAV,BASE(IQEXT),LFIXUP,CURR,FUNKNO,
     3      BASE(IAFGL),BASE(IFLXB))
         ENDIF
         CALL RLSARA(BASE(IFLXB))
         CALL RLSARA(BASE(IAFGL))
      ELSE IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6)) THEN
         NSCT=ISCAT*(ISCAT+1)/2
         IF(NSCT*NREG.GT.NUN) CALL XABORT('SNF: QEXT OVERFLOW(4)')
         IGE=0
         IF(ITYPE.EQ.6) IGE=1
         DO 100 IR=1,NREG
         IBM=MAT(IR)
         IF(IBM.EQ.0) GO TO 100
         IOF=0
         DO 90 IL=0,MIN(ISCAT,NANI)-1
         FACT=REAL(2*IL+1)
         DO 90 IM=-IL,IL
         IF(MOD(IL+IM,2).EQ.1) GO TO 90
         IOF=IOF+1
         DO 85 IEL=1,IELEM*IELEM
         IND=(IR-1)*NSCT*IELEM*IELEM+(IOF-1)*IELEM*IELEM+IEL
   85    BASE(IQEXT+IND-1)=BASE(IQEXT+IND-1)+
     1                     FACT*BASE(ISGAS+IL*(NBMIX+1)+IBM)*FUNKNO(IND)
   90    CONTINUE
  100    CONTINUE
         CALL LCMLEN(IPTRK,'DU',NPQ,ITYLCM)
         CALL LCMIOF(IPTRK,'DU',IDU)
         CALL LCMIOF(IPTRK,'DE',IDE)
         CALL LCMIOF(IPTRK,'W',IW)
         CALL LCMIOF(IPTRK,'MRM',IMRM)
         CALL LCMIOF(IPTRK,'MRMY',IMRMY)
         CALL LCMIOF(IPTRK,'DB',IDB)
         CALL LCMIOF(IPTRK,'DA',IDA)
         IF(IGE.GT.0) CALL LCMIOF(IPTRK,'DAL',IDAL)
         CALL LCMIOF(IPTRK,'PL',IPL)
         CALL LCMGET(IPTRK,'NCODE',NCODE)
         CALL LCMGET(IPTRK,'ZCODE',ZCODE)
*
         CALL SETARA(BASE,2*IELEM*LY,IXNI)
         CALL SETARA(BASE,LX*LY,IXARN)
         CALL SNFT12(IGE,LX,LY,IELEM,NBMIX,NPQ,ISCAT,NSCT,MAT,VOL,
     1   BASE(ISGAR),NCODE,ZCODE,BASE(IQEXT),LFIXUP,BASE(IDU),BASE(IDE),
     2   BASE(IW),BASE(IMRM),BASE(IMRMY),BASE(IDB),BASE(IDA),BASE(IDAL),
     3   BASE(IPL),FUNKNO,FUNKNO(L4+1),FUNKNO(L4+IELEM*LY*NPQ+1),
     4   BASE(IXNI),BASE(IXARN))
         CALL RLSARA(BASE(IXARN))
         CALL RLSARA(BASE(IXNI))
      ELSE
         CALL XABORT('SNF: TYPE OF DISCRETIZATION NOT IMPLEMENTED.')
      ENDIF
*----
*  SYNTHETIC ACCELERATION.
*----
      IF(LDSA) THEN
         CALL LCMSIX(IPTRK,'DSA',1)
         CALL LCMGET(IPTRK,'STATE-VECTOR',IPAR)
         IF(NREG.NE.IPAR(1)) CALL XABORT('SNF: INVALID NREG (DSA).')
         NUNSA=IPAR(2)
         CALL LCMSIX(IPTRK,' ',2)
         CALL SETARA(BASE,NREG,IIDL)
         CALL SETARA(BASE,NUNSA,IFUNSA)
         CALL SETARA(BASE,NUNSA,ISUNSA)
         CALL SNDSA(KPSYS,IPTRK,IFTRAK,IMPX,IDIR,NREG,NBMIX,NUN,
     1   ISCAT,MAT,VOL,KEYFLX,TITR,NUNSA,ZCODE,BASE(IOLD2),FUNKNO,
     2   BASE(IIDL),BASE(IFUNSA),BASE(ISUNSA))
         CALL RLSARA(BASE(ISUNSA))
         CALL RLSARA(BASE(IFUNSA))
         CALL RLSARA(BASE(IIDL))
      ENDIF
*----
*  VARIATIONAL ACCELERATION.
*----
      IF(MOD(ITER-1,ICL1+ICL2).GE.ICL1) THEN
         F1=0.0
         F2=0.0
         DO 170 I=1,NUN
         R1=BASE(IOLD2+I-1)-BASE(IOLD1+I-1)
         R2=FUNKNO(I)-BASE(IOLD2+I-1)
         F1=F1+R1*(R2-R1)
  170    F2=F2+(R2-R1)*(R2-R1)
         DMU=-F1/F2
         IF(DMU.GT.0.0) THEN
            DO 180 I=1,NUN
            FUNKNO(I)=BASE(IOLD2+I-1)+DMU*(FUNKNO(I)-BASE(IOLD2+I-1))
            BASE(IOLD2+I-1)=BASE(IOLD1+I-1)+DMU*(BASE(IOLD2+I-1)-
     1      BASE(IOLD1+I-1))
  180       CONTINUE
         ENDIF
      ELSE
         DMU=1.0D0
      ENDIF      
      AAA=0.0
      BBB=0.0
      DO 190 I=1,NREG
      AAA=MAX(AAA,ABS(FUNKNO(KEYFLX(I))-BASE(IOLD2+KEYFLX(I)-1)))
      BBB=MAX(BBB,ABS(FUNKNO(KEYFLX(I))))
  190 CONTINUE
      IF(IMPX.GT.1) WRITE(IUNOUT,300) ITER,AAA,BBB,DMU
      IF(AAA.LE.EPSINR*BBB) GO TO 200
      IF(ITER.EQ.1) TEST=AAA
      IF((ITER.GE.10).AND.(AAA.GT.TEST)) THEN
         WRITE(IUNOUT,'(43H SNF: UNABLE TO CONVERGE ONE-SPEED ITERATIO,
     1   3HNS.)')
         GO TO 200
      ENDIF
      GO TO 10
  200 CALL RLSARA(BASE(IOLD2))
      CALL RLSARA(BASE(IOLD1))
      CALL RLSARA(BASE(ISGAS))
      CALL RLSARA(BASE(ISGAR))
      CALL RLSARA(BASE(IQEXT))
*
      IF((IMPX.GT.0).AND.(NLF.GT.2)) WRITE(IUNOUT,'(15H SNF: NUMBER OF,
     1 22H ONE-SPEED ITERATIONS=,I5,1H.)') ITER
      IF(IMPX.GT.2) THEN
         WRITE(IUNOUT,'(//33H N E U T R O N    F L U X E S   :)')
         WRITE(IUNOUT,'(1P,6(5X,E15.7))') (FUNKNO(KEYFLX(I)),I=1,NREG)
      ENDIF
      RETURN
  300 FORMAT(25H SNF: ONE-SPEED ITERATION,I3,8H  ERROR=,1P,E11.4,
     1 5H OVER,E11.4,22H  ACCELERATION FACTOR=,0P,F7.3)
      END
