*DECK SPHPIJ
      SUBROUTINE SPHPIJ(IPTRK2,NPSYS,KSPH,NREG,NUN,NMERGE,NGCOND,NBMIX,
     1 MAT,VOL,KEY,MERG,COURIN,SUNMER,FLXMER,IEX,FUNKNO,SUNKNO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* collision probability source calculation over the macro-geometry.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK2  pointer to the tracking of the macro-geometry
*         (L_TRACK signature).
* NPSYS   group masks.
* KSPH    type of SPH factor normalization:
*         =1 average flux normalization;
*         =2 Selengut normalization;
*         =3 Selengut normalization with surface leakage.
*         Note: if KSPH=2 or 3, the 'ALBS' option should be used in ASM.
* NREG    number of macro-regions (in the macro calculation).
* NUN     number of unknowns in the macro-calculation.
* NMERGE  number of merged regions.
* NGCOND  number of condensed groups.
* NBMIX   number of macro-mixtures.
* MAT     mixture index per macro-region.
* VOL     volume of macro-regions.
* KEY     position of the flux components associated with each volume.
* MERG    index of merged macro-regions.
* COURIN  averaged flux if KSPH=1. Equal to 4 times the incoming current
*         per unit surface if KSPH=2 or 3.
* SUNMER  incoming source (scattering+fission) cross sections.
* FLXMER  flux estimate per mixture.
* IEX     iteration index.
* FUNKNO  neutron flux.
*
*Parameters: output
* SUNKNO  neutron sources.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK2,NPSYS(NGCOND),KSPH,NREG,NUN,NMERGE,NGCOND,NBMIX,
     1 MAT(NREG),KEY(NREG),MERG(NREG),IEX
      REAL VOL(NREG),COURIN(NGCOND),SUNMER(NMERGE,NGCOND,NGCOND),
     1 FLXMER(NMERGE,NGCOND),FUNKNO(NUN,NGCOND),SUNKNO(NUN,NGCOND)
*----
*  COMPUTE THE NEUTRON SOURCES IN GROUP IGR
*----
      IF(NREG.GT.NUN) CALL XABORT('SPHPIJ: INVALID VALUE OF NUN.')
      DO 40 IGR=1,NGCOND
      CALL XDRSET(SUNKNO(1,IGR),NUN,0.0)
      IF(NPSYS(IGR).EQ.0) GO TO 40
      COUR=0.0
      IF(KSPH.GT.1) COUR=COURIN(IGR)
      DO 10 I=1,NREG
      IF(I.NE.KEY(I)) CALL XABORT('SPHPIJ: INVALID VALUE OF KEY.')
      IF(MAT(I).GT.NBMIX) CALL XABORT('SPHPIJ: NBMIX OVERFLOW.')
   10 CONTINUE
      DO 30 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 30
      IF(VOL(K).EQ.0.0) GO TO 30
      IND1=KEY(K)
      IMER=MERG(K)
      PV=0.0
      DO 20 JGR=1,NGCOND
   20 PV=PV+SUNMER(IMER,JGR,IGR)*FLXMER(IMER,JGR)
      SUNKNO(IND1,IGR)=SUNKNO(IND1,IGR)+PV
   30 CONTINUE
   40 CONTINUE
      RETURN
      END
