*DECK SPHTRA
      SUBROUTINE SPHTRA(JPSYS,NPSYS,KSPH,NREG,NGCOND,NBMIX,MAT,VOL,
     1 COURIN,SUNKNO,FUNKNO,PIJ,VMIX,SIGMA,WORK2,WORK3)
*
*-----------------------------------------------------------------------
*
*Purpose:
* transport calculation over the macro-geometry using the collision
* probability technique. Use the Bell factor acceleration strategy.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* JPSYS   pointer to the 'GROUP' directory in the system LCM object.
* NPSYS   group masks.
* KSPH    type of SPH factor normalization:
*         =1 average flux normalization;
*         =2 Selengut normalization;
*         =3 Selengut normalization with surface leakage.
*         Note: if KSPH=2 or 3, the 'ALBS' option should be used in ASM.
* NREG    number of macro-regions (in the macro calculation).
* NGCOND  number of condensed groups.
* NBMIX   number of material mixtures.
* MAT     mixture index per macro-region.
* VOL     volume of macro-regions.
* COURIN  averaged flux if KSPH=1. Equal to 4 times the incoming current
*         per unit surface if KSPH=2 or 3.
* SUNKNO  fission and out-of-group diffusion sources.
*
*Parameters: output
* FUNKNO  neutron flux.
*
*Parameters: scratch
* PIJ
* VMIX
* SIGMA
* WORK2
* WORK3
*
*Reference(s):
* P. Blanc-Tranchant, A. Santamarina, G. Willermoz and A. Hebert,
* "Definition and Validation of a 2-D Transport Scheme for PWR Control
* Rod Clusters", paper presented at the Int. Conf. on Mathematics and
* Computation, Reactor Physics and Environmental Analysis in Nuclear
* Applications, Madrid, Spain, September 27-30, 1999.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER JPSYS,NPSYS(NGCOND),KSPH,NREG,NGCOND,NBMIX,MAT(NREG)
      REAL VOL(NREG),COURIN(NGCOND),SUNKNO(NREG,NGCOND),
     1 FUNKNO(NREG,NGCOND),PIJ(NREG,NREG),VMIX(NBMIX+1),
     2 SIGMA(0:NBMIX)
      DOUBLE PRECISION WORK2(NBMIX+1,NBMIX+1),WORK3(NBMIX,NBMIX+2)
*----
*  COMPUTE THE WORK2 MATRIX: NREG --> NBMIX.
*----
      DO 100 IGR=1,NGCOND
      CALL XDRSET(FUNKNO(1,IGR),NREG,0.0)
      IF(NPSYS(IGR).EQ.0) GO TO 100
      KPSYS=LCMGIL(JPSYS,IGR)
      CALL LCMGET(KPSYS,'DRAGON-TXSC',SIGMA)
      CALL LCMGET(KPSYS,'DRAGON-PCSCT',PIJ)
      DO 20 IBM=1,NBMIX
      VMIX(IBM)=0.0
      DO 20 JBM=1,NBMIX+1
   20 WORK2(IBM,JBM)=0.0
      DO 40 I=1,NREG
      IBM=MAT(I)
      IF(IBM.EQ.0) GO TO 40
      VMIX(IBM)=VMIX(IBM)+VOL(I)
      DO 30 J=1,NREG
      WORK2(IBM,NBMIX+1)=WORK2(IBM,NBMIX+1)+PIJ(I,J)*VOL(I)*
     1 SUNKNO(J,IGR)
      JBM=MAT(J)
      IF(JBM.GT.0) WORK2(IBM,JBM)=WORK2(IBM,JBM)+PIJ(I,J)*VOL(I)
   30 CONTINUE
   40 CONTINUE
*----
*  COMPUTE THE NEUTRON FLUXES.
*----
      IF(KSPH.GT.1) THEN
*        SELENGUT NORMALIZATION.
         DO 60 IBM=1,NBMIX
         WORK2(IBM,NBMIX+1)=WORK2(IBM,NBMIX+1)+COURIN(IGR)*VMIX(IBM)
         DO 50 JBM=1,NBMIX
         WORK2(IBM,JBM)=-SIGMA(JBM)*WORK2(IBM,JBM)
   50    WORK2(IBM,NBMIX+1)=WORK2(IBM,NBMIX+1)-COURIN(IGR)*SIGMA(JBM)*
     1   WORK2(IBM,JBM)
   60    WORK2(IBM,IBM)=WORK2(IBM,IBM)+VMIX(IBM)
         CALL ALSBD(NBMIX,1,WORK2,IER,NBMIX+1)
         IF(IER.NE.0) CALL XABORT('SPHTRA: SINGULAR MATRIX.')
      ELSE
*        INTEGRATED FLUX NORMALIZATION. USE A SVD SOLUTION.
         VOLTOT=0.0
         DO 80 IBM=1,NBMIX
         VOLTOT=VOLTOT+VMIX(IBM)
         WORK2(NBMIX+1,IBM)=VMIX(IBM)
         DO 70 JBM=1,NBMIX
   70    WORK2(IBM,JBM)=-SIGMA(JBM)*WORK2(IBM,JBM)
   80    WORK2(IBM,IBM)=WORK2(IBM,IBM)+VMIX(IBM)
         WORK2(NBMIX+1,NBMIX+1)=COURIN(IGR)*VOLTOT
         CALL ALSVDF(WORK2,NBMIX+1,NBMIX,NBMIX+1,NBMIX,WORK3(1,NBMIX+1),
     1   WORK3,WORK3(1,NBMIX+2))
         CALL ALSVDS(WORK2,WORK3(1,NBMIX+1),WORK3,NBMIX+1,NBMIX,NBMIX+1,
     1   NBMIX,WORK2(1,NBMIX+1),WORK2(1,NBMIX+1),WORK3(1,NBMIX+2))
      ENDIF
      DO 90 IREG=1,NREG
      IBM=MAT(IREG)
      IF(IBM.GT.0) FUNKNO(IREG,IGR)=WORK2(IBM,NBMIX+1)
   90 CONTINUE
  100 CONTINUE
      RETURN
      END
