*DECK SYBB01
      SUBROUTINE SYBB01 (NSUPCE,NPIJ,NPIS,NPIS0,SIGT,SIGW,IMPX,IQUAD,
     1 NMC,RAYRE,IR1,IR2,NG,NSMAX,MICRO,IQUAB,NS,IBI,RS,FRACT,VOLK,
     2 NMCB,PIJW,PISW,PSJW,PSSW,PLS,PSL,PIS,PSI)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the cellwise scattering-reduced collision, escape and
* transmission probabilities for the 'do-it-yourself' approach.
* Special version with double heterogeneity capability.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NSUPCE  total number of cells.
* NPIJ    length of cellwise scattering-reduced collision probability
*         matrices in the composite geometry.
* NPIS    length of cellwise scattering-reduced escape probability
*         matrices in the composite geometry (NPIS=NMCB(NSUPCE+1)).
* NPIS0   length of cellwise scattering-reduced escape probability
*         matrices in the macro geometry (NPIS0=NMC(NSUPCE+1)).
* SIGT    total macroscopic cross sections.
* SIGW    P0 within-group scattering macroscopic cross sections.
* IMPX    print flag (equal to 0 for no print).
* IQUAD   quadrature parameter for the macro geometry.
* NMC     offset of the first macro-volume in each cell.
* RAYRE   radius of the tubes in each cell.
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUAB   quadrature parameter for the treatment of the micro volumes.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
* NMCB    offset of the first composite volume in each generating cell.
*
*Parameters: output
* PIJW    cellwise scattering-reduced collision probability matrices.
* PISW    cellwise scattering-reduced escape probability matrices.
* PSJW    cellwise scattering-reduced collision probability matrices
*         for incoming neutrons.
* PSSW    cellwise scattering-reduced transmission probability
*         matrices.
*
*Parameters: scratch
* PLS
* PSL
* PIS
* PSI
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NSUPCE,NPIJ,NPIS,NPIS0,IMPX,IQUAD,NMC(NSUPCE+1),IR1,IR2,
     1 NG,NSMAX,MICRO,IQUAB,NS(NG),IBI(NPIS0),NMCB(NSUPCE+1)
      REAL SIGT(NPIS),SIGW(NPIS),RAYRE(NPIS0+NSUPCE),RS(NSMAX+1,NG),
     1 FRACT(NG,IR2),VOLK(NG,NSMAX),PIJW(NPIJ),PISW(NPIS),PSJW(NPIS),
     2 PSSW(NSUPCE),PLS(NPIS0),PSL(NPIS0),PIS(NPIS),PSI(NPIS)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (PI=3.141592654)
      LOGICAL LSKIP
      COMMON BASE(1)
*
      IPIJ=0
      DO 160 IKK=1,NSUPCE
      J1=NMC(IKK)
      J2=NMC(IKK+1)-J1
      J1B=NMCB(IKK)
      J2B=NMCB(IKK+1)-J1B
*
      CALL SETARA(BASE,J2B*J2B,IPB)
      CALL SETARA(BASE,J2*J2,IP)
      CALL SETARA(BASE,J2,IRRR)
      CALL SETARA(BASE,J2,ISIGT0)
      CALL SETARA(BASE,NG*NSMAX*J2,IQKD)
*----
*  DOUBLE HETEROGENEITY PART 1
*----
      CALL SETARA(BASE,NSMAX*NSMAX,IQKN)
      CALL SYBHT1(IR1,IR2,J2B,SIGT(J1B+1),J2,NG,NSMAX,MICRO,IQUAB,NS,
     1 IBI(J1+1),RS,FRACT,VOLK,BASE(IPB),BASE(IRRR),BASE(ISIGT0),
     2 BASE(IQKD),BASE(IQKN))
      CALL RLSARA(BASE(IQKN))
*----
*  COMPUTE THE REDUCED COLLISION PROBABILITY MATRIX
*----
      CALL SETARA(BASE,3*J2,IX)
      CALL SETARA(BASE,1+IQUAD*((J2*(5+J2))/2),IZ)
      CALL SYBT1D(J2,RAYRE(J1+IKK),.FALSE.,IQUAD,BASE(IZ))
      CALL SYBALC(J2,J2,RAYRE(J1+IKK),BASE(ISIGT0),IQUAD,0.0,BASE(IZ),
     1 BASE(IP),BASE(IX))
      CALL RLSARA(BASE(IZ))
      CALL RLSARA(BASE(IX))
      SURFA=2.0*PI*RAYRE(J1+J2+IKK)
      PSS=0.0
      RJN=0.0
      DO 20 I=1,J2
      PLS(I)=0.0
      DO 10 J=1,J2
   10 PLS(I)=PLS(I)+BASE(IP+(J-1)*J2+I-1)*BASE(ISIGT0+J-1)
      PLS(I)=1.0-PLS(I)
      RJN1=RAYRE(I+J1+IKK)**2
      PSL(I)=4.0*PI*(RJN1-RJN)*PLS(I)/SURFA
      PSS=PSS+PSL(I)*BASE(ISIGT0+I-1)
   20 RJN=RJN1
      PSS=1.0-PSS
      IF(IMPX.GE.8) CALL SYBPRX(1,J2,IKK,BASE(ISIGT0),BASE(IP),PLS,PSS)
*----
*  DOUBLE HETEROGENEITY PART 2
*----
      CALL SYBHT3(IR1,IR2,J2B,J2,1,NG,NSMAX,NS,IBI(J1+1),FRACT,VOLK,
     1 BASE(IPB),PIS,PSI,BASE(IRRR),BASE(ISIGT0),BASE(IQKD),BASE(IP),
     2 PLS,PSL)
      CALL RLSARA(BASE(IQKD))
      CALL RLSARA(BASE(ISIGT0))
      CALL RLSARA(BASE(IRRR))
      CALL RLSARA(BASE(IP))
      IF(IMPX.GE.8) CALL SYBPRX(1,J2B,IKK,SIGT(J1B+1),BASE(IPB),PIS,
     1 PSS)
*----
*  CHECK IF SCATTERING REDUCTION IS REQUIRED
*----
      LSKIP=.TRUE.
      DO 30 I=1,J2B
   30 LSKIP=LSKIP.AND.(SIGW(J1B+I).EQ.0.0)
      IF(LSKIP) THEN
*        DO NOT PERFORM SCATTERING REDUCTION.
         DO 40 I=1,J2B
         DO 40 J=1,J2B
   40    PIJW(IPIJ+(J-1)*J2B+I)=BASE(IPB+(J-1)*J2B+I-1)
         DO 50 I=1,J2B
         PISW(J1B+I)=PIS(I)
   50    PSJW(J1B+I)=PSI(I)
         PSSW(IKK)=PSS
      ELSE
*        COMPUTE THE SCATTERING-REDUCED COLLISION AND ESCAPE MATRICES.
         DO 70 I=1,J2B
         DO 60 J=1,J2B
   60    PIJW(IPIJ+(J-1)*J2B+I)=-BASE(IPB+(J-1)*J2B+I-1)*SIGW(J1B+J)
   70    PIJW(IPIJ+(I-1)*J2B+I)=1.0+PIJW(IPIJ+(I-1)*J2B+I)
         CALL SETARA(BASE,J2B,IWORK)
         CALL ALINV(J2B,PIJW(IPIJ+1),J2B,IER,BASE(IWORK))
         IF(IER.NE.0) CALL XABORT('SYBB01: SINGULAR MATRIX.')
         DO 120 I=1,J2B
         DO 80 K=1,J2B
   80    BASE(IWORK+K-1)=PIJW(IPIJ+(K-1)*J2B+I)
         DO 100 J=1,J2B
         WGAR=0.0
         DO 90 K=1,J2B
   90    WGAR=WGAR+BASE(IWORK+K-1)*BASE(IPB+(J-1)*J2B+K-1)
  100    PIJW(IPIJ+(J-1)*J2B+I)=WGAR
         WGAR=0.0
         DO 110 K=1,J2B
  110    WGAR=WGAR+BASE(IWORK+K-1)*PIS(K)
  120    PISW(J1B+I)=WGAR
         CALL RLSARA(BASE(IWORK))
*
*        COMPUTE THE SCATTERING-REDUCED COLLISION PROBABILITY MATRIX
*        FOR INCOMING NEUTRONS.
         DO 140 J=1,J2B
         WGAR=PSI(J)
         DO 130 K=1,J2B
  130    WGAR=WGAR+PSI(K)*SIGW(J1B+K)*PIJW(IPIJ+(J-1)*J2B+K)
  140    PSJW(J1B+J)=WGAR
*
*        COMPUTE THE SCATTERING-REDUCED TRANSMISSION PROBABILITY MATRIX.
         WGAR=PSS
         DO 150 K=1,J2B
  150    WGAR=WGAR+PSI(K)*SIGW(J1B+K)*PISW(J1B+K)
         PSSW(IKK)=WGAR
      ENDIF
      CALL RLSARA(BASE(IPB))
  160 IPIJ=IPIJ+J2B*J2B
      RETURN
      END
