*DECK SYBB02
      SUBROUTINE SYBB02 (NGEN,NPIJ,NPIS,NPIS0,SIGT2,SIGW2,IMPX,NCOUR,
     1 IWIGN,IQUAD,XX,YY,NMC,RAYRE,MAIL,RZMAIL,IR1,IR2,NG,NSMAX,MICRO,
     2 IQUAB,NS,IBI,RS,FRACT,VOLK,NMCB,PIJW,PISW,PSJW,PSSW,PLS,PSL,PIS,
     3 PSI)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the cellwise scattering-reduced collision, escape and
* transmission probabilities in a 2-D Cartesian or hexagonal assembly
* with Roth approximation. Special version with double heterogeneity
* capability.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NGEN    total number of generating cells.
* NPIJ    length of cellwise scattering-reduced collision probability
*         matrices in the composite geometry.
* NPIS    length of cellwise scattering-reduced escape probability
*         matrices in the composite geometry (NPIS=NMCB(NGEN+1)).
* NPIS0   length of cellwise scattering-reduced escape probability
*         matrices in the macro geometry (NPIS0=NMC(NGEN+1)).
* SIGT2   total macroscopic cross sections.
* SIGW2   P0 within-group scattering macroscopic cross sections.
* IMPX    print flag (equal to 0 for no print).
* NCOUR   number of interface currents per cell (=4: Cartesian lattice;
*         =6: hexagonal lattice).
* IWIGN   type of cylinderization.
* IQUAD   quadrature parameters.
* XX      X-thickness of the generating cells.
* YY      Y-thickness of the generating cells.
* NMC     offset of the first macro-volume in each generating cell.
* RAYRE   radius of the tubes in each generating cell.
* MAIL    offset of the first tracking information in each generating
*         cell.
* RZMAIL  real tracking information.
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUAB   quadrature parameter for the treatment of the micro volumes.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
* NMCB    offset of the first composite volume in each generating cell.
*
*Parameters: output
* PIJW    cellwise scattering-reduced collision probability matrices.
* PISW    cellwise scattering-reduced escape probability matrices.
* PSJW    cellwise scattering-reduced collision probability matrices
*         for incoming neutrons.
* PSSW    cellwise scattering-reduced transmission probability
*         matrices.
*
*Parameters: scratch
* PLS
* PSL
* PIS
* PSI
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NGEN,NPIJ,NPIS,NPIS0,IMPX,NCOUR,IWIGN,IQUAD(4),
     1 NMC(NGEN+1),MAIL(2,NGEN),IR1,IR2,NG,NSMAX,MICRO,IQUAB,NS(NG),
     2 IBI(NPIS0),NMCB(NGEN+1)
      REAL SIGT2(NPIS),SIGW2(NPIS),XX(NGEN),YY(NGEN),RAYRE(NPIS0),
     1 RZMAIL(*),RS(NSMAX+1,NG),FRACT(NG,IR2),VOLK(NG,NSMAX),PIJW(NPIJ),
     2 PISW(NPIS),PSJW(NPIS),PSSW(NGEN),PLS(NPIS0),PSL(NPIS0),PIS(NPIS),
     3 PSI(NPIS)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (PI=3.141592654)
      LOGICAL LSKIP
      COMMON BASE(1)
      MR=IQUAD(4)
*
      IPIJ=0
      DO 220 JKG=1,NGEN
      J1=NMC(JKG)
      J2=NMC(JKG+1)-J1
      J1B=NMCB(JKG)
      J2B=NMCB(JKG+1)-J1B
*
      CALL SETARA(BASE,J2B*J2B,IPB)
      CALL SETARA(BASE,J2*J2,IP)
      CALL SETARA(BASE,J2,IRRR)
      CALL SETARA(BASE,J2,ISIGT0)
      CALL SETARA(BASE,NG*NSMAX*J2,IQKD)
*----
*  DOUBLE HETEROGENEITY PART 1
*----
      CALL SETARA(BASE,NSMAX*NSMAX,IQKN)
      CALL SYBHT1(IR1,IR2,J2B,SIGT2(J1B+1),J2,NG,NSMAX,MICRO,IQUAB,
     1 NS,IBI(J1+1),RS,FRACT,VOLK,BASE(IPB),BASE(IRRR),BASE(ISIGT0),
     2 BASE(IQKD),BASE(IQKN))
      CALL RLSARA(BASE(IQKN))
*----
*  CYLINDERIZATION OPTIONS
*----
      A=XX(JKG)
      B=YY(JKG)
      IB=MAIL(2,JKG)
      RJ1=RAYRE(NMC(JKG+1))
      IF((NCOUR.EQ.4).AND.(IWIGN.EQ.1)) THEN
*        ASKEW CYLINDERIZATION CARTESIAN.
         RJ2=(A+B)/PI
         SCALE1=(A*B-PI*RJ1**2)/(PI*RJ2**2-PI*RJ1**2)
         SCALE2=1.0
         ROUT=RJ2
      ELSE IF((NCOUR.EQ.4).AND.(IWIGN.EQ.2)) THEN
*        WIGNER CYLINDERIZATION CARTESIAN.
         SCALE1=1.0
         SCALE2=1.0
         ROUT=SQRT(A*B/PI)
      ELSE IF((NCOUR.EQ.4).AND.(IWIGN.EQ.3)) THEN
*        SANCHEZ CYLINDERIZATION CARTESIAN.
         SCALE1=1.0
         SCALE2=SQRT(PI*A*B)/(A+B)
         ROUT=SQRT(A*B/PI)
      ELSE IF(IWIGN.EQ.1) THEN
*        ASKEW CYLINDERIZATION HEXAGONAL.
         RJ2=3.0*A/PI
         SCALE1=(1.5*SQRT(3.0)*A*A-PI*RJ1**2)/(PI*RJ2**2-PI*RJ1**2)
         SCALE2=1.0
         ROUT=RJ2
      ELSE IF(IWIGN.EQ.2) THEN
*        WIGNER CYLINDERIZATION HEXAGONAL.
         SCALE1=1.0
         SCALE2=1.0
         ROUT=SQRT(1.5*SQRT(3.0)/PI)*A
      ELSE IF(IWIGN.EQ.3) THEN
*        SANCHEZ CYLINDERIZATION HEXAGONAL.
         SCALE1=1.0
         SCALE2=SQRT(PI*SQRT(3.0)/6.0)
         ROUT=SQRT(1.5*SQRT(3.0)/PI)*A
      ENDIF
      IF(ROUT.LE.RJ1) CALL XABORT('SYBB02: CYLINDERIZATION ERROR.')
*----
*  COMPUTE THE REDUCED COLLISION PROBABILITY MATRIX
*----
      SURFA=0.5*PI*ROUT
      CALL SETARA(BASE,J2+1,IRAYR)
      DO 10 I=1,J2
   10 BASE(IRAYR+I-1)=RAYRE(J1+I)
      BASE(IRAYR+J2)=ROUT
      BASE(ISIGT0+J2-1)=BASE(ISIGT0+J2-1)*SCALE1
      CALL SETARA(BASE,3*J2,IX)
      CALL SYBALC(J2,J2,BASE(IRAYR),BASE(ISIGT0),MR,0.0,RZMAIL(IB),
     1 BASE(IP),BASE(IX))
      CALL RLSARA(BASE(IX))
      PSS=0.0
      RJ1=0.0
      DO 30 I=1,J2
      PLS(I)=1.0
      RJ2=BASE(IRAYR+I)**2
      VV=PI*(RJ2-RJ1)
      DO 20 J=1,J2
   20 PLS(I)=PLS(I)-BASE(IP+(J-1)*J2+I-1)*BASE(ISIGT0+J-1)
      PSS=PSS+PLS(I)*BASE(ISIGT0+I-1)*VV/SURFA
   30 RJ1=RJ2
      CALL RLSARA(BASE(IRAYR))
      PSS=1.0-SCALE2*PSS
      IF(IMPX.GE.8) CALL SYBPRX(1,J2,JKG,BASE(ISIGT0),BASE(IP),PLS,PSS)
      BASE(ISIGT0+J2-1)=BASE(ISIGT0+J2-1)/SCALE1
      DO 35 I=1,J2
   35 BASE(IP+(J2-1)*J2+I-1)=BASE(IP+(J2-1)*J2+I-1)*SCALE1
*----
*  COMPUTE THE REDUCED COLLISION PROBABILITY MATRIX FOR INCOMING
*  NEUTRONS
*----
      SURFA=(0.5*PI*ROUT)/SCALE2
      RJ1=0.0
      DO 40 I=1,J2-1
      RJ2=PI*RAYRE(J1+I+1)**2
      PSL(I)=PLS(I)*(RJ2-RJ1)/SURFA
   40 RJ1=RJ2
      RJ2=PI*ROUT**2
      PSL(J2)=PLS(J2)*(RJ2-RJ1)*SCALE1/SURFA
*----
*  DOUBLE HETEROGENEITY PART 2
*----
      CALL SYBHT3(IR1,IR2,J2B,J2,1,NG,NSMAX,NS,IBI(J1+1),FRACT,VOLK,
     1 BASE(IPB),PIS,PSI,BASE(IRRR),BASE(ISIGT0),BASE(IQKD),BASE(IP),
     2 PLS,PSL)
      CALL RLSARA(BASE(IQKD))
      CALL RLSARA(BASE(ISIGT0))
      CALL RLSARA(BASE(IRRR))
      CALL RLSARA(BASE(IP))
      IF(IMPX.GE.8) CALL SYBPRX(1,J2B,JKG,SIGT2(J1B+1),BASE(IPB),PIS,
     1 PSS)
*----
*  CHECK IF SCATTERING REDUCTION IS REQUIRED
*----
      LSKIP=.TRUE.
      DO 70 I=1,J2B
   70 LSKIP=LSKIP.AND.(SIGW2(J1B+I).EQ.0.0)
      IF(LSKIP) THEN
*        DO NOT PERFORM SCATTERING REDUCTION.
         DO 90 I=1,J2B
         DO 80 J=1,J2B-1
   80    PIJW(IPIJ+(J-1)*J2B+I)=BASE(IPB+(J-1)*J2B+I-1)
   90    PIJW(IPIJ+(J2B-1)*J2B+I)=BASE(IPB+(J2B-1)*J2B+I-1)
         DO 100 I=1,J2B
         PISW(J1B+I)=PIS(I)
  100    PSJW(J1B+I)=PSI(I)
         PSSW(JKG)=PSS
      ELSE
*        COMPUTE THE SCATTERING-REDUCED COLLISION AND ESCAPE MATRICES.
         DO 120 I=1,J2B
         DO 110 J=1,J2B-1
  110    PIJW(IPIJ+(J-1)*J2B+I)=-BASE(IPB+(J-1)*J2B+I-1)*SIGW2(J1B+J)
         PIJW(IPIJ+(J2B-1)*J2B+I)=-BASE(IPB+(J2B-1)*J2B+I-1)*
     1   SIGW2(J1B+J2B)
  120    PIJW(IPIJ+(I-1)*J2B+I)=1.0+PIJW(IPIJ+(I-1)*J2B+I)
         CALL SETARA(BASE,J2B,IWORK)
         CALL ALINV(J2B,PIJW(IPIJ+1),J2B,IER,BASE(IWORK))
         IF(IER.NE.0) CALL XABORT('SYBB02: SINGULAR MATRIX.')
         DO 170 I=1,J2B
         DO 130 K=1,J2B
  130    BASE(IWORK+K-1)=PIJW(IPIJ+(K-1)*J2B+I)
         DO 150 J=1,J2B-1
         WGAR=0.0
         DO 140 K=1,J2B
  140    WGAR=WGAR+BASE(IWORK+K-1)*BASE(IPB+(J-1)*J2B+K-1)
  150    PIJW(IPIJ+(J-1)*J2B+I)=WGAR
         WGAR=0.0
         DO 155 K=1,J2B
  155    WGAR=WGAR+BASE(IWORK+K-1)*BASE(IPB+(J2B-1)*J2B+K-1)
         PIJW(IPIJ+(J2B-1)*J2B+I)=WGAR
         WGAR=0.0
         DO 160 K=1,J2B
  160    WGAR=WGAR+BASE(IWORK+K-1)*PIS(K)
  170    PISW(J1B+I)=WGAR
         CALL RLSARA(BASE(IWORK))
*
*        COMPUTE THE SCATTERING-REDUCED COLLISION PROBABILITY MATRIX
*        FOR INCOMING NEUTRONS.
         DO 190 J=1,J2B
         WGAR=PSI(J)
         DO 180 K=1,J2B
  180    WGAR=WGAR+PSI(K)*SIGW2(J1B+K)*PIJW(IPIJ+(J-1)*J2B+K)
  190    PSJW(J1B+J)=WGAR
*
*        COMPUTE THE SCATTERING-REDUCED TRANSMISSION PROBABILITY MATRIX.
         WGAR=PSS
         DO 200 K=1,J2B
  200    WGAR=WGAR+PSI(K)*SIGW2(J1B+K)*PISW(J1B+K)
         PSSW(JKG)=WGAR
      ENDIF
      CALL RLSARA(BASE(IPB))
  220 IPIJ=IPIJ+J2B*J2B
      RETURN
      END
