*DECK SYBB03
      SUBROUTINE SYBB03 (NGEN,NPIJ,NPIS,NPIS0,SIGT2,SIGW2,IMPX,NCOUR,
     1 IWIGN,IQUAD,XX,YY,NMC,RAYRE,MAIL,RZMAIL,IR1,IR2,NG,NSMAX,MICRO,
     2 IQUAB,NS,IBI,RS,FRACT,VOLK,NMCB,PIJW,PISW,PSJW,PSSW,PLS,PSL,PIS,
     3 PSI)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the cellwise scattering-reduced collision, escape and
* transmission probabilities in a 2-D Cartesian or hexagonal assembly
* with Roth X 4 or Roth X 6 approximation. Special version with double
* heterogeneity capability.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NGEN    total number of generating cells.
* NPIJ    length of cellwise scattering-reduced collision probability
*         matrices in the composite geometry.
* NPIS    length of cellwise scattering-reduced escape probability
*         matrices in the composite geometry (NPIS=NMCB(NGEN+1)).
* NPIS0   length of cellwise scattering-reduced escape probability
*         matrices in the macro geometry (NPIS0=NMC(NGEN+1)).
* SIGT2   total macroscopic cross sections.
* SIGW2   P0 within-group scattering macroscopic cross sections.
* IMPX    print flag (equal to 0 for no print).
* NCOUR   number of interface currents per cell (=4: Cartesian lattice;
*         =6: hexagonal lattice).
* IWIGN   type of cylinderization.
* IQUAD   quadrature parameters.
* XX      X-thickness of the generating cells.
* YY      Y-thickness of the generating cells.
* NMC     offset of the first macro-volume in each generating cell.
* RAYRE   radius of the tubes in each generating cell.
* MAIL    offset of the first tracking information in each generating
*         cell.
* RZMAIL  real tracking information.
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUAB   quadrature parameter for the treatment of the micro volumes.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
* NMCB    offset of the first composite volume in each generating cell.
*
*Parameters: output
* PIJW    cellwise scattering-reduced collision probability matrices.
* PISW    cellwise scattering-reduced escape probability matrices.
* PSJW    cellwise scattering-reduced collision probability matrices
*         for incoming neutrons.
* PSSW    cellwise scattering-reduced transmission probability
*         matrices.
*
*Parameters: scratch
* PLS
* PSL
* PIS
* PSI
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NGEN,NPIJ,NPIS,NPIS0,IMPX,NCOUR,IWIGN,IQUAD(4),
     1 NMC(NGEN+1),MAIL(2,NGEN),IR1,IR2,NG,NSMAX,MICRO,IQUAB,NS(NG),
     2 IBI(NPIS0),NMCB(NGEN+1)
      REAL SIGT2(NPIS),SIGW2(NPIS),XX(NGEN),YY(NGEN),RAYRE(NPIS0),
     1 RZMAIL(*),RS(NSMAX+1,NG),FRACT(NG,IR2),VOLK(NG,NSMAX),PIJW(NPIJ),
     2 PISW(NCOUR*NPIS),PSJW(NCOUR*NPIS),PSSW(NGEN*NCOUR*NCOUR),
     3 PLS(NCOUR,NPIS0),PSL(NCOUR,NPIS0),PIS(NCOUR,NPIS),PSI(NCOUR,NPIS)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (PI=3.141592654)
      LOGICAL LSKIP
      INTEGER ISLR(4,4),ISLH(6,6)
      REAL PSS(36),PBB(6)
      COMMON BASE(1)
      SAVE ISLR,ISLH
      DATA ISLR/0,4,1,1,4,0,1,1,3,3,0,2,3,3,2,0/
      DATA ISLH/0,1,2,3,2,1,1,0,1,2,3,2,2,1,0,1,2,3,
     1          3,2,1,0,1,2,2,3,2,1,0,1,1,2,3,2,1,0/
      MR=IQUAD(4)
*
      IPIJ=0
      IPIS=0
      IPSS=0
      DO 220 JKG=1,NGEN
      J1=NMC(JKG)
      J2=NMC(JKG+1)-J1
      J1B=NMCB(JKG)
      J2B=NMCB(JKG+1)-J1B
*
      CALL SETARA(BASE,J2B*J2B,IPB)
      CALL SETARA(BASE,J2*J2,IP)
      CALL SETARA(BASE,J2,IRRR)
      CALL SETARA(BASE,J2,ISIGT0)
      CALL SETARA(BASE,NG*NSMAX*J2,IQKD)
*----
*  DOUBLE HETEROGENEITY PART 1
*----
      CALL SETARA(BASE,NSMAX*NSMAX,IQKN)
      CALL SYBHT1(IR1,IR2,J2B,SIGT2(J1B+1),J2,NG,NSMAX,MICRO,IQUAB,
     1 NS,IBI(J1+1),RS,FRACT,VOLK,BASE(IPB),BASE(IRRR),BASE(ISIGT0),
     2 BASE(IQKD),BASE(IQKN))
      CALL RLSARA(BASE(IQKN))
*----
*  CYLINDERIZATION OPTIONS
*----
      A=XX(JKG)
      B=YY(JKG)
      IB=MAIL(2,JKG)
      RJ1=RAYRE(NMC(JKG+1))
      IF((NCOUR.EQ.4).AND.(IWIGN.EQ.1)) THEN
*        ASKEW CYLINDERIZATION CARTESIAN.
         RJ2=(A+B)/PI
         SCALE1=(A*B-PI*RJ1**2)/(PI*RJ2**2-PI*RJ1**2)
         SCALE2=1.0
         ROUT=RJ2
      ELSE IF((NCOUR.EQ.4).AND.(IWIGN.EQ.2)) THEN
*        WIGNER CYLINDERIZATION CARTESIAN.
         SCALE1=1.0
         SCALE2=1.0
         ROUT=SQRT(A*B/PI)
      ELSE IF((NCOUR.EQ.4).AND.(IWIGN.EQ.3)) THEN
*        SANCHEZ CYLINDERIZATION CARTESIAN.
         SCALE1=1.0
         SCALE2=SQRT(PI*A*B)/(A+B)
         ROUT=SQRT(A*B/PI)
      ELSE IF(IWIGN.EQ.1) THEN
*        ASKEW CYLINDERIZATION HEXAGONAL.
         RJ2=3.0*A/PI
         SCALE1=(1.5*SQRT(3.0)*A*A-PI*RJ1**2)/(PI*RJ2**2-PI*RJ1**2)
         SCALE2=1.0
         ROUT=RJ2
      ELSE IF(IWIGN.EQ.2) THEN
*        WIGNER CYLINDERIZATION HEXAGONAL.
         SCALE1=1.0
         SCALE2=1.0
         ROUT=SQRT(1.5*SQRT(3.0)/PI)*A
      ELSE IF(IWIGN.EQ.3) THEN
*        SANCHEZ CYLINDERIZATION HEXAGONAL.
         SCALE1=1.0
         SCALE2=SQRT(PI*SQRT(3.0)/6.0)
         ROUT=SQRT(1.5*SQRT(3.0)/PI)*A
      ENDIF
      IF(ROUT.LE.RJ1) CALL XABORT('SYBB03: CYLINDERIZATION ERROR.')
*----
*  COMPUTE THE REDUCED COLLISION PROBABILITY MATRIX
*----
      SURFA=0.5*PI*ROUT
      CALL SETARA(BASE,J2+1,IRAYR)
      DO 10 I=1,J2
   10 BASE(IRAYR+I-1)=RAYRE(J1+I)
      BASE(IRAYR+J2)=ROUT
      BASE(ISIGT0+J2-1)=BASE(ISIGT0+J2-1)*SCALE1
      CALL SETARA(BASE,3*J2,IX)
      CALL SYBALC(J2,J2,BASE(IRAYR),BASE(ISIGT0),MR,0.0,RZMAIL(IB),
     1 BASE(IP),BASE(IX))
      CALL RLSARA(BASE(IX))
*
      PSSX=0.0
      RJ1=0.0
      DO 50 I=1,J2
      PISX=1.0
      RJ2=BASE(IRAYR+I)**2
      VV=PI*(RJ2-RJ1)
      DO 20 J=1,J2
   20 PISX=PISX-BASE(IP+(J-1)*J2+I-1)*BASE(ISIGT0+J-1)
      PSSX=PSSX+PISX*BASE(ISIGT0+I-1)*VV/SURFA
      IF(NCOUR.EQ.4) THEN
         DEN1=2.0*(A+B)
         PLS(1,I)=PISX*B/DEN1
         PLS(2,I)=PISX*B/DEN1
         PLS(3,I)=PISX*A/DEN1
         PLS(4,I)=PISX*A/DEN1
      ELSE
         DO 30 IC=1,NCOUR
   30    PLS(IC,I)=PISX/6.0
      ENDIF
*----
*  COMPUTE THE REDUCED COLLISION PROBABILITY MATRIX FOR INCOMING
*  NEUTRONS
*----
      SURFA2=(0.5*PI*ROUT)/SCALE2
      DO 40 IC=1,NCOUR
      IF(I.LE.J2-1) THEN
         PSL(IC,I)=PISX*VV/SURFA2
      ELSE IF(I.EQ.J2) THEN
         PSL(IC,I)=PISX*VV*SCALE1/SURFA2
      ENDIF
   40 CONTINUE
   50 RJ1=RJ2
      CALL RLSARA(BASE(IRAYR))
*----
*  COMPUTE THE TRANSMISSION PROBABILITIES
*----
      PSSX=1.0-SCALE2*PSSX
      IF(NCOUR.EQ.4) THEN
         A=XX(JKG)
         B=YY(JKG)
         DEN1=MAX(2.0*B+A,2.0*A+B)
         PBB(1)=B/DEN1
         PBB(2)=MAX(A,2.0*A-B)/DEN1
         PBB(3)=A/DEN1
         PBB(4)=MAX(B,2.0*B-A)/DEN1
      ELSE
         PBB(1)=1.0/5.0
         PBB(2)=1.0/5.0
         PBB(3)=1.0/5.0
      ENDIF
      DO 60 JC=1,NCOUR
      DO 60 IC=1,NCOUR
      IF(NCOUR.EQ.4) THEN
         IB=ISLR(IC,JC)
      ELSE
         IB=ISLH(IC,JC)
      ENDIF
      IF(IB.GT.0) THEN
         PSS((JC-1)*NCOUR+IC)=PSSX*PBB(IB)
      ELSE
         PSS((JC-1)*NCOUR+IC)=0.0
      ENDIF
   60 CONTINUE
      IF(IMPX.GE.8) CALL SYBPRX(NCOUR,J2,JKG,BASE(ISIGT0),BASE(IP),
     1 PLS,PSS)
      BASE(ISIGT0+J2-1)=BASE(ISIGT0+J2-1)/SCALE1
      DO 65 I=1,J2
   65 BASE(IP+(J2-1)*J2+I-1)=BASE(IP+(J2-1)*J2+I-1)*SCALE1
*----
*  DOUBLE HETEROGENEITY PART 2
*----
      CALL SYBHT3(IR1,IR2,J2B,J2,NCOUR,NG,NSMAX,NS,IBI(J1+1),FRACT,VOLK,
     1 BASE(IPB),PIS,PSI,BASE(IRRR),BASE(ISIGT0),BASE(IQKD),BASE(IP),
     2 PLS,PSL)
      CALL RLSARA(BASE(IQKD))
      CALL RLSARA(BASE(ISIGT0))
      CALL RLSARA(BASE(IRRR))
      CALL RLSARA(BASE(IP))
      IF(IMPX.GE.8) CALL SYBPRX(NCOUR,J2B,JKG,SIGT2(J1B+1),BASE(IPB),
     1 PIS,PSS)
*----
*  CHECK IF SCATTERING REDUCTION IS REQUIRED
*----
      LSKIP=.TRUE.
      DO 70 I=1,J2B
   70 LSKIP=LSKIP.AND.(SIGW2(J1B+I).EQ.0.0)
      IF(LSKIP) THEN
*        DO NOT PERFORM SCATTERING REDUCTION.
         DO 85 I=1,J2B
         DO 80 J=1,J2B-1
   80    PIJW(IPIJ+(J-1)*J2B+I)=BASE(IPB+(J-1)*J2B+I-1)
   85    PIJW(IPIJ+(J2B-1)*J2B+I)=BASE(IPB+(J2B-1)*J2B+I-1)
         DO 90 I=1,J2B
         DO 90 JC=1,NCOUR
         PISW(IPIS+(JC-1)*J2B+I)=PIS(JC,I)
   90    PSJW(IPIS+(I-1)*NCOUR+JC)=PSI(JC,I)
         DO 100 IC=1,NCOUR
         DO 100 JC=1,NCOUR
  100    PSSW(IPSS+(JC-1)*NCOUR+IC)=PSS((JC-1)*NCOUR+IC)
      ELSE
*        COMPUTE THE SCATTERING-REDUCED COLLISION AND ESCAPE MATRICES.
         DO 120 I=1,J2B
         DO 110 J=1,J2B-1
  110    PIJW(IPIJ+(J-1)*J2B+I)=-BASE(IPB+(J-1)*J2B+I-1)*SIGW2(J1B+J)
         PIJW(IPIJ+(J2B-1)*J2B+I)=-BASE(IPB+(J2B-1)*J2B+I-1)*
     1   SIGW2(J1B+J2B)
  120    PIJW(IPIJ+(I-1)*J2B+I)=1.0+PIJW(IPIJ+(I-1)*J2B+I)
         CALL SETARA(BASE,J2B,IWORK)
         CALL ALINV(J2B,PIJW(IPIJ+1),J2B,IER,BASE(IWORK))
         IF(IER.NE.0) CALL XABORT('SYBB03: SINGULAR MATRIX.')
         DO 170 I=1,J2B
         DO 130 K=1,J2B
  130    BASE(IWORK+K-1)=PIJW(IPIJ+(K-1)*J2B+I)
         DO 150 J=1,J2B-1
         WGAR=0.0
         DO 140 K=1,J2B
  140    WGAR=WGAR+BASE(IWORK+K-1)*BASE(IPB+(J-1)*J2B+K-1)
  150    PIJW(IPIJ+(J-1)*J2B+I)=WGAR
         WGAR=0.0
         DO 155 K=1,J2B
  155    WGAR=WGAR+BASE(IWORK+K-1)*BASE(IPB+(J2B-1)*J2B+K-1)
         PIJW(IPIJ+(J2B-1)*J2B+I)=WGAR
         DO 170 JC=1,NCOUR
         WGAR=0.0
         DO 160 K=1,J2B
  160    WGAR=WGAR+BASE(IWORK+K-1)*PIS(JC,K)
  170    PISW(IPIS+(JC-1)*J2B+I)=WGAR
         CALL RLSARA(BASE(IWORK))
*
*        COMPUTE THE SCATTERING-REDUCED COLLISION PROBABILITY MATRIX
*        FOR INCOMING NEUTRONS.
         DO 190 IC=1,NCOUR
         DO 190 J=1,J2B
         WGAR=PSI(IC,J)
         DO 180 K=1,J2B
  180    WGAR=WGAR+PSI(IC,K)*SIGW2(J1B+K)*PIJW(IPIJ+(J-1)*J2B+K)
  190    PSJW(IPIS+(J-1)*NCOUR+IC)=WGAR
*
*        COMPUTE THE SCATTERING-REDUCED TRANSMISSION PROBABILITY MATRIX.
         DO 210 IC=1,NCOUR
         DO 210 JC=1,NCOUR
         WGAR=PSS((JC-1)*NCOUR+IC)
         DO 200 K=1,J2B
  200    WGAR=WGAR+PSI(IC,K)*SIGW2(J1B+K)*PISW(IPIS+(JC-1)*J2B+K)
  210    PSSW(IPSS+(JC-1)*NCOUR+IC)=WGAR
      ENDIF
      CALL RLSARA(BASE(IPB))
      IPIJ=IPIJ+J2B*J2B
      IPIS=IPIS+J2B*NCOUR
  220 IPSS=IPSS+NCOUR*NCOUR
      RETURN
      END
