*DECK SYBB04
      SUBROUTINE SYBB04 (NGEN,NPIJ,NPIS,NPIS0,NRAYRE,SIGT2,SIGW2,IMPX,
     1 NCOUR,IQUAD,XX,YY,LSECT,NMC,NMCR,RAYRE,MAIL,IZMAIL,RZMAIL,IR1,
     2 IR2,NG,NSMAX,MICRO,IQUAB,NS,IBI,RS,FRACT,VOLK,NMCB,PIJW,PISW,
     3 PSJW,PSSW,PLS,PSL,PIS,PSI)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the cellwise scattering-reduced collision, escape and
* transmission probabilities in a 2-D Cartesian or hexagonal assembly
* with DP-0 approximation. Special version with double heterogeneity
* capability
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NGEN    total number of generating cells.
* NPIJ    length of cellwise scattering-reduced collision probability
*         matrices in the composite geometry.
* NPIS    length of cellwise scattering-reduced escape probability
*         matrices in the composite geometry (NPIS=NMCB(NGEN+1)).
* NPIS0   length of cellwise scattering-reduced escape probability
*         matrices in the macro geometry (NPIS0=NMC(NGEN+1)).
* NRAYRE  size of array RAYRE (NRAYRE=NMCR(NGEN+1)).
* SIGT2   total macroscopic cross sections.
* SIGW2   P0 within-group scattering macroscopic cross sections.
* IMPX    print flag (equal to 0 for no print).
* NCOUR   number of interface currents per cell (=4: Cartesian lattice;
*         =6: hexagonal lattice).
* IQUAD   quadrature parameters.
* XX      X-thickness of the generating cells.
* YY      Y-thickness of the generating cells.
* LSECT   type of sectorization.
* NMC     offset of the first macro-volume in each generating cell.
* NMCR    offset of the first radius in each generating cell.
* RAYRE   radius of the tubes in each generating cell.
* MAIL    offset of the first tracking information in each generating
*         cell.
* IZMAIL  integer tracking information.
* RZMAIL  real tracking information.
*
*Parameters: input
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUAB   quadrature parameter for the treatment of the micro volumes.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
* NMCB    offset of the first composite volume in each generating cell.
*
*Parameters: output
* PIJW    cellwise scattering-reduced collision probability matrices.
* PISW    cellwise scattering-reduced escape probability matrices.
* PSJW    cellwise scattering-reduced collision probability matrices
*         for incoming neutrons.
* PSSW    cellwise scattering-reduced transmission probability
*         matrices.
*
*Parameters: scratch
* PLS
* PSL
* PIS
* PSI
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NGEN,NPIJ,NPIS,NPIS0,NRAYRE,IMPX,NCOUR,IQUAD(4),
     1 LSECT(NGEN),NMC(NGEN+1),NMCR(NGEN+1),MAIL(2,NGEN),IZMAIL(*),
     2 IR1,IR2,NG,NSMAX,MICRO,IQUAB,NS(NG),IBI(NPIS0),NMCB(NGEN+1)
      REAL SIGT2(NPIS),SIGW2(NPIS),XX(NGEN),YY(NGEN),RAYRE(NRAYRE),
     1 RZMAIL(*),RS(NSMAX+1,NG),FRACT(NG,IR2),VOLK(NG,NSMAX),PIJW(NPIJ),
     2 PISW(NCOUR*NPIS),PSJW(NCOUR*NPIS),PSSW(NGEN*NCOUR*NCOUR),
     3 PLS(NCOUR,NPIS0),PSL(NCOUR,NPIS0),PIS(NCOUR,NPIS),PSI(NCOUR,NPIS)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (PI=3.141592654,SMALL=5.0E-3,SQRT3=1.732050807568877)
      CHARACTER HSMG*131
      LOGICAL LSKIP
      REAL PSS(36),SURFA(6),ALPA(64),PWA(64)
      COMMON BASE(1)
*
      IPIJ=0
      IPIS=0
      IPSS=0
      DO 220 JKG=1,NGEN
      J1=NMC(JKG)
      J2=NMC(JKG+1)-J1
      J1R=NMCR(JKG)
      J2R=NMCR(JKG+1)-J1R
      J1B=NMCB(JKG)
      J2B=NMCB(JKG+1)-J1B
*
      CALL SETARA(BASE,J2B*J2B,IPB)
      CALL SETARA(BASE,J2*J2,IP)
      CALL SETARA(BASE,J2,IRRR)
      CALL SETARA(BASE,J2,ISIGT0)
      CALL SETARA(BASE,NG*NSMAX*J2,IQKD)
      CALL SETARA(BASE,J2,IVOL)
*----
*  DOUBLE HETEROGENEITY PART 1
*----
      CALL SETARA(BASE,NSMAX*NSMAX,IQKN)
      CALL SYBHT1(IR1,IR2,J2B,SIGT2(J1B+1),J2,NG,NSMAX,MICRO,IQUAB,
     1 NS,IBI(J1+1),RS,FRACT,VOLK,BASE(IPB),BASE(IRRR),BASE(ISIGT0),
     2 BASE(IQKD),BASE(IQKN))
      CALL RLSARA(BASE(IQKN))
*----
*  COMPUTE THE REDUCED COLLISION PROBABILITY MATRIX
*----
      A=XX(JKG)
      B=YY(JKG)
      IF((NCOUR.EQ.4).AND.(LSECT(JKG).NE.0)) THEN
*        SECTORIZED CARTESIAN CELL.
         IB1=MAIL(1,JKG)
         IB2=MAIL(2,JKG)
         IF(LSECT(JKG).EQ.-999) THEN
            NSECT=4
         ELSE IF((LSECT(JKG).EQ.-1).OR.(LSECT(JKG).EQ.-101)) THEN
            NSECT=8
         ELSE
            NSECT=4*MOD(ABS(LSECT(JKG)),100)
         ENDIF
         MNA4=4*IQUAD(1)
         CALL SETARA(BASE,J2R*NSECT,INUMRE)
         CALL SETARA(BASE,J2R*NSECT,IVOLIN)
         CALL SETARA(BASE,J2,ILGFUL)
         CALL SETARA(BASE,(J2+4)*(J2+5)/2,IWORKI)
         CALL SETARA(BASE,4*J2,IPSIX)
         CALL SETARA(BASE,J2+4,IG)
         CALL SYB4QG(IMPX,1,MNA4,J2R,NSECT,LSECT(JKG),J2,RZMAIL(IB2),
     1   IZMAIL(IB1),A,B,RAYRE(J1R+2),BASE(ISIGT0),SMALL,BASE(IVOL),
     2   BASE(IP),PLS,PSS,BASE(INUMRE),BASE(IVOLIN),BASE(ILGFUL),
     3   BASE(IWORKI),BASE(IPSIX),BASE(IG))
         CALL RLSARA(BASE(IG))
         CALL RLSARA(BASE(IPSIX))
         CALL RLSARA(BASE(IWORKI))
         CALL RLSARA(BASE(ILGFUL))
         CALL RLSARA(BASE(IVOLIN))
         CALL RLSARA(BASE(INUMRE))
      ELSE IF(LSECT(JKG).NE.0) THEN
*        SECTORIZED HEXAGONAL CELL.
         IB1=MAIL(1,JKG)
         IB2=MAIL(2,JKG)
         NSECT=6
         MNA4=12*IQUAD(1)
         CALL SETARA(BASE,J2R*NSECT,INUMRE)
         CALL SETARA(BASE,J2R,IVOLIN)
         CALL SETARA(BASE,J2,ILGFUL)
         CALL SETARA(BASE,(J2+6)*(J2+7)/2,IWORKI)
         CALL SETARA(BASE,6*J2,IPSIX)
         CALL SETARA(BASE,J2+6,IG)
         CALL SYB7QG(IMPX,1,MNA4,J2R,NSECT,LSECT(JKG),J2,RZMAIL(IB2),
     1   IZMAIL(IB1),A,RAYRE(J1R+2),BASE(ISIGT0),SMALL,BASE(IVOL),
     2   BASE(IP),PLS,PSS,BASE(INUMRE),BASE(IVOLIN),BASE(ILGFUL),
     3   BASE(IWORKI),BASE(IPSIX),BASE(IG))
         CALL RLSARA(BASE(IG))
         CALL RLSARA(BASE(IPSIX))
         CALL RLSARA(BASE(IWORKI))
         CALL RLSARA(BASE(ILGFUL))
         CALL RLSARA(BASE(IVOLIN))
         CALL RLSARA(BASE(INUMRE))
      ELSE IF((NCOUR.EQ.4).AND.(J2.EQ.1)) THEN
         CALL ALGPT(IQUAD(3),-1.0,1.0,ALPA,PWA)
         CALL RECT1(IQUAD(3),A,B,BASE(ISIGT0),SMALL,BASE(IP),PLS,
     1   PSS,ALPA,PWA)
         BASE(IVOL)=A*B
      ELSE IF(J2.EQ.1) THEN
         CALL ALGPT(IQUAD(3),-1.0,1.0,ALPA,PWA)
         CALL XHX2D0(IQUAD(3),ALPA,PWA,A,BASE(ISIGT0),SMALL,BASE(IP),
     1   PLS,PSS)
         BASE(IVOL)=1.5*SQRT3*A*A
      ELSE
         JJJ=J2-1
         IB1=MAIL(1,JKG)
         IB2=MAIL(2,JKG)
         IB3=IB2+1+IQUAD(4)*((JJJ*(5+JJJ))/2)
         IF(NCOUR.EQ.4) THEN
            CALL SYBRN1(J2,A,B,RZMAIL(IB3),IZMAIL(IB1),RAYRE(J1R+1),
     1      BASE(ISIGT0),SMALL,PLS,PSS)
            X=2.0*A*B/(A+B)
         ELSE
            CALL SETARA(BASE,J2,IPA)
            CALL SYBHN1(J2,RZMAIL(IB3),IZMAIL(IB1),A,BASE(ISIGT0),SMALL,
     1      RAYRE(J1R+2),BASE(IPA),PLS,PSS)
            CALL RLSARA(BASE(IPA))
            X=SQRT3*A
         ENDIF
         SIGTOT=BASE(ISIGT0+J2-1)
         IF(X*SIGTOT.LE.SMALL) THEN
            WRITE (HSMG,300) JKG
            CALL XABORT(HSMG)
         ENDIF
         CALL SETARA(BASE,3*JJJ,IX)
         CALL SYBALC(JJJ,J2,RAYRE(J1R+1),BASE(ISIGT0),IQUAD(4),0.0,
     1   RZMAIL(IB2),BASE(IP),BASE(IX))
         CALL RLSARA(BASE(IX))
         RJ1=0.0
         DO 10 I=1,J2R
         IF((NCOUR.EQ.4).AND.(I.EQ.J2R)) THEN
            RJ2=A*B
         ELSE IF(I.EQ.J2R) THEN
            RJ2=1.5*SQRT3*A*A
         ELSE
            RJ2=PI*RAYRE(J1R+I+1)**2
         ENDIF
         BASE(IVOL+I-1)=RJ2-RJ1
   10    RJ1=RJ2
*
         W=1.0
         DO 15 IC=1,NCOUR
   15    W=W-PLS(IC,J2)
         Y=BASE(IVOL+JJJ)
         DO 40 I=1,JJJ
         X=1.0
         DO 20 IC=1,NCOUR
   20    X=X-PLS(IC,I)
         DO 30 J=1,JJJ
   30    X=X-BASE(ISIGT0+J-1)*BASE(IP+(J-1)*J2+I-1)
         BASE(IP+(J2-1)*J2+I-1)=X/SIGTOT
         BASE(IP+I*J2-1)=BASE(IVOL+I-1)*BASE(IP+(J2-1)*J2+I-1)/Y
   40    W=W-BASE(ISIGT0+I-1)*BASE(IP+I*J2-1)
         BASE(IP+J2*J2-1)=W/SIGTOT
      ENDIF
      IF(IMPX.GE.8) CALL SYBPRX(NCOUR,J2,JKG,BASE(ISIGT0),BASE(IP),
     1 PLS,PSS)
*----
*  COMPUTE THE REDUCED COLLISION PROBABILITY MATRIX FOR INCOMING
*  NEUTRONS
*----
      RJ1=0.0
      DO 60 I=1,J2
      IF(NCOUR.EQ.4) THEN
         SURFA(1)=0.25*B
         SURFA(2)=0.25*B
         SURFA(3)=0.25*A
         SURFA(4)=0.25*A
      ELSE
         DO 50 JC=1,6
   50    SURFA(JC)=0.25*A
      ENDIF
      DO 60 JC=1,NCOUR
   60 PSL(JC,I)=PLS(JC,I)*BASE(IVOL+I-1)/SURFA(JC)
      CALL RLSARA(BASE(IVOL))
*----
*  DOUBLE HETEROGENEITY PART 2
*----
      CALL SYBHT3(IR1,IR2,J2B,J2,NCOUR,NG,NSMAX,NS,IBI(J1+1),FRACT,VOLK,
     1 BASE(IPB),PIS,PSI,BASE(IRRR),BASE(ISIGT0),BASE(IQKD),BASE(IP),
     2 PLS,PSL)
      CALL RLSARA(BASE(IQKD))
      CALL RLSARA(BASE(ISIGT0))
      CALL RLSARA(BASE(IRRR))
      CALL RLSARA(BASE(IP))
      IF(IMPX.GE.8) CALL SYBPRX(NCOUR,J2B,JKG,SIGT2(J1B+1),BASE(IPB),
     1 PIS,PSS)
*----
*  CHECK IF SCATTERING REDUCTION IS REQUIRED
*----
      LSKIP=.TRUE.
      DO 70 I=1,J2B
   70 LSKIP=LSKIP.AND.(SIGW2(J1B+I).EQ.0.0)
      IF(LSKIP) THEN
*        DO NOT PERFORM SCATTERING REDUCTION.
         DO 80 I=1,J2B
         DO 80 J=1,J2B
   80    PIJW(IPIJ+(J-1)*J2B+I)=BASE(IPB+(J-1)*J2B+I-1)
         DO 90 I=1,J2B
         DO 90 JC=1,NCOUR
         PISW(IPIS+(JC-1)*J2B+I)=PIS(JC,I)
   90    PSJW(IPIS+(I-1)*NCOUR+JC)=PSI(JC,I)
         DO 100 IC=1,NCOUR
         DO 100 JC=1,NCOUR
  100    PSSW(IPSS+(JC-1)*NCOUR+IC)=PSS((JC-1)*NCOUR+IC)
      ELSE
*        COMPUTE THE SCATTERING-REDUCED COLLISION AND ESCAPE MATRICES.
         DO 120 I=1,J2B
         DO 110 J=1,J2B
  110    PIJW(IPIJ+(J-1)*J2B+I)=-BASE(IPB+(J-1)*J2B+I-1)*SIGW2(J1B+J)
  120    PIJW(IPIJ+(I-1)*J2B+I)=1.0+PIJW(IPIJ+(I-1)*J2B+I)
         CALL SETARA(BASE,J2B,IWORK)
         CALL ALINV(J2B,PIJW(IPIJ+1),J2B,IER,BASE(IWORK))
         IF(IER.NE.0) CALL XABORT('SYBB04: SINGULAR MATRIX.')
         DO 170 I=1,J2B
         DO 130 K=1,J2B
  130    BASE(IWORK+K-1)=PIJW(IPIJ+(K-1)*J2B+I)
         DO 150 J=1,J2B
         WGAR=0.0
         DO 140 K=1,J2B
  140    WGAR=WGAR+BASE(IWORK+K-1)*BASE(IPB+(J-1)*J2B+K-1)
  150    PIJW(IPIJ+(J-1)*J2B+I)=WGAR
         DO 170 JC=1,NCOUR
         WGAR=0.0
         DO 160 K=1,J2B
  160    WGAR=WGAR+BASE(IWORK+K-1)*PIS(JC,K)
  170    PISW(IPIS+(JC-1)*J2B+I)=WGAR
         CALL RLSARA(BASE(IWORK))
*
*        COMPUTE THE SCATTERING-REDUCED COLLISION PROBABILITY MATRIX
*        FOR INCOMING NEUTRONS.
         DO 190 IC=1,NCOUR
         DO 190 J=1,J2B
         WGAR=PSI(IC,J)
         DO 180 K=1,J2B
  180    WGAR=WGAR+PSI(IC,K)*SIGW2(J1B+K)*PIJW(IPIJ+(J-1)*J2B+K)
  190    PSJW(IPIS+(J-1)*NCOUR+IC)=WGAR
*
*        COMPUTE THE SCATTERING-REDUCED TRANSMISSION PROBABILITY MATRIX.
         DO 210 IC=1,NCOUR
         DO 210 JC=1,NCOUR
         WGAR=PSS((JC-1)*NCOUR+IC)
         DO 200 K=1,J2B
  200    WGAR=WGAR+PSI(IC,K)*SIGW2(J1B+K)*PISW(IPIS+(JC-1)*J2B+K)
  210    PSSW(IPSS+(JC-1)*NCOUR+IC)=WGAR
      ENDIF
      CALL RLSARA(BASE(IPB))
      IPIJ=IPIJ+J2B*J2B
      IPIS=IPIS+J2B*NCOUR
  220 IPSS=IPSS+NCOUR*NCOUR
      RETURN
*
  300 FORMAT (30HSYBB04: GENERATING CELL NUMBER,I6,14H HAS A VOIDED ,
     1 6HCROWN.)
      END
