*DECK SYBBFL
      SUBROUTINE SYBBFL (IPSYS,IPTRK,IFTRAK,IMPX,IDIR,IPAS,NUNKNO,
     1 MAT,VOL,KEYFLX,FUNKNO,SUNKNO,TITR,IPER)
*
*-----------------------------------------------------------------------
*
*Purpose:
* solve one group transport equation for fluxes using the SYBIL current
* iteration method. Special version for double heterogeneity cases.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
* 
*Parameters: input
* IPSYS   pointer to the assembly matrices.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  not used.
* IMPX    print flag (equal to zero for no print).
* IDIR    not used (=0 only for SYBIL).
* IPAS    total number of regions for which specific values of the
*         neutron flux and reactions rates are required.
* NUNKNO  total number of unknowns in vectors SUNKNO and FUNKNO.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KEYFLX  position of flux elements in FUNKNO vector.
* SUNKNO  input source vector.
* TITR    title.
* 
*Parameters: input/output
* FUNKNO  unknown vector.
*
*Parameters: scratch
* IPER
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER   TITR*72
      INTEGER     IPSYS,IPTRK,IFTRAK,IMPX,IDIR,IPAS,NUNKNO,MAT(IPAS),
     1            KEYFLX(IPAS),IPER(IPAS)
      REAL        VOL(IPAS),FUNKNO(NUNKNO),SUNKNO(NUNKNO)
*----
*  LOCAL VARIABLES
*----
      PARAMETER  (IUNOUT=6,NSTATE=40)
      CHARACTER   NAMLCM*12,NAMMY*12
      INTEGER     JPAR(NSTATE),IPAR(16),IBASE(1)
      LOGICAL     EMPTY,LCM
      COMMON      BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      CALL LCMINF(IPSYS,NAMLCM,NAMMY,EMPTY,ILONG,LCM)
      IF(IDIR.NE.0) CALL XABORT('SYBBFL: PIJK FORBIDDEN HERE')
*----
*  RECOVER SYBIL SPECIFIC PARAMETERS
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',JPAR)
      ITG=JPAR(6)
      IF(ITG.NE.10) CALL XABORT('SYBBFL: THIS GEOMETRY IS NOT A DOUBLE'
     1 //'-HETEROGENEITY CASE.')
      CALL LCMSIX(IPTRK,'BIHET',1)
      CALL LCMGET(IPTRK,'PARAM',IPAR)
      IPAS2=IPAR(3)
      ITG=IPAR(6)
      CALL LCMSIX(IPTRK,' ',2)
*
      IF(ITG.EQ.1) THEN
         CALL XABORT('SYBBFL: THIS GEOMETRY CANNOT BE PROCESSED WITH A'
     1   //' CURRENT ITERATION METHOD. USE KEYWORD PIJ IN ASM: (1).')
      ELSE IF(ITG.EQ.2) THEN
         CALL XABORT('SYBBFL: THIS GEOMETRY CANNOT BE PROCESSED WITH A'
     1   //' CURRENT ITERATION METHOD. USE KEYWORD PIJ IN ASM: (2).')
      ELSE IF(ITG.EQ.3) THEN
         CALL LCMSIX(IPTRK,'DOITYOURSELF',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         NSUPCE=IPAR(1)
         ISTAT=IPAR(3)
         CALL LCMIOF(IPTRK,'NMC',INMC3)
         CALL LCMIOF(IPTRK,'PROCEL',IPROCE)
         CALL LCMSIX(IPTRK,' ',2)
         IF(LCM) THEN
            CALL LCMIOF(IPSYS,'NMC$SYBIL',INMCB)
         ELSE
            CALL SETARA(BASE,NGEN,INMCB)
            CALL LCMGET(IPSYS,'NMC$SYBIL',IBASE(INMCB))
         ENDIF
         NPIJ=0
         DO 10 IKG=1,NSUPCE
         J2=IBASE(INMCB+IKG)-IBASE(INMCB+IKG-1)
   10    NPIJ=NPIJ+J2*J2
         IF(IBASE(INMCB+NSUPCE).NE.IPAS) CALL XABORT('SYBBFL: ABORT.')
*
*        SET THE PERMUTATION VECTOR AND PERMUTE THE SOURCES.
         I1=0
         I2=IPAS2
         I3=0
         DO 40 IKK=1,NSUPCE
         J2=IBASE(INMC3+IKK)-IBASE(INMC3+IKK-1)
         J2B=IBASE(INMCB+IKK)-IBASE(INMCB+IKK-1)
         DO 20 I=1,J2
         I3=I3+1
   20    IPER(I3)=I1+I
         DO 30 I=1,J2B-J2
         I3=I3+1
   30    IPER(I3)=I2+I
         I1=I1+J2
   40    I2=I2+J2B-J2
         CALL SETARA(BASE,2*IPAS,IWORK)
         DO 50 I=1,IPAS
         BASE(IWORK+I-1)=FUNKNO(I)
   50    BASE(IWORK+IPAS+I-1)=SUNKNO(I)
         DO 60 I=1,IPAS
         FUNKNO(I)=BASE(IWORK+IPER(I)-1)
   60    SUNKNO(I)=BASE(IWORK+IPAS+IPER(I)-1)
*
         IF(LCM) THEN
            CALL LCMIOF(IPSYS,'PIJW$SYBIL',IPIJW)
            CALL LCMIOF(IPSYS,'PISW$SYBIL',IPISW)
            CALL LCMIOF(IPSYS,'PSJW$SYBIL',IPSJW)
            CALL LCMIOF(IPSYS,'PSSW$SYBIL',IPSSW)
         ELSE
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,IPAS,IPISW)
            CALL SETARA(BASE,IPAS,IPSJW)
            CALL SETARA(BASE,NSUPCE,IPSSW)
            CALL LCMGET(IPSYS,'PIJW$SYBIL',BASE(IPIJW))
            CALL LCMGET(IPSYS,'PISW$SYBIL',BASE(IPISW))
            CALL LCMGET(IPSYS,'PSJW$SYBIL',BASE(IPSJW))
            CALL LCMGET(IPSYS,'PSSW$SYBIL',BASE(IPSSW))
         ENDIF
*
         CALL SETARA(BASE,NSUPCE,IINDPI)
         CALL SETARA(BASE,3*NSUPCE,ICITR)
         CALL SETARA(BASE,NSUPCE,ICIT0)
         CALL SETARA(BASE,2*NSUPCE,IAITR)
         CALL SETARA(BASE,2*NSUPCE,IWCURR)
         CALL SYBJJ0 (IPAS,NSUPCE,NPIJ,NUNKNO,FUNKNO,SUNKNO,IMPX,
     1   ISTAT,IBASE(INMCB),BASE(IPROCE),BASE(IPIJW),BASE(IPISW),
     2   BASE(IPSJW),BASE(IPSSW),BASE(IINDPI),BASE(ICITR),BASE(ICIT0),
     3   BASE(IAITR),BASE(IWCURR))
         CALL RLSARA(BASE(IWCURR))
         CALL RLSARA(BASE(IAITR))
         CALL RLSARA(BASE(ICIT0))
         CALL RLSARA(BASE(ICITR))
         CALL RLSARA(BASE(IINDPI))
         DO 70 I=1,IPAS
         BASE(IWORK+I-1)=FUNKNO(I)
   70    BASE(IWORK+IPAS+I-1)=SUNKNO(I)
         DO 80 I=1,IPAS
         FUNKNO(IPER(I))=BASE(IWORK+I-1)
   80    SUNKNO(IPER(I))=BASE(IWORK+IPAS+I-1)
         CALL RLSARA(BASE(IWORK))
         IF(.NOT.LCM) THEN
            CALL RLSARA(BASE(INMCB))
            CALL RLSARA(BASE(IPSSW))
            CALL RLSARA(BASE(IPSJW))
            CALL RLSARA(BASE(IPISW))
            CALL RLSARA(BASE(IPIJW))
         ENDIF
      ELSE IF(ITG.EQ.4) THEN
         CALL LCMSIX(IPTRK,'EURYDICE',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         IHEX=IPAR(1)
         MULTC=IPAR(2)
         NMCEL=IPAR(4)
         NMERGE=IPAR(5)
         NGEN=IPAR(6)
         IJAT=IPAR(7)
         CALL LCMIOF(IPTRK,'XX',IXX4)
         CALL LCMIOF(IPTRK,'YY',IYY4)
         CALL LCMIOF(IPTRK,'NMC',INMC4)
         CALL LCMIOF(IPTRK,'IFR',IIFR)
         CALL LCMIOF(IPTRK,'ALB',IALB)
         CALL LCMIOF(IPTRK,'INUM',IINUM)
         CALL LCMIOF(IPTRK,'MIX',IMIX)
         CALL LCMIOF(IPTRK,'DVX',IDVX)
         CALL LCMIOF(IPTRK,'IGEN',IIGEN)
         CALL LCMSIX(IPTRK,' ',2)
*
         IF(LCM) THEN
            CALL LCMIOF(IPSYS,'NMC$SYBIL',INMCB)
         ELSE
            CALL SETARA(BASE,NGEN,INMCB)
            CALL LCMGET(IPSYS,'NMC$SYBIL',IBASE(INMCB))
         ENDIF
         NCOUR=4
         IF(IHEX.NE.0) NCOUR=6
         NPIJ=0
         DO 90 IKG=1,NGEN
         J2=IBASE(INMCB+IKG)-IBASE(INMCB+IKG-1)
   90    NPIJ=NPIJ+J2*J2
         NPIS=IBASE(INMCB+NGEN)
*
*        SET THE PERMUTATION VECTOR AND PERMUTE THE SOURCES.
         I1=0
         I2=IPAS2
         I3=0
         DO 120 IKK=1,NMERGE
         IKG=IBASE(IIGEN+IKK-1)
         J2=IBASE(INMC4+IKG)-IBASE(INMC4+IKG-1)
         J2B=IBASE(INMCB+IKG)-IBASE(INMCB+IKG-1)
         DO 100 I=1,J2
         I3=I3+1
  100    IPER(I3)=I1+I
         DO 110 I=1,J2B-J2
         I3=I3+1
  110    IPER(I3)=I2+I
         I1=I1+J2
  120    I2=I2+J2B-J2
         CALL SETARA(BASE,2*IPAS,IWORK)
         DO 130 I=1,IPAS
         BASE(IWORK+I-1)=FUNKNO(I)
  130    BASE(IWORK+IPAS+I-1)=SUNKNO(I)
         DO 140 I=1,IPAS
         FUNKNO(I)=BASE(IWORK+IPER(I)-1)
  140    SUNKNO(I)=BASE(IWORK+IPAS+IPER(I)-1)
*
         IF(MULTC.EQ.1) THEN
            IF(LCM) THEN
               CALL LCMIOF(IPSYS,'PIJW$SYBIL',IPIJW)
               CALL LCMIOF(IPSYS,'PISW$SYBIL',IPISW)
               CALL LCMIOF(IPSYS,'PSJW$SYBIL',IPSJW)
               CALL LCMIOF(IPSYS,'PSSW$SYBIL',IPSSW)
            ELSE
               CALL SETARA(BASE,NPIJ,IPIJW)
               CALL SETARA(BASE,NPIS,IPISW)
               CALL SETARA(BASE,NPIS,IPSJW)
               CALL SETARA(BASE,NGEN,IPSSW)
               CALL LCMGET(IPSYS,'PIJW$SYBIL',BASE(IPIJW))
               CALL LCMGET(IPSYS,'PISW$SYBIL',BASE(IPISW))
               CALL LCMGET(IPSYS,'PSJW$SYBIL',BASE(IPSJW))
               CALL LCMGET(IPSYS,'PSSW$SYBIL',BASE(IPSSW))
            ENDIF
*
            CALL SETARA(BASE,NGEN,IINDPI)
            CALL SETARA(BASE,NMERGE,IINDNM)
            CALL SETARA(BASE,3*NMERGE,ICITR)
            CALL SETARA(BASE,NMERGE,ICIT0)
            CALL SETARA(BASE,2*NMERGE,IAITR)
            CALL SETARA(BASE,2*NMERGE,IWCURR)
            CALL SYBJJ1 (IPAS,NMCEL,NMERGE,NGEN,NPIJ,NPIS,NUNKNO,
     1      FUNKNO,SUNKNO,IMPX,NCOUR,BASE(IXX4),BASE(IYY4),
     2      IBASE(INMCB),IBASE(IIFR),BASE(IALB),IBASE(IINUM),
     3      IBASE(IIGEN),BASE(IPIJW),BASE(IPISW),BASE(IPSJW),
     4      BASE(IPSSW),BASE(IINDPI),BASE(IINDNM),BASE(ICITR),
     5      BASE(ICIT0),BASE(IAITR),BASE(IWCURR))
            CALL RLSARA(BASE(IWCURR))
            CALL RLSARA(BASE(IAITR))
            CALL RLSARA(BASE(ICIT0))
            CALL RLSARA(BASE(ICITR))
            CALL RLSARA(BASE(IINDNM))
            CALL RLSARA(BASE(IINDPI))
         ELSE
            IF(MULTC.EQ.4) NCOUR=3*NCOUR
            IF(LCM) THEN
               CALL LCMIOF(IPSYS,'PIJW$SYBIL',IPIJW)
               CALL LCMIOF(IPSYS,'PISW$SYBIL',IPISW)
               CALL LCMIOF(IPSYS,'PSJW$SYBIL',IPSJW)
               CALL LCMIOF(IPSYS,'PSSW$SYBIL',IPSSW)
            ELSE
               CALL SETARA(BASE,NPIJ,IPIJW)
               CALL SETARA(BASE,NCOUR*NPIS,IPISW)
               CALL SETARA(BASE,NCOUR*NPIS,IPSJW)
               CALL SETARA(BASE,NCOUR*NCOUR*NGEN,IPSSW)
               CALL LCMGET(IPSYS,'PIJW$SYBIL',BASE(IPIJW))
               CALL LCMGET(IPSYS,'PISW$SYBIL',BASE(IPISW))
               CALL LCMGET(IPSYS,'PSJW$SYBIL',BASE(IPSJW))
               CALL LCMGET(IPSYS,'PSSW$SYBIL',BASE(IPSSW))
            ENDIF
*
            CALL SETARA(BASE,NGEN,IINDPI)
            CALL SETARA(BASE,NMERGE,IINDNM)
            CALL SETARA(BASE,3*IJAT,ICITR)
            CALL SETARA(BASE,IJAT,ICIT0)
            CALL SETARA(BASE,2*IJAT,IAITR)
            CALL SETARA(BASE,2*IJAT,IWCURR)
            CALL SYBJJ2 (IPAS,NMCEL,NMERGE,NGEN,IJAT,NPIJ,NPIS,
     1      NUNKNO,FUNKNO,SUNKNO,IMPX,NCOUR,IBASE(INMCB),IBASE(IIFR),
     2      BASE(IALB),IBASE(IINUM),IBASE(IMIX),BASE(IDVX),
     3      IBASE(IIGEN),BASE(IPIJW),BASE(IPISW),BASE(IPSJW),
     4      BASE(IPSSW),BASE(IINDPI),BASE(IINDNM),BASE(ICITR),
     5      BASE(ICIT0),BASE(IAITR),BASE(IWCURR))
            CALL RLSARA(BASE(IWCURR))
            CALL RLSARA(BASE(IAITR))
            CALL RLSARA(BASE(ICIT0))
            CALL RLSARA(BASE(ICITR))
            CALL RLSARA(BASE(IINDNM))
            CALL RLSARA(BASE(IINDPI))
         ENDIF
         DO 150 I=1,IPAS
         BASE(IWORK+I-1)=FUNKNO(I)
  150    BASE(IWORK+IPAS+I-1)=SUNKNO(I)
         DO 160 I=1,IPAS
         FUNKNO(IPER(I))=BASE(IWORK+I-1)
  160    SUNKNO(IPER(I))=BASE(IWORK+IPAS+I-1)
         CALL RLSARA(BASE(IWORK))
         IF(.NOT.LCM) THEN
            CALL RLSARA(BASE(INMCB))
            CALL RLSARA(BASE(IPSSW))
            CALL RLSARA(BASE(IPSJW))
            CALL RLSARA(BASE(IPISW))
            CALL RLSARA(BASE(IPIJW))
         ENDIF
      ELSE
         CALL XABORT('SYBBFL: UNKNOWN CP MODULE(2).')
      ENDIF
      RETURN
      END
