*DECK SYBBX2
      SUBROUTINE SYBBX2 (IPAS,NPIJ,NPIS,NPIS0,SIGT,SIGW,P,IMPX,NCOUR,
     1 IWIGN,NMCEL,NMERGE,NGEN,IQUAD,XX,YY,NMC,RAYRE,MAIL,RZMAIL,IFR,
     2 ALB,INUM,IGEN,IR1,IR2,IPAS2,NG,NSMAX,MICRO,IQUAB,NS,IBI,RS,
     3 FRACT,VOLK,NMCB,PSSB,SIGT2,SIGW2,PIJW,PISW,PSJW,PSSW,IBI2,IPER)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the global scattering-reduced collision probabilities in a
* 2-D Cartesian or hexagonal assembly using the interface current
* method with Roth approximation. Special version with double
* heterogeneity capability.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPAS    number of volumes in the composite geometry.
* NPIJ    length of cellwise scattering-reduced collision probability
*         matrices in the composite geometry.
* NPIS    length of cellwise scattering-reduced escape probability
*         matrices in the composite geometry (NPIS=NMCB(NGEN+1)).
* NPIS0   length of cellwise scattering-reduced escape probability
*         matrices in the macro geometry (NPIS0=NMC(NGEN+1)).
* SIGT    total macroscopic cross sections.
* IMPX    print flag (equal to 0 for no print).
* NCOUR   number of interface currents per cell (=4: Cartesian lattice;
*         =6: hexagonal lattice).
* IWIGN   type of cylinderization.
* IQUAD   quadrature parameters.
* NMCEL   total number of cells in the domain.
* IFR     index-number of in-currents.
* ALB     transmission/albedo associated with each in-current.
*         Note: IFR and ALB contains information to rebuild the
*         geometrical 'A' matrix.
* NMERGE  total number of merged cells for which specific values
*         of the neutron flux and reactions rates are required.
*         Many cells with different position in the domain can
*         be merged before the neutron flux calculation if they
*         own the same generating cell. This allows some reduction
*         in cpu time and memory (NMERGE.le.NMCEL).
* INUM    index-number of the merged cell associated to each cell.
* NGEN    total number of generating cells. a generating cell is
*         defined by its material and dimensions, irrespective of
*         its position in the domain (NGEN.le.NMERGE).
* XX      X-thickness of the generating cells.
* YY      Y-thickness of the generating cells.
* NMC     offset of the first volume in each generating cell.
* RAYRE   radius of the tubes in each generating cell.
* MAIL    offset of the first tracking information in each generating
*         cell.
* RZMAIL  real tracking information.
* IGEN    index-number of the generating cell associated with each
*         merged cell.
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* IPAS2   number of volumes in the macro geometry.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUAB   quadrature parameter for the treatment of the micro volume.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
* NMCB    offset of the first composite volume in each generating cell.
*
*Parameters: output
* P       reduced collision probabilities.
*
*Parameters: scratch
* PSSB
* SIGT2
* SIGW2
* PIJW
* PISW
* PSJW
* PSSW
* IBI2
* IPER
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPAS,NPIJ,NPIS,NPIS0,IMPX,NCOUR,IWIGN,NMCEL,NMERGE,NGEN,
     1 IQUAD(4),NMC(NGEN+1),MAIL(2,NGEN),IFR(NCOUR*NMCEL),INUM(NMCEL),
     2 IGEN(NMERGE),IR1,IR2,IPAS2,NG,NSMAX,MICRO,IQUAB,NS(NG),
     3 IBI(IPAS2),NMCB(NGEN+1),IBI2(NPIS0),IPER(IPAS)
      REAL SIGT(IPAS),SIGW(IPAS),P(IPAS,IPAS),XX(NGEN),YY(NGEN),
     1 RAYRE(NPIS0),RZMAIL(*),ALB(NCOUR*NMCEL),RS(NSMAX+1,NG),
     2 FRACT(NG,IR2),VOLK(NG,NSMAX),PSSB(NMERGE,2*NMERGE),SIGT2(NPIS),
     3 SIGW2(NPIS),PIJW(NPIJ),PISW(NPIS),PSJW(NPIS),PSSW(NGEN)
*----
*  LOCAL VARIABLES
*----
      REAL PIBB(6)
      COMMON BASE(1)
*
      DO 10 I=1,IPAS
      DO 10 J=1,IPAS
   10 P(I,J)=0.0
      I1=0
      I2=IPAS2
      I3=0
      DO 30 IKK=1,NMERGE
      IKG=IGEN(IKK)
      J1=NMC(IKG)
      J2=NMC(IKG+1)-J1
      J1B=NMCB(IKG)
      J2B=NMCB(IKG+1)-J1B
      DO 20 I=1,J2
      I3=I3+1
      IPER(I3)=I1+I
      IBI2(J1+I)=IBI(I1+I)
      SIGT2(J1B+I)=SIGT(I1+I)
   20 SIGW2(J1B+I)=SIGW(I1+I)
      DO 25 I=1,J2B-J2
      I3=I3+1
      IPER(I3)=I2+I
      SIGT2(J1B+J2+I)=SIGT(I2+I)
   25 SIGW2(J1B+J2+I)=SIGW(I2+I)
      I1=I1+J2
   30 I2=I2+J2B-J2
*
      CALL SETARA(BASE,NPIS0,IPLS)
      CALL SETARA(BASE,NPIS0,IPSL)
      CALL SETARA(BASE,NPIS,IPIS)
      CALL SETARA(BASE,NPIS,IPSI)
      CALL SYBB02(NGEN,NPIJ,NPIS,NPIS0,SIGT2,SIGW2,IMPX,NCOUR,IWIGN,
     1 IQUAD,XX,YY,NMC,RAYRE,MAIL,RZMAIL,IR1,IR2,NG,NSMAX,MICRO,IQUAB,
     2 NS,IBI2,RS,FRACT,VOLK,NMCB,PIJW,PISW,PSJW,PSSW,BASE(IPLS),
     3 BASE(IPSL),BASE(IPIS),BASE(IPSI))
      CALL RLSARA(BASE(IPSI))
      CALL RLSARA(BASE(IPIS))
      CALL RLSARA(BASE(IPSL))
      CALL RLSARA(BASE(IPLS))
*
      IPIJ=0
      DO 95 JKG=1,NGEN
      J2B=NMCB(JKG+1)-NMCB(JKG)
      I1=0
      DO 90 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I2B=NMCB(IKG+1)-NMCB(IKG)
      IF(IKG.EQ.JKG) THEN
         DO 70 J=1,J2B
         DO 70 I=1,J2B
   70    P(IPER(I1+I),IPER(I1+J))=PIJW(IPIJ+(J-1)*J2B+I)
      ENDIF
   90 I1=I1+I2B
   95 IPIJ=IPIJ+J2B*J2B
*----
*  COMPUTATION OF PSSB=A*(I-PSS*A)**-1
*----
      DO 110 I=1,NMERGE
      DO 100 J=1,NMERGE
      PSSB(I,J)=0.0
  100 PSSB(I,NMERGE+J)=0.0
  110 PSSB(I,I)=1.0
      DO 120 ICEL=1,NMCEL
      IKK=INUM(ICEL)
      IKG=IGEN(IKK)
      IS=NCOUR*(ICEL-1)
      IF(NCOUR.EQ.4) THEN
         A=XX(IKG)
         B=YY(IKG)
         DEN1=2.0*(A+B)
         PIBB(1)=B/DEN1
         PIBB(2)=B/DEN1
         PIBB(3)=A/DEN1
         PIBB(4)=A/DEN1
      ELSE
         DO 115 JC=1,NCOUR
  115    PIBB(JC)=1.0/6.0
      ENDIF
      ZZZ=PSSW(IKG)
      DO 120 JC=1,NCOUR
      J1=IFR(IS+JC)
      ALBEDO=PIBB(JC)*ALB(IS+JC)
      PSSB(J1,NMERGE+IKK)=PSSB(J1,NMERGE+IKK)+ALBEDO
  120 PSSB(J1,IKK)=PSSB(J1,IKK)-ZZZ*ALBEDO
      CALL ALSB(NMERGE,NMERGE,PSSB,IER,NMERGE)
      IF(IER.NE.0) CALL XABORT('SYBBX2: SINGULAR MATRIX.')
*----
*  COMPUTATION OF PIS*PSSB*PSJ
*----
      I1=0
      DO 150 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I1P=NMCB(IKG)
      I2B=NMCB(IKG+1)-I1P
      DO 140 I=1,I2B
      ZZZ=PISW(I1P+I)
      J1=0
      DO 140 JKK=1,NMERGE
      JKG=IGEN(JKK)
      J1P=NMCB(JKG)
      J2B=NMCB(JKG+1)-J1P
      DO 130 J=1,J2B
  130 P(IPER(I1+I),IPER(J1+J))=P(IPER(I1+I),IPER(J1+J))+ZZZ*
     1 PSSB(JKK,NMERGE+IKK)*PSJW(J1P+J)
  140 J1=J1+J2B
  150 I1=I1+I2B
      RETURN
      END
