*DECK SYBBX3
      SUBROUTINE SYBBX3 (MULTC,IPAS,NPIJ,NPIS,NPIS0,NRAYRE,SIGT,SIGW,P,
     1 IMPX,NCOUR,IWIGN,NMCEL,NMERGE,NGEN,IJAT,IQUAD,XX,YY,LSECT,NMC,
     2 NMCR,RAYRE,MAIL,IZMAIL,RZMAIL,IFR,ALB,INUM,MIX,DVX,IGEN,IR1,IR2,
     3 IPAS2,NG,NSMAX,MICRO,IQUAB,NS,IBI,RS,FRACT,VOLK,NMCB,PSSB,SIGT2,
     4 SIGW2,PIJW,PISW,PSJW,PSSW,IBI2,IPER)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the global scattering-reduced collision probabilities in a
* 2-D Cartesian or hexagonal assembly using the interface current
* method with Roth X 4, Roth X 6, DP-0 or DP-1 approximation. Special
* version with double heterogeneity capability.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* MULTC   type of interface current approximation:
*         =2: Roth X 4 or Roth X 6 approximation;
*         =3: DP-0 approximation; =4: DP-1 approximation.
* IPAS    number of volumes in the composite geometry.
* NPIJ    length of cellwise scattering-reduced collision probability
*         matrices in the composite geometry.
* NPIS    length of cellwise scattering-reduced escape probability
*         matrices in the composite geometry (NPIS=NMCB(NGEN+1)).
* NPIS0   length of cellwise scattering-reduced escape probability
*         matrices in the macro geometry (NPIS0=NMC(NGEN+1)).
* NRAYRE  size of array RAYRE (NRAYRE=NMCR(NGEN+1)).
* SIGT    total macroscopic cross sections.
* SIGW    P0 within-group scattering macroscopic cross sections.
* IMPX    print flag (equal to 0 for no print).
* NCOUR   number of interface currents per cell (=4 or 12: Cartesian
*         lattice; =6 or 18: hexagonal lattice).
* IWIGN   type of cylinderization if MULTC=2.
* IQUAD   quadrature parameters.
* NMCEL   total number of cells in the domain.
* IFR     index-number of in-currents.
* ALB     transmission/albedo associated with each in-current.
* NMERGE  total number of merged cells for which specific values
*         of the neutron flux and reactions rates are required.
*         Many cells with different position in the domain can
*         be merged before the neutron flux calculation if they
*         own the same generating cell. This allows some reduction
*         in cpu time and memory (NMERGE.le.NMCEL).
* IJAT    total number of distinct out-currents.
* INUM    index-number of the merged cell associated to each cell.
* MIX     index-number of out-currents.
* DVX     weight associated with each out-current.
*         Note: IFR, ALB, MIX and DVX contains information to rebuild
*         the geometrical 'A' matrix.
* NGEN    total number of generating cells. A generating cell is
*         defined by its material and dimensions, irrespective of
*         its position in the domain (NGEN.le.NMERGE).
* XX      X-thickness of the generating cells.
* YY      Y-thickness of the generating cells.
* LSECT   type of sectorization.
* NMC     offset of the first macro volume in each generating cell.
* NMCR    offset of the first radius in each generating cell.
* RAYRE   radius of the tubes in each generating cell.
* MAIL    offset of the first tracking information in each generating
*         cell.
* IZMAIL  integer tracking information.
* RZMAIL  real tracking information.
* IGEN    index-number of the generating cell associated with each
*         merged cell.
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* IPAS2   number of volumes in the macro geometry.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUAB   quadrature parameter for the treatment of the micro volumes.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
* NMCB    offset of the first composite volume in each generating cell.
*
*Parameters: output
* P       reduced collision probabilities.
*
*Parameters: scratch
* PSSB
* SIGT2
* SIGW2
* PIJW
* PISW
* PSJW
* PSSW
* IBI2
* IPER
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER MULTC,NPIJ,NPIS,NPIS0,NRAYRE,IMPX,NCOUR,IWIGN,NMCEL,
     1 NMERGE,NGEN,IJAT,IQUAD(4),LSECT(NGEN),NMC(NGEN+1),NMCR(NGEN+1),
     2 MAIL(2,NGEN),IZMAIL(*),IFR(NCOUR*NMCEL),INUM(NMCEL),
     3 MIX(NCOUR*NMERGE),IGEN(NMERGE),IR1,IR2,IPAS2,NG,NSMAX,MICRO,
     4 IQUAB,NS(NG),IBI(IPAS2),NMCB(NGEN+1),IBI2(NPIS0),IPER(IPAS)
      REAL SIGT(NPIS),SIGW(NPIS),P(IPAS,IPAS),XX(NGEN),YY(NGEN),
     1 RAYRE(NRAYRE),RZMAIL(*),ALB(NCOUR*NMCEL),DVX(NCOUR*NMERGE),
     2 RS(NSMAX+1,NG),FRACT(NG,IR2),VOLK(NG,NSMAX),PSSB(IJAT,2*IJAT),
     3 SIGT2(NPIS),SIGW2(NPIS),PIJW(NPIJ),PISW(NCOUR*NPIS),
     4 PSJW(NCOUR*NPIS),PSSW(NGEN*NCOUR*NCOUR)
*----
*  LOCAL VARIABLES
*----
      COMMON BASE(1)
*
      DO 10 I=1,IPAS
      DO 10 J=1,IPAS
   10 P(I,J)=0.0
      I1=0
      I2=IPAS2
      I3=0
      DO 30 IKK=1,NMERGE
      IKG=IGEN(IKK)
      J1=NMC(IKG)
      J2=NMC(IKG+1)-J1
      J1B=NMCB(IKG)
      J2B=NMCB(IKG+1)-J1B
      DO 20 I=1,J2
      I3=I3+1
      IPER(I3)=I1+I
      IBI2(J1+I)=IBI(I1+I)
      SIGT2(J1B+I)=SIGT(I1+I)
   20 SIGW2(J1B+I)=SIGW(I1+I)
      DO 25 I=1,J2B-J2
      I3=I3+1
      IPER(I3)=I2+I
      SIGT2(J1B+J2+I)=SIGT(I2+I)
   25 SIGW2(J1B+J2+I)=SIGW(I2+I)
      I1=I1+J2
   30 I2=I2+J2B-J2
*
      CALL SETARA(BASE,NCOUR*NPIS0,IPLS)
      CALL SETARA(BASE,NCOUR*NPIS0,IPSL)
      CALL SETARA(BASE,NCOUR*NPIS,IPIS)
      CALL SETARA(BASE,NCOUR*NPIS,IPSI)
      IF(MULTC.EQ.2) THEN
*        ROTH X 4 OR ROTH X 6 APPROXIMATION.
         CALL SYBB03(NGEN,NPIJ,NPIS,NPIS0,SIGT2,SIGW2,IMPX,NCOUR,IWIGN,
     1   IQUAD,XX,YY,NMC,RAYRE,MAIL,RZMAIL,IR1,IR2,NG,NSMAX,MICRO,IQUAB,
     2   NS,IBI2,RS,FRACT,VOLK,NMCB,PIJW,PISW,PSJW,PSSW,BASE(IPLS),
     3   BASE(IPSL),BASE(IPIS),BASE(IPSI))
      ELSE IF(MULTC.EQ.3) THEN
*        DP-0 APPROXIMATION.
         CALL SYBB04(NGEN,NPIJ,NPIS,NPIS0,NRAYRE,SIGT2,SIGW2,IMPX,NCOUR,
     1   IQUAD,XX,YY,LSECT,NMC,NMCR,RAYRE,MAIL,IZMAIL,RZMAIL,IR1,IR2,NG,
     2   NSMAX,MICRO,IQUAB,NS,IBI2,RS,FRACT,VOLK,NMCB,PIJW,PISW,PSJW,
     3   PSSW,BASE(IPLS),BASE(IPSL),BASE(IPIS),BASE(IPSI))
      ELSE IF(MULTC.EQ.4) THEN
*        DP-1 APPROXIMATION.
         CALL SYBB05(NGEN,NPIJ,NPIS,NPIS0,NRAYRE,SIGT2,SIGW2,IMPX,NCOUR,
     1   IQUAD,XX,YY,LSECT,NMC,NMCR,RAYRE,MAIL,IZMAIL,RZMAIL,IR1,IR2,NG,
     2   NSMAX,MICRO,IQUAB,NS,IBI2,RS,FRACT,VOLK,NMCB,PIJW,PISW,PSJW,
     3   PSSW,BASE(IPLS),BASE(IPSL),BASE(IPIS),BASE(IPSI))
      ELSE
         CALL XABORT('SYBBX3: UNKNOWN CP MODULE.')
      ENDIF
      CALL RLSARA(BASE(IPSI))
      CALL RLSARA(BASE(IPIS))
      CALL RLSARA(BASE(IPSL))
      CALL RLSARA(BASE(IPLS))
*
      IPIJ=0
      DO 60 JKG=1,NGEN
      J2B=NMCB(JKG+1)-NMCB(JKG)
      I1=0
      DO 50 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I2B=NMCB(IKG+1)-NMCB(IKG)
      IF(IKG.EQ.JKG) THEN
         DO 40 J=1,J2B
         DO 40 I=1,J2B
   40    P(IPER(I1+I),IPER(I1+J))=PIJW(IPIJ+(J-1)*J2B+I)
      ENDIF
   50 I1=I1+I2B
   60 IPIJ=IPIJ+J2B*J2B
*----
*  COMPUTATION OF PSSB=A*(I-PSS*A)**-1
*----
      DO 80 I=1,IJAT
      DO 70 J=1,IJAT
      PSSB(I,J)=0.0
   70 PSSB(I,IJAT+J)=0.0
   80 PSSB(I,I)=1.0
      DO 90 ICEL=1,NMCEL
      IKK=INUM(ICEL)
      IT=NCOUR*(IKK-1)
      IS=NCOUR*(ICEL-1)
      IKG=IGEN(IKK)
      IPSS=(IKG-1)*NCOUR*NCOUR
      DO 90 JC=1,NCOUR
      J1=IFR(IS+JC)
      J2=MIX(IT+JC)
      ALBEDO=ALB(IS+JC)
      PSSB(J1,IJAT+J2)=PSSB(J1,IJAT+J2)+ALBEDO*DVX(IT+JC)
      DO 90 IC=1,NCOUR
      J2=MIX(IT+IC)
   90 PSSB(J1,J2)=PSSB(J1,J2)-PSSW(IPSS+(JC-1)*NCOUR+IC)*ALBEDO*
     1 DVX(IT+IC)
      CALL ALSB(IJAT,IJAT,PSSB,IER,IJAT)
      IF(IER.NE.0) CALL XABORT('SYBBX3: SINGULAR MATRIX.')
*----
*  COMPUTATION OF PISW*PSSB*PSJW
*----
      I1=0
      DO 120 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I1P=NMCB(IKG)
      I2B=NMCB(IKG+1)-I1P
      IT=NCOUR*(IKK-1)
      DO 110 I=1,I2B
      DO 110 IC=1,NCOUR
      ICC=MIX(IT+IC)
      ZZZ=PISW(I1P*NCOUR+(IC-1)*I2B+I)*SIGN(1.0,DVX(IT+IC))
      J1=0
      DO 110 JKK=1,NMERGE
      JKG=IGEN(JKK)
      J1P=NMCB(JKG)
      J2B=NMCB(JKG+1)-J1P
      JT=NCOUR*(JKK-1)
      DO 100 J=1,J2B
      DO 100 JC=1,NCOUR
      JCC=MIX(JT+JC)
      PBJ=PSJW(J1P*NCOUR+(J-1)*NCOUR+JC)
  100 P(IPER(I1+I),IPER(J1+J))=P(IPER(I1+I),IPER(J1+J))+ZZZ*DVX(JT+JC)*
     1 PSSB(JCC,IJAT+ICC)*PBJ
  110 J1=J1+J2B
  120 I1=I1+I2B
      RETURN
      END
