*DECK SYBBXE
      SUBROUTINE SYBBXE(IPAS,NPIJ,NSUPCE,RAYRE,SIGT,SIGW,P,IQUAD,ISTAT,
     1 NMC,PROCEL,IMPX,IR1,IR2,IPAS2,NG,NSMAX,MICRO,IQUAB,NS,IBI,RS,
     2 FRACT,VOLK,NMCB,IAPPAR,PIJW,PISW,PSJW,PSSB,PSSW,SIGT2,SIGW2,IPER)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the reduced collision probabilities for the
* 'do-it-yourself' approach. Special version with double heterogeneity
* capability
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPAS    number of volumes in the composite geometry
*         (IPAS=NMCB(NSUPCE+1)).
* NPIJ    length of cellwise scattering-reduced collision probability
*         matrices for the composite geometry.
* NSUPCE  number of cells.
* RAYRE   radius of the tubes in each cell.
* SIGT    total macroscopic cross sections.
* SIGW    P0 within-group scattering macroscopic cross sections.
* IQUAD   quadrature parameter.
* ISTAT   statistical approximation flag (set with ISTAT=1).
* NMC     offset of the first macro-volume in each cell.
* PROCEL  user supplied geometrical matrix.
* IMPX    print flag (equal to 0 for no print).
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* IPAS2   number of volumes in the macro geometry (IPAS2=NMC(NSUPCE+1)).
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUAB   quadrature parameter for the treatment of the micro volume
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
* NMCB    offset of the first composite volume in each generating cell.
*
*Parameters: output
* P       reduced collision probabilities.
*
*Parameters: scratch
*  IAPPAR
*  PIJW
*  PISW
*  PSJW
*  PSSB
*  PSSW
*  SIGT2
*  SIGW2
*  IPER
*
*-----------------------------------------------------------------------
*
* REFERENCE: A. HOFFMANN, F. JEANPIERRE, A. KAVENOKY, M. LIVOLANT
* AND H. LORAIN, 'APOLLO, CODE MULTIGROUPE DE RESOLUTION DE L'EQUATION
* DU TRANSPORT POUR LES NEUTRONS RAPIDES ET THERMIQUES', REPORT
* CEA-N-1610, COMMISSARIAT A L'ENERGIE ATOMIQUE, P. 13 (1973).
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPAS,NPIJ,NSUPCE,IQUAD,ISTAT,NMC(NSUPCE+1),IMPX,IR1,IR2,
     1 IPAS2,NG,NSMAX,MICRO,IQUAB,NS(NG),IBI(IPAS2),NMCB(NSUPCE+1),
     2 IAPPAR(IPAS),IPER(IPAS)
      REAL RAYRE(NSUPCE+IPAS),SIGT(IPAS),SIGW(IPAS),P(IPAS,IPAS),
     1 PROCEL(NSUPCE,NSUPCE),RS(NSMAX+1,NG),FRACT(NG,IR2),
     2 VOLK(NG,NSMAX),PIJW(NPIJ),PISW(IPAS),PSJW(IPAS),
     3 PSSB(NSUPCE,2*NSUPCE),PSSW(NSUPCE),SIGT2(IPAS),SIGW2(IPAS)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (PI=3.141592654)
      COMMON BASE(1)
*
      I1=0
      I2=IPAS2
      I3=0
      DO 30 IKK=1,NSUPCE
      J1=NMC(IKK)
      J2=NMC(IKK+1)-J1
      J1B=NMCB(IKK)
      J2B=NMCB(IKK+1)-J1B
      DO 10 I=1,J2
      I3=I3+1
      IPER(I3)=I1+I
      SIGT2(J1B+I)=SIGT(I1+I)
   10 SIGW2(J1B+I)=SIGW(I1+I)
      DO 20 I=1,J2B-J2
      I3=I3+1
      IPER(I3)=I2+I
      SIGT2(J1B+J2+I)=SIGT(I2+I)
   20 SIGW2(J1B+J2+I)=SIGW(I2+I)
      I1=I1+J2
   30 I2=I2+J2B-J2
*
      CALL SETARA(BASE,IPAS2,IPLS)
      CALL SETARA(BASE,IPAS2,IPSL)
      CALL SETARA(BASE,IPAS,IPIS)
      CALL SETARA(BASE,IPAS,IPSI)
      CALL SYBB01(NSUPCE,NPIJ,IPAS,IPAS2,SIGT2,SIGW2,IMPX,IQUAD,NMC,
     1 RAYRE,IR1,IR2,NG,NSMAX,MICRO,IQUAB,NS,IBI,RS,FRACT,VOLK,NMCB,
     2 PIJW,PISW,PSJW,PSSW,BASE(IPLS),BASE(IPSL),BASE(IPIS),BASE(IPSI))
      CALL RLSARA(BASE(IPSI))
      CALL RLSARA(BASE(IPIS))
      CALL RLSARA(BASE(IPSL))
      CALL RLSARA(BASE(IPLS))
*
      IPIJ=0
      DO 50 IKK=1,NSUPCE
      J1B=NMCB(IKK)
      J2B=NMCB(IKK+1)-J1B
      DO 40 I=1,J2B
      IAPPAR(I+J1B)=IKK
      DO 40 J=1,J2B
   40 P(IPER(J1B+I),IPER(J1B+J))=PIJW(IPIJ+(J-1)*J2B+I)
   50 IPIJ=IPIJ+J2B*J2B
*----
*  COMPUTATION OF QIJ MATRIX
*----
      IF(ISTAT.EQ.0) THEN
         DO 70 I=1,NSUPCE
         DO 60 J=1,NSUPCE
         PSSB(I,J)=-PROCEL(I,J)*PSSW(J)
   60    PSSB(I,NSUPCE+J)=PROCEL(I,J)
   70    PSSB(I,I)=1.0+PSSB(I,I)
         CALL ALSB(NSUPCE,NSUPCE,PSSB,IER,NSUPCE)
         IF(IER.NE.0) CALL XABORT('SYBBXE: SINGULAR MATRIX.')
*        COMPUTATION OF PIJ MATRIX.
         DO 80 I=1,IPAS
         K=IAPPAR(I)
         DO 80 J=1,IPAS
         L=IAPPAR(J)
         XX=PISW(I)*PSSB(K,NSUPCE+L)*PSJW(J)
         IF(L.EQ.K) THEN
            P(IPER(I),IPER(J))=P(IPER(I),IPER(J))+XX
         ELSE
            P(IPER(I),IPER(J))=XX
         ENDIF
   80    CONTINUE
      ELSE
         X1=0.0
         DO 90 I=1,NSUPCE
   90    X1=X1+PSSW(I)*PROCEL(1,I)
         X1=1.0/(1.0-X1)
         DO 100 J=1,IPAS
         L=IAPPAR(J)
         ZZZ=PROCEL(1,L)*X1*PSJW(J)
         DO 100 I=1,IPAS
         K=IAPPAR(I)
         XX=PISW(I)*ZZZ
         IF(L.EQ.K) THEN
            P(IPER(I),IPER(J))=P(IPER(I),IPER(J))+XX
         ELSE
            P(IPER(I),IPER(J))=XX
         ENDIF
  100    CONTINUE
      ENDIF
      RETURN
      END
