*DECK SYBCP2
      SUBROUTINE SYBCP2 (IPTRK,ITG,IMPX,NREG,SIGT,SIGW,IR1,IR2,NREG2,
     1 NG,NSMAX,MICRO,IQUA10,NS,IBI,RS,FRACT,VOLK,PIJ)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the scattering-reduced collision probabilities for
* geometries with double heterogeneity in Sybil.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   pointer to the Sybil tracking (L_TRACK signature).
* ITG     type of Sybil one-speed solution operator.
* IMPX    print flag (equal to zero for no print).
* NREG    number of volumes in the composite geometry.
* SIGT    total macroscopis cross sections in each volume of the
*         composite geometry.
* SIGW    P0 within-group scattering macroscopic cross sections
*         ordered by volume.
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* NREG2   number of volumes in the macro geometry.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUA10  quadrature parameter for the treatment of the micro volumes.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro
*         volumes.
*
*Parameters: output
* PIJ     scattering-reduced collision probabilities matrix.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,ITG,IMPX,NREG,IR1,IR2,NREG2,NG,NSMAX,MICRO,IQUA10,
     1 NS(NG),IBI(NREG2)
      REAL SIGT(NREG),SIGW(NREG),RS(NSMAX+1,NG),FRACT(NG,IR2),
     1 VOLK(NG,NSMAX),PIJ(NREG,NREG)
*----
*  LOCAL VARIABLES
*----
      INTEGER IPAR(16),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      IF(ITG.EQ.1) THEN
         CALL SETARA(BASE,NREG2,IRRR)
         CALL SETARA(BASE,NREG2,ISIGT0)
         CALL SETARA(BASE,NG*NSMAX*NREG2,IQKD)
*
         CALL SETARA(BASE,NSMAX*NSMAX,IQKN)
         CALL SYBHT1(IR1,IR2,NREG,SIGT,NREG2,NG,NSMAX,MICRO,IQUA10,NS,
     1   IBI,RS,FRACT,VOLK,PIJ,BASE(IRRR),BASE(ISIGT0),BASE(IQKD),
     2   BASE(IQKN))
         CALL RLSARA(BASE(IQKN))
*
         PIL=1.0/BASE(ISIGT0)
         CALL SYBHT2(IR1,IR2,NREG,NREG2,NG,NSMAX,NS,IBI,FRACT,VOLK,PIJ,
     1   BASE(IRRR),BASE(ISIGT0),BASE(IQKD),PIL)
         CALL RLSARA(BASE(ISIGT0))
         CALL RLSARA(BASE(IQKD))
         CALL RLSARA(BASE(IRRR))
         CALL SYBWIJ(NREG,NREG,SIGT,SIGW,PIJ)
      ELSE IF(ITG.EQ.2) THEN
         CALL LCMSIX(IPTRK,'PURE-GEOM',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         ITYPE=IPAR(1)
         IHEX=IPAR(2)
         IQUA2=IPAR(3)
         IF(ITYPE.GE.8) CALL LCMGET(IPTRK,'SIDE',SIDE)
         CALL LCMIOF(IPTRK,'XXX',IXX2)
         CALL LCMIOF(IPTRK,'YYY',IYY2)
         CALL LCMIOF(IPTRK,'ZZZ',IZZ2)
         CALL LCMIOF(IPTRK,'NCODE',INCODE)
         CALL LCMIOF(IPTRK,'ZCODE',IZCODE)
         CALL LCMSIX(IPTRK,' ',2)
*
         CALL SETARA(BASE,NREG2,IRRR)
         CALL SETARA(BASE,NREG2,ISIGT0)
         CALL SETARA(BASE,NG*NSMAX*NREG2,IQKD)
*
         CALL SETARA(BASE,NSMAX*NSMAX,IQKN)
         CALL SYBHT1(IR1,IR2,NREG,SIGT,NREG2,NG,NSMAX,MICRO,IQUA10,NS,
     1   IBI,RS,FRACT,VOLK,PIJ,BASE(IRRR),BASE(ISIGT0),BASE(IQKD),
     2   BASE(IQKN))
         CALL RLSARA(BASE(IQKN))
*
         CALL SETARA(BASE,NREG2*NREG2,IPIL)
*
         CALL SETARA(BASE,6*NREG2,IX)
         IF(ITYPE.EQ.2) THEN
            CALL SETARA(BASE,4*NREG2*NREG2,IF2)
            CALL SYBALP(NREG2,NREG2,BASE(IXX2),BASE(ISIGT0),
     1      BASE(INCODE),BASE(IZCODE),BASE(IPIL),BASE(IX),BASE(IF2))
            CALL RLSARA(BASE(IF2))
         ELSE IF(ITYPE.EQ.3) THEN
            CALL SETARA(BASE,1+IQUA2*((NREG2*(5+NREG2))/2),IZ)
            CALL SYBT1D(NREG2,BASE(IXX2),.FALSE.,IQUA2,BASE(IZ))
            CALL SYBALC(NREG2,NREG2,BASE(IXX2),BASE(ISIGT0),IQUA2,
     1      BASE(IZCODE+1),BASE(IZ),BASE(IPIL),BASE(IX))
            CALL RLSARA(BASE(IZ))
         ELSE IF(ITYPE.EQ.4) THEN
            CALL SETARA(BASE,1+IQUA2*((NREG2*(5+NREG2))/2),IZ)
            CALL SYBT1D(NREG2,BASE(IXX2),.TRUE.,IQUA2,BASE(IZ))
            CALL SYBALS(NREG2,NREG2,BASE(IXX2),BASE(ISIGT0),IQUA2,
     1      BASE(IZCODE+1),BASE(IZ),BASE(IPIL),BASE(IX))
            CALL RLSARA(BASE(IZ))
         ENDIF
         CALL RLSARA(BASE(IX))
*
         CALL SYBHT2(IR1,IR2,NREG,NREG2,NG,NSMAX,NS,IBI,FRACT,VOLK,PIJ,
     1   BASE(IRRR),BASE(ISIGT0),BASE(IQKD),BASE(IPIL))
         CALL RLSARA(BASE(IPIL))
         CALL RLSARA(BASE(ISIGT0))
         CALL RLSARA(BASE(IQKD))
         CALL RLSARA(BASE(IRRR))
         CALL SYBWIJ(NREG,NREG,SIGT,SIGW,PIJ)
      ELSE IF(ITG.EQ.3) THEN
         CALL LCMSIX(IPTRK,'DOITYOURSELF',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         NSUPCE=IPAR(1)
         IQUA3=IPAR(2)
         ISTAT=IPAR(3)
         CALL LCMIOF(IPTRK,'NMC',INMC3)
         CALL LCMIOF(IPTRK,'RAYRE',IRAYR3)
         CALL LCMIOF(IPTRK,'PROCEL',IPROCE)
         CALL LCMSIX(IPTRK,' ',2)
*
         CALL SETARA(IBASE,NSUPCE+1,INMCB)
         NPIJ=0
         IBASE(INMCB)=0
         DO 30 IKG=1,NSUPCE
         J1=IBASE(INMC3+IKG-1)
         J2=IBASE(INMC3+IKG)-J1
         DO 20 I=1,J2
         IF(IBI(J1+I).GT.0) THEN
            DO 10 J=1,NG
            IF(FRACT(J,IBI(J1+I)).GT.0.00001) J2=J2+NS(J)
   10       CONTINUE
         ENDIF
   20    CONTINUE
         IBASE(INMCB+IKG)=IBASE(INMCB+IKG-1)+J2
   30    NPIJ=NPIJ+J2*J2
         IF(NREG.NE.IBASE(INMCB+NSUPCE)) CALL XABORT('SYBCP2: ABORT1.')
         IF(NREG2.NE.IBASE(INMC3+NSUPCE)) CALL XABORT('SYBCP2: ABORT2.')
         CALL SETARA(BASE,NREG,IIAP)
         CALL SETARA(BASE,NPIJ,IPIJW)
         CALL SETARA(BASE,NREG,IPISW)
         CALL SETARA(BASE,NREG,IPSJW)
         CALL SETARA(BASE,2*NSUPCE**2,IPSSB)
         CALL SETARA(BASE,NSUPCE,IPSSW)
         CALL SETARA(BASE,NREG,ISIGT2)
         CALL SETARA(BASE,NREG,ISIGW2)
         CALL SETARA(BASE,NREG,IIPER)
         CALL SYBBXE(NREG,NPIJ,NSUPCE,BASE(IRAYR3),SIGT,SIGW,PIJ,IQUA3,
     1   ISTAT,IBASE(INMC3),BASE(IPROCE),IMPX,IR1,IR2,NREG2,NG,NSMAX,
     2   MICRO,IQUA10,NS,IBI,RS,FRACT,VOLK,IBASE(INMCB),BASE(IIAP),
     3   BASE(IPIJW),BASE(IPISW),BASE(IPSJW),BASE(IPSSB),BASE(IPSSW),
     4   BASE(ISIGT2),BASE(ISIGW2),BASE(IIPER))
         CALL RLSARA(BASE(IIPER))
         CALL RLSARA(BASE(ISIGW2))
         CALL RLSARA(BASE(ISIGT2))
         CALL RLSARA(BASE(IPSSW))
         CALL RLSARA(BASE(IPSSB))
         CALL RLSARA(BASE(IPSJW))
         CALL RLSARA(BASE(IPISW))
         CALL RLSARA(BASE(IPIJW))
         CALL RLSARA(BASE(IIAP))
         CALL RLSARA(IBASE(INMCB))
      ELSE IF(ITG.EQ.4) THEN
         CALL LCMSIX(IPTRK,'EURYDICE',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         IHEX=IPAR(1)
         MULTC=IPAR(2)
         IWIGN=IPAR(3)
         NMCEL=IPAR(4)
         NMERGE=IPAR(5)
         NGEN=IPAR(6)
         IJAT=IPAR(7)
         LMAILI=IPAR(15)
         LMAILR=IPAR(16)
         CALL LCMIOF(IPTRK,'XX',IXX4)
         CALL LCMIOF(IPTRK,'YY',IYY4)
         CALL LCMIOF(IPTRK,'LSECT',ILSEC4)
         CALL LCMIOF(IPTRK,'NMC',INMC4)
         CALL LCMIOF(IPTRK,'NMCR',INMCR4)
         CALL LCMIOF(IPTRK,'RAYRE',IRAYR4)
         CALL LCMIOF(IPTRK,'MAIL',IMAIL)
         IF(LMAILI.GT.0) THEN
            CALL LCMIOF(IPTRK,'ZMAILI',IIZMAI)
         ELSE
*           THIS INFO IS NOT REQUIRED IN THE CALLED ROUTINE.
            IIZMAI=IMAIL
         ENDIF
         IF(LMAILR.GT.0) THEN
            CALL LCMIOF(IPTRK,'ZMAILR',IRZMAI)
         ELSE
*           THIS INFO IS NOT REQUIRED IN THE CALLED ROUTINE.
            IRZMAI=IMAIL
         ENDIF
         CALL LCMIOF(IPTRK,'IFR',IIFR)
         CALL LCMIOF(IPTRK,'ALB',IALB)
         CALL LCMIOF(IPTRK,'INUM',IINUM)
         CALL LCMIOF(IPTRK,'MIX',IMIX)
         CALL LCMIOF(IPTRK,'DVX',IDVX)
         CALL LCMIOF(IPTRK,'IGEN',IIGEN)
         CALL LCMSIX(IPTRK,' ',2)
*
         CALL SETARA(IBASE,NGEN+1,INMCB)
         CALL SETARA(IBASE,NREG2,IIBI2)
         NCOUR=4
         IF(IHEX.NE.0) NCOUR=6
         J3=0
         DO 50 IKK=1,NMERGE
         IKG=IBASE(IIGEN+IKK-1)
         J1=IBASE(INMC4+IKG-1)
         J2=IBASE(INMC4+IKG)-J1
         DO 40 I=1,J2
   40    IBASE(IIBI2+J1+I-1)=IBI(J3+I)
   50    J3=J3+J2
         NPIJ=0
         NPIS=0
         IBASE(INMCB)=0
         DO 80 IKG=1,NGEN
         J1=IBASE(INMC4+IKG-1)
         J2=IBASE(INMC4+IKG)-J1
         DO 70 I=1,J2
         IF(IBASE(IIBI2+J1+I-1).GT.0) THEN
            DO 60 J=1,NG
            IF(FRACT(J,IBASE(IIBI2+J1+I-1)).GT.0.00001) J2=J2+NS(J)
   60       CONTINUE
         ENDIF
   70    CONTINUE
         IBASE(INMCB+IKG)=IBASE(INMCB+IKG-1)+J2
         NPIJ=NPIJ+J2*J2
   80    NPIS=NPIS+J2
         NPIS0=IBASE(INMC4+NGEN)
         NRAYRE=IBASE(INMCR4+NGEN)
         CALL SETARA(BASE,2*IJAT*IJAT,IPSSB)
         CALL SETARA(BASE,NPIS,ISIGT2)
         CALL SETARA(BASE,NPIS,ISIGW2)
         CALL SETARA(BASE,NREG,IIPER)
         IF(MULTC.EQ.1) THEN
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,NPIS,IPISW)
            CALL SETARA(BASE,NPIS,IPSJW)
            CALL SETARA(BASE,NGEN,IPSSW)
            CALL SYBBX2(NREG,NPIJ,NPIS,NPIS0,SIGT,SIGW,PIJ,IMPX,NCOUR,
     1      IWIGN,NMCEL,NMERGE,NGEN,IPAR(8),BASE(IXX4),BASE(IYY4),
     2      IBASE(INMC4),BASE(IRAYR4),IBASE(IMAIL),BASE(IRZMAI),
     3      IBASE(IIFR),BASE(IALB),IBASE(IINUM),IBASE(IIGEN),IR1,
     4      IR2,NREG2,NG,NSMAX,MICRO,IQUA10,NS,IBI,RS,FRACT,VOLK,
     5      IBASE(INMCB),BASE(IPSSB),BASE(ISIGT2),BASE(ISIGW2),
     6      BASE(IPIJW),BASE(IPISW),BASE(IPSJW),BASE(IPSSW),
     7      BASE(IIBI2),BASE(IIPER))
         ELSE
            IF(MULTC.EQ.4) NCOUR=3*NCOUR
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,NCOUR*NPIS,IPISW)
            CALL SETARA(BASE,NCOUR*NPIS,IPSJW)
            CALL SETARA(BASE,NGEN*NCOUR*NCOUR,IPSSW)
            CALL SYBBX3(MULTC,NREG,NPIJ,NPIS,NPIS0,NRAYRE,SIGT,SIGW,PIJ,
     1      IMPX,NCOUR,IWIGN,NMCEL,NMERGE,NGEN,IJAT,IPAR(8),BASE(IXX4),
     2      BASE(IYY4),IBASE(ILSEC4),IBASE(INMC4),IBASE(INMCR4),
     3      BASE(IRAYR4),IBASE(IMAIL),BASE(IIZMAI),BASE(IRZMAI),
     4      IBASE(IIFR),BASE(IALB),IBASE(IINUM),IBASE(IMIX),BASE(IDVX),
     5      IBASE(IIGEN),IR1,IR2,NREG2,NG,NSMAX,MICRO,IQUA10,NS,IBI,RS,
     6      FRACT,VOLK,IBASE(INMCB),BASE(IPSSB),BASE(ISIGT2),
     7      BASE(ISIGW2),BASE(IPIJW),BASE(IPISW),BASE(IPSJW),
     8      BASE(IPSSW),BASE(IIBI2),BASE(IIPER))
         ENDIF
         CALL RLSARA(BASE(IPSSW))
         CALL RLSARA(BASE(IPSJW))
         CALL RLSARA(BASE(IPISW))
         CALL RLSARA(BASE(IPIJW))
         CALL RLSARA(BASE(IIPER))
         CALL RLSARA(BASE(ISIGW2))
         CALL RLSARA(BASE(ISIGT2))
         CALL RLSARA(BASE(IPSSB))
         CALL RLSARA(IBASE(IIBI2))
         CALL RLSARA(IBASE(INMCB))
      ELSE
         CALL XABORT('SYBCP2: UNKNOWN CP MODULE.')
      ENDIF
      RETURN
      END
