*DECK SYBHN1
      SUBROUTINE SYBHN1(NR,Z,ZI,COTE,SIGT,TRONC,R,LGZERO,PIS,PSS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute DP-0 leakage and transmission probabilities for heterogeneous
* hexagonal 2D geometry.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): M. Ouisloumen
*
*Parameters: input
*  NR      number regions in the cell.
*  COTE    length of one of sides of the hexagon.
*  SIGT    total cross sections.
*  TRONC   voided block cutoff criterion.
*  R       radius of the tubes.
*  Z       real integration mesh.
*  ZI      integer integration mesh.
*
*Parameters: output
*  PIS     leakage probability:
*          PIS(j,i)  -> volume i to side j, (i=1,nr;j=1,6).
*  PSS     transmission probability:
*          PSS(i,j) -> side i to side j, (i=1,6;j=1,6).
*
*Parameters: scratch
*  LGZERO
*
*-----------------------------------------------------------------------
*
*                                        side a
*                   side 2               dir a -> isotropic
*                  xxxxxxxx
*                 x        x
*        side 3  x          x side 1
*               x            x
*              x              x
*               x            x
*        side 4  x          x side 6
*                 x        x
*                  xxxxxxxx
*                   side 5
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   NR,ZI(*)
      REAL      Z(*),COTE,SIGT(NR),TRONC,R(NR),PIS(6,NR),PSS(6,6)
      LOGICAL   LGZERO(NR)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MKI3=600,MKI4=600,MKI5=600)
      PARAMETER (SQRT3=1.732050807568877,PI=3.141592653589793)
*
      REAL      P(3)
      INTEGER   IROT(6,6)
      LOGICAL   LPASS
*----
*  ASSUME THAT BICKLEY KI TABLES HAVE THE SAME TABULATION POINTS AND
*  THE SAME TRUNCATION LIMIT.
*----
      COMMON /BICKL3/BI3(0:MKI3),BI31(0:MKI3),BI32(0:MKI3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4),BI41(0:MKI4),BI42(0:MKI4),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5),BI51(0:MKI5),BI52(0:MKI5),PAS5,XLIM5,L5
*
      SAVE IROT
      DATA IROT /
     +          0, 1, 2, 3, 2, 1,
     +          1, 0, 1, 2, 3, 2,
     +          2, 1, 0, 1, 2, 3,
     +          3, 2, 1, 0, 1, 2,
     +          2, 3, 2, 1, 0, 1,
     +          1, 2, 3, 2, 1, 0/
*
      FUNC3(X,K)=BI3(K)+X*(BI31(K)+X*BI32(K))
      FUNC4(X,K)=BI4(K)+X*(BI41(K)+X*BI42(K))
      FUNC5(X,K)=BI5(K)+X*(BI51(K)+X*BI52(K))
*----
*  INITIALIZATION
*----
      DO 40 I=1,3
      P(I)=0.
   40 CONTINUE
      VOID=TRONC/(SQRT3*COTE)
      DO 10 I=1,NR
           IF(SIGT(I).LE.VOID) THEN
              LGZERO(I)=.TRUE.
           ELSE
              LGZERO(I)=.FALSE.
           ENDIF
      DO 10 J=1,6
           PIS(J,I)=0.
   10 CONTINUE
*
      NA=ZI(1)
      NX=ZI(2)
      IZ0=2
      IZ=4
*
          DO 150 IA=1,NA
      IZ0=IZ0+3
      NR5=ZI(IZ0)
      IMIN=ZI(IZ0-1)
      IMAX=NR5+IMIN-1
      IMIN6=ZI(IZ0-2)
      IZZ1=IZ
      IZ=IZ+NR5+IMIN+NR-IMIN6+1
*
         DO 120 IX=1,NX
      IZ=IZ+9
      W600=Z(IZ)
*----
*  TRANSMISSION PAR LES DIFFERENTES FACES 6,5,4, SUIVANT LA VALEUR DE LB
*     LB=1  ==> FACE 6
*     LB=2  ==> FACE 5
*     LB=3  ==> FACE 4
*
*----
      IMINX=IMIN6
      LB=1
      NMAX=NR
      WW600=W600
      IZ1=IZZ1
*
   22 PKI3=0.
*
          DO 77 I=NMAX,IMINX,-1
      IZ=IZ+1
      TAU0=SIGT(I)*Z(IZ)
      DO 33 IPP=I+1,NR
         IZ=IZ+1
         TAU0=TAU0+SIGT(IPP)*Z(IZ)
   33 CONTINUE
      IZ=IZ+1
      IZZ=IZ
      RC=Z(IZ)
      IZ=IZ-1
      RMC=Z(IZ)-RC
      IZ1=IZ1+1
      DELR=Z(IZ1)
      DW600=DELR*WW600
      F3D0=BI3(0)
      F2D0=1.
      IF(TAU0.LT.XLIM3) THEN
           K=NINT(TAU0*PAS3)
           F3DP=FUNC3(TAU0,K)
           F2DP=(4.*FUNC5(TAU0,K)-3.*F3DP)/TAU0+FUNC4(TAU0,K)
*----
*  PSS CALCULATION
*----
           PKI3=PKI3+DELR*F3DP
      ELSE
           F2DP=0.
           F3DP=0.
      ENDIF
      F2DM=F2DP
      F3DM=F3DP
      F2UP=F2DP
      F3UP=F3DP
      F2UM=F2DP
      F3UM=F3DP
      F3U0=F3D0
      F2U0=F2D0
      TAUPD0=0.
      TAUPU0=0.
      TAUUM=TAU0
      TAUDM=TAU0
      LPASS=.FALSE.
      R1=RC
      R2=RMC
          DO 44 J=NR,I+1,-1
      IF(LGZERO(J)) THEN
         PIS(1,J)=PIS(1,J)+DW600*(R1*F2D0+R2*F2DM+R2*F2U0+R1*F2UM)
      ELSE
         TAU2=SIGT(J)*R2
         TAU1=SIGT(J)*R1
         TAUDM=TAUDM-TAU2
         TAUPD0=TAUPD0+TAU1
         TAUUM=TAUUM-TAU1
         TAUPU0=TAUPU0+TAU2
         IF(TAUDM.LT.XLIM3) THEN
              K=NINT(PAS3*TAUDM)
              F3DM=FUNC3(TAUDM,K)
              F2DM=(4.*FUNC5(TAUDM,K)-3.*F3DM)/TAUDM+FUNC4(TAUDM,K)
         ELSE
              F3DM=0.
              F2DM=0.
         ENDIF
         F3D=0.
         F2D=0.
         IF(TAUPD0.LT.XLIM3) THEN
               IF(TAUPD0.EQ.0.) THEN
                 F3D=BI3(0)
                 F2D=1.
               ELSE
                 K=NINT(PAS3*TAUPD0)
                 F3D=FUNC3(TAUPD0,K)
                 F2D=(4.*FUNC5(TAUPD0,K)-3.*F3D)/TAUPD0+FUNC4(TAUPD0,K)
               ENDIF
         ENDIF
         IF(TAUUM.LT.XLIM3) THEN
             K=NINT(PAS3*TAUUM)
             F3UM=FUNC3(TAUUM,K)
             F2UM=(4.*FUNC5(TAUUM,K)-3.*F3UM)/TAUUM+FUNC4(TAUUM,K)
         ELSE
             F3UM=0.
             F2UM=0.
         ENDIF
         F3U=0.
         F2U=0.
         IF(TAUPU0.LT.XLIM3) THEN
             IF(TAUPU0.EQ.0.) THEN
                F3U=BI3(0)
                F2U=1.
             ELSE
                K=NINT(PAS3*TAUPU0)
                F3U=FUNC3(TAUPU0,K)
                F2U=(4.*FUNC5(TAUPU0,K)-3.*F3U)/TAUPU0+FUNC4(TAUPU0,K)
             ENDIF
         ENDIF
         PIS(1,J)=PIS(1,J)+DW600*(F3D0-F3D+F3DM-F3DP
     +                           +F3U0-F3U+F3UM-F3UP)
         F3D0=F3D
         F2D0=F2D
         F3U0=F3U
         F2U0=F2U
         F3DP=F3DM
         F3UP=F3UM
      ENDIF
      IZ=IZ-1
      R1=.5*Z(IZ)
      R2=R1
      LPASS=.TRUE.
   44     CONTINUE
*
      IF(LGZERO(I)) THEN
          IF(LPASS) RC=2.*R1
          PIS(1,I)=PIS(1,I)+DW600*RC*(F2D0+F2U0)
      ELSE
          PIS(1,I)=PIS(1,I)+DW600*(F3D0-F3DP+F3U0-F3UP)
      ENDIF
      IZ=IZZ
   77     CONTINUE
*
      GOTO (55,56,57),LB
*
   55 P(1)=P(1)+W600*PKI3
      LB=2
      IMINX=IMIN
      NMAX=IMAX
      GOTO 22
*
   56 P(2)=P(2)+W600*PKI3
      LB=3
      IMINX=1
      NMAX=IMIN
      WW600=.5*W600
      GOTO 22
*
   57 P(3)=P(3)+W600*PKI3
*
  120    CONTINUE
  150     CONTINUE
*----
*  PSS ORTHONORMALIZATION
*----
      E1=Z(1)
      AUX=1./(6.*COTE)
      P(1)=P(1)*AUX*E1*E1
      P(2)=P(2)*AUX*E1*E1
      P(3)=P(3)*AUX*E1*E1
*----
*  PIS ORTHONORMALIZATION
*----
      S4=COTE*.25
      VAUX=0.
      DO 200 I=1,NR-1
         VV=PI*R(I)*R(I)
         V=VV-VAUX
         VAUX=VV
         COEF=S4/V
         IF(LGZERO(I)) THEN
           PIS(1,I)=COEF*AUX*E1*E1*PIS(1,I)
         ELSE
           SIGTI=COEF/SIGT(I)
           PIS(1,I)=AUX*SIGTI*E1*E1*PIS(1,I)
         ENDIF
  200 CONTINUE
         COEF=S4/(1.5*SQRT3*COTE*COTE-VAUX)
         IF(LGZERO(NR)) THEN
           PIS(1,NR)=COEF*AUX*E1*E1*PIS(1,NR)
         ELSE
           SIGTI=COEF/SIGT(NR)
           PIS(1,NR)=AUX*SIGTI*E1*E1*PIS(1,NR)
         ENDIF
*
*                  *STOCKAGE DES PIS ET DES PSS*
*----
*  TRANSMISSION MATRIX
*----
      DO  58 I=1,6
      DO  58 J=1,6
      IB=IROT(J,I)
      IF(IB.GT.0) THEN
         PSS(I,J)=P(IB)
      ELSE
         PSS(I,J)=0.
      ENDIF
   58 CONTINUE
*----
*  LEAKAGE MARTIX
*----
      DO 61 J=1,NR
      DO 61 I=2,6
        PIS(I,J)=PIS(1,J)
   61 CONTINUE
*
      RETURN
      END
