*DECK SYBHN2
      SUBROUTINE SYBHN2(NR,Z,ZI,COTE,SIGT,TRONC,R,LGZERO,PIS,PSS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute DP-1 leakage and transmission probabilities for heterogeneous
* hexagonal 2D geometry.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): M. Ouisloumen
*
*Parameters: input
*  NR      number regions in the cell.
*  COTE    length of one of sides of the hexagon.
*  SIGT    total cross sections.
*  TRONC   voided block cutoff criterion.
*  R       radius of the tubes.
*  Z       real integration mesh.
*  ZI      integer integration mesh.
*
*Parameters: output
*  PIS     leakage probability:
*          PIS(j,i)  -> volume i to side j, (i=1,nr;j=1,18).
*  PSS     transmission probability:
*          PSS(i,j) -> side i to side j, (i=1,18;j=1,18).
*
*Parameters: scratch
*  LGZERO
*
*-----------------------------------------------------------------------
*
*                                        side a,b,c
*                 side 4,5,6             dir a -> isotropic
*                  xxxxxxxx              dir c -> tangent to surface
*                 x        x             dir b -> normal  to surface
*   side 7,8,9   x          x side 1,2,3
*               x            x
*              x              x
*               x            x
* side 10,11,12  x          x side 16,17,18
*                 x        x
*                  xxxxxxxx
*                side 13,14,15
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   NR,ZI(*)
      REAL      Z(*),COTE,SIGT(NR),TRONC,R(NR),PIS(18,NR),PSS(18,18)
      LOGICAL   LGZERO(NR)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MKI3=600,MKI4=600,MKI5=600)
      PARAMETER (SQRT3=1.732050807568877,SQRT2=1.414213562373095,
     +           PI=3.141592653589793)
*
      REAL      P(16)
      INTEGER   IROT(18,18)
      LOGICAL   LPASS
*----
*  ASSUME THAT BICKLEY KI TABLES HAVE THE SAME TABULATION POINTS AND
*  THE SAME TRUNCATION LIMIT
*----
      COMMON /BICKL3/BI3(0:MKI3),BI31(0:MKI3),BI32(0:MKI3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4),BI41(0:MKI4),BI42(0:MKI4),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5),BI51(0:MKI5),BI52(0:MKI5),PAS5,XLIM5,L5
*
      SAVE IROT
      DATA IROT /
     +          0, 0, 0, 1, 2,-3, 7, 8,-9,13,14, 0, 7, 8, 9, 1, 2, 3,
     +          0, 0, 0, 2, 4, 5, 8,10,11,14,15, 0, 8,10,-11, 2, 4,-5,
     +          0, 0, 0, 3,-5, 6, 9,-11,12,0, 0,16,-9,11,12,-3, 5,6,
     +          1, 2, 3, 0, 0, 0, 1, 2,-3, 7, 8,-9,13,14, 0, 7, 8, 9,
     +          2, 4,-5, 0, 0, 0, 2, 4, 5, 8,10,11,14,15, 0, 8,10,-11,
     +         -3, 5, 6, 0, 0, 0, 3,-5, 6, 9,-11,12,0, 0,16,-9,11,12,
     +          7, 8, 9, 1, 2, 3, 0, 0, 0, 1, 2,-3, 7, 8,-9,13,14, 0,
     +          8,10,-11, 2, 4,-5, 0, 0, 0, 2, 4, 5, 8,10,11,14,15, 0,
     +         -9,11,12,-3, 5, 6, 0, 0, 0, 3,-5, 6, 9,-11,12,0, 0,16,
     +          13,14, 0, 7, 8, 9, 1, 2, 3, 0, 0, 0, 1, 2,-3, 7, 8,-9,
     +          14,15, 0, 8,10,-11, 2, 4,-5, 0, 0, 0, 2, 4, 5, 8,10,11,
     +          0, 0,16,-9,11,12,-3, 5, 6, 0, 0, 0, 3,-5, 6, 9,-11,12,
     +          7, 8,-9, 13,14, 0, 7, 8, 9, 1, 2, 3, 0, 0, 0, 1, 2,-3,
     +          8,10,11,14,15, 0, 8,10,-11, 2, 4,-5, 0, 0, 0, 2, 4, 5,
     +          9,-11,12,  0, 0,16,-9,11,12,-3, 5, 6, 0, 0, 0, 3,-5, 6,
     +          1, 2,-3, 7, 8,-9, 13,14, 0, 7, 8, 9, 1, 2, 3, 0, 0, 0,
     +          2, 4, 5, 8,10,11,14,15, 0, 8,10,-11, 2, 4,-5, 0, 0, 0,
     +          3,-5, 6, 9,-11,12,  0, 0,16,-9,11,12,-3, 5, 6, 0, 0, 0/
*
      FUNC3(X,K)=BI3(K)+X*(BI31(K)+X*BI32(K))
      FUNC4(X,K)=BI4(K)+X*(BI41(K)+X*BI42(K))
      FUNC5(X,K)=BI5(K)+X*(BI51(K)+X*BI52(K))
*----
*  INITIALISATION
*----
      DO 40 I=1,16
      P(I)=0.
   40 CONTINUE
      VOID=TRONC/(SQRT3*COTE)
      DO 10 I=1,NR
           IF(SIGT(I).LE.VOID) THEN
              LGZERO(I)=.TRUE.
           ELSE
              LGZERO(I)=.FALSE.
           ENDIF
      DO 10 J=1,18
           PIS(J,I)=0.
   10 CONTINUE
*
      NA=ZI(1)
      NX=ZI(2)
      IZ0=2
      IZ=4
*
          DO 150 IA=1,NA
      IZ0=IZ0+3
      NR5=ZI(IZ0)
      IMIN=ZI(IZ0-1)
      IMAX=NR5+IMIN-1
      IMIN6=ZI(IZ0-2)
      IZZ1=IZ
      IZ=IZ+NR5+IMIN+NR-IMIN6+1
*
         DO 120 IX=1,NX
      IZ=IZ+9
      W600=Z(IZ)
      W411=Z(IZ-1)
      W420=Z(IZ-2)
      W511=Z(IZ-3)
      W611=Z(IZ-4)
      W610=Z(IZ-5)
      WW1=Z(IZ-6)
      WW2=Z(IZ-7)
      WW3=Z(IZ-8)
*----
*  TRANSMISSION PAR LES DIFFERENTES FACES 6,5,4, SUIVANT LA VALEUR DE LB
*     LB=1  ==> FACE 6
*     LB=2  ==> FACE 5
*     LB=3  ==> FACE 4
*----
      IMINX=IMIN6
      LB=1
      NMAX=NR
      IZ1=IZZ1
      W1=WW1
      W2=WW3
      WW600=W600
*
   22 PKI3=0.
      PKI4=0.
      PKI5=0.
*
          DO 77 I=NMAX,IMINX,-1
      IZ=IZ+1
      TAU0=SIGT(I)*Z(IZ)
      DO 33 IPP=I+1,NR
         IZ=IZ+1
         TAU0=TAU0+SIGT(IPP)*Z(IZ)
   33 CONTINUE
      IZ=IZ+1
      IZZ=IZ
      RC=Z(IZ)
      IZ=IZ-1
      RMC=Z(IZ)-RC
      IZ1=IZ1+1
      DELR=Z(IZ1)
      DW600=DELR*WW600
      DW1=DELR*W1
      DW2=DELR*W2
      F3D0=BI3(0)
      F4D0=BI4(0)
      F2D0=1.
      IF(TAU0.LT.XLIM3) THEN
           K=NINT(TAU0*PAS3)
           F3DP=FUNC3(TAU0,K)
           F4DP=FUNC4(TAU0,K)
           F5DP=FUNC5(TAU0,K)
           F2DP=(4.*F5DP-3.*F3DP)/TAU0+F4DP
*----
*  PSS CALCULATION
*----
           PKI3=PKI3+DELR*F3DP
           PKI4=PKI4+DELR*F4DP
           PKI5=PKI5+DELR*F5DP
      ELSE
           F2DP=0.
           F3DP=0.
           F4DP=0.
      ENDIF
      F2DM=F2DP
      F3DM=F3DP
      F4DM=F4DP
      F2UP=F2DP
      F3UP=F3DP
      F4UP=F4DP
      F2UM=F2DP
      F3UM=F3DP
      F4UM=F4DP
      F3U0=F3D0
      F4U0=F4D0
      F2U0=F2D0
      TAUPD0=0.
      TAUPU0=0.
      TAUUM=TAU0
      TAUDM=TAU0
      LPASS=.FALSE.
      R1=RC
      R2=RMC
          DO 44 J=NR,I+1,-1
      IF(LGZERO(J)) THEN
      PIS(1,J)=PIS(1,J)+DW600*(R1*F2D0+R2*F2DM+R2*F2U0+R1*F2UM)
      PIS(2,J)=PIS(2,J)+DW1*(R1*F3D0+R2*F3DM)+DW2*(R2*F3U0+R1*F3UM)
      ELSE
         TAU2=SIGT(J)*R2
         TAU1=SIGT(J)*R1
         TAUDM=TAUDM-TAU2
         TAUPD0=TAUPD0+TAU1
         TAUUM=TAUUM-TAU1
         TAUPU0=TAUPU0+TAU2
         IF(TAUDM.LT.XLIM3) THEN
              K=NINT(PAS3*TAUDM)
              F3DM=FUNC3(TAUDM,K)
              F4DM=FUNC4(TAUDM,K)
              F2DM=(4.*FUNC5(TAUDM,K)-3.*F3DM)/TAUDM+F4DM
         ELSE
              F3DM=0.
              F4DM=0.
              F2DM=0.
         ENDIF
         F3D=0.
         F4D=0.
         F2D=0.
         IF(TAUPD0.LT.XLIM3) THEN
               IF(TAUPD0.EQ.0.) THEN
                  F3D=BI3(0)
                  F4D=BI4(0)
                  F2D=1.
               ELSE
                  K=NINT(PAS3*TAUPD0)
                  F3D=FUNC3(TAUPD0,K)
                  F4D=FUNC4(TAUPD0,K)
                  F2D=(4.*FUNC5(TAUPD0,K)-3.*F3D)/TAUPD0+F4D
               ENDIF
         ENDIF
         IF(TAUUM.LT.XLIM3) THEN
             K=NINT(PAS3*TAUUM)
             F3UM=FUNC3(TAUUM,K)
             F4UM=FUNC4(TAUUM,K)
             F2UM=(4.*FUNC5(TAUUM,K)-3.*F3UM)/TAUUM+F4UM
         ELSE
             F3UM=0.
             F4UM=0.
             F2UM=0.
         ENDIF
         F3U=0.
         F4U=0.
         F2U=0.
         IF(TAUPU0.LT.XLIM3) THEN
             IF(TAUPU0.EQ.0.) THEN
                F3U=BI3(0)
                F4U=BI4(0)
                F2U=1.
             ELSE
                K=NINT(PAS3*TAUPU0)
                F3U=FUNC3(TAUPU0,K)
                F4U=FUNC4(TAUPU0,K)
                F2U=(4.*FUNC5(TAUPU0,K)-3.*F3U)/TAUPU0+F4U
             ENDIF
         ENDIF
         PIS(1,J)=PIS(1,J)+DW600*(F3D0-F3D+F3DM-F3DP
     +                           +F3U0-F3U+F3UM-F3UP)
         PIS(2,J)=PIS(2,J)+DW1*(F4D0-F4D+F4DM-F4DP)
     +                    +DW2*(F4U0-F4U+F4UM-F4UP)
         F3D0=F3D
         F4D0=F4D
         F2D0=F2D
         F3U0=F3U
         F4U0=F4U
         F2U0=F2U
         F3DP=F3DM
         F4DP=F4DM
         F3UP=F3UM
         F4UP=F4UM
      ENDIF
      IZ=IZ-1
      R1=.5*Z(IZ)
      R2=R1
      LPASS=.TRUE.
   44     CONTINUE
*
      IF(LGZERO(I)) THEN
          IF(LPASS) RC=2.*R1
          PIS(1,I)=PIS(1,I)+DW600*RC*(F2D0+F2U0)
          PIS(2,I)=PIS(2,I)+RC*(DW1*F3D0+DW2*F3U0)
      ELSE
          PIS(1,I)=PIS(1,I)+DW600*(F3D0-F3DP+F3U0-F3UP)
          PIS(2,I)=PIS(2,I)+DW1*(F4D0-F4DP)+DW2*(F4U0-F4UP)
      ENDIF
      IZ=IZZ
   77     CONTINUE
*
      GOTO (55,56,57),LB
*
   55 P(1)=P(1)+W600*PKI3
      P(3)=P(3)+W610*PKI4
      P(6)=P(6)+W611*PKI5
      P(5)=P(5)+W600*PKI5
      LB=2
      IMINX=IMIN
      NMAX=IMAX
      W1=W420
      W2=WW2
      GOTO 22
*
   56 P(7)=P(7)+W600*PKI3
      P(9)=P(9)+W610*PKI4
      P(12)=P(12)+W511*PKI5
      P(11)=P(11)+W600*PKI5
      LB=3
      IMINX=1
      NMAX=IMIN
      W1=W420*.5
      W2=W1
      WW600=W600*.5
      GOTO 22
*
   57 P(13)=P(13)+W600*PKI3
      P(14)=P(14)+W420*PKI4
      P(16)=P(16)+W411*PKI5
      P(15)=P(15)+(W600-W411)*PKI5
*
  120    CONTINUE
  150     CONTINUE
*----
*  PSS ORTHONORMALIZATION
*----
      E1=Z(1)
      E2=Z(2)
      E3=Z(3)
      E4=Z(4)
      AUX=1./(6.*COTE)
      P13=P(13)*AUX
      P(1)=P(1)*AUX*E1*E1
      P(7)=P(7)*AUX*E1*E1
      P(13)=P13*E1*E1
      P(3)=-.25*P(3)*AUX*E1*E4*SQRT3
      P(9)=-.25*P(9)*AUX*E1*E4
      P(14)=E1*(AUX*P(14)*E2-E3*P13)
      SQRT6=.25*SQRT3*AUX*E4*E2
      P(5)=SQRT6*P(5)+E3*P(3)/E1
      P(11)=SQRT6*P(11)+E3*P(9)/E1
      P(6)=-.5*AUX*E4*E4*P(6)
      P(12)=-.5*AUX*E4*E4*P(12)
      P(16)=AUX*E4*E4*P(16)
      P(15)=AUX*E2*E2*P(15)-E3*(2.*P(14)/E1+E3*P13)
*----
*  PIS NORMALIZATION
*----
      S4=COTE*.25
      VAUX=0.
      DO 200 I=1,NR-1
         VV=PI*R(I)*R(I)
         V=VV-VAUX
         VAUX=VV
         COEF=S4/V
         IF(LGZERO(I)) THEN
           PIS(2,I)=COEF*AUX*E1*(E2*PIS(2,I)-E3*PIS(1,I))
           PIS(1,I)=COEF*AUX*PIS(1,I)*E1*E1
         ELSE
           SIGTI=COEF/SIGT(I)
           PIS(2,I)=AUX*SIGTI*E1*(E2*PIS(2,I)-E3*PIS(1,I))
           PIS(1,I)=AUX*SIGTI*PIS(1,I)*E1*E1
         ENDIF
  200 CONTINUE
         COEF=S4/(1.5*SQRT3*COTE*COTE-VAUX)
         IF(LGZERO(NR)) THEN
           PIS(2,NR)=COEF*AUX*E1*(E2*PIS(2,NR)-E3*PIS(1,NR))
           PIS(1,NR)=COEF*AUX*PIS(1,NR)*E1*E1
         ELSE
           SIGTI=COEF/SIGT(NR)
           PIS(2,NR)=AUX*SIGTI*E1*(E2*PIS(2,NR)-E3*PIS(1,NR))
           PIS(1,NR)=AUX*SIGTI*PIS(1,NR)*E1*E1
         ENDIF
*
*                  *STOCKAGE DES PIS ET DES PSS*
*----
*  OTHER PROBABILITIES COMPUTATION
*----
      P(2)=(-SQRT3*P(3)-4.*P(1))/SQRT2
      P(4)=-4.5*P(6)+SQRT3*(-SQRT2*P(5)+8.*P(3))+8.*P(1)
      P(8)=(-3.*SQRT3*P(9)-4.*P(7))/SQRT2
      P(10)=-4.5*P(12)+SQRT3*(SQRT2*P(11)+8.*P(9))+8.*P(7)
*----
*  TRANSMISSION MATRIX
*----
      DO  58 I=1,18
      DO  58 J=1,18
      IB=IROT(J,I)
      IF(IB.LT.0) THEN
         PSS(I,J)=-P(-IB)
      ELSEIF(IB.GT.0) THEN
         PSS(I,J)=P(IB)
      ELSE
         PSS(I,J)=0.
      ENDIF
   58 CONTINUE
*----
*  LEAKAGE MARTIX
*
      DO 61 J=1,NR
      K=3
      DO 61 I=1,5
        K=K+3
        PIS(K-1,J)=PIS(2,J)
        PIS(K-2,J)=PIS(1,J)
   61 CONTINUE
*
      RETURN
      END
