*DECK SYBHT1
      SUBROUTINE SYBHT1 (IR1,IR2,IPAS,SIGT,IPAS2,NG,NSMAX,MICRO,IQUAD,
     1 NS,IBI,RS,FRACT,VOLK,P,RRRR,SIGT0,QKDEL,QKN)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the reduced collision probabilities for the double
* heterogeneity option of Sybil (part 1).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* IPAS    number of volumes in the composite geometry.
* SIGT    total macroscopis cross sections in each volume of the
*         composite geometry.
* IPAS2   number of volumes in the macro geometry.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind of
*         micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUAD   quadrature parameter for the treatment of the micro volumes.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
*
*Parameters: output
* P       reduced collision probabilities for the composite geometry.
* RRRR    information used by SYBHT2 or SYBHT3.
* SIGT0   equivalent macroscopic cross section in each macro volume.
* QKDEL   information used by SYBHT2 or SYBHT3.
*
*Parameters: scratch
* QKN
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IR1,IR2,IPAS,IPAS2,NG,NSMAX,MICRO,IQUAD,NS(NG),IBI(IPAS2)
      REAL SIGT(IPAS),RS(NSMAX+1,NG),FRACT(NG,IR2),VOLK(NG,NSMAX),
     1 P(IPAS,IPAS),RRRR(IPAS2),SIGT0(IPAS2),QKDEL(NG,NSMAX,IPAS2),
     2 QKN(NSMAX,NSMAX)
*----
*  LOCAL VARIABLES
*----
      REAL RGAR(2),X2(3)
      CHARACTER*131 HSMG
      DOUBLE PRECISION DP0,DP1,RAPP,QKD,FACT
      COMMON BASE(1)
*
      DO 10 I=1,IPAS
      DO 10 L=1,IPAS
   10 P(I,L)=0.0
      IND1=IPAS2
      DO 80 I=1,IPAS2
      MIL=IBI(I)
      SIGT0(I)=SIGT(I)
      RRRR(I)=1.0
      IF(MIL.GT.IR1) THEN
         DILF=1.0
         DO 20 J=1,NG
   20    DILF=DILF-FRACT(J,MIL)
         DP0=DILF*SIGT(I)
         DP1=DP0
         DO 70 J=1,NG
         FRT=FRACT(J,MIL)
         IF(FRT.LE.0.00001) GO TO 70
         RGAR(1)=0.0
         RGAR(2)=RS(NS(J)+1,J)
         CHORD=4.0D0*RGAR(2)/REAL(MICRO-1)
         CALL SETARA(BASE,1+IQUAD*3,IZ)
         IF(MICRO.EQ.3) THEN
            CALL SYBT1D(1,RGAR,.FALSE.,IQUAD,BASE(IZ))
            CALL SYBALC(1,NSMAX,RGAR,SIGT(I),IQUAD,0.0,BASE(IZ),QKN,X2)
         ELSE IF(MICRO.EQ.4) THEN
            CALL SYBT1D(1,RGAR,.TRUE.,IQUAD,BASE(IZ))
            CALL SYBALS(1,NSMAX,RGAR,SIGT(I),IQUAD,0.0,BASE(IZ),QKN,X2)
         ENDIF
         CALL RLSARA(BASE(IZ))
         RAPP=1.0D0/(1.0D0-QKN(1,1)*SIGT(I))
         CALL SETARA(BASE,3*NS(J),IX2)
         CALL SETARA(BASE,1+IQUAD*((NS(J)*(5+NS(J)))/2),IZ)
         IF(MICRO.EQ.3) THEN
            CALL SYBT1D(NS(J),RS(1,J),.FALSE.,IQUAD,BASE(IZ))
            CALL SYBALC(NS(J),NSMAX,RS(1,J),SIGT(IND1+1),IQUAD,0.0,
     1      BASE(IZ),QKN,BASE(IX2))
         ELSE IF(MICRO.EQ.4) THEN
            CALL SYBT1D(NS(J),RS(1,J),.TRUE.,IQUAD,BASE(IZ))
            CALL SYBALS(NS(J),NSMAX,RS(1,J),SIGT(IND1+1),IQUAD,0.0,
     1      BASE(IZ),QKN,BASE(IX2))
         ENDIF
         CALL RLSARA(BASE(IZ))
         CALL RLSARA(BASE(IX2))
         IND1P=IND1
         DO 40 K=1,NS(J)
         IND1P=IND1P+1
         IF(CHORD*SIGT(IND1P).GE.1.0E4) THEN
            FACT=FRT/CHORD
            QKDEL(J,K,I)=1.0/(VOLK(J,K)*SIGT(IND1P)*CHORD)
         ELSE
            QKD=1.0D0
            DO 30 N=1,NS(J)
   30       QKD=QKD-QKN(K,N)*SIGT(IND1+N)
            QKDEL(J,K,I)=QKD
            FACT=FRT*VOLK(J,K)*QKD*SIGT(IND1P)
         ENDIF
         DP0=DP0+FACT
   40    DP1=DP1+FACT*RAPP
         IF(CHORD*SIGT(IND1P).GE.1.0E4) THEN
            IND1P=IND1
            DO 50 K=1,NS(J)
            IND1P=IND1P+1
   50       P(IND1P,IND1P)=1.0/SIGT(IND1P)
         ELSE
            IND1P=IND1
            DO 60 K=1,NS(J)
            IND1P=IND1P+1
            IND2P=IND1
            DO 60 N=1,NS(J)
            IND2P=IND2P+1
   60       P(IND1P,IND2P)=QKN(K,N)
         ENDIF
         IND1=IND1+NS(J)
   70    CONTINUE
         SIGT0(I)=DP1
         RRRR(I)=DP1/DP0
      ENDIF
   80 CONTINUE
      IF(IND1.EQ.IPAS) RETURN
      WRITE (HSMG,100) IND1,IPAS
  100 FORMAT (46HSYBHT1: IND1 IS INCONSISTENT WITH IPAS. IND1 =,I5,5X,
     1 6HIPAS =,I5)
      CALL XABORT(HSMG)
      END
