*DECK SYBHT2
      SUBROUTINE SYBHT2 (IR1,IR2,IPAS,IPAS2,NG,NSMAX,NS,IBI,FRACT,VOLK,
     1 P,RRRR,SIGT0,QKDEL,PIL)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the reduced collision probabilities for the double
* heterogeneity option of Sybil (part 2).
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* IPAS    number of volumes in the composite geometry.
* IPAS2   number of volumes in the macro geometry.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given
*         macro volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro volumes.
* RRRR    information computed by SYBHT1.
* SIGT0   equivalent macroscopic cross section in each macro volume.
* QKDEL   information computed by SYBHT1.
* PIL     reduced collision probabilities between the macro volumes.
*
*Parameters: output
* P       reduced collision probabilities for the composite geometry.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IR1,IR2,IPAS,IPAS2,NG,NSMAX,NS(NG),IBI(IPAS2)
      REAL FRACT(NG,IR2),VOLK(NG,NSMAX),P(IPAS,IPAS),RRRR(IPAS2),
     1 SIGT0(IPAS2),QKDEL(NG,NSMAX,IPAS2),PIL(IPAS2,IPAS2)
*
      IND1=IPAS2
      DO 140 I=1,IPAS2
      IMIL=IBI(I)
      I1PL=0
      IF(IMIL.GT.IR1) THEN
         RSLN=(1.0-RRRR(I))*RRRR(I)/SIGT0(I)
         DILF=1.0
         DO 10 J=1,NG
   10    DILF=DILF-FRACT(J,IMIL)
         P(I,I)=P(I,I)+DILF*RSLN
         IND1P=IND1
         DO 40 J=1,NG
         IF(FRACT(J,IMIL).LE.0.00001) GO TO 40
         I1PL=I1PL+NS(J)
         DO 30 K=1,NS(J)
         IND1P=IND1P+1
         RIKS=QKDEL(J,K,I)*RSLN
         P(IND1P,I)=P(IND1P,I)+DILF*RIKS
         P(I,IND1P)=P(I,IND1P)+FRACT(J,IMIL)*VOLK(J,K)*RIKS
         IND2P=IND1
         DO 30 M=1,NG
         FRT=FRACT(M,IMIL)
         IF(FRT.LE.0.00001) GO TO 30
         DO 20 N=1,NS(M)
         IND2P=IND2P+1
   20    P(IND1P,IND2P)=P(IND1P,IND2P)+FRT*VOLK(M,N)*QKDEL(M,N,I)*RIKS
   30    CONTINUE
   40    CONTINUE
      ENDIF
*
      IND2=IPAS2
      DO 130 L=1,IPAS2
      LMIL=IBI(L)
      I2PL=0
      IF(LMIL.GT.IR1) THEN
         DILF=1.0
         DO 50 M=1,NG
   50    DILF=DILF-FRACT(M,LMIL)
         RSL0=DILF*RRRR(L)
         P(I,L)=P(I,L)+RRRR(I)*PIL(I,L)*RSL0
         IND2P=IND2
         DO 70 M=1,NG
         FRT=FRACT(M,LMIL)
         IF(FRT.LE.0.00001) GO TO 70
         I2PL=I2PL+NS(M)
         DO 60 N=1,NS(M)
         IND2P=IND2P+1
         RSLN=QKDEL(M,N,L)*FRT*VOLK(M,N)*RRRR(L)
   60    P(I,IND2P)=P(I,IND2P)+RRRR(I)*PIL(I,L)*RSLN
   70    CONTINUE
         IND1P=IND1
         DO 100 J=1,NG
         IF(FRACT(J,IMIL).LE.0.00001) GO TO 100
         DO 90 K=1,NS(J)
         IND1P=IND1P+1
         RIKS=QKDEL(J,K,I)*RRRR(I)
         P(IND1P,L)=P(IND1P,L)+RIKS*PIL(I,L)*RSL0
         IND2P=IND2
         DO 90 M=1,NG
         FRT=FRACT(M,LMIL)
         IF(FRT.LE.0.00001) GO TO 90
         DO 80 N=1,NS(M)
         IND2P=IND2P+1
         RSLN=QKDEL(M,N,L)*FRT*VOLK(M,N)*RRRR(L)
   80    P(IND1P,IND2P)=P(IND1P,IND2P)+RIKS*PIL(I,L)*RSLN
   90    CONTINUE
  100    CONTINUE
      ELSE
         P(I,L)=P(I,L)+RRRR(I)*PIL(I,L)
         IND1P=IND1
         DO 120 J=1,NG
         IF(FRACT(J,IMIL).LE.0.00001) GO TO 120
         DO 110 K=1,NS(J)
         IND1P=IND1P+1
         RIKS=QKDEL(J,K,I)*RRRR(I)
  110    P(IND1P,L)=P(IND1P,L)+RIKS*PIL(I,L)
  120    CONTINUE
      ENDIF
      IND2=IND2+I2PL
  130 CONTINUE
      IND1=IND1+I1PL
  140 CONTINUE
      RETURN
      END
