*DECK SYBJP2
      SUBROUTINE SYBJP2 (IPSYS,IPTRK,ITG,IMPX,NREG,NBMIX,MAT,SIGT0,
     1 SIGW0,IR1,IR2,NREG2,NG,NSMAX,MICRO,IQUA10,NS,IBI,RS,FRACT,VOLK)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the cellwise scattering-reduced collision, escape and
* transmission probabilities for the current iteration method in Sybil.
* Special version with double heterogeneity capability.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPSYS   pointer to the system matrices.
* IPTRK   pointer to the Sybil tracking (L_TRACK signature).
* ITG     type of Sybil module.
* IMPX    print flag (equal to zero for no print).
* NREG    number of volumes in the composite geometry.
* NBMIX   number of mixtures.
* MAT     index-number of the mixture type assigned to each volume
*         of the composite geometry.
* SIGT0   total macroscopic cross sections ordered by mixture of
*         the composite geometry.
* SIGS0   within-group scattering macroscopic cross section ordered
*         by mixture of the composite geometry.
* IR1     number of mixtures in the domain.
* IR2     number of mixtures in the domain, including the composite
*         mixtures.
* NREG2   number of volumes in the macro geometry.
* NG      number of different kind of micro structures. A kind of
*         micro structure is characterized by the radius of its
*         micro volumes. All the micro volumes of the same kind
*         should own the same nuclear properties in a given macro
*         volume.
* NSMAX   maximum number of volumes (tubes or shells) in each kind
*         of micro structure.
* MICRO   type of micro volumes (=3: cylinder; =4: sphere).
* IQUA10  quadrature parameter for the treatment of the micro volumes.
* NS      number of volumes in each kind of micro structure.
* IBI     type of mixture in each volume of the macro geometry.
*         If IBI(IKK) is greater than IR1, the volume IKK contains a
*         micro structure.
* RS      radius of the micro volumes.
* FRACT   volumic fractions of the micro volumes.
* VOLK    volumic fractions of the tubes or shells in the micro
*         volumes.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPSYS,IPTRK,ITG,IMPX,NREG,NBMIX,MAT(NREG),IR1,IR2,NREG2,
     1 NG,NSMAX,MICRO,IQUA10,NS(NG),IBI(NREG2)
      REAL SIGT0(0:NBMIX),SIGW0(0:NBMIX),RS(NSMAX+1,NG),FRACT(NG,IR2),
     1 VOLK(NG,NSMAX)
*----
*  LOCAL VARIABLES
*----
      INTEGER IPAR(16),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*
      IF(ITG.EQ.1) THEN
         CALL XABORT('SYBJP2: THIS GEOMETRY CANNOT BE PROCESSED WITH A'
     1   //' CURRENT ITERATION METHOD. USE KEYWORD PIJ IN ASM: (1).')
      ELSE IF(ITG.EQ.2) THEN
         CALL XABORT('SYBJP2: THIS GEOMETRY CANNOT BE PROCESSED WITH A'
     1   //' CURRENT ITERATION METHOD. USE KEYWORD PIJ IN ASM: (2).')
      ELSE IF(ITG.EQ.3) THEN
         CALL LCMSIX(IPTRK,'DOITYOURSELF',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         NSUPCE=IPAR(1)
         IQUA3=IPAR(2)
         ISTAT=IPAR(3)
         CALL LCMIOF(IPTRK,'NMC',INMC3)
         CALL LCMIOF(IPTRK,'RAYRE',IRAYR3)
         CALL LCMIOF(IPTRK,'PROCEL',IPROCE)
         CALL LCMIOF(IPTRK,'POURCE',IPOURC)
         CALL LCMIOF(IPTRK,'SURFA',ISURFA)
         CALL LCMSIX(IPTRK,' ',2)
*
         CALL SETARA(IBASE,NSUPCE+1,INMCB)
         NPIJ=0
         IBASE(INMCB)=0
         DO 30 IKG=1,NSUPCE
         J1=IBASE(INMC3+IKG-1)
         J2=IBASE(INMC3+IKG)-J1
         DO 20 I=1,J2
         IF(IBI(J1+I).GT.0) THEN
            DO 10 J=1,NG
            IF(FRACT(J,IBI(J1+I)).GT.0.00001) J2=J2+NS(J)
   10       CONTINUE
         ENDIF
   20    CONTINUE
         IBASE(INMCB+IKG)=IBASE(INMCB+IKG-1)+J2
   30    NPIJ=NPIJ+J2*J2
         IF(NREG.NE.IBASE(INMCB+NSUPCE)) CALL XABORT('SYBJP2: ABORT1.')
         IF(NREG2.NE.IBASE(INMC3+NSUPCE)) CALL XABORT('SYBJP2: ABORT2.')
         CALL SETARA(BASE,NPIJ,IPIJW)
         CALL SETARA(BASE,NREG,IPISW)
         CALL SETARA(BASE,NREG,IPSJW)
         CALL SETARA(BASE,NSUPCE,IPSSW)
         CALL SETARA(BASE,NREG,ISIGT2)
         CALL SETARA(BASE,NREG,ISIGW2)
*
         I1=0
         I2=NREG2
         I3=0
         DO 60 IKK=1,NSUPCE
         J1=IBASE(INMC3+IKK-1)
         J2=IBASE(INMC3+IKK)-J1
         J1B=IBASE(INMCB+IKK-1)
         J2B=IBASE(INMCB+IKK)-J1B
         DO 40 I=1,J2
         I3=I3+1
         BASE(ISIGT2+J1B+I-1)=SIGT0(MAT(I1+I))
   40    BASE(ISIGW2+J1B+I-1)=SIGW0(MAT(I1+I))
         DO 50 I=1,J2B-J2
         I3=I3+1
         BASE(ISIGT2+J1B+J2+I-1)=SIGT0(MAT(I2+I))
   50    BASE(ISIGW2+J1B+J2+I-1)=SIGW0(MAT(I2+I))
         I1=I1+J2
   60    I2=I2+J2B-J2
*
         CALL SETARA(BASE,NREG2,IPLS)
         CALL SETARA(BASE,NREG2,IPSL)
         CALL SETARA(BASE,NREG,IPIS)
         CALL SETARA(BASE,NREG,IPSI)
         CALL SYBB01(NSUPCE,NPIJ,NREG,NREG2,BASE(ISIGT2),BASE(ISIGW2),
     1   IMPX,IQUA3,IBASE(INMC3),BASE(IRAYR3),IR1,IR2,NG,NSMAX,MICRO,
     2   IQUA10,NS,IBI,RS,FRACT,VOLK,IBASE(INMCB),BASE(IPIJW),
     3   BASE(IPISW),BASE(IPSJW),BASE(IPSSW),BASE(IPLS),BASE(IPSL),
     4   BASE(IPIS),BASE(IPSI))
         CALL RLSARA(BASE(IPSI))
         CALL RLSARA(BASE(IPIS))
         CALL RLSARA(BASE(IPSL))
         CALL RLSARA(BASE(IPLS))
         CALL LCMPOF(IPSYS,'PSSW$SYBIL',NSUPCE,2,IPSSW)
         CALL LCMPOF(IPSYS,'PSJW$SYBIL',NREG,2,IPSJW)
         CALL LCMPOF(IPSYS,'PISW$SYBIL',NREG,2,IPISW)
         CALL LCMPOF(IPSYS,'PIJW$SYBIL',NPIJ,2,IPIJW)
         CALL LCMPOF(IPSYS,'NMC$SYBIL',NSUPCE+1,1,INMCB)
         CALL RLSARA(BASE(ISIGW2))
         CALL RLSARA(BASE(ISIGT2))
      ELSE IF(ITG.EQ.4) THEN
         CALL LCMSIX(IPTRK,'EURYDICE',1)
         CALL LCMGET(IPTRK,'PARAM',IPAR)
         IHEX=IPAR(1)
         MULTC=IPAR(2)
         IWIGN=IPAR(3)
         NMCEL=IPAR(4)
         NMERGE=IPAR(5)
         NGEN=IPAR(6)
         IJAT=IPAR(7)
         LMAILI=IPAR(15)
         LMAILR=IPAR(16)
         CALL LCMIOF(IPTRK,'XX',IXX4)
         CALL LCMIOF(IPTRK,'YY',IYY4)
         CALL LCMIOF(IPTRK,'LSECT',ILSEC4)
         CALL LCMIOF(IPTRK,'NMC',INMC4)
         CALL LCMIOF(IPTRK,'NMCR',INMCR4)
         CALL LCMIOF(IPTRK,'RAYRE',IRAYR4)
         CALL LCMIOF(IPTRK,'MAIL',IMAIL)
         IF(LMAILI.GT.0) THEN
            CALL LCMIOF(IPTRK,'ZMAILI',IIZMAI)
         ELSE
*           THIS INFO IS NOT REQUIRED IN THE CALLED ROUTINE.
            IIZMAI=IMAIL
         ENDIF
         IF(LMAILR.GT.0) THEN
            CALL LCMIOF(IPTRK,'ZMAILR',IRZMAI)
         ELSE
*           THIS INFO IS NOT REQUIRED IN THE CALLED ROUTINE.
            IRZMAI=IMAIL
         ENDIF
         CALL LCMIOF(IPTRK,'IFR',IIFR)
         CALL LCMIOF(IPTRK,'ALB',IALB)
         CALL LCMIOF(IPTRK,'INUM',IINUM)
         CALL LCMIOF(IPTRK,'MIX',IMIX)
         CALL LCMIOF(IPTRK,'DVX',IDVX)
         CALL LCMIOF(IPTRK,'IGEN',IIGEN)
         CALL LCMSIX(IPTRK,' ',2)
*
         CALL SETARA(IBASE,NGEN+1,INMCB)
         CALL SETARA(IBASE,NREG2,IIBI2)
         NCOUR=4
         IF(IHEX.NE.0) NCOUR=6
         J3=0
         DO 110 IKK=1,NMERGE
         IKG=IBASE(IIGEN+IKK-1)
         J1=IBASE(INMC4+IKG-1)
         J2=IBASE(INMC4+IKG)-J1
         DO 70 I=1,J2
   70    IBASE(IIBI2+J1+I-1)=IBI(J3+I)
   80    J3=J3+J2
         NPIJ=0
         NPIS=0
         IBASE(INMCB)=0
         DO 110 IKG=1,NGEN
         J1=IBASE(INMC4+IKG-1)
         J2=IBASE(INMC4+IKG)-J1
         DO 100 I=1,J2
         IF(IBASE(IIBI2+J1+I-1).GT.0) THEN
            DO 90 J=1,NG
            IF(FRACT(J,IBASE(IIBI2+J1+I-1)).GT.0.00001) J2=J2+NS(J)
   90       CONTINUE
         ENDIF
  100    CONTINUE
         IBASE(INMCB+IKG)=IBASE(INMCB+IKG-1)+J2
         NPIJ=NPIJ+J2*J2
  110    NPIS=NPIS+J2
         NPIS0=IBASE(INMC4+NGEN)
         CALL SETARA(BASE,NPIS,ISIGT2)
         CALL SETARA(BASE,NPIS,ISIGW2)
         I1=0
         I2=NREG2
         I3=0
         DO 140 IKK=1,NMERGE
         IKG=IBASE(IIGEN+IKK-1)
         J1=IBASE(INMC4+IKG-1)
         J2=IBASE(INMC4+IKG)-J1
         J1B=IBASE(INMCB+IKG-1)
         J2B=IBASE(INMCB+IKG)-J1B
         DO 120 I=1,J2
         I3=I3+1
         IBASE(IIBI2+J1+I-1)=IBI(I1+I)
         BASE(ISIGT2+J1B+I-1)=SIGT0(MAT(I1+I))
  120    BASE(ISIGW2+J1B+I-1)=SIGW0(MAT(I1+I))
         DO 130 I=1,J2B-J2
         I3=I3+1
         BASE(ISIGT2+J1B+J2+I-1)=SIGT0(MAT(I2+I))
  130    BASE(ISIGW2+J1B+J2+I-1)=SIGW0(MAT(I2+I))
         I1=I1+J2
  140    I2=I2+J2B-J2
         IF(MULTC.EQ.1) THEN
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,NPIS,IPISW)
            CALL SETARA(BASE,NPIS,IPSJW)
            CALL SETARA(BASE,NGEN,IPSSW)
*
            CALL SETARA(BASE,NPIS0,IPLS)
            CALL SETARA(BASE,NPIS0,IPSL)
            CALL SETARA(BASE,NPIS,IPIS)
            CALL SETARA(BASE,NPIS,IPSI)
            CALL SYBB02(NGEN,NPIJ,NPIS,NPIS0,BASE(ISIGT2),BASE(ISIGW2),
     1      IMPX,NCOUR,IWIGN,IPAR(8),BASE(IXX4),BASE(IYY4),IBASE(INMC4),
     2      BASE(IRAYR4),IBASE(IMAIL),BASE(IRZMAI),IR1,IR2,NG,NSMAX,
     3      MICRO,IQUA10,NS,IBASE(IIBI2),RS,FRACT,VOLK,IBASE(INMCB),
     4      BASE(IPIJW),BASE(IPISW),BASE(IPSJW),BASE(IPSSW),BASE(IPLS),
     5      BASE(IPSL),BASE(IPIS),BASE(IPSI))
            CALL RLSARA(BASE(IPSI))
            CALL RLSARA(BASE(IPIS))
            CALL RLSARA(BASE(IPSL))
            CALL RLSARA(BASE(IPLS))
            CALL LCMPOF(IPSYS,'PSSW$SYBIL',NGEN,2,IPSSW)
            CALL LCMPOF(IPSYS,'PSJW$SYBIL',NPIS,2,IPSJW)
            CALL LCMPOF(IPSYS,'PISW$SYBIL',NPIS,2,IPISW)
            CALL LCMPOF(IPSYS,'PIJW$SYBIL',NPIJ,2,IPIJW)
         ELSE
            NRAYRE=IBASE(INMCR4+NGEN)
            IF(MULTC.EQ.4) NCOUR=3*NCOUR
            CALL SETARA(BASE,NPIJ,IPIJW)
            CALL SETARA(BASE,NCOUR*NPIS,IPISW)
            CALL SETARA(BASE,NCOUR*NPIS,IPSJW)
            CALL SETARA(BASE,NGEN*NCOUR*NCOUR,IPSSW)
*
            CALL SETARA(BASE,NCOUR*NPIS0,IPLS)
            CALL SETARA(BASE,NCOUR*NPIS0,IPSL)
            CALL SETARA(BASE,NCOUR*NPIS,IPIS)
            CALL SETARA(BASE,NCOUR*NPIS,IPSI)
            IF(MULTC.EQ.2) THEN
*              ROTH X 4 OR ROTH X 6 APPROXIMATION.
               CALL SYBB03(NGEN,NPIJ,NPIS,NPIS0,BASE(ISIGT2),
     1         BASE(ISIGW2),IMPX,NCOUR,IWIGN,IPAR(8),BASE(IXX4),
     2         BASE(IYY4),IBASE(INMC4),BASE(IRAYR4),IBASE(IMAIL),
     3         BASE(IRZMAI),IR1,IR2,NG,NSMAX,MICRO,IQUA10,NS,
     4         IBASE(IIBI2),RS,FRACT,VOLK,IBASE(INMCB),BASE(IPIJW),
     4         BASE(IPISW),BASE(IPSJW),BASE(IPSSW),BASE(IPLS),
     5         BASE(IPSL),BASE(IPIS),BASE(IPSI))
            ELSE IF(MULTC.EQ.3) THEN
*              DP-0 APPROXIMATION.
               CALL SYBB04(NGEN,NPIJ,NPIS,NPIS0,NRAYRE,BASE(ISIGT2),
     1         BASE(ISIGW2),IMPX,NCOUR,IPAR(8),BASE(IXX4),BASE(IYY4),
     2         IBASE(ILSEC4),IBASE(INMC4),IBASE(INMCR4),BASE(IRAYR4),
     3         IBASE(IMAIL),IBASE(IIZMAI),BASE(IRZMAI),IR1,IR2,NG,
     4         NSMAX,MICRO,IQUA10,NS,IBASE(IIBI2),RS,FRACT,VOLK,
     5         IBASE(INMCB),BASE(IPIJW),BASE(IPISW),BASE(IPSJW),
     6         BASE(IPSSW),BASE(IPLS),BASE(IPSL),BASE(IPIS),BASE(IPSI))
            ELSE IF(MULTC.EQ.4) THEN
*              DP-1 APPROXIMATION.
               CALL SYBB05(NGEN,NPIJ,NPIS,NPIS0,NRAYRE,BASE(ISIGT2),
     1         BASE(ISIGW2),IMPX,NCOUR,IPAR(8),BASE(IXX4),BASE(IYY4),
     2         IBASE(ILSEC4),IBASE(INMC4),IBASE(INMCR4),BASE(IRAYR4),
     3         IBASE(IMAIL),IBASE(IIZMAI),BASE(IRZMAI),IR1,IR2,NG,
     4         NSMAX,MICRO,IQUA10,NS,IBASE(IIBI2),RS,FRACT,VOLK,
     5         IBASE(INMCB),BASE(IPIJW),BASE(IPISW),BASE(IPSJW),
     6         BASE(IPSSW),BASE(IPLS),BASE(IPSL),BASE(IPIS),BASE(IPSI))
            ELSE
               CALL XABORT('SYBJP2: UNKNOWN CP MODULE(1).')
            ENDIF
            CALL RLSARA(BASE(IPSI))
            CALL RLSARA(BASE(IPIS))
            CALL RLSARA(BASE(IPSL))
            CALL RLSARA(BASE(IPLS))
            CALL LCMPOF(IPSYS,'PSSW$SYBIL',NCOUR*NCOUR*NGEN,2,IPSSW)
            CALL LCMPOF(IPSYS,'PSJW$SYBIL',NCOUR*NPIS,2,IPSJW)
            CALL LCMPOF(IPSYS,'PISW$SYBIL',NCOUR*NPIS,2,IPISW)
            CALL LCMPOF(IPSYS,'PIJW$SYBIL',NPIJ,2,IPIJW)
         ENDIF
         CALL RLSARA(BASE(ISIGW2))
         CALL RLSARA(BASE(ISIGT2))
         CALL RLSARA(IBASE(IIBI2))
         CALL LCMPOF(IPSYS,'NMC$SYBIL',NGEN+1,1,INMCB)
      ELSE
         CALL XABORT('SYBJP2: UNKNOWN CP MODULE(2).')
      ENDIF
      RETURN
      END
