*DECK SYBRN1
      SUBROUTINE SYBRN1 (NZ,A,B,Z,IZ,RAYRE,SIGT,TRONC,PVS,PSS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* integration of the escape and transmission probabilities of a 2-D
* square or rectangular crown with DP-0 surface angular flux expansion
* on the external boundary. The tracks are computed by SYBRTK.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NZ      number of regions in the cell.
* A       size of external X side.
* B       size of external Y side.
* Z       real tracking information.
* IZ      integer tracking information.
* RAYRE   radius of the tubes.
* SIGT    total macroscopic cross section.
* TRONC   voided block criterion.
*
*Parameters: output
* PVS     volume to surface probabilities:
*         XINF surface 1; XSUP: surface 2;
*         YINF surface 3; YSUP: surface 4.
* PSS     surface to surface probabilities in the following order:
*         PSS(i,j) is the probability from surface i to surface j.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NZ,IZ(*)
      REAL A,B,Z(*),RAYRE(NZ),SIGT(NZ),TRONC,PVS(4,4),PSS(4,NZ)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MKI3=600)
      PARAMETER (PI=3.141592654,ZI30=0.785398164)
      LOGICAL ICARE
      REAL KI3,PBB(5)
      INTEGER ISL(4,4)
      COMMON /BICKL3/BI3(0:MKI3),BI31(0:MKI3),BI32(0:MKI3),PAS3,XLIM3,L3
      SAVE ISL
      DATA ISL/5,4,1,1,
     1         4,5,1,1,
     2         3,3,5,2,
     3         3,3,2,5/
*
      ICARE=IZ(1).EQ.2
      ZERO=TRONC*(A+B)/(2.0*A*B)
      AOB=A/B
      DO 5 I=1,5
    5 PBB(I)=0.0
      DO 10 I=1,4
      DO 10 J=1,NZ
   10 PVS(I,J)=0.0
      L=6
      NA=IZ(6)
      DO 170 IA=1,NA
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 102
      L=L+2
      DO 101 IX=1,NLX
      Z1=Z(L+1)
      KMAX=IZ(L+2)
      L=L+2
      MMAX=KMAX+KMAX-1
      KI3=ZI30
      POP=0.0
      DO 70 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+I)
      IF (POP-XLIM3) 50,45,45
   45 IF (SIGTI.LE.ZERO) GO TO 80
      PVS(3,III)=PVS(3,III)+2.0*KI3*Z1
      GO TO 80
   50 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+I))
         PVS(3,III)=PVS(3,III)+2.0*WI2*Z(L+I)*Z1
      ELSE
         PVS(3,III)=PVS(3,III)+2.0*(KI3-WI3)*Z1
      ENDIF
   70 KI3=WI3
   80 KI3=ZI30
      POP=0.0
      DO 95 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+MMAX+1-I)
      IF (POP-XLIM3) 90,85,85
   85 IF (SIGTI.LE.ZERO) GO TO 100
      PVS(1,III)=PVS(1,III)+2.0*KI3*Z1*AOB
      GO TO 100
   90 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+MMAX+1-I))
         PVS(1,III)=PVS(1,III)+2.0*WI2*Z(L+MMAX+1-I)*Z1*AOB
      ELSE
         PVS(1,III)=PVS(1,III)+2.0*(KI3-WI3)*Z1*AOB
      ENDIF
   95 KI3=WI3
      PBB(1)=PBB(1)+KI3*Z1
  100 L=L+MMAX
  101 CONTINUE
*
  102 L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 104
      L=L+1
      DO 103 IX=1,NLX
      Z1=Z(L+1)
      KMAX=IZ(L+2)
      L=L+2
      MMAX=KMAX+KMAX-1
      KI3=ZI30
      POP=0.0
      DO 30 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+I)
      IF (POP-XLIM3) 20,15,15
   15 IF (SIGTI.LE.ZERO) GO TO 40
      PVS(3,III)=PVS(3,III)+KI3*Z1
      GO TO 40
   20 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+I))
         PVS(3,III)=PVS(3,III)+WI2*Z(L+I)*Z1
      ELSE
         PVS(3,III)=PVS(3,III)+(KI3-WI3)*Z1
      ENDIF
   30 KI3=WI3
   40 KI3=ZI30
      POP=0.0
      DO 350 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+MMAX+1-I)
      IF (POP-XLIM3) 320,310,310
  310 IF (SIGTI.LE.ZERO) GO TO 370
      PVS(3,III)=PVS(3,III)+KI3*Z1
      GO TO 370
  320 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+MMAX+1-I))
         PVS(3,III)=PVS(3,III)+WI2*Z(L+MMAX+1-I)*Z1
      ELSE
         PVS(3,III)=PVS(3,III)+(KI3-WI3)*Z1
      ENDIF
  350 KI3=WI3
      PBB(2)=PBB(2)+2.0*KI3*Z1
  370 L=L+MMAX
  103 CONTINUE
*
  104 IF (ICARE) GO TO 170
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 170
      L=L+1
      DO 165 IX=1,NLX
      Z1=Z(L+1)
      KMAX=IZ(L+2)
      L=L+2
      MMAX=KMAX+KMAX-1
      KI3=ZI30
      POP=0.0
      DO 120 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+I)
      IF (POP-XLIM3) 110,105,105
  105 IF (SIGTI.LE.ZERO) GO TO 160
      PVS(1,III)=PVS(1,III)+KI3*Z1
      GO TO 160
  110 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+I))
         PVS(1,III)=PVS(1,III)+WI2*Z(L+I)*Z1
      ELSE
         PVS(1,III)=PVS(1,III)+(KI3-WI3)*Z1
      ENDIF
  120 KI3=WI3
  160 KI3=ZI30
      POP=0.0
      DO 450 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+MMAX+1-I)
      IF (POP-XLIM3) 420,410,410
  410 IF (SIGTI.LE.ZERO) GO TO 470
      PVS(1,III)=PVS(1,III)+KI3*Z1
      GO TO 470
  420 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+MMAX+1-I))
         PVS(1,III)=PVS(1,III)+WI2*Z(L+MMAX+1-I)*Z1
      ELSE
         PVS(1,III)=PVS(1,III)+(KI3-WI3)*Z1
      ENDIF
  450 KI3=WI3
      PBB(4)=PBB(4)+2.0*KI3*Z1
  470 L=L+MMAX
  165 CONTINUE
  170 CONTINUE
      IF (ICARE) THEN
         PBB(1)=PBB(1)+PBB(1)
         PBB(2)=PBB(2)+PBB(4)
         PBB(4)=PBB(2)
         DO 180 I=1,NZ
         PVS(3,I)=PVS(3,I)+PVS(1,I)
  180    PVS(1,I)=PVS(3,I)
      ENDIF
      PBB(3)=PBB(1)*AOB
*
      Z1=Z(2)
      DO 191 I=1,4
  191 PBB(I)=Z1*Z1*PBB(I)
      DO 192 J=1,NZ
      PVS(3,J)=Z1*Z1*PVS(3,J)
  192 PVS(1,J)=Z1*Z1*PVS(1,J)
      VOL=A*B
      Z1=0.0
      Z2=0.0
      DO 210 J=1,NZ
      JJJ=NZ+1-J
      R2=PI*RAYRE(JJJ)**2
      X=4.0*(VOL-R2)
      SIGTI=SIGT(JJJ)
      IF (SIGTI.LE.ZERO) GO TO 195
      Z1=Z1+PVS(3,JJJ)
      Z2=Z2+PVS(1,JJJ)
      X=X*SIGTI
  195 PVS(3,JJJ)=PVS(3,JJJ)*A/X
      PVS(1,JJJ)=PVS(1,JJJ)*B/X
  210 VOL=R2
      IF ((Z1.GT.TRONC).AND.(Z2.GT.TRONC)) THEN
         Z1=(1.0-2.0*PBB(1)-PBB(2))/Z1
         Z2=(1.0-2.0*PBB(3)-PBB(4))/Z2
         DO 220 J=1,NZ
         PVS(3,J)=PVS(3,J)*Z1
  220    PVS(1,J)=PVS(1,J)*Z2
      ENDIF
      DO 240 I=1,NZ
      PVS(2,I)=PVS(1,I)
  240 PVS(4,I)=PVS(3,I)
      DO 270 JC=1,4
      DO 270 IC=1,4
  270 PSS(IC,JC)=PBB(ISL(IC,JC))
      RETURN
      END
