*DECK SYBRN2
      SUBROUTINE SYBRN2 (NZ,A,B,Z,IZ,RAYRE,SIGT,TRONC,PVS,PSS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* integration of the escape and transmission probabilities of a 2-D
* square or rectangular crown with DP-1 surface angular flux expansion
* on the external boundary. The tracks are computed by SYBRTK.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NZ      number of regions in the cell.
* A       dimension of the external X sides.
* B       dimension of the external Y sides.
* Z       real tracking information.
* IZ      integer tracking information.
* RAYRE   radius of the tubes.
* SIGT    total macroscopic cross section.
* TRONC   voided block criterion.
*
*Parameters: output
* PVS     volume to surface probabilities:
*         XINF surfaces 1, 2 and 3;   XSUP: surfaces 4, 5 and 6;
*         YINF surfaces 7, 8 and 9;   YSUP: surfaces 10, 11 and 12.
* PSS     surface to surface probabilities in the following order:
*         PSS(i,j) is the probability from surface i to surface j.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NZ,IZ(*)
      REAL A,B,Z(*),RAYRE(NZ),SIGT(NZ),TRONC,PVS(12,NZ),PSS(12,12)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MKI3=600,MKI4=600,MKI5=600)
      PARAMETER (PI=3.141592654,ZI30=0.785398164,ZI40=0.666666667)
      LOGICAL ICARE
      REAL KI3,KI4,KI5
      INTEGER ISN(12,12)
      REAL PBB(28)
      COMMON /BICKL3/BI3(0:MKI3),BI31(0:MKI3),BI32(0:MKI3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4),BI41(0:MKI4),BI42(0:MKI4),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5),BI51(0:MKI5),BI52(0:MKI5),PAS5,XLIM5,L5
      SAVE ISN
      DATA ISN/  0,  0,  0,  4, 12,  0,  1,  9,-19,  1,  9, 19,
     1           0,  0,  0,  8, 16,  0,  5, 13,-23,  5, 13, 23,
     2           0,  0,  0,  0,  0, 28, 17, 21,-25,-17,-21,-25,
     3           4, 12,  0,  0,  0,  0,  1,  9, 19,  1,  9,-19,
     4           8, 16,  0,  0,  0,  0,  5, 13, 23,  5, 13,-23,
     5           0,  0, 28,  0,  0,  0,-17,-21,-25, 17, 21,-25,
     6           3, 11, 20,  3, 11,-20,  0,  0,  0,  2, 10,  0,
     7           7, 15, 24,  7, 15,-24,  0,  0,  0,  6, 14,  0,
     8         -18,-22,-27, 18, 22,-27,  0,  0,  0,  0,  0, 26,
     9           3, 11,-20,  3, 11, 20,  2, 10,  0,  0,  0,  0,
     1           7, 15,-24,  7, 15, 24,  6, 14,  0,  0,  0,  0,
     2          18, 22,-27,-18,-22,-27,  0,  0, 26,  0,  0,  0/
*
      ICARE=IZ(1).EQ.2
      ZERO=TRONC*(A+B)/(2.0*A*B)
      AOB=A/B
      DO 5 I=1,28
    5 PBB(I)=0.0
      DO 10 I=1,12
      DO 10 J=1,NZ
   10 PVS(I,J)=0.0
      L=6
      NA=IZ(6)
      DO 170 IA=1,NA
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 102
      SI=Z(L+1)
      CO=Z(L+2)
      L=L+2
      DO 101 IX=1,NLX
      Z1=Z(L+1)
      Z2=Z1*CO
      Z3=Z1*SI
      Z4=Z3*CO
      Z5=Z3*SI
      KMAX=IZ(L+2)
      L=L+2
      MMAX=KMAX+KMAX-1
      KI3=ZI30
      KI4=ZI40
      POP=0.0
      DO 70 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+I)
      IF (POP-XLIM3) 50,45,45
   45 IF (SIGTI.LE.ZERO) GO TO 80
      PVS(7,III)=PVS(7,III)+2.0*KI3*Z1
      PVS(8,III)=PVS(8,III)+2.0*KI4*Z3
      GO TO 80
   50 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      WI4=BI4(K)+POP*(BI41(K)+POP*BI42(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+I))
         PVS(7,III)=PVS(7,III)+2.0*WI2*Z(L+I)*Z1
         PVS(8,III)=PVS(8,III)+2.0*KI3*Z(L+I)*Z3
      ELSE
         PVS(7,III)=PVS(7,III)+2.0*(KI3-WI3)*Z1
         PVS(8,III)=PVS(8,III)+2.0*(KI4-WI4)*Z3
      ENDIF
      KI3=WI3
   70 KI4=WI4
   80 KI3=ZI30
      KI4=ZI40
      POP=0.0
      DO 95 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+MMAX+1-I)
      IF (POP-XLIM3) 90,85,85
   85 IF (SIGTI.LE.ZERO) GO TO 100
      PVS(1,III)=PVS(1,III)+2.0*KI3*Z1*AOB
      PVS(2,III)=PVS(2,III)+2.0*KI4*Z2*AOB
      GO TO 100
   90 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      WI4=BI4(K)+POP*(BI41(K)+POP*BI42(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+MMAX+1-I))
         PVS(1,III)=PVS(1,III)+2.0*WI2*Z(L+MMAX+1-I)*Z1*AOB
         PVS(2,III)=PVS(2,III)+2.0*KI3*Z(L+MMAX+1-I)*Z2*AOB
      ELSE
         PVS(1,III)=PVS(1,III)+2.0*(KI3-WI3)*Z1*AOB
         PVS(2,III)=PVS(2,III)+2.0*(KI4-WI4)*Z2*AOB
      ENDIF
      KI3=WI3
   95 KI4=WI4
      KI5=BI5(K)+POP*(BI51(K)+POP*BI52(K))
      PBB(1)=PBB(1)+KI3*Z1
      PBB(5)=PBB(5)+KI4*Z2
      PBB(9)=PBB(9)+KI4*Z3
      PBB(13)=PBB(13)+KI5*Z4
      PBB(21)=PBB(21)+KI5*Z5
      PBB(23)=PBB(23)+KI5*(Z1-Z5)
  100 L=L+MMAX
  101 CONTINUE
*
  102 L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 104
      SI=Z(L+1)
      L=L+1
      DO 103 IX=1,NLX
      Z1=Z(L+1)
      Z2=Z1*SI
      Z3=Z2*SI
      KMAX=IZ(L+2)
      L=L+2
      MMAX=KMAX+KMAX-1
      KI3=ZI30
      KI4=ZI40
      POP=0.0
      DO 30 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+I)
      IF (POP-XLIM3) 20,15,15
   15 IF (SIGTI.LE.ZERO) GO TO 40
      PVS(7,III)=PVS(7,III)+KI3*Z1
      PVS(8,III)=PVS(8,III)+KI4*Z2
      GO TO 40
   20 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      WI4=BI4(K)+POP*(BI41(K)+POP*BI42(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+I))
         PVS(7,III)=PVS(7,III)+WI2*Z(L+I)*Z1
         PVS(8,III)=PVS(8,III)+KI3*Z(L+I)*Z2
      ELSE
         PVS(7,III)=PVS(7,III)+(KI3-WI3)*Z1
         PVS(8,III)=PVS(8,III)+(KI4-WI4)*Z2
      ENDIF
      KI3=WI3
   30 KI4=WI4
   40 KI3=ZI30
      KI4=ZI40
      POP=0.0
      DO 350 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+MMAX+1-I)
      IF (POP-XLIM3) 320,310,310
  310 IF (SIGTI.LE.ZERO) GO TO 370
      PVS(7,III)=PVS(7,III)+KI3*Z1
      PVS(8,III)=PVS(8,III)+KI4*Z2
      GO TO 370
  320 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      WI4=BI4(K)+POP*(BI41(K)+POP*BI42(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+MMAX+1-I))
         PVS(7,III)=PVS(7,III)+WI2*Z(L+MMAX+1-I)*Z1
         PVS(8,III)=PVS(8,III)+KI3*Z(L+MMAX+1-I)*Z2
      ELSE
         PVS(7,III)=PVS(7,III)+(KI3-WI3)*Z1
         PVS(8,III)=PVS(8,III)+(KI4-WI4)*Z2
      ENDIF
      KI3=WI3
  350 KI4=WI4
      KI5=BI5(K)+POP*(BI51(K)+POP*BI52(K))
      PBB(2)=PBB(2)+2.0*KI3*Z1
      PBB(6)=PBB(6)+2.0*KI4*Z2
      PBB(14)=PBB(14)+2.0*KI5*Z3
      PBB(26)=PBB(26)+2.0*KI5*(Z1-Z3)
  370 L=L+MMAX
  103 CONTINUE
*
  104 IF (ICARE) GO TO 170
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 170
      SI=Z(L+1)
      L=L+1
      DO 165 IX=1,NLX
      Z1=Z(L+1)
      Z2=Z1*SI
      Z3=Z2*SI
      KMAX=IZ(L+2)
      L=L+2
      MMAX=KMAX+KMAX-1
      KI3=ZI30
      KI4=ZI40
      POP=0.0
      DO 120 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+I)
      IF (POP-XLIM3) 110,105,105
  105 IF (SIGTI.LE.ZERO) GO TO 160
      PVS(1,III)=PVS(1,III)+KI3*Z1
      PVS(2,III)=PVS(2,III)+KI4*Z2
      GO TO 160
  110 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      WI4=BI4(K)+POP*(BI41(K)+POP*BI42(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+I))
         PVS(1,III)=PVS(1,III)+WI2*Z(L+I)*Z1
         PVS(2,III)=PVS(2,III)+KI3*Z(L+I)*Z2
      ELSE
         PVS(1,III)=PVS(1,III)+(KI3-WI3)*Z1
         PVS(2,III)=PVS(2,III)+(KI4-WI4)*Z2
      ENDIF
      KI3=WI3
  120 KI4=WI4
  160 KI3=ZI30
      KI4=ZI40
      POP=0.0
      DO 450 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+MMAX+1-I)
      IF (POP-XLIM3) 420,410,410
  410 IF (SIGTI.LE.ZERO) GO TO 470
      PVS(1,III)=PVS(1,III)+KI3*Z1
      PVS(2,III)=PVS(2,III)+KI4*Z2
      GO TO 470
  420 K=NINT(POP*PAS3)
      WI3=BI3(K)+POP*(BI31(K)+POP*BI32(K))
      WI4=BI4(K)+POP*(BI41(K)+POP*BI42(K))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+MMAX+1-I))
         PVS(1,III)=PVS(1,III)+WI2*Z(L+MMAX+1-I)*Z1
         PVS(2,III)=PVS(2,III)+KI3*Z(L+MMAX+1-I)*Z2
      ELSE
         PVS(1,III)=PVS(1,III)+(KI3-WI3)*Z1
         PVS(2,III)=PVS(2,III)+(KI4-WI4)*Z2
      ENDIF
      KI3=WI3
  450 KI4=WI4
      KI5=BI5(K)+POP*(BI51(K)+POP*BI52(K))
      PBB(4)=PBB(4)+2.0*KI3*Z1
      PBB(8)=PBB(8)+2.0*KI4*Z2
      PBB(16)=PBB(16)+2.0*KI5*Z3
      PBB(28)=PBB(28)+2.0*KI5*(Z1-Z3)
  470 L=L+MMAX
  165 CONTINUE
  170 CONTINUE
      IF (ICARE) THEN
         PBB(1)=2.0*PBB(1)
         PBB(5)=PBB(5)+PBB(9)
         PBB(9)=PBB(5)
         PBB(13)=2.0*PBB(13)
         PBB(21)=PBB(21)+PBB(23)
         PBB(23)=PBB(21)
         PBB(4)=PBB(2)
         PBB(8)=PBB(6)
         PBB(16)=PBB(14)
         PBB(28)=PBB(26)
         DO 180 I=1,NZ
         PVS(7,I)=PVS(7,I)+PVS(1,I)
         PVS(8,I)=PVS(8,I)+PVS(2,I)
         PVS(1,I)=PVS(7,I)
  180    PVS(2,I)=PVS(8,I)
      ENDIF
      PBB(10)=PBB(6)
      PBB(12)=PBB(8)
      PBB(17)=PBB(9)
      PBB(19)=PBB(5)
      PBB(25)=PBB(13)
      PBB(3)=PBB(1)*AOB
      PBB(7)=PBB(9)*AOB
      PBB(11)=PBB(5)*AOB
      PBB(15)=PBB(13)*AOB
      PBB(18)=PBB(19)*AOB
      PBB(20)=PBB(17)*AOB
      PBB(22)=PBB(23)*AOB
      PBB(24)=PBB(21)*AOB
      PBB(27)=PBB(25)*AOB
*----
*  ORTHONORMALIZATION
*----
      Z1=Z(2)
      Z2=Z(3)
      Z3=Z(4)
      Z4=Z(5)
      DO 280 I=1,4
      DEN0=PBB(I)
      DEN1=PBB(4+I)
      DEN2=PBB(8+I)
      PBB(I)=Z1*Z1*DEN0
      PBB(4+I)=Z1*(Z2*DEN1-Z3*DEN0)
      PBB(8+I)=Z1*(Z2*DEN2-Z3*DEN0)
      PBB(12+I)=Z2*Z2*PBB(12+I)-Z2*Z3*(DEN1+DEN2)+Z3*Z3*DEN0
  280 PBB(24+I)=Z4*Z4*PBB(24+I)
      DO 290 I=1,2
      DEN1=PBB(16+I)
      DEN2=PBB(18+I)
      PBB(16+I)=Z1*Z4*DEN1
      PBB(18+I)=Z1*Z4*DEN2
      PBB(20+I)=(Z2*PBB(20+I)-Z3*DEN1)*Z4
  290 PBB(22+I)=(Z2*PBB(22+I)-Z3*DEN2)*Z4
      DO 300 J=1,NZ
      DEN1=PVS(7,J)
      DEN2=PVS(1,J)
      PVS(7,J)=Z1*Z1*DEN1
      PVS(1,J)=Z1*Z1*DEN2
      PVS(8,J)=Z1*(Z2*PVS(8,J)-Z3*DEN1)
  300 PVS(2,J)=Z1*(Z2*PVS(2,J)-Z3*DEN2)
*
      VOL=A*B
      Z1=0.0
      Z2=0.0
      Z4=0.0
      Z5=0.0
      DO 510 J=1,NZ
      JJJ=NZ+1-J
      R2=PI*RAYRE(JJJ)**2
      X=4.0*(VOL-R2)
      SIGTI=SIGT(JJJ)
      IF (SIGTI.GT.ZERO) THEN
         Z1=Z1+PVS(7,JJJ)
         Z2=Z2+PVS(8,JJJ)
         Z4=Z4+PVS(1,JJJ)
         Z5=Z5+PVS(2,JJJ)
         X=X*SIGTI
      ENDIF
      PVS(7,JJJ)=PVS(7,JJJ)*A/X
      PVS(8,JJJ)=PVS(8,JJJ)*A/X
      PVS(1,JJJ)=PVS(1,JJJ)*B/X
      PVS(2,JJJ)=PVS(2,JJJ)*B/X
  510 VOL=R2
      IF ((Z1.GT.TRONC).AND.(Z2.GT.TRONC).AND.(Z4.GT.TRONC).AND.(Z5.GT.
     $ TRONC)) THEN
         Z1=(1.0-2.0*PBB(1)-PBB(2))/Z1
         Z2=(-2.0*PBB(9)-PBB(10))/Z2
         Z4=(1.0-2.0*PBB(3)-PBB(4))/Z4
         Z5=(-2.0*PBB(11)-PBB(12))/Z5
         DO 520 J=1,NZ
         PVS(7,J)=PVS(7,J)*Z1
         PVS(8,J)=PVS(8,J)*Z2
         PVS(1,J)=PVS(1,J)*Z4
  520    PVS(2,J)=PVS(2,J)*Z5
      ENDIF
*
      DO 540 I=1,NZ
      PVS(4,I)=PVS(1,I)
      PVS(5,I)=PVS(2,I)
      PVS(10,I)=PVS(7,I)
  540 PVS(11,I)=PVS(8,I)
      DO 550 JC=1,12
      DO 550 IC=1,12
      IB=ISN(IC,JC)
      IF (IB.LT.0) THEN
         PSS(IC,JC)=-PBB(-IB)
      ELSE IF (IB.GT.0) THEN
         PSS(IC,JC)=PBB(IB)
      ELSE
         PSS(IC,JC)=0.0
      ENDIF
  550 CONTINUE
      RETURN
      END
