*DECK SYBRTK
      SUBROUTINE SYBRTK (NA,NX,NZ,A,B,RAYRE,ILIGN,INORM,IRECT,IQW,L,Z,
     1 IZ,PREC,VAP)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the tracking information related to a square or rectangular
* heterogeneous cell.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NA      number of angles in (0,pi/2).
* NX      number of tracks in each sub domain for a given angle.
* NZ      number of regions in the cell.
* A       Cartesian dimension of the cell along the X axis.
* B       Cartesian dimension of the cell along the Y axis.
* RAYRE   radius of each cylinder (RAYRE(1)=0.0).
* ILIGN   tracking print flag (=1 to print the tracking).
* INORM   track normalization flag (=1 to avoid track normalization).
* IRECT   rectangular flag (=1 to avoid considering symmetries of
*         square cells).
* IQW     equal weight quadrature flag (=1 to use equal weight
*         quadratures in angle and space).
*
*Parameters: output
* L       size of vectors Z and IZ. If the cell is rectangular, then
*         L.LE.5+NA*(7+NX*(2*NZ+1)*(2*NZ+1)). If the cell is square,
*         then L.LE.5+NA*(5+NX*(NZ+1)*(2*NZ+1)).
* Z       real tracking information.
*         Z(2) to Z(5) contain the numerical orthonormalization
*         factors.
* IZ      integer tracking information. Note: vectors Z and IZ should
*         be in equivalence in the calling subroutine.
*         IZ(1)=1 if the cell is rectangular or if IRECT=1;
*         IZ(1)=2 if the cell is square.
*  PREC   accuracy obtained if the non-normalized tracks are used
*         to integrate the volumes.
*
*Parameters: scratch
*  VAP
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NA,NX,NZ,ILIGN,INORM,IRECT,IQW,L,IZ(*)
      REAL A,B,RAYRE(NZ),Z(*),PREC,VAP(NZ)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (PIO2=1.570796327)
      REAL ZX(64),WX(64),ZA(64),WA(64),ZXJ(64),WXJ(64)
*
      IF(MOD(2+NA,2).NE.0) CALL XABORT('SYBRTK: NA SHOULD BE EVEN.')
      IF(NX.GT.64) CALL XABORT('SYBRTK: NX IS GREATER THAN 64.')
      IF(NA.GT.64) CALL XABORT('SYBRTK: NA IS GREATER THAN 64.')
      IF(2.0*RAYRE(NZ).GT.MIN(A,B)) CALL XABORT('SYBRTK: A RADIUS IS G'
     1 //'REATER THAN HALF A SIDE.')
      IF(IQW.EQ.0) THEN
*        GAUSS-LEGENDRE AND GAUSS-JACOBI INTEGRATION POINTS.
         CALL ALGPT(NX,-1.,1.,ZX,WX)
         CALL ALGJP(NX,ZXJ,WXJ)
         CALL ALGPT(NA/2,-1.,0.,ZA,WA)
         CALL ALGPT(NA/2,0.,1.,ZA(NA/2+1),WA(NA/2+1))
      ELSE
*        EQUAL WEIGHT INTEGRATION POINTS.
         DO 5 I=1,NX
         ZX(I)=(2.0*REAL(I)-1.0)/REAL(NX)-1.0
         WX(I)=2.0/REAL(NX)
         ZXJ(I)=0.5*(2.0*REAL(I)-1.0)/REAL(NX)
5        WXJ(I)=ZXJ(I)/REAL(NX)
         DO 6 I=1,NA
         ZA(I)=(2.0*REAL(I)-1.0)/REAL(NA)-1.0
6        WA(I)=2.0/REAL(NA)
      ENDIF
      IZ(1)=2
      IF((A.NE.B).OR.(IRECT.EQ.1)) IZ(1)=1
      AI=1.0/A
      BI=1.0/B
      AO2=0.5*A
      BO2=0.5*B
      AB=A*B
      PREC=0.0
      L=6
      IZ(6)=NA
*----
*  INTEGRATION IN ANGLE FROM 0. TO PIO2
*----
      ZN1=0.0
      ZN2=0.0
      ZN3=0.0
      DO 360 IA=1,NA
      PHI=0.5*PIO2*(ZA(IA)+1.0)
      SI=SIN(PHI)
      CO=COS(PHI)
      TA=SI/CO
      ZN1=ZN1+SI*WA(IA)
      ZN2=ZN2+SI*SI*WA(IA)
      ZN3=ZN3+SI*SI*SI*WA(IA)
*----
*  FIRST ANGULAR DOMAIN
*----
      L=L+1
      IZ(L)=0
      IF((IZ(1).EQ.2).AND.(PHI.GT.0.5*PIO2)) GO TO 120
      X1=0.0
      XLIM=MIN(A,B/TA)
      DLIM=BO2*CO+(AO2-XLIM)*SI
      L4=L
      Z(L+1)=SI
      Z(L+2)=CO
      L=L+2
      DO 115 K1=1,NZ
      K0=NZ-K1+1
      X2=MIN(XLIM,XLIM-(RAYRE(K0)-DLIM)/SI)
      L3=L
      DO 10 I=1,NZ
10    VAP(I)=0.0
      DO 60 IX=1,NX
      IF(K1.EQ.1) THEN
         S=0.5*(X2-X1)*SI*WX(IX)
         X=X1+0.5*(X2-X1)*(1.0+ZX(IX))
      ELSE
*        FLURIG CHANGE OF VARIABLE.
         S=2.0*(X2-X1)*SI*WXJ(IX)
         X=X1+(X2-X1)*ZXJ(IX)**2
      ENDIF
      Z(L+1)=0.5*S*WA(IA)/A
      L=L+2
      C=BO2*SI-(AO2-X)*CO
      D=BO2*CO+(AO2-X)*SI
      D=D*D
      SUM=0.0
      L1=1
20    K=NZ+1-L1
      RR=RAYRE(K)**2-D
      IF(K.LE.K0) GO TO 30
      CORDE=SQRT(RR)
      DEL=C-CORDE
      SUM=SUM+DEL
      Z(L+L1)=DEL
      VAP(K)=VAP(K)+DEL*S
      C=CORDE
      L1=L1+1
      GO TO 20
30    IF(L1.EQ.1) GO TO 50
      DEL=2.0*CORDE
      SUM=SUM+DEL
      Z(L+L1)=DEL
      VAP(K)=VAP(K)+DEL*S
      DO 40 I=1,L1-2
      DEL=Z(L+L1-I)
      SUM=SUM+DEL
      Z(L+L1+I)=DEL
40    VAP(K+I)=VAP(K+I)+DEL*S
50    IZ(L)=L1
      L=L+L1+L1-1
      DEL=X/CO-SUM
      Z(L)=DEL
      VAP(NZ)=VAP(NZ)+DEL*S
60    CONTINUE
*----
*  VOLUME NORMALIZATION
*----
      IF((INORM.EQ.0).AND.(K0.LT.NZ)) THEN
         DLIM1=BO2*CO+(AO2-X2)*SI
         DLIM2=BO2*CO+(AO2-X1)*SI
         VW1=0.0
         SUMVAP=0.0
         DO 80 I=K0,NZ-1
         SUMVAP=SUMVAP+VAP(I)
         RW=RAYRE(I+1)
         VEX1=RW*RW*ACOS(DLIM1/RW)-DLIM1*SQRT(RW*RW-DLIM1*DLIM1)
         IF(RW.GT.DLIM2)
     1   VEX1=VEX1-(RW*RW*ACOS(DLIM2/RW)-DLIM2*SQRT(RW*RW-DLIM2*DLIM2))
         SUM=(VEX1-VW1)/VAP(I)
         PREC=MAX(PREC,ABS(1.0-SUM)*(VEX1-VW1)/AB)
         VW1=VEX1
80       VAP(I)=SUM
         VEX1=0.5*(B*SI-(A-X1-X2)*CO)*(X2-X1)*SI
         VEX2=0.5*TA*(X2*X2-X1*X1)-VEX1
         SUM=(VEX1-0.5*VW1)/(VEX1-0.5*SUMVAP)
         PREC=MAX(PREC,ABS(1.0-SUM)*(VEX1-0.5*VW1)/AB)
         VEX1=SUM
         SUM=(VEX2-0.5*VW1)/(VEX2-0.5*SUMVAP)
         PREC=MAX(PREC,ABS(1.0-SUM)*(VEX2-0.5*VW1)/AB)
         VEX2=SUM
         DO 110 IX=1,NX
         L3=L3+2
         L1=IZ(L3)
         K=NZ+1-L1
         Z(L3+L1)=Z(L3+L1)*VAP(K)
         DO 100 I=1,L1-2
         Z(L3+L1-I)=Z(L3+L1-I)*VAP(K+I)
100      Z(L3+L1+I)=Z(L3+L1+I)*VAP(K+I)
         Z(L3+1)=Z(L3+1)*VEX1
         Z(L3+L1+L1-1)=Z(L3+L1+L1-1)*VEX2
110      L3=L3+L1+L1-1
      ENDIF
      IZ(L4)=IZ(L4)+NX
      IF(X2.GE.XLIM) GO TO 120
      X1=X2
115   CONTINUE
*----
*  SECOND ANGULAR DOMAIN
*----
120   L=L+1
      IZ(L)=0
      IF(PHI.LE.ATAN(B*AI)) GO TO 240
      X1=B/TA
      XLIM=0.5*(A+B/TA)
      L4=L
      Z(L+1)=SI
      L=L+1
      DO 235 K1=1,NZ
      K0=NZ-K1+1
      X2=XLIM-RAYRE(K0)/SI
      IF(X2.LE.X1) GO TO 235
      L3=L
      DO 130 I=1,NZ
130   VAP(I)=0.0
      DO 180 IX=1,NX
      IF(K1.EQ.1) THEN
         S=0.5*(X2-X1)*SI*WX(IX)
         X=X1+0.5*(X2-X1)*(1.0+ZX(IX))
      ELSE
*        FLURIG CHANGE OF VARIABLE.
         S=2.0*(X2-X1)*SI*WXJ(IX)
         X=X1+(X2-X1)*ZXJ(IX)**2
      ENDIF
      Z(L+1)=S*WA(IA)/A
      L=L+2
      C=BO2*SI-(AO2-X)*CO
      D=BO2*CO+(AO2-X)*SI
      D=D*D
      SUM=0.0
      L1=1
140   K=NZ+1-L1
      RR=RAYRE(K)**2-D
      IF(K.LE.K0) GO TO 150
      CORDE=SQRT(RR)
      DEL=C-CORDE
      SUM=SUM+DEL
      Z(L+L1)=DEL
      VAP(K)=VAP(K)+DEL*S
      C=CORDE
      L1=L1+1
      GO TO 140
150   IF(L1.EQ.1) GO TO 170
      DEL=2.0*CORDE
      SUM=SUM+DEL
      Z(L+L1)=DEL
      VAP(K)=VAP(K)+DEL*S
      DO 160 I=1,L1-2
      DEL=Z(L+L1-I)
      SUM=SUM+DEL
      Z(L+L1+I)=DEL
160   VAP(K+I)=VAP(K+I)+DEL*S
170   IZ(L)=L1
      L=L+L1+L1-1
      DEL=B/SI-SUM
      Z(L)=DEL
      VAP(NZ)=VAP(NZ)+DEL*S
180   CONTINUE
*----
*  VOLUME NORMALIZATION
*----
      IF((INORM.EQ.0).AND.(K0.LT.NZ)) THEN
         DLIM1=BO2*CO+(AO2-X2)*SI
         DLIM2=BO2*CO+(AO2-X1)*SI
         VW1=0.0
         SUMVAP=0.0
         DO 200 I=K0,NZ-1
         SUMVAP=SUMVAP+VAP(I)
         RW=RAYRE(I+1)
         VEX1=RW*RW*ACOS(DLIM1/RW)-DLIM1*SQRT(RW*RW-DLIM1*DLIM1)
         IF(RW.GT.DLIM2)
     1   VEX1=VEX1-(RW*RW*ACOS(DLIM2/RW)-DLIM2*SQRT(RW*RW-DLIM2*DLIM2))
         SUM=(VEX1-VW1)/VAP(I)
         PREC=MAX(PREC,ABS(1.0-SUM)*(VEX1-VW1)/AB)
         VW1=VEX1
200      VAP(I)=SUM
         VEX1=0.5*(B*SI-(A-X1-X2)*CO)*(X2-X1)*SI
         VEX2=(X2-X1)*B-VEX1
         SUM=(VEX1-0.5*VW1)/(VEX1-0.5*SUMVAP)
         PREC=MAX(PREC,ABS(1.0-SUM)*(VEX1-0.5*VW1)/AB)
         VEX1=SUM
         SUM=(VEX2-0.5*VW1)/(VEX2-0.5*SUMVAP)
         PREC=MAX(PREC,ABS(1.0-SUM)*(VEX2-0.5*VW1)/AB)
         VEX2=SUM
         DO 230 IX=1,NX
         L3=L3+2
         L1=IZ(L3)
         K=NZ+1-L1
         Z(L3+L1)=Z(L3+L1)*VAP(K)
         DO 220 I=1,L1-2
         Z(L3+L1-I)=Z(L3+L1-I)*VAP(K+I)
220      Z(L3+L1+I)=Z(L3+L1+I)*VAP(K+I)
         Z(L3+1)=Z(L3+1)*VEX1
         Z(L3+L1+L1-1)=Z(L3+L1+L1-1)*VEX2
230      L3=L3+L1+L1-1
      ENDIF
      IZ(L4)=IZ(L4)+NX
      X1=X2
235   CONTINUE
*----
*  THIRD ANGULAR DOMAIN
*----
240   IF(IZ(1).EQ.2) GO TO 360
      L=L+1
      IZ(L)=0
      IF(PHI.LE.ATAN(A*BI)) GO TO 360
      X1=A/TA
      XLIM=0.5*(B+A/TA)
      L4=L
      Z(L+1)=SI
      L=L+1
      DO 355 K1=1,NZ
      K0=NZ-K1+1
      X2=XLIM-RAYRE(K0)/SI
      IF(X2.LE.X1) GO TO 355
      L3=L
      DO 250 I=1,NZ
250   VAP(I)=0.0
      DO 300 IX=1,NX
      IF(K1.EQ.1) THEN
         S=0.5*(X2-X1)*SI*WX(IX)
         X=X1+0.5*(X2-X1)*(1.0+ZX(IX))
      ELSE
*        FLURIG CHANGE OF VARIABLE.
         S=2.0*(X2-X1)*SI*WXJ(IX)
         X=X1+(X2-X1)*ZXJ(IX)**2
      ENDIF
      Z(L+1)=S*WA(IA)/B
      L=L+2
      C=AO2*SI-(BO2-X)*CO
      D=AO2*CO+(BO2-X)*SI
      D=D*D
      SUM=0.0
      L1=1
260   K=NZ+1-L1
      RR=RAYRE(K)**2-D
      IF(K.LE.K0) GO TO 270
      CORDE=SQRT(RR)
      DEL=C-CORDE
      SUM=SUM+DEL
      Z(L+L1)=DEL
      VAP(K)=VAP(K)+DEL*S
      C=CORDE
      L1=L1+1
      GO TO 260
270   IF(L1.EQ.1) GO TO 290
      DEL=2.0*CORDE
      SUM=SUM+DEL
      Z(L+L1)=DEL
      VAP(K)=VAP(K)+DEL*S
      DO 280 I=1,L1-2
      DEL=Z(L+L1-I)
      SUM=SUM+DEL
      Z(L+L1+I)=DEL
280   VAP(K+I)=VAP(K+I)+DEL*S
290   IZ(L)=L1
      L=L+L1+L1-1
      DEL=A/SI-SUM
      Z(L)=DEL
      VAP(NZ)=VAP(NZ)+DEL*S
300   CONTINUE
*----
*  VOLUME NORMALIZATION
*----
      IF((INORM.EQ.0).AND.(K0.LT.NZ)) THEN
         DLIM1=AO2*CO+(BO2-X2)*SI
         DLIM2=AO2*CO+(BO2-X1)*SI
         VW1=0.0
         SUMVAP=0.0
         DO 320 I=K0,NZ-1
         SUMVAP=SUMVAP+VAP(I)
         RW=RAYRE(I+1)
         VEX1=RW*RW*ACOS(DLIM1/RW)-DLIM1*SQRT(RW*RW-DLIM1*DLIM1)
         IF(RW.GT.DLIM2)
     1   VEX1=VEX1-(RW*RW*ACOS(DLIM2/RW)-DLIM2*SQRT(RW*RW-DLIM2*DLIM2))
         SUM=(VEX1-VW1)/VAP(I)
         PREC=MAX(PREC,ABS(1.0-SUM)*(VEX1-VW1)/AB)
         VW1=VEX1
320      VAP(I)=SUM
         VEX1=0.5*(A*SI-(B-X1-X2)*CO)*(X2-X1)*SI
         VEX2=(X2-X1)*A-VEX1
         SUM=(VEX1-0.5*VW1)/(VEX1-0.5*SUMVAP)
         PREC=MAX(PREC,ABS(1.0-SUM)*(VEX1-0.5*VW1)/AB)
         VEX1=SUM
         SUM=(VEX2-0.5*VW1)/(VEX2-0.5*SUMVAP)
         PREC=MAX(PREC,ABS(1.0-SUM)*(VEX2-0.5*VW1)/AB)
         VEX2=SUM
         DO 350 IX=1,NX
         L3=L3+2
         L1=IZ(L3)
         K=NZ+1-L1
         Z(L3+L1)=Z(L3+L1)*VAP(K)
         DO 340 I=1,L1-2
         Z(L3+L1-I)=Z(L3+L1-I)*VAP(K+I)
340      Z(L3+L1+I)=Z(L3+L1+I)*VAP(K+I)
         Z(L3+1)=Z(L3+1)*VEX1
         Z(L3+L1+L1-1)=Z(L3+L1+L1-1)*VEX2
350      L3=L3+L1+L1-1
      ENDIF
      IZ(L4)=IZ(L4)+NX
      X1=X2
355   CONTINUE
360   CONTINUE
      ZN1=0.5*ZN1*PIO2
      ZN2=0.5*ZN2*PIO2
      ZN3=0.5*ZN3*PIO2
      Z(2)=1.0/SQRT(ZN1)
      Z(3)=1.0/SQRT(0.75*ZN3-0.7205061948*ZN2*ZN2/ZN1)
      Z(4)=Z(3)*0.8488263632*ZN2/ZN1
      Z(5)=2.0/SQRT(3.0*(ZN1-ZN3))
*----
*  TRACKING INFORMATION OUTPUT
*----
      IF(ILIGN.NE.1) RETURN
      WRITE (6,500) (Z(I),I=2,5)
      L1=6
      L2=0
      DO 450 IA=1,NA
      L1=L1+1
      NLX=IZ(L1)
      IF(NLX.EQ.0) GO TO 390
      DO 380 IX=1,NLX
      L2=L2+1
      IF(IX.NE.1) GO TO 370
      L3=2*IZ(L1+4)-1
      WRITE (6,510) L2,(Z(L1+I),I=1,3),IZ(L1+4),(Z(L1+I+4),I=1,L3)
      L1=L1+L3+4
      GO TO 380
370   L3=2*IZ(L1+2)-1
      WRITE (6,530) L2,Z(L1+1),IZ(L1+2),(Z(L1+I+2),I=1,L3)
      L1=L1+L3+2
380   CONTINUE
390   L1=L1+1
      NLX=IZ(L1)
      IF(NLX.EQ.0) GO TO 420
      DO 410 IX=1,NLX
      L2=L2+1
      IF(IX.NE.1) GO TO 400
      L3=2*IZ(L1+3)-1
      WRITE (6,520) L2,(Z(L1+I),I=1,2),IZ(L1+3),(Z(L1+I+3),I=1,L3)
      L1=L1+L3+3
      GO TO 410
400   L3=2*IZ(L1+2)-1
      WRITE (6,530) L2,Z(L1+1),IZ(L1+2),(Z(L1+I+2),I=1,L3)
      L1=L1+L3+2
410   CONTINUE
420   IF(IZ(1).EQ.2) GO TO 450
      L1=L1+1
      NLX=IZ(L1)
      IF(NLX.EQ.0) GO TO 450
      DO 440 IX=1,NLX
      L2=L2+1
      IF(IX.NE.1) GO TO 430
      L3=2*IZ(L1+3)-1
      WRITE (6,520) L2,(Z(L1+I),I=1,2),IZ(L1+3),(Z(L1+I+3),I=1,L3)
      L1=L1+L3+3
      GO TO 440
430   L3=2*IZ(L1+2)-1
      WRITE (6,530) L2,Z(L1+1),IZ(L1+2),(Z(L1+I+2),I=1,L3)
      L1=L1+L3+2
440   CONTINUE
450   CONTINUE
      RETURN
*
500   FORMAT (1H1//30H TRACKING INFORMATION LISTING.//12H NUMERICAL O,
     1 27HRTHONORMALIZATION FACTORS =,1P,4E12.4//6H TRACK)
510   FORMAT (1X,I5,7H  SIN =,1P,E10.3,7H  COS =,E10.3,9H WEIGHT =,
     1 E10.3,6H  MZ =,I3,12H  SEGMENTS =,5E10.3/(80X,5E10.3))
520   FORMAT (1X,I5,7H  SIN =,1P,E10.3,17X,9H WEIGHT =,E10.3,6H  MZ =,
     1 I3,12H  SEGMENTS =,5E10.3/(80X,5E10.3))
530   FORMAT (1X,I5,34X,9H WEIGHT =,1P,E10.3,6H  MZ =,I3,10H  SEGMENTS,
     1 2H =,5E10.3/(80X,5E10.3))
      END
