*DECK SYBUQV
      SUBROUTINE SYBUQV(ZZR,ZZI,NSURF,NREG,SIGT,MNA,LGFULL,PIJ,POPTX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the one-group collision, DP-0 leakage and DP-0 transmission
* probabilities in a Cartesian or hexagonal sectorized cell.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* ZZR     real tracking elements.
* ZZI     integer tracking elements.
* NSURF   number of surfaces (4 or 6).
* NREG    number of regions.
* SIGT    total macroscopic cross section.
* MNA     number of angles.
* LGFULL  void flad (=.true. in voided regions).
*
*Parameters: output
* PIJ     collision, DP-0 leakage and DP-0 transmission probabilities
*         in lower triangular form.
*
*Parameters: scratch
* POPTX
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER   ZZI(*),NSURF,NREG,MNA
      REAL      ZZR(*),SIGT(NSURF:NSURF+NREG-1),
     >          PIJ(0:(NREG+NSURF)*(NREG+NSURF+1)/2-1),POPTX(2*NREG+4)
      LOGICAL   LGFULL(NSURF:NSURF+NREG-1)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (MKI3=600,ZI30=0.785398164)
      LOGICAL   LGEMPT
      COMMON /BICKL3/BI3(0:MKI3),BI31(0:MKI3),BI32(0:MKI3),PAS3,XLIM3,L3
*----
*  CHECK FOR VOIDED REGIONS
*----
      LGEMPT=.FALSE.
      DO 10 IR=1,NREG
   10 LGEMPT=LGEMPT.OR.(.NOT.LGFULL(IR+NSURF-1))
*----
*  INITIALIZATION
*----
      IZ0=0
      IZR=4
      DO 20 I=0,(NREG+NSURF)*(NREG+NSURF+1)/2-1
   20 PIJ(I)=0.
*
      IF(LGEMPT) THEN
*        VOIDED REGION DETECTED.
*        LOOP OVER THE ANGLE(FROM 0 TO PI/3).
         DO 80 IA=1,MNA
         MNT=ZZI(IZ0+1)
         IZ0=IZ0+2
         IZR=IZR+NSURF*2+1
*
*        LOOP OVER COMPLETE TRACKS.
         DO 70 ITT=1,MNT
         IZ0=IZ0+1
         NH=ZZI(IZ0)
         IZ0=IZ0+1
         NX=ZZI(IZ0)
*
*        WEST SIDE.
         IZ0=IZ0+1
         ISW=ZZI(IZ0)
         JZI=IZ0
         DO 60 ITX=1,NX
         IZR=IZR+1
         WITRAJ=ZZR(IZR)
         JZR=IZR
         IZ0=JZI
         IF(WITRAJ.LT.0.0) CALL XABORT('SYBUQV: FAILURE 1.')
         IF(NH.GT.2*NREG+4) CALL XABORT('SYBUQV: FAILURE 2.')
*
*        COMPUTE OPTICAL LENGTHS.
         DO 30 IX=1,NH
         IZ0=IZ0+1
         IR=ZZI(IZ0)
         IZR=IZR+1
         SEG=ZZR(IZR)
   30    POPTX(IX)=SEG*SIGT(IR)
*
*        EXTERNAL IS IR=0
         IZ0=JZI
         IRS=ISW
         POP1=0.
         DO 50 IXI=1,NH
         IRT=IRS
         IZ00=IZ0
         IZ0=IZ0+1
         IRS=ZZI(IZ0)
*
*        PREVIOUS REGION J
         JZ0=IZ00
         PKI3=ZI30*WITRAJ
         TAUX=0.
*
*        BEGINNING WITH IRS == JRS, P(i,i)=Taup  -(B3(0)-P3)
         IIJ=IRS*(IRS+3)/2
         IF(LGFULL(IRS)) THEN
            PIJ(IIJ)=PIJ(IIJ)+POPTX(IXI)*WITRAJ
         ELSE
            CALL SYB33C(PPLUS,POPTX(IXI))
            PIJ(IIJ)=PIJ(IIJ)+PPLUS*WITRAJ
         ENDIF
         DO 40 IXJ=IXI,NH
         JZ0=JZ0+1
         JRS=ZZI(JZ0)
         QIJ3=PKI3
         TAUP=TAUX
         TAUX=TAUX+POPTX(IXJ)
         IF(TAUX.GE.XLIM3) THEN
            PKI3=0.0
         ELSE
            K=NINT(TAUX*PAS3)
            PKI3=(BI3(K)+TAUX*(BI31(K)+TAUX*BI32(K)))*WITRAJ
         ENDIF
         SIJ3=QIJ3-PKI3
*
*        BEGINNING OF PIJ : REGION IRS, JRS
         IF(JRS.LE.IRS) THEN
            IIJ=IRS*(IRS+1)/2+JRS
         ELSE
            IIJ=JRS*(JRS+1)/2+IRS
         ENDIF
         IF(LGFULL(IRS) .AND. LGFULL(JRS)) THEN
            PIJ(IIJ)=PIJ(IIJ)-SIJ3
         ENDIF
*
*        REMAINING OF PIJ : Region IRS, JRS
*        OR ... PSI IFF IXI=1, IRT IS THE WEST SIDE
         IF(JRS.LE.IRT) THEN
            IIJ=IRT*(IRT+1)/2+JRS
         ELSE
            IIJ=JRS*(JRS+1)/2+IRT
         ENDIF
         IF(LGFULL(JRS)) THEN
            IF(IRT.LT.NSURF) THEN
               PIJ(IIJ)=PIJ(IIJ)+SIJ3
            ELSE IF(LGFULL(IRT)) THEN
               PIJ(IIJ)=PIJ(IIJ)+SIJ3
            ELSE
               CALL SYB31C(PPLUS,TAUP,POP1,POPTX(IXJ))
               PIJ(IIJ)=PIJ(IIJ)+PPLUS*WITRAJ
            ENDIF
         ELSE IF(IRT.LT.NSURF) THEN
            CALL SYB32C(PPLUS,TAUP,POPTX(IXJ),2)
            PIJ(IIJ)=PIJ(IIJ)+PPLUS*WITRAJ
         ELSE
            CALL SYB31C(PPLUS,TAUP,POP1,POPTX(IXJ))
            PIJ(IIJ)=PIJ(IIJ)+PPLUS*WITRAJ
         ENDIF
   40    CONTINUE
*
*        COMPUTE LEAKAGE AND TRANSMISSION PROBABILITIES PSI, PSS
         JZ0=JZ0+1
         ISE=ZZI(JZ0)
*        NOTE: THE CONDITION ISE.LE.IRS IS TRUE SINCE NH > 0
*        ISE IS THE EAST SIDE(0 <= ISE <= NSURF-1)
*        IRS IS A REGION(IRS >= NSURF)
*
*        BEGINNING OF PSI
         IIJ=IRS*(IRS+1)/2+ISE
         IF(LGFULL(IRS)) PIJ(IIJ)=PIJ(IIJ)-PKI3
*
*        NOTE: IF IXI=1, CONDITION(ISE.LE.IRT) MAY BE FALSE.
*        COMPUTE PSS BETWEEN SIDES IRT AND ISE, --OR--
*        COMPURE PSI BETWEEN REGION IRT AND EAST SIDE ISE
         IF(ISE.LE.IRT) THEN
            IIJ=IRT*(IRT+1)/2+ISE
         ELSE
            IIJ=ISE*(ISE+1)/2+IRT
         ENDIF
         IF(IRT.LT.NSURF) THEN
            PIJ(IIJ)=PIJ(IIJ)+PKI3
         ELSE IF(LGFULL(IRT)) THEN
            PIJ(IIJ)=PIJ(IIJ)+PKI3
         ELSE
            CALL SYB32C(PPLUS,TAUX,POP1,2)
            PIJ(IIJ)=PIJ(IIJ)+PPLUS*WITRAJ
         ENDIF
   50    POP1=POPTX(IXI)
*
*        COMPUTE REMAINING PSI FROM LAST REGION TO EAST SIDE.
         IZ0=IZ0+1
         ISE=ZZI(JZ0)
         IF(ISE.LE.IRS) THEN
            IIJ=IRS*(IRS+1)/2+ISE
         ELSE
            IIJ=ISE*(ISE+1)/2+IRS
         ENDIF
         IF(LGFULL(IRS)) THEN
            PIJ(IIJ)=PIJ(IIJ)+ZI30*WITRAJ
         ELSE
            TAUX=0.
            POP1=POPTX(NH)
            CALL SYB32C(PPLUS,TAUX,POP1,2)
            PIJ(IIJ)=PIJ(IIJ)+PPLUS*WITRAJ
         ENDIF
*        END OF TRACK
   60    CONTINUE
   70    CONTINUE
*        END OF ANGLE
   80    CONTINUE
      ELSE
*        NO VOIDED REGION DETECTED. FAST INTEGRATION METHOD.
*        LOOP OVER THE ANGLE(FROM 0 TO PI/3).
         DO 140 IA=1,MNA
         MNT=ZZI(IZ0+1)
         IZ0=IZ0+2
         IZR=IZR+NSURF*2+1
*
*        LOOP OVER COMPLETE TRACKS.
         DO 130 ITT=1,MNT
         IZ0=IZ0+1
         NH=ZZI(IZ0)
         IZ0=IZ0+1
         NX=ZZI(IZ0)
*
*        WEST SIDE
         IZ0=IZ0+1
         ISW=ZZI(IZ0)
         JZI=IZ0
         DO 120 ITX=1,NX
         IZR=IZR+1
         WITRAJ=ZZR(IZR)
         JZR=IZR
         IZ0=JZI
         IF(WITRAJ.LT.0.0) CALL XABORT('SYBUQV: FAILURE 3.')
         IF(NH.GT.2*NREG+4) CALL XABORT('SYBUQV: FAILURE 4.')
*
*        COMPUTE OPTICAL LENGTHS.
         DO 90 IX=1,NH
         IZ0=IZ0+1
         IR=ZZI(IZ0)
         IZR=IZR+1
         SEG=ZZR(IZR)
   90    POPTX(IX)=SEG*SIGT(IR)
*
*        EXTERNAL IS IR=0
         IZ0=JZI
         IRS=ISW
         DO 110 IXI=1,NH
         IRT=IRS
         IZ00=IZ0
         IZ0=IZ0+1
         IRS=ZZI(IZ0)
*
*        PREVIOUS REGION J
         JZ0=IZ00
         PKI3=ZI30*WITRAJ
         TAUX=0.
*
*        BEGINNING WITH IRS == JRS,P(i,i)=Taup  -(B3(0)-P3)
         IIJ=IRS*(IRS+3)/2
         PIJ(IIJ)=PIJ(IIJ)+POPTX(IXI)*WITRAJ
         DO 100 IXJ=IXI,NH
         JZ0=JZ0+1
         JRS=ZZI(JZ0)
         QIJ3=PKI3
         TAUX=TAUX+POPTX(IXJ)
         IF(TAUX.GE.XLIM3) THEN
            PKI3=0.0
         ELSE
            K=NINT(TAUX*PAS3)
            PKI3=(BI3(K)+TAUX*(BI31(K)+TAUX*BI32(K)))*WITRAJ
         ENDIF
         SIJ3=QIJ3-PKI3
*
*        BEGINNING OF PIJ : REGION IRS,JRS
         IF(JRS.LE.IRS) THEN
           IIJ=IRS*(IRS+1)/2+JRS
         ELSE
           IIJ=JRS*(JRS+1)/2+IRS
         ENDIF
         PIJ(IIJ)=PIJ(IIJ)-SIJ3
*
*        REMAINING OF PIJ : Region IRS,JRS
*        OR ... PSI IFF IXI=1, IRT IS THE WEST SIDE
         IF(JRS.LE.IRT) THEN
           IIJ=IRT*(IRT+1)/2+JRS
         ELSE
           IIJ=JRS*(JRS+1)/2+IRT
         ENDIF
  100    PIJ(IIJ)=PIJ(IIJ)+SIJ3
*
*        COMPUTE LEAKAGE AND TRANSMISSION PROBABILITIES PSI, PSS
         JZ0=JZ0+1
         ISE=ZZI(JZ0)
         IIJ=IRS*(IRS+1)/2+ISE
         PIJ(IIJ)=PIJ(IIJ)-PKI3
         IF(ISE.LE.IRT) THEN
            IIJ=IRT*(IRT+1)/2+ISE
         ELSE
            IIJ=ISE*(ISE+1)/2+IRT
         ENDIF
  110    PIJ(IIJ)=PIJ(IIJ)+PKI3
*
*        COMPUTE REMAINING PSI FROM LAST REGION TO EAST SIDE.
         IZ0=IZ0+1
         ISE=ZZI(JZ0)
         IF(ISE.LE.IRS) THEN
            IIJ=IRS*(IRS+1)/2+ISE
         ELSE
            IIJ=ISE*(ISE+1)/2+IRS
         ENDIF
         PIJ(IIJ)=PIJ(IIJ)+ZI30*WITRAJ
*        END OF TRACK
  120    CONTINUE
  130    CONTINUE
*        END OF ANGLE
  140    CONTINUE
      ENDIF
*----
*  APPLY SYMMETRY FACTORS
*----
      IIJ=0
      DO 160 JS=0,NSURF-1
      DO 150 IS=0,JS-1
      PIJ(IIJ)=PIJ(IIJ) / 4.0
  150 IIJ=IIJ+1
  160 IIJ=IIJ+1
      IIJ= NSURF*(NSURF+1) / 2
      IIJ= IIJ-1
      DO 180 JR=1,NREG
      IIJ=IIJ+NSURF
      DO 170 IR=1,JR-1
  170 PIJ(IIJ+IR)=PIJ(IIJ+IR) / 4.0
      IIJ=IIJ+JR
  180 PIJ(IIJ)=PIJ(IIJ) / 2.0
      IIJ= NSURF *(NSURF+1) / 2
      DO 200 IR=1,NREG
      DO 190 JS=0,NSURF-1
  190 PIJ(IIJ+JS)=PIJ(IIJ+JS) / 4.0
  200 IIJ=IIJ+NSURF+IR
*
      RETURN
      END
