*DECK USSFLU
      SUBROUTINE USSFLU(MAXNOR,IPTRK,IPLIB,IPLI0,IFTRAK,NREG,NUN,NBMIX,
     1 NBISO,NIRES,NL,NED,NDEL,ISONAM,ISOBIS,HCAL,MAT,VOL,KEYFLX,CDOOR,
     2 LEAKSW,IMPX,DEN,MIX,IAPT,ITRANC,IPHASE,NGRP,IGRMIN,IGRMAX,NBNRS,
     3 IREX,TITR,ICORR,ISUBG,UNGAR,PHGAR,STGAR,SFGAR,SSGAR,S0GAR,SAGAR,
     4 SDGAR,SWGAR,SIGT,SIGS0,SIGS1,ISMIN,ISMAX,MASKG,GOLD,GAS,GA2,NOR,
     5 IPPT,IWRK,SIGGAR,ISM,XFLUX,CONR,VOLMER,DELTA,MASKH)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the self-shielded cross sections in each energy group using a
* subgroup approach.
*
*Copyright:
* Copyright (C) 2003 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* MAXNOR  maximum order of the probability tables (PT).
* IPTRK   pointer to the tracking (L_TRACK signature).
* IPLIB   pointer to the internal microscopic cross section library
*         with subgroups (L_LIBRARY signature).
* IPLI0   pointer to the internal microscopic cross section library
*         builded by the self-shielding module.
* IFTRAK  file unit number used to store the tracks.
* NREG    number of regions.
* NUN     number of unknowns per energy group and band.
* NBMIX   number of mixtures in the internal library.
* NBISO   number of isotopes.
* NIRES   number of correlated resonant isotopes.
* NL      number of legendre orders required in the calculation
*         (NL=1 or higher).
* NED     number of extra vector edits.
* NDEL    number of delayed neutron precursor groups.
* ISONAM  alias name of isotopes in IPLIB.
* ISOBIS  alias name of isotopes in IPLI0.
* HCAL    name of the self-shielding calculation.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KEYFLX  pointers of fluxes in unknown vector.
* CDOOR   name of the geometry/solution operator.
* LEAKSW  leakage flag (LEAKSW=.true. if neutron leakage through
*         external boundary is present).
* IMPX    print flag (equal to zero for no print).
* DEN     density of each isotope.
* MIX     mix number of each isotope (can be zero).
* IAPT    resonant isotope index associated with isotope I. Mixed
*         moderator if IAPT(I)=NIRES+1. Out-of-fuel isotope if
*         IAPT(I)=0.
* ITRANC  type of transport correction.
* IPHASE  type of flux solution (=1: use a native flux solution door;
*         =2: use collision probabilities).
* NGRP    number of energy groups.
* IGRMIN  first group where the self-shielding is applied.
* IGRMAX  most thermal group where the self-shielding is applied.
* NBNRS   number of correlated fuel regions. Note that NBNRS=max(IREX).
* IREX    fuel region index assigned to each mixture. Equal to zero
*         in non-resonant mixtures or in mixtures not used.
* TITR    title.
* ICORR   mutual resonance shielding flag (=1: to suppress the model
*         in cases it is required in LIB operator).
* ISUBG   type of self-shielding model (=1: use physical probability
*         tables; =4: use Ribon extended method).
*
*Parameters: output
* UNGAR   averaged fluxes per volume.
* PHGAR   averaged fluxes.
* STGAR   microscopic self-shielded total x-s.
* SFGAR   microscopic self-shielded fission x-s.
* SSGAR   microscopic self-shielded scattering x-s.
* S0GAR   microscopic transfer scattering xs (isotope,secondary,
*         primary).
* SAGAR   microscopic self-shielded additional xs.
* SDGAR   microscopic self-shielded delayed nu-sigf xs.
* SWGAR   microscopic secondary slowing-down cross sections (ISUBG=4).
* SIGT    microscopic total x-s.
* SIGS0   microscopic P0 scattering x-s.
* SIGS1   microscopic transport correction.
* ISMIN   first secondary group indices.
* ISMAX   last secondary group indices.
* MASKG   energy group mask pointing on self-shielded groups.
*
*Parameters: scratch
* GOLD    Goldstein-Cohen parameters.
* GAS
* GA2
* NOR
* IPPT
* IWRK
* SIGGAR
* ISM
* XFLUX
* CONR
* VOLMER
* DELTA
* MASKH
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER MAXNOR,IPTRK,IPLIB,IPLI0,IFTRAK,NREG,NUN,NBMIX,NBISO,
     1 NIRES,NL,NED,NDEL,ISONAM(3,NBISO),ISOBIS(3,NBISO),MAT(NREG),
     2 KEYFLX(NREG),IMPX,MIX(NBISO),IAPT(NBISO),ITRANC,IPHASE,NGRP,
     3 IGRMIN,IGRMAX,NBNRS,IREX(NBMIX),ICORR,ISUBG,ISMIN(NL,NGRP),
     4 ISMAX(NL,NGRP),NOR(NIRES,NGRP),IPPT(NIRES,5),IWRK(NGRP),ISM(2,NL)
      REAL VOL(NREG),DEN(NBISO),UNGAR(NREG,NIRES,NGRP),
     1 PHGAR(NBNRS,NIRES,NGRP),STGAR(NBNRS,NIRES,NGRP),
     2 SFGAR(NBNRS,NIRES,NGRP),SSGAR(NBNRS,NIRES,NL,NGRP),
     3 S0GAR(NBNRS,NIRES,NL,NGRP,NGRP),SAGAR(NBNRS,NIRES,NED,NGRP),
     4 SDGAR(NBNRS,NIRES,NDEL,NGRP),SWGAR(NBNRS,NIRES,NGRP),
     5 SIGT(NGRP,NBISO),SIGS0(NGRP,NBISO),SIGS1(NGRP,NBISO),
     6 GOLD(NIRES,NGRP),GAS(NGRP),GA2(NGRP,NGRP),
     7 SIGGAR(NBMIX,0:NIRES,NGRP,3),XFLUX(NBNRS,MAXNOR,NL,NIRES),
     8 CONR(NBNRS,NIRES),VOLMER(0:NBNRS),DELTA(NGRP)
      LOGICAL LEAKSW,MASKG(NGRP),MASKH(NGRP)
      CHARACTER HCAL*12,CDOOR*12,TITR*72
*----
*  LOCAL VARIABLES
*----
      LOGICAL LRES,LLIB
      PARAMETER (MAXED=50)
      CHARACTER HNAMIS*12,TEXT12*12,HVECT(MAXED)*6,CBDPNM*12
      COMMON BASE(1)
*
      CALL KDRCPU(TK1)
      DO 10 IG1=1,NGRP
      DO 10 IL=1,NL
      ISMIN(IL,IG1)=NGRP
   10 ISMAX(IL,IG1)=1
      DO 20 IRES=1,NIRES
   20 NOR(IRES,1)=-1
*
      IF(NED.GT.0) THEN
         IF(NED.GT.MAXED) CALL XABORT('USSFLU: INVALID VALUE OF MAXED.')
         CALL SETARA(BASE,2*NED,IJCEDM)
         CALL LCMGET(IPLIB,'ADDXSNAME-P0',BASE(IJCEDM))
         IDEB=IJCEDM
         DO 30 J=1,NED
         WRITE(HVECT(J),'(A4,A2)') BASE(IDEB),BASE(IDEB+1)
   30    IDEB=IDEB+2
         CALL RLSARA(BASE(IJCEDM))
      ENDIF
*
      DO 190 ISOT=1,NBISO
      IRES=IAPT(ISOT)
      WRITE(HNAMIS,'(3A4)') (ISONAM(J,ISOT),J=1,3)
      CALL LCMLEN(IPLIB,HNAMIS,LENGT,ITYLCM)
      IF(LENGT.EQ.0) CALL XABORT('USSFLU: UNABLE TO FIND ISOTOPE '''
     1 //HNAMIS//'''.')
*----
*  RECOVER INFINITE DILUTION OR SELF-SHIELDED CROSS SECTIONS.
*----
      WRITE(TEXT12,'(3A4)') (ISOBIS(I0,ISOT),I0=1,3)
      CALL LCMLEN(IPLI0,TEXT12,ILEN0,ITYLCM)
      IF(ILEN0.NE.0) THEN
         CALL LCMSIX(IPLI0,TEXT12,1)
         CALL LCMLEN(IPLI0,'NTOT0',ILEN0,ITYLCM)
         IF(ILEN0.NE.0) THEN
            LLIB=.FALSE.
            IPP=IPLI0
         ELSE
            LLIB=.TRUE.
            TEXT12=HNAMIS
            IPP=IPLIB
            CALL LCMSIX(IPLI0,' ',2)
            CALL LCMSIX(IPP,TEXT12,1)
         ENDIF
      ELSE
         LLIB=.TRUE.
         TEXT12=HNAMIS
         IPP=IPLIB
         CALL LCMSIX(IPP,TEXT12,1)
      ENDIF
      IF((ILEN0.NE.0).AND.(IMPX.GT.2)) WRITE(6,'(/14H USSFLU: RECOV,
     1 12HER ISOTOPE '',A12,23H'' FROM THE NEW LIBRARY.)') TEXT12
      CALL LCMLEN(IPP,'NTOT0',LENGT,ITYLCM)
      IF(LENGT.NE.NGRP) CALL XABORT('USSFLU: INVALID X-SECTIONS.')
      CALL LCMGET(IPP,'NTOT0',SIGT(1,ISOT))
      CALL LCMGET(IPP,'SIGS00',SIGS0(1,ISOT))
      CALL LCMLEN(IPP,'TRANC',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
         CALL LCMGET(IPP,'TRANC',SIGS1(1,ISOT))
      ELSE
         DO 40 IGRP=1,NGRP
   40    SIGS1(IGRP,ISOT)=0.0
      ENDIF
      CALL LCMSIX(IPP,' ',2)
      CALL LCMGET(IPP,'DELTAU',DELTA)
      IF(.NOT.LLIB) IPLI0=IPP
*----
*  RECOVER PROBABILITY TABLE INFORMATION.
*----
      IF((IRES.GT.0).AND.(IRES.LE.NIRES)) THEN
         IF(NOR(IRES,1).EQ.-1) THEN
            CALL LCMSIX(IPLIB,HNAMIS,1)
*
*           RECOVER INFINITE DILUTION VALUES.
            CALL LCMGET(IPLIB,'NTOT0',GAS)
            DO 55 IG=1,NGRP
            DO 55 IND=1,NBNRS
            STGAR(IND,IRES,IG)=GAS(IG)
            SFGAR(IND,IRES,IG)=0.0
            SWGAR(IND,IRES,IG)=0.0
            DO 50 IED=1,NED
   50       SAGAR(IND,IRES,IED,IG)=0.0
            DO 55 IDEL=1,NDEL
   55       SDGAR(IND,IRES,IDEL,IG)=0.0
            CALL LCMLEN(IPLIB,'NUSIGF',ILENGT,ITYLCM)
            IF(ILENGT.GT.0) THEN
               CALL LCMGET(IPLIB,'NUSIGF',GAS)
               DO 60 IG=1,NGRP
               DO 60 IND=1,NBNRS
   60          SFGAR(IND,IRES,IG)=GAS(IG)
            ENDIF
            CALL SETARA(BASE,NGRP*NGRP,IXSSCM)
            CALL SETARA(BASE,NGRP,INJJ)
            CALL SETARA(BASE,NGRP,IIJJ)
            DO 80 IL=1,NL
            CALL XDRLGS(IPLIB,-1,IL-1,IL-1,1,NGRP,GAS,GA2,ITYPRO,
     1      BASE(IXSSCM),BASE(INJJ),BASE(IIJJ))
*           JG IS THE SECONDARY GROUP.
            DO 70 IG=1,NGRP
            DO 70 IND=1,NBNRS
            SSGAR(IND,IRES,IL,IG)=GAS(IG)
            DO 70 JG=1,NGRP
            IF(IL.EQ.1) THEN
              SWGAR(IND,IRES,JG)=SWGAR(IND,IRES,JG)+GA2(JG,IG)*DELTA(IG)
            ENDIF
   70       S0GAR(IND,IRES,IL,JG,IG)=GA2(JG,IG)
   80       CONTINUE
            DO 90 IG=1,NGRP
            DO 90 IND=1,NBNRS
   90       SWGAR(IND,IRES,IG)=SWGAR(IND,IRES,IG)/DELTA(IG)
            CALL RLSARA(BASE(IIJJ))
            CALL RLSARA(BASE(INJJ))
            CALL RLSARA(BASE(IXSSCM))
            DO 110 IED=1,NED
            CALL LCMLEN(IPLIB,HVECT(IED),ILENGT,ITYLCM)
            IF(ILENGT.GT.0) THEN
               CALL LCMGET(IPLIB,HVECT(IED),GAS)
               DO 100 IG=1,NGRP
               DO 100 IND=1,NBNRS
  100          SAGAR(IND,IRES,IED,IG)=GAS(IG)
            ENDIF
  110       CONTINUE
            DO 130 IDEL=1,NDEL
            WRITE(TEXT12,'(6HNUSIGF,I2.2)') IDEL
            CALL LCMLEN(IPLIB,TEXT12,ILENGT,ITYLCM)
            IF(ILENGT.GT.0) THEN
               CALL LCMGET(IPLIB,TEXT12,GAS)
               DO 120 IG=1,NGRP
               DO 120 IND=1,NBNRS
  120          SDGAR(IND,IRES,IDEL,IG)=GAS(IG)
            ENDIF
  130       CONTINUE
*
            CALL LCMSIX(IPLIB,'PT-PHYS',1)
               CALL LCMGET(IPLIB,'NOR',IWRK)
               JPLIB=LCMGID(IPLIB,'GROUP')
               DO 150 IG1=1,NGRP
               IF(IWRK(IG1).GT.1) THEN
                  KPLIB=LCMGIL(JPLIB,IG1)
                  CALL LCMGET(KPLIB,'ISM-LIMITS',ISM)
                  DO 140 IL=1,NL
                  ISMIN(IL,IG1)=MIN(ISMIN(IL,IG1),ISM(1,IL))
  140             ISMAX(IL,IG1)=MAX(ISMAX(IL,IG1),ISM(2,IL))
               ENDIF
  150          NOR(IRES,IG1)=IWRK(IG1)
            CALL LCMSIX(IPLIB,' ',2)
            CALL LCMLEN(IPLIB,'NGOLD',LENGT,ITYLCM)
            IF(LENGT.GT.0) THEN
               CALL SETARA(BASE,NGRP,IGOLD)
               CALL LCMGET(IPLIB,'NGOLD',BASE(IGOLD))
               DO 160 IG1=1,NGRP
  160          GOLD(IRES,IG1)=BASE(IGOLD+IG1-1)
               CALL RLSARA(BASE(IGOLD))
            ELSE
               DO 170 IG1=1,NGRP
  170          GOLD(IRES,IG1)=1.0
            ENDIF
            CALL LCMLEN(IPLIB,'BIN-NFS',LENGT,ITYLCM)
            IF(LENGT.GT.0) THEN
               CALL LCMGET(IPLIB,'BIN-NFS',IWRK)
               DO 180 IG1=1,NGRP
               IF((GOLD(IRES,IG1).EQ.-999.).AND.(IWRK(IG1).EQ.0)) THEN
                  GOLD(IRES,IG1)=1.0
               ENDIF
  180          CONTINUE
            ENDIF
            CALL LCMSIX(IPLIB,' ',2)
         ENDIF
      ENDIF
  190 CONTINUE
      CALL KDRCPU(TK2)
      IF(IMPX.GT.1) WRITE(6,'(/34H USSFLU: CPU TIME SPENT TO RECOVER,
     1 23H INFINITE-DILUTION XS =,F8.1,8H SECOND./)') TK2-TK1
*
      CALL KDRCPU(TK1)
      TK4=0.0
      TK5=0.0
      ICPIJ=0
*----
*  COMPUTE THE MERGED VOLUMES AND NUMBER DENSITIES.
*----
      DO 200 I=0,NBNRS
  200 VOLMER(I)=0.0
      DO 210 I=1,NREG
      IBM=MAT(I)
      IF(IBM.GT.0) VOLMER(IREX(IBM))=VOLMER(IREX(IBM))+VOL(I)
  210 CONTINUE
      DO 220 JRES=1,NIRES
      DO 220 IND=1,NBNRS
  220 CONR(IND,JRES)=0.0
      DO 240 ISO=1,NBISO
      JRES=IAPT(ISO)
      IF((JRES.GT.0).AND.(JRES.LE.NIRES)) THEN
         DENN=DEN(ISO)
         DO 230 IREG=1,NREG
         IBM=MAT(IREG)
         IF(MIX(ISO).EQ.IBM) THEN
            IND0=IREX(IBM)
            IF(IND0.EQ.0) CALL XABORT('USSFLU: IREX FAILURE.')
            CONR(IND0,JRES)=CONR(IND0,JRES)+DENN*VOL(IREG)/VOLMER(IND0)
         ENDIF
  230    CONTINUE
      ENDIF
  240 CONTINUE
*----
*  RECOVER POSITION OF PROBABILITY TABLES AND NAME OF RESONANT ISOTOPE.
*----
      DO 270 IRES=1,NIRES
      DO 250 JSOT=1,NBISO
      IF(IAPT(JSOT).EQ.IRES) THEN
         ISOT=JSOT
         GO TO 260
      ENDIF
  250 CONTINUE
      CALL XABORT('USSFLU: UNABLE TO FIND A RESONANT ISOTOPE.')
  260 WRITE(TEXT12,'(3A4)') (ISONAM(J,ISOT),J=1,3)
      CALL LCMSIX(IPLIB,TEXT12,1)
      CALL LCMLEN(IPLIB,'PT-PHYS',ILONG,ITYLCM)
      IF(ILONG.EQ.0) CALL XABORT('USSFLU: BUG1.')
      CALL LCMSIX(IPLIB,'PT-PHYS',1)
      CALL LCMLEN(IPLIB,'GROUP',LENG,ITYLCM)
      IF(ILONG.EQ.0) CALL XABORT('USSFLU: BUG2.')
      IPPT(IRES,1)=LCMGID(IPLIB,'GROUP')
      CALL LCMSIX(IPLIB,' ',2)
      IPPT(IRES,2)=IREX(MIX(ISOT))
      IPPT(IRES,3)=ISONAM(1,ISOT)
      IPPT(IRES,4)=ISONAM(2,ISOT)
      IPPT(IRES,5)=ISONAM(3,ISOT)
      CALL LCMSIX(IPLIB,' ',2)
      IF(IPPT(IRES,2).LE.0) CALL XABORT('USSFLU: BUG3.')
  270 CONTINUE
*----
*  DETERMINE WHICH GROUPS ARE SELF-SHIELDED.
*----
      DO 290 IGRP=1,NGRP
      LRES=.FALSE.
      DO 280 IRES=1,NIRES
  280 LRES=LRES.OR.(NOR(IRES,IGRP).GT.1)
  290 MASKG(IGRP)=((IGRP.GE.IGRMIN).AND.(IGRP.LE.IGRMAX).AND.LRES)
*----
*  INITIALIZATION OF THE MULTIBAND FLUXES AND SOURCES.
*----
      CALL LCMSIX(IPLI0,'SHIBA_SG',1)
      CALL LCMSIX(IPLI0,HCAL,1)
      DO 310 IRES=1,NIRES
      WRITE(CBDPNM,'(3HCOR,I4.4,1H/,I4.4)') IRES,NIRES
      CALL LCMSIX(IPLI0,CBDPNM,1)
      JPLI0=LCMLID(IPLI0,'GROUP_INFO',NGRP)
      CALL XDRSET(XFLUX(1,1,1,IRES),NBNRS*MAXNOR*NL,1.0)
      DO 300 IGRP=1,NGRP
      IF(MASKG(IGRP).AND.(NOR(IRES,IGRP).GT.1)) THEN
         CALL LCMLEL(JPLI0,IGRP,LENG0,ITYLCM)
         IF(LENG0.EQ.0) THEN
            KPLI0=LCMDIL(JPLI0,IGRP)
            CALL LCMPUT(KPLI0,'NWT0-PT',NBNRS*MAXNOR*NL,2,
     1      XFLUX(1,1,1,IRES))
         ENDIF
      ENDIF
  300 CONTINUE
      CALL LCMSIX(IPLI0,' ',2)
  310 CONTINUE
*
      CALL KDRCPU(TKA)
*----
*  COMPUTE OUT-OF-FUEL MACROSCOPIC CROSS SECTIONS.
*----
      CALL XDRSET(SIGGAR,NBMIX*(NIRES+1)*NGRP*3,0.0)
      DO 320 IGRP=1,NGRP
      DO 320 ISO=1,NBISO
      IBM=MIX(ISO)
      DENN=DEN(ISO)
      IRES=IAPT(ISO)
      IF(IRES.EQ.NIRES+1) IRES=0
      IF((DENN.NE.0.0).AND.(IBM.NE.0)) THEN
         SIGGAR(IBM,IRES,IGRP,1)=SIGGAR(IBM,IRES,IGRP,1)+SIGT(IGRP,ISO)
     1   *DENN
         SIGGAR(IBM,IRES,IGRP,3)=SIGGAR(IBM,IRES,IGRP,3)+SIGS0(IGRP,ISO)
     1   *DENN
         IF(ITRANC.NE.0) THEN
            SIGGAR(IBM,IRES,IGRP,2)=SIGGAR(IBM,IRES,IGRP,2)+
     1      SIGS1(IGRP,ISO)*DENN
         ENDIF
      ENDIF
  320 CONTINUE
*
      DO 340 IRES=1,NIRES
      DO 330 IGRP=1,NGRP
      MASKH(IGRP)=MASKG(IGRP).AND.(NOR(IRES,IGRP).GT.1)
      IF(MASKH(IGRP)) ICPIJ=ICPIJ+NOR(IRES,IGRP)
  330 CONTINUE
      CALL SETARA(BASE,NBMIX+1,ISIGGA)
      CALL SETARA(BASE,NBMIX+1,ISIGGS)
      CALL SETARA(BASE,MAXNOR*NIRES,IWEIGH)
      CALL SETARA(BASE,MAXNOR*NIRES,ITOTPT)
      CALL SETARA(BASE,MAXNOR*NIRES,ISIGWS)
*----
*  ITERATIVE APPROACH FOR THE HELIOS/WIMS-7 METHOD.
*----
      CALL SETARA(BASE,NBNRS,IFLNEW)
      CALL SETARA(BASE,NBNRS*MAXNOR,IOFLUX)
      CALL SETARA(BASE,MAXNOR*NGRP,INPSYS)
      CALL USSIT1(MAXNOR,NGRP,MASKH,IRES,IPLI0,IPTRK,IFTRAK,CDOOR,
     1 IMPX,NBMIX,NREG,NUN,NL,IPHASE,MAT,VOL,KEYFLX,LEAKSW,IREX,
     2 SIGGAR(1,0,1,1),SIGGAR(1,0,1,2),SIGGAR(1,0,1,3),TITR,NIRES,
     3 NBNRS,NOR,CONR,GOLD,IPPT,STGAR,SSGAR,VOLMER(1),UNGAR,XFLUX,
     4 BASE(ISIGGA),BASE(ISIGGS),BASE(IWEIGH),BASE(ITOTPT),BASE(ISIGWS),
     5 BASE(IFLNEW),BASE(IOFLUX),BASE(INPSYS))
      CALL RLSARA(BASE(INPSYS))
      CALL RLSARA(BASE(IOFLUX))
      CALL RLSARA(BASE(IFLNEW))
*----
*  RESPONSE MATRIX APPROACH FOR THE RIBON EXTENDED METHOD.
*----
      CALL SETARA(BASE,2*(NBNRS*MAXNOR)*(NBNRS*MAXNOR+1),IMATRI)
      CALL SETARA(BASE,NBNRS+1,IAWPHI)
      CALL SETARA(BASE,MAXNOR*MAXNOR*NIRES,IWSLD)
      CALL SETARA(BASE,(NBNRS+1)**2,IPAV)
      CALL SETARA(BASE,NBNRS*NIRES,ISIGX)
      CALL USSIT0(MAXNOR,NGRP,MASKH,IRES,IPLI0,IPTRK,IFTRAK,CDOOR,
     1 IMPX,NBMIX,NREG,NUN,NL,IPHASE,MAT,VOL,KEYFLX,LEAKSW,IREX,
     2 SIGGAR(1,0,1,1),SIGGAR(1,0,1,2),SIGGAR(1,0,1,3),TITR,ICORR,NIRES,
     3 NBNRS,NOR,CONR,GOLD,IPPT,STGAR,SSGAR,SWGAR,VOLMER,UNGAR,XFLUX,
     4 BASE(ISIGGA),BASE(ISIGGS),BASE(IMATRI),BASE(IAWPHI),BASE(IWEIGH),
     5 BASE(ITOTPT),BASE(IWSLD),BASE(ISIGWS),BASE(IPAV),BASE(ISIGX))
      CALL RLSARA(BASE(ISIGX))
      CALL RLSARA(BASE(IPAV))
      CALL RLSARA(BASE(IWSLD))
      CALL RLSARA(BASE(IAWPHI))
      CALL RLSARA(BASE(IMATRI))
*
      CALL RLSARA(BASE(ISIGWS))
      CALL RLSARA(BASE(ITOTPT))
      CALL RLSARA(BASE(IWEIGH))
      CALL RLSARA(BASE(ISIGGS))
      CALL RLSARA(BASE(ISIGGA))
  340 CONTINUE
      CALL KDRCPU(TKB)
      TK4=TK4+(TKB-TKA)
*----
*  COMPUTE THE SELF-SHIELDED REACTION RATES.
*----
      DO 360 IGRP=1,NGRP
      IF(MASKG(IGRP)) THEN
         MAXXS=2+NL+NED+NDEL
         DO 350 IL=1,NL
  350    MAXXS=MAXXS+MAX(ISMAX(IL,IGRP)-ISMIN(IL,IGRP)+1,0)
         IF(ISUBG.EQ.4) MAXXS=MAXXS+1
         CALL KDRCPU(TKA)
         CALL SETARA(BASE,MAXXS,ICGAR)
         CALL SETARA(BASE,MAXNOR*NIRES,IWEIGH)
         CALL SETARA(BASE,MAXNOR*NIRES,ITOTPT)
         CALL SETARA(BASE,MAXNOR*NIRES,ISIGFP)
         CALL SETARA(BASE,MAXNOR*NIRES*NL,ISIGSP)
         CALL SETARA(BASE,MAXNOR*NIRES*NL*NGRP,ISIG0P)
         CALL SETARA(BASE,MAXNOR*NIRES*NED,ISIGAP)
         CALL SETARA(BASE,MAXNOR*NIRES*NDEL,ISIGDP)
         CALL SETARA(BASE,MAXNOR*NIRES,ISIGWP)
         CALL SETARA(BASE,NBNRS*MAXNOR*NL,IXFLUX)
         CALL USSIT2(MAXNOR,IPLI0,IGRP,NGRP,ISMIN(1,IGRP),ISMAX(1,IGRP),
     1   NIRES,NBNRS,NL,NED,NDEL,NOR(1,IGRP),IPPT,GOLD(1,IGRP),MAXXS,
     2   ISUBG,PHGAR(1,1,IGRP),STGAR(1,1,IGRP),SFGAR(1,1,IGRP),
     3   SSGAR(1,1,1,IGRP),S0GAR(1,1,1,1,IGRP),SAGAR(1,1,1,IGRP),
     4   SDGAR(1,1,1,IGRP),SWGAR(1,1,IGRP),BASE(ICGAR),BASE(IWEIGH),
     5   BASE(ITOTPT),BASE(ISIGFP),BASE(ISIGSP),BASE(ISIG0P),
     6   BASE(ISIGAP),BASE(ISIGDP),BASE(ISIGWP),ISM,BASE(IXFLUX))
         CALL RLSARA(BASE(IXFLUX))
         CALL RLSARA(BASE(ISIGWP))
         CALL RLSARA(BASE(ISIGDP))
         CALL RLSARA(BASE(ISIGAP))
         CALL RLSARA(BASE(ISIG0P))
         CALL RLSARA(BASE(ISIGSP))
         CALL RLSARA(BASE(ISIGFP))
         CALL RLSARA(BASE(ITOTPT))
         CALL RLSARA(BASE(IWEIGH))
         CALL RLSARA(BASE(ICGAR))
         CALL KDRCPU(TKB)
         TK5=TK5+(TKB-TKA)
      ENDIF
  360 CONTINUE
*     ***************************************************************
      CALL LCMSIX(IPLI0,' ',2)
      CALL LCMSIX(IPLI0,' ',2)
      CALL LCMVAL(IPLI0,' ')
      CALL KDRCPU(TK2)
      IF(IMPX.GT.1) WRITE(6,'(/34H USSFLU: CPU TIME SPENT TO COMPUTE,
     1 31H SELF-SHIELDED REACTION RATES =,F8.1,19H SECOND, INCLUDING:
     2 /9X,F8.1,46H SECOND TO BUILD/SOLVE SUBGROUP MATRIX SYSTEM;/9X,
     4 F8.1,38H SECOND TO COMPUTE THE REACTION RATES./9X,9HNUMBER OF,
     5 23H ASSEMBLY DOORS CALLS =,I5,1H.)') TK2-TK1,TK4,TK5,ICPIJ
      RETURN
      END
