*DECK USSIT1
      SUBROUTINE USSIT1(MAXNOR,NGRP,MASKG,IRES,IPLI0,IPTRK,IFTRAK,
     1 CDOOR,IMPX,NBMIX,NREG,NUN,NL,IPHASE,MAT,VOL,KEYFLX,LEAKSW,
     2 IREX,SIGGA1,SIGGA2,SIGGA3,TITR,NIRES,NBNRS,NOR,CONR,GOLD,IPPT,
     3 STGAR,SSGAR,VOLMER,UNGAR,XFLUX,SIGGAR,SIGGAS,WEIGH,TOTPT,SIGWS,
     4 FLNEW,OFLUX,NPSYS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute the multiband fluxes as required by the subgroup method using
* an iterative approach:
* a) assume a single resonant isotope;
* b) use the standard solution doors of Dragon.
*
*Copyright:
* Copyright (C) 2003 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* MAXNOR  maximum order of the probability tables (PT).
* NGRP    number of energy group.
* MASKG   energy group mask pointing on self-shielded groups.
* IRES    index of the resonant isotope.
* IPLI0   pointer to the internal microscopic cross section library
*         builded by the self-shielding module.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  file unit number used to store the tracks.
* CDOOR   name of the geometry/solution operator.
* IMPX    print flag (equal to zero for no print).
* NBMIX   number of mixtures in the internal library.
* NREG    number of regions.
* NUN     number of unknowns in the flux or source vector in one
*         energy group and one band.
* NL      number of Legendre orders required in the calculation
*         (NL=1 or higher).
* IPHASE  type of flux solution (=1: use a native flux solution door;
*         =2: use collision probabilities).
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KEYFLX  pointers of fluxes in unknown vector.
* LEAKSW  leakage switch (LEAKSW=.true. if neutron leakage through
*         external boundary is present).
* IREX    fuel region index assigned to each mixture. Equal to zero
*         in non-resonant mixtures or in mixtures not used.
* SIGGA1  macroscopic total xs of the non-resonant isotopes in each
*         mixture.
* SIGGA2  macroscopic transport correction of the non-resonant
*         isotopes in each mixture.
* SIGGA3  macroscopic P0 scattering xs of the non-resonant
*         isotopes in each mixture.
* TITR    title.
* NIRES   exact number of correlated resonant isotopes.
* NBNRS   number of correlated fuel regions.
* NOR     exact order of the probability table.
* CONR    number density of the resonant isotopes.
* GOLD    Goldstein-Cohen parameter.
* IPPT    information related to each resonant isotope:
*         IPPT(:,1): pointer to LCM directory;
*         IPPT(:,2): index of a resonant region (used with infinite
*         dilution case);
*         IPPT(:,3:5): alias name of resonant isotope.
* STGAR   averaged microscopic total xs in resonant region.
* SSGAR   averaged microscopic scattering xs in resonant region.
* VOLMER  volumes of the resonant regions.
*
*Parameters: output
* UNGAR   averaged fluxes per volume.
*
*Parameters: scratch
* XFLUX
* SIGGAR
* SIGGAS
* WEIGH
* TOTPT
* SIGWS
* FLNEW
* OFLUX
* NPSYS
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER MAXNOR,NGRP,IRES,IPLI0,IPTRK,IFTRAK,IMPX,NBMIX,NREG,
     1 NUN,NL,IPHASE,MAT(NREG),KEYFLX(NREG),IREX(NBMIX),NIRES,NBNRS,
     2 NOR(NIRES,NGRP),IPPT(NIRES,5),NPSYS(MAXNOR*NGRP)
      REAL VOL(NREG),SIGGA1(NBMIX,0:NIRES,NGRP),
     1 SIGGA2(NBMIX,0:NIRES,NGRP),SIGGA3(NBMIX,0:NIRES,NGRP),
     2 CONR(NBNRS,NIRES),GOLD(NIRES,NGRP),STGAR(NBNRS,NIRES,NGRP),
     3 SSGAR(NBNRS,NIRES,NL,NGRP),VOLMER(NBNRS),UNGAR(NREG,NIRES,NGRP),
     4 XFLUX(NBNRS,MAXNOR,NL,NIRES),SIGGAR(0:NBMIX),SIGGAS(0:NBMIX),
     5 WEIGH(MAXNOR,NIRES),TOTPT(MAXNOR,NIRES),SIGWS(MAXNOR,NIRES),
     6 FLNEW(NBNRS),OFLUX(NBNRS,MAXNOR)
      CHARACTER CDOOR*12,TITR*72
      LOGICAL LEAKSW,MASKG(NGRP)
*----
*  LOCAL VARIABLES
*----
      CHARACTER CBDPNM*12,TEXT12*12,TEXX12*12,HSMG*131
      LOGICAL EMPTY,LCM,LEXAC
      COMMON BASE(1)
*----
*  FIND THE NUMBER OF COMPONENTS REQUIRED AND ALLOCATE THE LIST OF
*  ASSEMBLY MATRICES.
*----
      NASM=0
      DO 10 IGRP=1,NGRP
      IF(MASKG(IGRP).AND.(GOLD(IRES,IGRP).NE.-999.)) THEN
         NASM=NASM+NOR(IRES,IGRP)
      ENDIF
   10 CONTINUE
      IF(NASM.EQ.0) RETURN
*----
*  CREATE A SPECIFIC DIRECTORY FOR IRES-TH RESONANT ISOTOPE.
*----
      WRITE(CBDPNM,'(3HCOR,I4.4,1H/,I4.4)') IRES,NIRES
      CALL LCMSIX(IPLI0,CBDPNM,1)
      JPLI0=LCMGID(IPLI0,'GROUP_INFO')
      IPSYS=LCMLID(IPLI0,'ASSEMB_PHYS',NASM)
      CALL LCMSIX(IPLI0,' ',2)
*----
*  LOOP OVER THE ENERGY GROUPS.
*----
      IASM=0
      DO 100 IGRP=1,NGRP
      IF(MASKG(IGRP).AND.(GOLD(IRES,IGRP).NE.-999.)) THEN
         IF(IMPX.GT.1) THEN
            WRITE(6,'(46H USSIT1: PROCESS CORRELATED ISOTOPE WITH INDEX,
     1      I3,9H IN GROUP,I4,20H (ITERATIVE METHOD).)') IRES,IGRP
         ENDIF
         DO 20 JRES=1,NIRES
         IF(GOLD(JRES,IGRP).EQ.-999.) THEN
            WRITE(HSMG,'(30HUSSIT1: PTSL SET FOR ISOTOPE '',3A4,
     1      10H'' IN GROUP,I4,1H.)') (IPPT(JRES,J0),J0=3,5),IGRP
            CALL XABORT(HSMG)
         ELSE IF(NOR(JRES,IGRP).GT.MAXNOR) THEN
            CALL XABORT('USSIT1: MAXNOR OVERFLOW.')
         ENDIF
   20    CONTINUE
         NORI=NOR(IRES,IGRP)
*----
*  COLLECT THE BASE POINTS IN TOTAL AND PARTIAL CROSS SECTION.
*----
         DO 40 JRES=1,NIRES
         CALL LCMLEL(IPPT(JRES,1),IGRP,ILONG,ITYLCM)
         IF(ILONG.NE.0) THEN
            KPLIB=LCMGIL(IPPT(JRES,1),IGRP)
            CALL LCMINF(KPLIB,TEXT12,TEXX12,EMPTY,ILONG,LCM)
            CALL LCMLEN(KPLIB,'PROB-TABLE',LENG,ITYLCM)
            NPART=LENG/MAXNOR
            IF(LCM) THEN
               CALL LCMIOF(KPLIB,'PROB-TABLE',ISIGP)
            ELSE
               CALL SETARA(BASE,MAXNOR*NPART,ISIGP)
               CALL LCMGET(KPLIB,'PROB-TABLE',BASE(ISIGP))
            ENDIF
            DO 30 INOR=1,NOR(JRES,IGRP)
            WEIGH(INOR,JRES)=BASE(ISIGP+INOR-1)
            TOTPT(INOR,JRES)=BASE(ISIGP+MAXNOR+INOR-1)
   30       SIGWS(INOR,JRES)=BASE(ISIGP+3*MAXNOR+INOR-1)
            IF(.NOT.LCM) CALL RLSARA(BASE(ISIGP))
         ELSE
            WEIGH(1,JRES)=1.0
            TOTPT(1,JRES)=STGAR(IPPT(JRES,2),JRES,IGRP)
            SIGWS(1,JRES)=SSGAR(IPPT(JRES,2),JRES,1,IGRP)
         ENDIF
   40    CONTINUE
*----
*  RECOVER THE PREVIOUS FLUXES.
*----
         WRITE(CBDPNM,'(3HCOR,I4.4,1H/,I4.4)') IRES,NIRES
         CALL LCMSIX(IPLI0,CBDPNM,1)
         JPLI1=LCMGID(IPLI0,'GROUP_INFO')
         KPLI1=LCMGIL(JPLI1,IGRP)
         CALL LCMGET(KPLI1,'NWT0-PT',XFLUX(1,1,1,IRES))
         CALL LCMSIX(IPLI0,' ',2)
*----
*  SET THE MIXTURE-DEPENDENT CROSS SECTIONS.
*----
         DO 90 INOR=1,NORI
         CALL XDRSET(SIGGAR(0),NBMIX+1,0.0)
         CALL XDRSET(SIGGAS(0),NBMIX+1,0.0)
         DO 80 IBM=1,NBMIX
         DO 70 JRES=0,NIRES
         IF(JRES.NE.IRES) THEN
            SIGGAR(IBM)=SIGGAR(IBM)+(SIGGA1(IBM,JRES,IGRP)-
     1      SIGGA2(IBM,JRES,IGRP))
            SIGGAS(IBM)=SIGGAS(IBM)-SIGGA2(IBM,JRES,IGRP)
         ENDIF
   70    CONTINUE
         IND=IREX(IBM)
         IF(IND.GT.0) THEN
            SIGGAR(IBM)=SIGGAR(IBM)+CONR(IND,IRES)*TOTPT(INOR,IRES)
            SIGGAS(IBM)=SIGGAS(IBM)+(1.0-GOLD(IRES,IGRP))*CONR(IND,IRES)
     1      *SIGWS(INOR,IRES)
         ENDIF
   80    CONTINUE
         IASM=IASM+1
         NPSYS(IASM)=IASM
         KPSYS=LCMDIL(IPSYS,IASM)
         CALL LCMPUT(KPSYS,'DRAGON-TXSC',NBMIX+1,2,SIGGAR)
         CALL LCMPUT(KPSYS,'DRAGON-S0XSC',NBMIX+1,2,SIGGAS)
   90    CONTINUE
      ELSE IF(GOLD(IRES,IGRP).NE.-999.) THEN
         CALL LCMLEL(JPLI0,IGRP,LENG0,ITYLCM)
         IF(LENG0.NE.0) THEN
            WRITE(HSMG,'(42HUSSIT1: UNEXPECTED SELF-SHIELDING DATA FOU,
     1      11HND IN GROUP,I5,1H.)') IGRP
            CALL XABORT(HSMG)
         ENDIF
      ENDIF
  100 CONTINUE
*----
*  ASSEMBLY MATRIX OR REDUCED COLLISION PROBABILITIES CALCULATION.
*----
      NANI=1
      KNORM=1
      IMPY=MAX(0,IMPX-3)
      IF(IPHASE.EQ.1) THEN
*        USE A NATIVE DOOR.
         CALL DOORAV(CDOOR,IPSYS,NPSYS,IPTRK,IFTRAK,IMPY,NASM,NREG,
     1   NBMIX,NANI,MAT,VOL,KNORM,LEAKSW,TITR)
      ELSE IF(IPHASE.EQ.2) THEN
*        USE A COLLISION PROBABILITY DOOR.
         IPIJK=1
         CALL DOORPV(CDOOR,IPSYS,NPSYS,IPTRK,IFTRAK,IMPY,NASM,NREG,
     1   NBMIX,NANI,MAT,VOL,KNORM,IPIJK,LEAKSW,.FALSE.,TITR)
      ENDIF
*----
*  LOOP OVER THE ENERGY GROUPS.
*----
      IASM=0
      DO 260 IGRP=1,NGRP
      IF(MASKG(IGRP).AND.(GOLD(IRES,IGRP).NE.-999.)) THEN
         NORI=NOR(IRES,IGRP)
*----
*  COLLECT THE BASE POINTS IN PARTIAL CROSS SECTION.
*----
         DO 120 JRES=1,NIRES
         CALL LCMLEL(IPPT(JRES,1),IGRP,ILONG,ITYLCM)
         IF(ILONG.NE.0) THEN
            KPLIB=LCMGIL(IPPT(JRES,1),IGRP)
            CALL LCMINF(KPLIB,TEXT12,TEXX12,EMPTY,ILONG,LCM)
            CALL LCMLEN(KPLIB,'PROB-TABLE',LENG,ITYLCM)
            NPART=LENG/MAXNOR
            IF(LCM) THEN
               CALL LCMIOF(KPLIB,'PROB-TABLE',ISIGP)
            ELSE
               CALL SETARA(BASE,MAXNOR*NPART,ISIGP)
               CALL LCMGET(KPLIB,'PROB-TABLE',BASE(ISIGP))
            ENDIF
            DO 110 INOR=1,NOR(JRES,IGRP)
            WEIGH(INOR,JRES)=BASE(ISIGP+INOR-1)
  110       SIGWS(INOR,JRES)=BASE(ISIGP+3*MAXNOR+INOR-1)
            IF(.NOT.LCM) CALL RLSARA(BASE(ISIGP))
         ELSE
            WEIGH(1,JRES)=1.0
            SIGWS(1,JRES)=SSGAR(IPPT(JRES,2),JRES,1,IGRP)
         ENDIF
  120    CONTINUE
*----
*  RECOVER THE PREVIOUS FLUXES.
*----
         WRITE(CBDPNM,'(3HCOR,I4.4,1H/,I4.4)') IRES,NIRES
         CALL LCMSIX(IPLI0,CBDPNM,1)
         JPLI1=LCMGID(IPLI0,'GROUP_INFO')
         KPLI1=LCMGIL(JPLI1,IGRP)
         CALL LCMGET(KPLI1,'NWT0-PT',XFLUX(1,1,1,IRES))
         CALL LCMSIX(IPLI0,' ',2)
*----
*  ITERATIVE PROCEDURE.
*----
        ITER=0
  140   ITER=ITER+1
        IF(ITER.GT.20) CALL XABORT('USSIT1: TOO MANY ITERATIONS.')
        ERR1=0.0
        ERR2=0.0
        CALL XDRSET(UNGAR(1,IRES,IGRP),NREG,0.0)
*----
*  COMPUTE THE AVERAGED SOURCE.
*----
        CALL SETARA(BASE,NUN*NORI,IFUN)
        CALL SETARA(BASE,NUN*NORI,ISUN)
        CALL XDRSET(BASE(ISUN),NUN*NORI,0.0)
        DO 150 INOR=1,NORI
        KPSYS=LCMGIL(IPSYS,IASM+INOR)
        CALL LCMLEN(KPSYS,'FUNKNO$USS',ILENG,ITYLCM)
        IF(ILENG.EQ.NUN) THEN
           CALL LCMGET(KPSYS,'FUNKNO$USS',BASE(IFUN+(INOR-1)*NUN))
        ELSE
           CALL XDRSET(BASE(IFUN+(INOR-1)*NUN),NUN,0.0)
        ENDIF
        DO 150 IND=1,NBNRS
  150   OFLUX(IND,INOR)=XFLUX(IND,INOR,1,IRES)
        DO 190 INOR=1,NORI
        NPSYS(INOR)=IASM+INOR
        DO 190 I=1,NREG
        IBM=MAT(I)
        IF(IBM.EQ.0) GO TO 190
        IOF=ISUN+(INOR-1)*NUN+KEYFLX(I)-1
        DO 155 JRES=0,NIRES
        IF(JRES.NE.IRES) THEN
           BASE(IOF)=BASE(IOF)+SIGGA3(IBM,JRES,IGRP)
        ENDIF
  155   CONTINUE
        IND=IREX(IBM)
        IF(IND.GT.0) THEN
           DO 160 JNOR=1,NORI
  160      BASE(IOF)=BASE(IOF)+GOLD(IRES,IGRP)*WEIGH(JNOR,IRES)*
     1     CONR(IND,IRES)*SIGWS(JNOR,IRES)*OFLUX(IND,JNOR)
        ENDIF
  190   CONTINUE
*----
*  SOLVE FOR THE MULTIBAND FLUX.
*----
        IDIR=0
        NLF=1
        LEXAC=.FALSE.
        CALL DOORFV (CDOOR,IPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NORI,NBMIX,
     1  NLF,IDIR,NREG,NUN,IPHASE,LEXAC,MAT,VOL,KEYFLX,TITR,BASE(ISUN),
     2  BASE(IFUN))
*----
*  HOMOGENIZE THE FLUX AT ITERATION ITER.
*----
        DO 230 INOR=1,NORI
        KPSYS=LCMGIL(IPSYS,IASM+INOR)
        CALL LCMPUT(KPSYS,'FUNKNO$USS',NUN,2,BASE(IFUN+(INOR-1)*NUN))
        CALL XDRSET(FLNEW,NBNRS,0.0)
        DO 200 I=1,NREG
        IF(MAT(I).EQ.0) GO TO 200
        IOF=IFUN+(INOR-1)*NUN+KEYFLX(I)-1
        IND=IREX(MAT(I))
        IF(IND.GT.0) FLNEW(IND)=FLNEW(IND)+BASE(IOF)*VOL(I)
  200   CONTINUE
        DO 210 IND=1,NBNRS
  210   FLNEW(IND)=FLNEW(IND)/VOLMER(IND)
*
        DO 220 I=1,NREG
        IOF=IFUN+(INOR-1)*NUN+KEYFLX(I)-1
  220   UNGAR(I,IRES,IGRP)=UNGAR(I,IRES,IGRP)+BASE(IOF)*WEIGH(INOR,IRES)
*----
*  CONVERGENCE CONTROL.
*----
        DO 230 IND=1,NBNRS
        ERR1=MAX(ERR1,ABS(FLNEW(IND)-XFLUX(IND,INOR,1,IRES)))
        ERR2=MAX(ERR2,ABS(FLNEW(IND)))
  230   XFLUX(IND,INOR,1,IRES)=FLNEW(IND)
        CALL RLSARA(BASE(ISUN))
        CALL RLSARA(BASE(IFUN))
*----
*  CONVERGENCE CONTROL.
*----
        IF((ERR1.GT.1.0E-4*ERR2).AND.(GOLD(IRES,IGRP).NE.0.0)) GO TO 140
        IF(IMPX.GT.0) THEN
           WRITE(TEXT12,'(3A4)') (IPPT(IRES,I),I=3,5)
           WRITE(6,'(15H USSIT1: GROUP=,I5,24H. SUBGROUP ITERATION CON,
     1     11HVERGENCE IN,I4,22H ITERATIONS. ISOTOPE='',A12,2H''.)')
     2     IGRP,ITER,TEXT12
        ENDIF
        IF(IMPX.GT.2) THEN
           DO 250 IND=1,NBNRS
           T1=0.0
           DO 240 INOR=1,NORI
  240      T1=T1+WEIGH(INOR,IRES)*XFLUX(IND,INOR,1,IRES)
           WRITE(6,'(31H USSIT1: AVERAGED FLUX IN GROUP,I4,9H AND RESO,
     1     11HNANT REGION,I4,21H FOR RESONANT ISOTOPE,I4,2H =,F9.5)')
     2     IGRP,IND,IRES,T1
  250      CONTINUE
        ENDIF
        KPLI0=LCMGIL(JPLI0,IGRP)
        CALL LCMPUT(KPLI0,'NWT0-PT',NBNRS*MAXNOR*NL,2,XFLUX(1,1,1,IRES))
        IASM=IASM+NORI
      ENDIF
  260 CONTINUE
*----
*  USE IL=1 VALUES FOR XFLUX AT HIGHER LEGENDRE ORDERS.
*----
      DO 280 IL=2,NL
      DO 280 IGRP=1,NGRP
      IF(MASKG(IGRP).AND.(GOLD(IRES,IGRP).NE.-999.)) THEN
        NORI=NOR(IRES,IGRP)
        KPLI0=LCMGIL(JPLI0,IGRP)
        CALL LCMGET(KPLI0,'NWT0-PT',XFLUX(1,1,1,IRES))
        DO 270 INOR=1,NORI
        DO 270 IND=1,NBNRS
  270   XFLUX(IND,INOR,IL,IRES)=XFLUX(IND,INOR,1,IRES)
        CALL LCMPUT(KPLI0,'NWT0-PT',NBNRS*MAXNOR*NL,2,XFLUX(1,1,1,IRES))
      ENDIF
  280 CONTINUE
      RETURN
      END
