*DECK XDRLPR
      SUBROUTINE XDRLPR(NDPROC,NPROC,NAMDXS,MINLEG,MAXLEG,IORD,NGROUP,
     >                  ITYPRO,XSREC,SCAT)
*
*-----------------------------------------------------------------------
*
*Purpose:
* print cross sections in a microlib
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): G. Marleau
*
*Parameters: input
* NDPROC  number of vector terms to process
* NPROC   number of terms to process
* NAMDXS  names of cross sections to process
* MINLEG  mimimum Legendre order to process for scattering
* MAXLEG  mimimum Legendre order to process for scattering
* IORD    cross section order:
*         =1 constant;
*         =2 linear;
*         =3 quadratic
* NGROUP  number of groups
* ITYPRO  vector for cross section processed indices:
*           = 0 -> absent  (not processed);
*           > 0 -> present (processed)
* XSREC   cross section records (scattering cross section of
*         order MINLEG to MAXLEG for IRPROC=MINLEG+1,MAXLEG+1)
* SCAT    complete scattering matrix (SCAT(JG,IG) is from IG to JG
*         for order MINLEG to MAXLEG)
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER          NDPROC,NPROC,MINLEG,MAXLEG,IORD,NGROUP,
     >                 ITYPRO(NPROC)
      REAL             XSREC(NGROUP,NPROC),
     >                 SCAT(NGROUP,NGROUP,MAXLEG-MINLEG+1)
      CHARACTER        NAMDXS(NDPROC)*6
*----
*  LOCAL VARIABLES
*----
      CHARACTER        NAMLEG*2,NORD*6
*----
*  LOCAL PARAMETERS
*  SCATTERING CROSS SECTIONS START AT NDPROC+1 WITH
*  NAME NAMSCT='SIGS'//NAMLEG AND NAMSCT='SCAT'//NAMLEG
*  WITH NAMLEG DEFINED BY
*  WRITE(NAMLEG ,'(I2.2)') ILEG
*  FOR ILEG=0 TO NDPROC-NPROC-1
*----
      PARAMETER       (IOUT=6)
*----
*  READ/INITIALIZE STATE VECTOR
*----
      IF(IORD.EQ.1) THEN
        NORD='      '
        IODIV=1
      ELSE IF(IORD.EQ.2) THEN
        NORD='   LIN'
        IODIV=2
      ELSE IF(IORD.EQ.3) THEN
        NORD='   QUA'
        IODIV=4
      ENDIF
*----
*  PRINT DEFAULT XS REQUIRED
*----
      DO 100 IXSR=1,NDPROC
        DO 110 IGR=1,NGROUP
          IF(XSREC(IGR,IXSR).NE.0.0) GO TO 115
 110    CONTINUE
        GO TO 100
 115    WRITE(IOUT,6000) NAMDXS(IXSR),NORD
        WRITE(IOUT,6010) (XSREC(IGR,IXSR),IGR=1,NGROUP)
 100  CONTINUE
*----
*  PRINT SCATTERING MATRIX
*----
      IL=0
      DO 130 ILEG=MINLEG+1,MAXLEG+1
        IL=IL+1
        IXSR=NDPROC+ILEG
        IXSTN=MOD(ITYPRO(ILEG)/IODIV,2)
        IF(IXSTN.NE.0) THEN
          WRITE(NAMLEG,'(I2.2)') ILEG-1
          WRITE(IOUT,6000) 'SIGS'//NAMLEG,NORD
          WRITE(IOUT,6010) (XSREC(IGR,IXSR),IGR=1,NGROUP)
*----
*  SCAT(IGTO,IGFROM) REPRESENT SCATTERING CROSS SECTION
*  FROM GROUP "IGFROM" TO GROUP "IGTO"
*----
          WRITE(IOUT,6000) 'SCAT'//NAMLEG,NORD
          DO 120 IGFROM=1,NGROUP
            WRITE(IOUT,6001) IGFROM
            WRITE(IOUT,6010) (SCAT(IGTO,IGFROM,IL),IGTO=1,NGROUP)
 120      CONTINUE
        ENDIF
 130  CONTINUE
*----
*  FORMAT
*----
 6000 FORMAT(/' CROSS SECTION TYPE = ',2A6)
 6001 FORMAT(/' SCATTERING FROM GROUP = ',I10)
 6010 FORMAT(1P,5E16.7)
      END
