#
#-----------------------------------------------------------------------
#
#Purpose:
# automatic generation of Njoy input data, including dragr data.
# Generation of DRAGLIB and ACELIB.
#
#Copyright:
# Copyright (C) 2003 Ecole Polytechnique de Montreal
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version
#
#Author(s): A. Hebert and R. Karthikeyan
#
#-----------------------------------------------------------------------
#
import os,time
class PyNjoy:
  def __init__(self):
    self.iwt = 4
    self.legendre = 1
    self.scatteringLaw = None
    self.branchingNG = None
    self.branchingN2N = None
  def pendf(self):
    print " --- make pendf for " + self.hmat + " ---"
    if not os.path.isdir(self.evaluationName): os.mkdir(self.evaluationName)
    os.chdir(self.evaluationName)
    textDil=""
    if self.dilutions:
      nbDil = len(self.dilutions)
      if nbDil > 10: raise "cannot have more than 10 dilutions"
      for dil in self.dilutions:
        textDil = textDil + " " + "%E"%dil
    else:
      nbDil = 0
    nbTmp = len(self.temperatures)
    if nbTmp > 10: raise "cannot have more than 10 temperatures"
    textTmp=""
    for tmp in self.temperatures:
      textTmp = textTmp + " " + "%E"%tmp
    if self.scatteringLaw:
      if self.scatteringMat == 1:
        matsab = 222
      elif self.scatteringMat == 7:
        matsab = 225
      elif self.scatteringMat == 11:
        matsab = 228
      elif self.scatteringMat == 26:
        matsab = 231
      else:
        matsab = 221
      unitLaw = -27
      matLaw = self.scatteringMat
      typeLaw = 4
      nbAtoms = 2
      os.system("ln -s " + self.scatteringLaw + " tape26")
    else:
      matsab = 221
      unitLaw = 0
      matLaw = 0
      typeLaw = 1
      nbAtoms = 1
    htime = time.ctime(time.time())
    self.__dict__.update({"textDil": textDil, \
                          "nbDil"  : nbDil,   \
                          "textTmp": textTmp, \
                          "nbTmp"  : nbTmp,   \
                          "unitLaw": unitLaw, \
                          "matLaw" : matLaw,  \
                          "typeLaw": typeLaw, \
                          "nbAtoms": nbAtoms, \
                          "htime"  : htime,   \
                          "matsab" : matsab})
    #
    if self.scatteringLaw:
      text_data = """
      moder
      20 -21
      moder
      26 -27
      """
    else:
      text_data = """
      moder
      20 -21
      """
    text_data = text_data + """
    reconr
    -21 -22
    'pendf tape from %(evaluationName)s'/
    %(mat)d 1/
    0.001  0.  0.005/
    '%(hmat)s from %(evaluationName)s at %(htime)s' /
    0/
    broadr
    -21 -22 -23
    %(mat)d %(nbTmp)d/
    0.001/
    %(textTmp)s/
    0/
    """%self.__dict__
    if self.dilutions:
      text_data = text_data + """
    purr
    -21 -23 -24
    %(mat)d %(nbTmp)d %(nbDil)d 20 32/
    %(textTmp)s/
    %(textDil)s/
    0/
    thermr
    0 -24 -35
    0 %(mat)d 16 %(nbTmp)d %(typeLaw)d 0 %(nbAtoms)d 221 0
    %(textTmp)s/
    0.001 4.0
    moder
    -35 29
    stop
      """%self.__dict__
    else:
     if self.scatteringLaw:
      text_data = text_data + """
    thermr
    %(unitLaw)d -23 -35
    %(matLaw)d %(mat)d 16 %(nbTmp)d %(typeLaw)d 0 %(nbAtoms)d %(matsab)d 0/
    %(textTmp)s/
    0.001 4.0
    moder
    -35 29
    stop
      """%self.__dict__
     else:
      text_data = text_data + """
    thermr
    0 -23 -35
    0 %(mat)d 16 %(nbTmp)d %(typeLaw)d 0 %(nbAtoms)d 221 0
    %(textTmp)s/
    0.001 4.0
    moder
    -35 29
    stop
      """%self.__dict__
    file_data = open("file_data",'w')
    file_data.write(text_data)
    file_data.close()
    os.system("ln -s " + self.evaluationFile + " tape20")
    os.system("../" + self.execDir + "/xnjoy<file_data")
    os.system("mv tape29 pendf" + self.hmat)
    os.system("mv file_data file_data_pendf" + self.hmat)
    os.system("mv output out_pendf_" + self.hmat)
    os.system("chmod 644 out_pendf_" + self.hmat)
    os.system("rm tape*")
    os.chdir("..")
  #
  def gendf(self):
    print " --- make gendf for " + self.hmat + " ---"
    os.chdir(self.evaluationName)
    matsab = 221
    if self.scatteringLaw:
      if self.scatteringMat == 1:
        matsab = 222
      elif self.scatteringMat == 7:
        matsab = 225
      elif self.scatteringMat == 11:
        matsab = 228
      elif self.scatteringMat == 26:
        matsab = 231
    if self.iwt:
      newiwt = self.iwt
    else:
      newiwt = 4
    if self.dilutions:
      newiwt = -abs(newiwt)
      nbDil = len(self.dilutions)
      if nbDil > 10: raise "cannot have more than 10 dilutions"
      textDil=""
      for dil in self.dilutions:
        textDil = textDil + " " + "%E"%dil
    else:
      newiwt = abs(newiwt)
      nbDil = 1
      textDil="1.0e10"
    nbTmp = len(self.temperatures)
    if nbTmp > 10: raise "cannot have more than 10 temperatures"
    textTmp=""
    for tmp in self.temperatures:
      textTmp = textTmp + " " + "%E"%tmp
    htime = time.ctime(time.time())
    self.__dict__.update({"textDil": textDil, \
                          "nbDil"  : nbDil,   \
                          "textTmp": textTmp, \
                          "nbTmp"  : nbTmp,   \
                          "htime"  : htime,   \
                          "newiwt" : newiwt,  \
                          "matsab" : matsab})
    #
    text_data = """
    moder
    20 -21
    moder
    29 -25
    groupr
    -21 -25 0 -26
    %(mat)d %(nstr)d 0 %(newiwt)d %(legendre)d %(nbTmp)d %(nbDil)d 1
    '%(hmat)s from %(evaluationName)s at %(htime)s' /
    %(textTmp)s/
    %(textDil)s/
    """%self.__dict__
    if self.dilutions:
      text_data = text_data + """
      677.2873 %(potential)f 20000 / Homog. Flux Calc.Param
      """%self.__dict__
    if newiwt == 1 or newiwt == -1:
      text_data = text_data + self.wght
    elif newiwt == 4 or newiwt == -4:
      text_data = text_data + """
      0.2 0.0253 820.3e3 1.40e6 / iwt=4 parameters
      """
    for tmp in self.temperatures:
      text_data = text_data + """
      3/
      3 %(matsab)d /
      """%self.__dict__
      if self.fission:
        text_data = text_data + """
        3 452 /
        """
      if self.fission == 2:
        text_data = text_data + """
        3 455 /
        5 455 /
        """
      text_data = text_data + """
      6 /
      6 %(matsab)d /
      0/
      """%self.__dict__
    text_data = text_data + """
    0/
    moder
    -26 30
    stop
    """
    file_data = open("file_data",'w')
    file_data.write(text_data)
    file_data.close()
    os.system("ln -s " + self.evaluationFile + " tape20")
    os.system("ln -s pendf" + self.hmat + " tape29")
    os.system("../" + self.execDir + "/xnjoy<file_data")
    os.system("mv file_data file_data_gendf" + self.hmat)
    os.system("mv tape30 gendf" + self.hmat)
    os.system("mv output out_gendf_" + self.hmat)
    os.system("chmod 644 out_gendf_" + self.hmat)
    os.system("rm tape*")
    os.chdir("..")
  #
  def draglib(self, fp=0):
    os.chdir(self.evaluationName)
    if os.path.isfile("drag"): os.remove("drag")
    if os.path.isfile("draglib" + self.evaluationName):
      iold = 29
      if not os.path.isfile("draglib" + self.evaluationName + ".bis.gz"):
        os.system("cp draglib" + self.evaluationName + \
        " draglib" + self.evaluationName + ".bis")
        os.system("gzip draglib" + self.evaluationName + ".bis")
      os.system("mv draglib" + self.evaluationName + " tape29")
      print " append data for " + self.hmat + " to existing draglib file"
    else:
      iold = 0
      print " create a new draglib file for " + self.hmat
    htime = time.ctime(time.time())
    #
    if self.dilutions:
      self.__dict__.update({"htime": htime,      \
                            "iold" : iold,       \
                            "ss0"  : self.ss[0], \
                            "ss1"  : self.ss[1], \
                            "auto0": self.autolib[0], \
                            "auto1": self.autolib[1], \
                            "fp"   : fp})
      text_data = """
      moder
      20 -21
      moder
      22 -23
      moder
      24 -25
      dragr
      -21 -23 -25 0 0 %(iold)d 30 %(fp)d/
      """%self.__dict__
      if iold == 0:
        text_data = text_data + """
        'draglib from %(evaluationName)s at %(htime)s'/
        """%self.__dict__
      text_data = text_data + """
      %(mat)d %(hmat)s /
      '%(hmat)s from %(evaluationName)s (%(mat)d) at %(htime)s' /
      %(ss0)E %(ss1)E /
      %(auto0)E %(auto1)E 0.00125 /
      0/
      stop
      """%self.__dict__
    else:
      self.__dict__.update({"htime": htime, "iold": iold, "fp": fp})
      text_data = """
      moder
      20 -21
      moder
      24 -25
      dragr
      -21 0 -25 0 0 %(iold)d 30 %(fp)d/
      """%self.__dict__
      if iold == 0:
        text_data = text_data + """
        'draglib from %(evaluationName)s at %(htime)s'/
        """%self.__dict__
      text_data = text_data + """
      %(mat)d %(hmat)s /
      '%(hmat)s from %(evaluationName)s (%(mat)d) at %(htime)s' /
      0.1 1.0E10 /
      0/
      stop
      """%self.__dict__
    file_data = open("file_data",'w')
    file_data.write(text_data)
    file_data.close()
    os.system("ln -s " + self.evaluationFile + " tape20")
    if self.dilutions:
      os.system("ln -s pendf" + self.hmat + " tape22")
    os.system("ln -s gendf" + self.hmat + " tape24")
    os.system("../" + self.execDir + "/xnjoy<file_data")
    os.system("mv file_data file_data_dendf" + self.hmat)
    stats = os.stat("tape30")                                 # get the stats of the file
    size = stats[6]                                           # extract the file size in bytes from the stats list
    if size > 8 :
      os.system("mv tape30 draglib"+self.evaluationName)
    else:
      os.system("mv tape29 draglib"+self.evaluationName)
      outp2=open("error.log",'a')
      outp2.write( " draglib file for" + self.hmat + " not created")
    file_in = open('output','r')
    file_out = open("out_draglib_" + self.hmat,'w')
    while 1:
      line=file_in.readline()
      if not line: break
      ind=line.find('???????????')
      if ind != -1:
        if not self.eFiss: raise "self.eFiss instance variable not set"
        line = line[:ind] + "%E"%self.eFiss + line[ind+11:]
      if self.branchingNG:
        ind=line.find(' ng ')
        if ind != -1:
          jnd=line[ind+3:].find(' 0.000 ')
          if jnd == -1: raise "unable to set the isomeric ng branching ratio"
          line = line[:ind+jnd+4] + "%5.3f"%self.branchingNG + line[ind+jnd+9:]
      if self.branchingN2N:
        ind=line.find(' n2n ')
        if ind != -1:
          jnd=line[ind+4:].find(' 0.000 ')
          if jnd == -1: raise "unable to set the isomeric n2n branching ratio"
          line = line[:ind+jnd+5] + "%5.3f"%self.branchingN2N + line[ind+jnd+10:]
      file_out.writelines(line)
    file_out.close()
    file_in.close()
    os.remove('output')
    os.system("chmod 644 out_draglib_" + self.hmat)
    os.system("rm tape*")
    os.chdir("..")
  #
  def matxs(self):
    print " --- make matxs for " + self.hmat + " ---"
    os.chdir(self.evaluationName)
    listGro = [ 0, 239, 30, 27, 50, 68, 100, 35, 69, 187, 70, 620, 80, 100,
    640, 174, 175, 172, 33, 1968, 315, 172, 175, 89 ]
    htime = time.ctime(time.time())
    self.__dict__.update({"nbGro": listGro[self.nstr - 1], \
                          "htime": htime})
    text_data = """
    moder
    24 -25
    matxsr
    -25 0 27/
    1 '%(hmat)s from %(evaluationName)s (%(mat)d) at %(htime)s'/
    1 2 1 1
    'neutron library'/
    'n'
    %(nbGro)d
    'nscat' 'ntherm'/
    1 1
    1 1
    %(hmat)s %(mat)d /
    stop
    """%self.__dict__
    file_data = open("file_data",'w')
    file_data.write(text_data)
    file_data.close()
    os.system("ln -s gendf" + self.hmat + " tape24")
    os.system("../" + self.execDir + "/xnjoy<file_data")
    os.system("mv file_data file_data_matxs" + self.hmat)
    os.system("mv tape27 matxs" + self.hmat)
    os.system("mv output out_matxs_" + self.hmat)
    os.system("chmod 644 out_matxs_" + self.hmat)
    os.system("rm tape*")
    os.chdir("..")
#
  def makeFp(self):
    self.scatteringLaw = None
    self.fission = None
    self.dilutions = None
    self.legendre = 0
    self.pendf()
    self.gendf()
    self.draglib(fp=1)
#
  def burnup(self):
    os.chdir(self.evaluationName)
    if os.path.isfile("drag"): os.remove("drag")
    if os.path.isfile("draglib" + self.evaluationName):
      iold = 29
      if not os.path.isfile("draglib" + self.evaluationName + ".bis.gz"):
        os.system("cp draglib" + self.evaluationName + \
        " draglib" + self.evaluationName + ".bis")
        os.system("gzip draglib" + self.evaluationName + ".bis")
      os.system("mv draglib" + self.evaluationName + " tape29")
    else:
      iold = 0
    htime = time.ctime(time.time())
    self.__dict__.update({"iold" : iold , "htime" : htime})
    text_data = """
    dragr
    0 0 0 23 24 %(iold)d 30/
"""%self.__dict__
    if iold == 0:
      text_data = text_data + """
      'draglib from %(evaluationName)s at %(htime)s'/
"""%self.__dict__
    file_data = open("file_data",'w')
    file_data.write(text_data)
    file_data.close()
    os.system("ln -s " + self.fissionFile + " tape23")
    os.system("ln -s " + self.decayFile + " tape24")
    chainFileName = 'chain' + self.evaluationName
    print 'Make the burnup chain file named',chainFileName
    listFiles = os.listdir(os.getcwd())
    data_dict = {}
    mat_dict = {}
    for fileName in listFiles:
      if fileName[:11] == 'out_draglib':
        file = open(fileName,'r')
        while 1:
          line=file.readline()
          if not line: break
          if line[:41] == ' isotopic specification line for material':
            mat = line[42:49]
            line=file.readline()
            line=file.readline()
            info = ''
            key = line[:8]
            while line[:6] != ' -----':
              info = info + line
              line=file.readline()
            data_dict[key] = info[:-1]
            mat_dict[key] = int(mat)
    dictKeys = data_dict.keys()
    dictKeys.sort(lambda a, b: mat_dict[a]-mat_dict[b])
    chainFile = open(chainFileName,'w')
    for key in dictKeys:
      line = data_dict[key]
      pos = 0
      while pos != -1:
         pos = line.find('\n')
         chainFile.write(line[:pos-1] + '\n')
         line = line[pos+1:]
    chainFile.write("end /\n")
    chainFile.write("stop\n")
    chainFile.close()
    os.system("cat file_data " + chainFileName + " > tempFile")
    os.system("../" + self.execDir + "/xnjoy < tempFile")
    os.system("mv tape30 draglib" + self.evaluationName)
    os.system("mv output out_draglib_burnup")
    os.system("chmod 644 out_draglib_burnup")
    os.system("rm tape*")
    os.remove("file_data")
    os.system("mv tempFile file_data_burnup")
    os.chdir("..")
#
  def acer(self):
    textTmp=""
    for tmp in self.tempace:
      textTmp = textTmp + " " + "%E"%tmp
    os.chdir(self.evaluationName)
    self.__dict__.update({"textTmp": textTmp})
    text_data = """
    moder
    20 -21
    moder
    29 -25
    """
    if self.scatteringLaw:
      text_data = text_data + """
    acer
    -21 -25 0 38 39
    1 0 1 %(suff)f/
    'pendf tape from %(evaluationName)s'/
    %(mat)d  %(textTmp)s /
    1 1/
    0.001/
    acer / Check ACE files
    0 38 0 40 41
    7 1 1 -1/
    /
    acer
    -21 -25 0 48 49
    2 0 1 %(suff)f/
    'pendf tape from %(evaluationName)s'/
    %(mat)d  %(textTmp)s %(scatName)s/
    %(za)d 0 0 /
    %(matsab)d 0 16 0 0 1 4.0 0 /
    acer / Check ACE files
    0 48 0 50 51
    7 1 1 -1/
    /
    stop
    """%self.__dict__
    else:
      text_data = text_data + """
    acer
    -21 -25 0 38 39
    1 0 1 %(suff)f/
    'pendf tape from %(evaluationName)s'/
    %(mat)d  %(textTmp)s /
    1 1/
    0.001/
    acer / Check ACE files
    0 38 0 40 41
    7 1 1 -1/
    /
    stop
    """%self.__dict__
    file_data = open("file_data",'w')
    file_data.write(text_data)
    file_data.close()
    os.system("ln -s " + self.evaluationFile + " tape20")
    os.system("ln -s pendf" + self.hmat + " tape29")
    os.system("../" + self.execDir + "/xnjoy<file_data")
    if os.path.isfile("tape48"):
     inp=open("tape48","r")
     outp=open("acecandu",'a')
     line = inp.readlines()
     outp.writelines(line)
     inp1=open("tape49","r")
     outp1=open("acexsdir",'a')
     line = inp1.readlines()
     outp1.writelines(line)
    if os.path.isfile("tape38"):
     inp=open("tape38","r")
     outp=open("acecandu",'a')
     line = inp.readlines()
     outp.writelines(line)
     inp1=open("tape39","r")
     outp1=open("acexsdir",'a')
     line = inp1.readlines()
     outp1.writelines(line)
     print " ace file for" + self.hmat + "created"
     if not os.path.isdir(self.dirName): os.mkdir(self.dirName)
     os.system("mv tape38 " + '   ' + self.dirName + "/"+self.hmat +'_'+str(int(self.tempace[0]))+".ace")
    else: 
     outp2=open("error.log",'a')
     outp2.write( " ace file for" + self.hmat + " not created")
#
    os.system("rm tape*")
    os.remove("file_data")
    os.chdir("..")
