*DECK BIVSPS
      SUBROUTINE BIVSPS (IPTRK,IPMACR,IPSYS,IMPX,NGRP,NEL,NUN,NLF,NANI,
     1 MAT,VOL,IDL,NBMIX,SGD,WORK,IJJ,NJJ,IPOS,IND,SCAT,SCATI)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover the cross-section data in LCM object with pointer IPMACR,
* compute and store the corresponding system matrices for a simplified
* PN approximation.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   L_TRACK pointer to the bivac tracking information.
* IPMACR  L_MACROLIB pointer to the cross sections.
* IPSYS   L_SYSTEM pointer to system matrices.
* IMPX    print parameter (equal to zero for no print).
* NGRP    number of energy groups.
* NEL     total number of finite elements.
* NUN     total number of unknowns per group.
* NLF     number of Legendre orders for the flux (even number).
* NANI    number of Legendre orders for the scattering cross sections.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* IDL     position of the flux component associated with each volume.
* NBMIX   total number of material mixtures in the macrolib.
*
*Parameters: scratch
* SGD
* WORK
* IJJ
* NJJ
* IPOS
* IND
* SCAT
* SCATI
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,IPMACR,IPSYS,IMPX,NGRP,NEL,NUN,NLF,NANI,MAT(NEL),
     1 IDL(NEL),NBMIX,IJJ(NBMIX),NJJ(NBMIX),IPOS(NBMIX),IND(NGRP)
      REAL VOL(NEL),SGD(NBMIX,2*(NANI+1)),WORK(NBMIX*NGRP),
     1 SCAT(NGRP,NGRP,NBMIX),SCATI(NGRP,NGRP,NBMIX)
*----
*  LOCAL VARIABLES
*----
      CHARACTER TEXT12*12,CM*2
      LOGICAL LFIS
*----
*  PROCESS MACROLIB INFORMATION FOR VARIOUS LEGENDRE ORDERS.
*----
      IF(NLF.EQ.0) CALL XABORT('BIVSPS: SPN APPROXIMATION REQUESTED.')
      JPMACR=LCMGID(IPMACR,'GROUP')
      JPSYS=LCMLID(IPSYS,'REMOVALXS',NGRP)
      DO 110 IL=1,NANI+1
      WRITE(CM,'(I2.2)') IL-1
      CALL XDRSET(SCAT,NGRP*NGRP*NBMIX,0.0)
      DO 50 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      KPMACR=LCMGIL(JPMACR,IGR)
      CALL XDRSET(SGD(1,1),NBMIX,0.0)
      IF(IL.EQ.1) THEN
         CALL LCMLEN(KPMACR,'NTOT0',LENGT,ITYLCM)
         IF(LENGT.EQ.0) THEN
            CALL XABORT('BIVSPS: NO TOTAL CROSS SECTIONS.')
         ELSE IF(LENGT.GT.NBMIX) THEN
            CALL XABORT('BIVSPS: INVALID LENGTH FOR TOTAL CROSS SECTIO'
     1      //'NS.')
         ENDIF
         CALL LCMGET(KPMACR,'NTOT0',SGD(1,2))
         CALL LCMLEN(KPMACR,'SIGW00',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            IF(LENGT.GT.NBMIX) CALL XABORT('BIVSPS: INVALID LENGTH FOR'
     1      //' SIGW00 CROSS SECTIONS.')
            CALL LCMGET(KPMACR,'SIGW00',SGD(1,1))
         ENDIF
      ELSE
         CALL LCMLEN(KPMACR,'NTOT1',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            IF(LENGT.GT.NBMIX) CALL XABORT('BIVSPS: INVALID LENGTH FOR'
     1      //' TOTAL-P1 CROSS SECTIONS.')
            CALL LCMGET(KPMACR,'NTOT1',SGD(1,2))
         ELSE
            CALL LCMGET(KPMACR,'NTOT0',SGD(1,2))
         ENDIF
         CALL LCMLEN(KPMACR,'SIGW'//CM,LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            IF(LENGT.GT.NBMIX) CALL XABORT('BIVSPS: INVALID LENGTH FOR'
     1      //' SIGW'//CM//' CROSS SECTIONS.')
            CALL LCMGET(KPMACR,'SIGW'//CM,SGD(1,1))
         ENDIF
      ENDIF
      DO 10 IBM=1,NBMIX
   10 SCAT(IGR,IGR,IBM)=SGD(IBM,2)-SGD(IBM,1)
      CALL LCMLEN(KPMACR,'NJJS'//CM,LENGT,ITYLCM)
      IF(LENGT.GT.NBMIX) CALL XABORT('BIVSPS: INVALID LENGTH FOR NJJS'
     1 //CM//' INFORMATION.')
      IF(LENGT.GT.0) THEN
         CALL LCMGET(KPMACR,'NJJS'//CM,NJJ)
         CALL LCMGET(KPMACR,'IJJS'//CM,IJJ)
         IGMIN=IGR
         IGMAX=IGR
         DO 20 IBM=1,NBMIX
         IGMIN=MIN(IGMIN,IJJ(IBM)-NJJ(IBM)+1)
   20    IGMAX=MAX(IGMAX,IJJ(IBM))
         CALL LCMGET(KPMACR,'IPOS'//CM,IPOS)
         CALL LCMGET(KPMACR,'SCAT'//CM,WORK)
         DO 40 JGR=IGMAX,IGMIN,-1
         IF(JGR.EQ.IGR) GO TO 40
         DO 30 IBM=1,NBMIX
         IF((JGR.GT.IJJ(IBM)-NJJ(IBM)).AND.(JGR.LE.IJJ(IBM))) THEN
            SCAT(IGR,JGR,IBM)=-WORK(IPOS(IBM)+IJJ(IBM)-JGR)
         ENDIF
   30    CONTINUE
   40    CONTINUE
      ENDIF
   50 CONTINUE
*----
*  INVERSION OF THE REMOVAL MATRIX FOR CASES WITH IELEM > 1.
*----
      DO 70 IBM=1,NBMIX
      DO 60 JGR=1,NGRP
      DO 60 IGR=1,NGRP
   60 SCATI(IGR,JGR,IBM)=SCAT(IGR,JGR,IBM)
      CALL ALINV(NGRP,SCATI(1,1,IBM),NGRP,IER,IND)
      IF(IER.NE.0) CALL XABORT('BIVSPS: SINGULAR MATRIX.')
   70 CONTINUE
*
      DO 110 IGR=1,NGRP
      KPSYS=LCMDIL(JPSYS,IGR)
      IPOSDE=0
      DO 90 IBM=1,NBMIX
      IPOS(IBM)=IPOSDE+1
      IGMIN=IGR
      IGMAX=IGR
      DO 80 JGR=NGRP,1,-1
      IF((SCAT(IGR,JGR,IBM).NE.0.0).OR.(SCATI(IGR,JGR,IBM).NE.0.0)) THEN
         IGMIN=MIN(IGMIN,JGR)
         IGMAX=MAX(IGMAX,JGR)
      ENDIF
   80 CONTINUE
      IJJ(IBM)=IGMAX
      NJJ(IBM)=IGMAX-IGMIN+1
      DO 90 JGR=IGMAX,IGMIN,-1
      IPOSDE=IPOSDE+1
   90 WORK(IPOSDE)=SCAT(IGR,JGR,IBM)
      CALL LCMPUT(KPSYS,'NJJS'//CM,NBMIX,1,NJJ)
      CALL LCMPUT(KPSYS,'IJJS'//CM,NBMIX,1,IJJ)
      CALL LCMPUT(KPSYS,'IPOS'//CM,NBMIX,1,IPOS)
      CALL LCMPUT(KPSYS,'SCAT'//CM,IPOSDE,2,WORK)
      IPOSDE=0
      DO 100 IBM=1,NBMIX
      DO 100 JGR=IJJ(IBM),IJJ(IBM)-NJJ(IBM)+1,-1
      IPOSDE=IPOSDE+1
  100 WORK(IPOSDE)=SCATI(IGR,JGR,IBM)
      CALL LCMPUT(KPSYS,'SCATI'//CM,IPOSDE,2,WORK)
  110 CONTINUE
*----
*  COMPUTE AND FACTORIZE THE DIAGONAL SYSTEM MATRICES.
*----
      DO 160 IGR=1,NGRP
      KPSYS=LCMDIL(JPSYS,IGR)
      DO 140 IL=1,NANI+1
      WRITE(CM,'(I2.2)') IL-1
      CALL LCMGET(KPSYS,'IJJS'//CM,IJJ)
      CALL LCMGET(KPSYS,'IPOS'//CM,IPOS)
      CALL LCMGET(KPSYS,'SCAT'//CM,WORK)
      DO 130 IBM=1,NBMIX
  130 SGD(IBM,IL)=WORK(IPOS(IBM)+IJJ(IBM)-IGR)
      CALL LCMGET(KPSYS,'SCATI'//CM,WORK)
      DO 140 IBM=1,NBMIX
  140 SGD(IBM,NANI+1+IL)=WORK(IPOS(IBM)+IJJ(IBM)-IGR)
      WRITE(TEXT12,'(1HA,2I3.3)') IGR,IGR
      CALL BIVASM(TEXT12,0,IPTRK,IPSYS,IMPX,NBMIX,NEL,NUN,NLF,
     1 2*(NANI+1),MAT,VOL,IDL,SGD)
*----
*  PUT A FLAG IN IPSYS TO IDENTIFY NON-ZERO SCATTERING TERMS.
*----
      DO 160 IL=1,NANI+1
      WRITE(CM,'(I2.2)') IL-1
      CALL LCMGET(KPSYS,'NJJS'//CM,NJJ)
      CALL LCMGET(KPSYS,'IJJS'//CM,IJJ)
      DO 160 IBM=1,NBMIX
      DO 160 JGR=IJJ(IBM),IJJ(IBM)-NJJ(IBM)+1,-1
      IF(JGR.NE.IGR) THEN
         WRITE (TEXT12,'(1HA,2I3.3)') IGR,JGR
         CALL LCMPUT(IPSYS,TEXT12,1,2,0.0)
      ENDIF
  160 CONTINUE
*----
*  PROCESS FISSION SPECTRUM TERMS.
*----
      KPMACR=LCMGIL(JPMACR,1)
      CALL LCMLEN(KPMACR,'CHI',LENGT,ITYLCM)
      IF(NBMIX.GT.NBMIX) CALL XABORT('BIVSPS: INVALID LENGTH FOR CHI'
     1 //' INFORMATION.')
      IF(LENGT.GT.0) THEN
         DO 170 IGR=1,NGRP
         KPMACR=LCMGIL(JPMACR,IGR)
         CALL LCMGET(KPMACR,'CHI',WORK((IGR-1)*NBMIX+1))
  170    CONTINUE
      ELSE
         DO 180 IBM=1,NBMIX
         WORK(IBM)=1.0
         DO 180 IGR=2,NGRP
  180    WORK((IGR-1)*NBMIX+IBM)=0.0
      ENDIF
*----
*  PROCESS FISSION NUSIGF TERMS.
*----
      DO 220 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      LFIS=.FALSE.
      DO 190 IBM=1,NBMIX
  190 LFIS=LFIS.OR.(WORK((IGR-1)*NBMIX+IBM).NE.0.0)
      IF(LFIS) THEN
         DO 210 JGR=1,NGRP
         KPMACR=LCMGIL(JPMACR,JGR)
         CALL LCMLEN(KPMACR,'NUSIGF',LENGT,ITYLCM)
         IF(LENGT.GT.NBMIX) CALL XABORT('BIVSPS: INVALID LENGTH FOR '
     1    //'NUSIGF INFORMATION.')
         IF(LENGT.GT.0) THEN
            CALL LCMGET(KPMACR,'NUSIGF',SGD)
            DO 200 IBM=1,LENGT
            SGD(IBM,1)=SGD(IBM,1)*WORK((IGR-1)*NBMIX+IBM)
            SGD(IBM,2)=0.0
            SGD(IBM,3)=0.0
  200       SGD(IBM,4)=0.0
            WRITE (TEXT12,'(1HB,2I3.3)') IGR,JGR
            CALL BIVASM(TEXT12,1,IPTRK,IPSYS,IMPX,NBMIX,NEL,NUN,2,
     1      4,MAT,VOL,IDL,SGD)
         ENDIF
  210    CONTINUE
      ENDIF
  220 CONTINUE
      RETURN
      END
