*DECK FLDBIV
      SUBROUTINE FLDBIV(IPTRK,NEL,NUN,EVECT,MAT,VOL,IDL)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the averaged flux in BIVAC.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   L_TRACK pointer to the BIVAC tracking information.
* NEL     total number of finite elements.
* NUN     total number of unknown per energy group.
* EVECT   variational coefficients of the flux. The information is
*         contained in position EVECT(1) to EVECT(LL4) where LL4 is
*         the order of the system matrices.
* MAT     mixture index assigned to each element.
* VOL     volume of each element.
* IDL     position of the average flux component associated with each
*         volume.
*
*Parameters: output
* EVECT   averaged fluxes. The information is contained in positions
*         EVECT(IDL(I)).
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,NEL,NUN,MAT(NEL),IDL(NEL)
      REAL EVECT(NUN),VOL(NEL)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      LOGICAL CYLIND
      INTEGER ITP(NSTATE),IBASE(1)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER BIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
      ITYPE=ITP(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      IELEM=ITP(8)
      ISPLH=ITP(10)
      LL4=ITP(11)
      LX=ITP(12)
      LY=ITP(13)
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
*
      IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
*        LAGRANGIAN FINITE ELEMENTS.
         CALL SETARA(BASE,LX*LY,IXX)
         CALL SETARA(BASE,LX*LY,IDD)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL LCMGET(IPTRK,'T',BASE(IT))
         CALL LCMGET(IPTRK,'TS',BASE(ITS))
         CALL LCMSIX(IPTRK,' ',2)
         CALL FLDBN2(NEL,LL4,-IELEM,CYLIND,EVECT,BASE(IXX),BASE(IDD),
     1   MAT,VOL,IDL,IBASE(IKN),LC,BASE(IT),BASE(ITS))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
         CALL RLSARA(BASE(IDD))
         CALL RLSARA(BASE(IXX))
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
*        MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,9,IRT)
         CALL LCMGET(IPTRK,'RH',BASE(IRH))
         CALL LCMGET(IPTRK,'RT',BASE(IRT))
         CALL LCMSIX(IPTRK,' ',2)
         IF(ISPLH.EQ.1) THEN
            NELEM=MAXKN/7
         ELSE
            NELEM=MAXKN/4
         ENDIF
         CALL SETARA(BASE,MAXKN,IQFR)
         CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
         CALL FLDBH2(ISPLH,NEL,NUN,NELEM,EVECT,MAT,VOL,IDL,IBASE(IKN),
     1   BASE(IQFR),BASE(IRH),BASE(IRT))
         CALL RLSARA(BASE(IQFR))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IRH))
      ELSE IF((IELEM.GT.0).AND.(ISPLH.GT.1).AND.(ITYPE.EQ.8)) THEN
*        MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
         CALL SETARA(BASE,MAXKN,IQFR)
         CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
         CALL FLDBH1(NEL,NUN,LL4,EVECT,VOL,IDL,IBASE(IKN),BASE(IQFR))
         CALL RLSARA(BASE(IQFR))
      ENDIF
*----
*  RELEASE BIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL RLSARA(IBASE(IKN))
      RETURN
      END
