*DECK FLDBSM
      SUBROUTINE FLDBSM(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,NGRP,F2,F3,SGD,
     1 WORK,IJJ,NJJ,IPOS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* LCM driver for the multiplication of a matrix by a vector.
* Special version for the simplified PN method in BIVAC.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NAMP    name of the coefficient matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* LL4     order of the matrix.
* NBMIX   total number of material mixtures in the macrolib.
* NAN     number of Legendre orders in the cross sections.
* NGRP    number of energy groups.
* F2      vector to multiply.
*
*Parameters: output
* F3      result of the multiplication.
*
*Parameters: scratch
* SGD
* WORK
* IJJ
* NJJ
* IPOS
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*(*)
      INTEGER IPTRK,IPSYS,LL4,NBMIX,NAN,NGRP,IJJ(NBMIX),NJJ(NBMIX),
     1 IPOS(NBMIX)
      REAL F2(LL4),F3(LL4),SGD(NBMIX,2*NAN),WORK(NBMIX*NGRP)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12,CM*2
      INTEGER IBASE(1),IPAR(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER ENERGY GROUP INDICES.
*----
      NAMT=NAMP
      READ(NAMT,'(1X,2I3)') IGR,JGR
      IF((IGR.GT.NGRP).OR.(JGR.GT.NGRP)) CALL XABORT('FLDBSM: INCONSIS'
     1 //'TENT GROUP INDICES.')
*----
*  RECOVER PN SPECIFIC PARAMETERS.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IPAR)
      NREG=IPAR(1)
      ITYPE=IPAR(6)
      IELEM=IPAR(8)
      ICOL=IPAR(9)
      L4=IPAR(11)
      NLF=IPAR(14)
      IF(IPAR(2).NE.L4*NLF/2) CALL XABORT('FLDBSM: INVALID NUN OR L4.')
*----
*  PROCESS A FISSION MATRIX.
*----
      IF(NAMT(1:1).EQ.'B') THEN
         CALL LCMLEN(IPTRK,'MU',LL4TS,ITYLCM)
         IF(L4.NE.LL4TS) CALL XABORT('FLDBSM: INVALID L4.')
         CALL LCMIOF(IPTRK,'MU',IMU)
         CALL LCMIOF(IPSYS,NAMT,IASS)
         CALL ALLDLM(L4,BASE(IASS),F2,F3,IBASE(IMU),1)
         RETURN
      ELSE IF(NAMT(1:1).NE.'A') THEN
         CALL XABORT('FLDBSM: ''A'' OR ''B'' MATRIX EXPECTED.')
      ENDIF
*----
*  RECOVER TRACKING INFORMATION.
*----
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(BASE,NREG,IVOL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      CALL SETARA(BASE,NREG,IXX)
      CALL SETARA(BASE,NREG,IYY)
      CALL LCMGET(IPTRK,'XX',BASE(IXX))
      CALL LCMGET(IPTRK,'YY',BASE(IYY))
*----
*  RECOVER THE CROSS SECTIONS.
*----
      JPSYS=LCMGID(IPSYS,'REMOVALXS')
      KPSYS=LCMGIL(JPSYS,IGR)
      DO 20 IL=1,NAN
      WRITE(CM,'(I2.2)') IL-1
      CALL LCMGET(KPSYS,'NJJS'//CM,NJJ)
      CALL LCMGET(KPSYS,'IJJS'//CM,IJJ)
      CALL LCMGET(KPSYS,'IPOS'//CM,IPOS)
      CALL LCMGET(KPSYS,'SCAT'//CM,WORK)
      CALL XDRSET(SGD(1,IL),NBMIX,0.0)
      DO 10 IBM=1,NBMIX
      IF((JGR.GT.IJJ(IBM)-NJJ(IBM)).AND.(JGR.LE.IJJ(IBM))) THEN
         SGD(IBM,IL)=WORK(IPOS(IBM)+IJJ(IBM)-JGR)
      ENDIF
   10 CONTINUE
      CALL LCMGET(KPSYS,'SCATI'//CM,WORK)
      CALL XDRSET(SGD(1,NAN+IL),NBMIX,0.0)
      DO 20 IBM=1,NBMIX
      IF((JGR.GT.IJJ(IBM)-NJJ(IBM)).AND.(JGR.LE.IJJ(IBM))) THEN
         SGD(IBM,NAN+IL)=WORK(IPOS(IBM)+IJJ(IBM)-JGR)
      ENDIF
   20 CONTINUE
*----
*  RECOVER THE FINITE ELEMENT UNIT STIFFNESS MATRIX.
*----
      CALL LCMSIX(IPTRK,'BIVCOL',1)
      CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
      CALL SETARA(BASE,LC*LC,IR)
      CALL LCMGET(IPTRK,'R',BASE(IR))
      CALL SETARA(BASE,LC*(LC-1),IV)
      CALL LCMGET(IPTRK,'V',BASE(IV))
      CALL LCMSIX(IPTRK,' ',2)
*----
*  COMPUTE THE SOURCE
*----
      ITY=0
      IF(IGR.NE.JGR) ITY=1
      CALL PNSZ2D(ITY,NREG,IELEM,ICOL,BASE(IXX),BASE(IYY),IBASE(IMAT),
     1 BASE(IVOL),NBMIX,NLF,NAN,SGD(1,1),SGD(1,NAN+1),L4,IBASE(IKN),
     2 BASE(IQFR),LC,BASE(IR),BASE(IV),F2,F3)
      IF(ITY.EQ.1) THEN
         DO 30 I=1,LL4
   30    F3(I)=-F3(I)
      ENDIF
      CALL RLSARA(BASE(IV))
      CALL RLSARA(BASE(IR))
      CALL RLSARA(BASE(IYY))
      CALL RLSARA(BASE(IXX))
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
