*DECK FLDBSS
      SUBROUTINE FLDBSS(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,NGRP,F1,SGD,
     1 WORK,IJJ,IPOS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform a one-group SPN flux iteration in Cartesian 2D geometry in
* BIVAC.
*
*Copyright:
* Copyright (C) 2004 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NAMP    name of the coefficient matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* LL4     order of the matrix.
* NBMIX   total number of material mixtures in the macrolib.
* NAN     number of Legendre orders in the cross sections.
* NGRP    number of energy groups.
* F1      source term of the linear system.
*
*Parameters: output
* F1      approached solution of the linear system.
*
*Parameters: scratch
* SGD
* WORK
* IJJ
* IPOS
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*(*)
      INTEGER IPTRK,IPSYS,LL4,NBMIX,NAN,NGRP,IJJ(NBMIX),IPOS(NBMIX)
      REAL F1(LL4),SGD(NBMIX,NAN),WORK(NBMIX*NGRP)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12,CM*2
      INTEGER IBASE(1),IPAR(NSTATE)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER ENERGY GROUP INDICES.
*----
      NAMT=NAMP
      READ(NAMT,'(1X,2I3)') IGR,JGR
      IF(NAMT(1:1).NE.'A') CALL XABORT('FLDBSS: ''A'' MATRIX EXPECTED.')
      IF((IGR.GT.NGRP).OR.(IGR.NE.JGR)) CALL XABORT('FLDBSS: INCONSIST'
     1 //'ENT GROUP INDICES.')
*----
*  RECOVER PN SPECIFIC PARAMETERS.
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',IPAR)
      NREG=IPAR(1)
      NUN=IPAR(2)
      IELEM=IPAR(8)
      ICOL=IPAR(9)
      L4=IPAR(11)
      NLF=IPAR(14)
      IF(NUN.NE.L4*NLF/2) CALL XABORT('FLDBSS: INVALID NUN OR L4.')
      IF(NUN.NE.LL4) CALL XABORT('FLDBSS: INVALID NUN OR LL4.')
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(IBASE,NREG,IKEY)
      CALL SETARA(BASE,NREG,IVOL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IKEY))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL SETARA(IBASE,L4,IMU)
      CALL LCMGET(IPTRK,'MU',IBASE(IMU))
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL LCMLEN(IPTRK,'BFR',MAXBF,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      CALL SETARA(BASE,NREG,IXX)
      CALL SETARA(BASE,NREG,IYY)
      CALL LCMGET(IPTRK,'XX',BASE(IXX))
      CALL LCMGET(IPTRK,'YY',BASE(IYY))
*----
*  RECOVER THE CROSS SECTIONS.
*----
      JPSYS=LCMGID(IPSYS,'REMOVALXS')
      KPSYS=LCMGIL(JPSYS,IGR)
      DO 10 IL=1,NAN
      WRITE(CM,'(I2.2)') IL-1
      CALL LCMGET(KPSYS,'IJJS'//CM,IJJ)
      CALL LCMGET(KPSYS,'IPOS'//CM,IPOS)
      CALL LCMGET(KPSYS,'SCATI'//CM,WORK)
      DO 10 IBM=1,NBMIX
   10 SGD(IBM,IL)=WORK(IPOS(IBM)+IJJ(IBM)-IGR)
*----
*  RECOVER THE FINITE ELEMENT UNIT STIFFNESS MATRIX.
*----
      CALL LCMSIX(IPTRK,'BIVCOL',1)
      CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
      CALL SETARA(BASE,LC*LC,IR)
      CALL LCMGET(IPTRK,'R',BASE(IR))
      CALL SETARA(BASE,LC*(LC-1),IV)
      CALL LCMGET(IPTRK,'V',BASE(IV))
      CALL LCMSIX(IPTRK,' ',2)
*----
*  SOLVE THE LINEAR SYSTEM.
*----
      IIMAX=IBASE(IMU+L4-1)*NLF/2
      CALL SETARA(BASE,IIMAX,ISYS)
      CALL SETARA(BASE,NUN,ISOUR)
      CALL LCMGET(IPSYS,'I'//NAMT,BASE(ISYS))
      DO 30 IUN=1,NUN
      BASE(ISOUR+IUN-1)=F1(IUN)
   30 F1(IUN)=0.0
      CALL PNFL2E(NREG,IELEM,ICOL,BASE(IXX),BASE(IYY),IBASE(IMAT),
     1 BASE(IVOL),NBMIX,NLF,NAN,SGD,L4,IBASE(IKN),BASE(IQFR),IBASE(IMU),
     2 IIMAX,LC,BASE(IR),BASE(IV),BASE(ISYS),BASE(ISOUR),F1)
      CALL RLSARA(BASE(ISOUR))
      CALL RLSARA(BASE(ISYS))
      CALL RLSARA(BASE(IV))
      CALL RLSARA(BASE(IR))
      CALL RLSARA(BASE(IYY))
      CALL RLSARA(BASE(IXX))
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(IBASE(IMU))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IKEY))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
