*DECK FLDSPN
      SUBROUTINE FLDSPN(NAMP,IPTRK,IPSYS,LL4,NBMIX,NAN,NGRP,F1,NADI,
     1 SGD,WORK,IJJ,IPOS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform NADI inner iterations with the ADI preconditionning.
* Special version for Thomas-Raviart basis (simplified PN).
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* NAMP    name of the ADI-splitted matrix.
* IPTRK   L_TRACK pointer to the tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* LL4     order of the matrix.
* NBMIX   total number of material mixtures in the macrolib.
* NAN     number of Legendre orders in the cross sections.
* NGRP    number of energy groups.
* F1      source term of the linear system.
* NADI    number of inner ADI iterations.
*
*Parameters: output
* F1      approached solution of the linear system.
*
*Parameters: scratch
* SGD
* WORK
* IJJ
* IPOS
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER NAMP*12
      INTEGER IPTRK,IPSYS,LL4,NBMIX,NAN,NGRP,NADI,IJJ(NBMIX),IPOS(NBMIX)
      REAL F1(LL4),SGD(NBMIX,NAN),WORK(NBMIX*NGRP)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      CHARACTER NAMT*12,CM*2
      INTEGER IBASE(1),ITP(NSTATE)
      LOGICAL LMUX,DIAG
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  INITIALIZATION
*----
      NAMT=NAMP
      READ(NAMT,'(1X,2I3)') IGR,JGR
      IF(NAMT(1:1).NE.'A') CALL XABORT('FLDBSS: ''A'' MATRIX EXPECTED.')
      IF((IGR.GT.NGRP).OR.(IGR.NE.JGR)) CALL XABORT('FLDSPN: INCONSIST'
     1 //'ENT GROUP INDICES.')
      CALL LCMGET(IPTRK,'STATE-VECTOR',ITP)
      NREG=ITP(1)
      NUN=ITP(2)
      ITYPE=ITP(6)
      IDIM=1
      IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
      IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
      IELEM=ITP(9)
      ICOL=ITP(10)
      L4=ITP(11)
      ISEG=ITP(17)
      LTSW=ITP(19)
      LL4F=ITP(25)
      LL4W=ITP(26)
      LL4X=ITP(27)
      LL4Y=ITP(28)
      LL4Z=ITP(29)
      NLF=ITP(30)
      IF(NUN.NE.L4*NLF/2) CALL XABORT('FLDSPN: INVALID NUN OR L4.')
      IF(NUN.NE.LL4) CALL XABORT('FLDSPN: INVALID NUN OR LL4.')
*----
*  RECOVER TRACKING-RELATED INFORMATIONS
*----
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(IBASE,NREG,IKEY)
      CALL SETARA(BASE,NREG,IVOL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IKEY))
      CALL LCMGET(IPTRK,'VOLUME',BASE(IVOL))
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL LCMLEN(IPTRK,'BFR',MAXBF,ITYLCM)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      CALL SETARA(BASE,NREG,IXX)
      CALL SETARA(BASE,NREG,IYY)
      CALL SETARA(BASE,NREG,IZZ)
      CALL LCMGET(IPTRK,'XX',BASE(IXX))
      CALL LCMGET(IPTRK,'YY',BASE(IYY))
      CALL LCMGET(IPTRK,'ZZ',BASE(IZZ))
*----
*  RECOVER THE CROSS SECTIONS.
*----
      JPSYS=LCMGID(IPSYS,'REMOVALXS')
      KPSYS=LCMGIL(JPSYS,IGR)
      DO 10 IL=1,NAN
      WRITE(CM,'(I2.2)') IL-1
      CALL LCMGET(KPSYS,'IJJS'//CM,IJJ)
      CALL LCMGET(KPSYS,'IPOS'//CM,IPOS)
      CALL LCMGET(KPSYS,'SCATI'//CM,WORK)
      DO 10 IBM=1,NBMIX
   10 SGD(IBM,IL)=WORK(IPOS(IBM)+IJJ(IBM)-IGR)
*----
*  RECOVER THE FINITE ELEMENT UNIT STIFFNESS MATRIX.
*----
      CALL LCMSIX(IPTRK,'BIVCOL',1)
      CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
      CALL SETARA(BASE,LC*LC,IR)
      CALL LCMGET(IPTRK,'R',BASE(IR))
      CALL SETARA(BASE,LC*(LC-1),IV)
      CALL LCMGET(IPTRK,'V',BASE(IV))
      CALL LCMSIX(IPTRK,' ',2)
*----
*  RECOVER INFORMATIONS RELATED TO SYSTEM MATRICES
*----
      CALL LCMLEN(IPTRK,'MUX',IDUM,ITYLCM)
      LMUX=(IDUM.NE.0).AND.(ITYLCM.EQ.1)
      DIAG=(LL4Y.GT.0).AND.(.NOT.LMUX)
      CALL LCMIOF(IPSYS,'TF'//NAMT,ITF)
*
      IF(DIAG) THEN
         CALL LCMIOF(IPSYS,'YI'//NAMT,IC11X)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUY',IMUX)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VY',LL4VX)
            CALL LCMIOF(IPTRK,'MUVY',IMUX)
            CALL LCMIOF(IPTRK,'IPVY',IIPVX)
            CALL LCMLEN(IPTRK,'NBLY',LONX,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBLX)
            CALL LCMIOF(IPTRK,'LBLY',ILBLX)
         ENDIF
      ELSE
         CALL LCMIOF(IPSYS,'XI'//NAMT,IC11X)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUX',IMUX)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VX',LL4VX)
            CALL LCMIOF(IPTRK,'MUVX',IMUX)
            CALL LCMIOF(IPTRK,'IPVX',IIPVX)
            CALL LCMLEN(IPTRK,'NBLX',LONX,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLX',INBLX)
            CALL LCMIOF(IPTRK,'LBLX',ILBLX)
         ENDIF
      ENDIF
      CALL LCMIOF(IPTRK,'IPBBX',IIPBX)
      CALL LCMIOF(IPTRK,'XB',IBX)
      IF(LL4Y.GT.0) THEN
         CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
         CALL LCMIOF(IPTRK,'YB',IBY)
         CALL LCMIOF(IPSYS,'YI'//NAMT,IC11Y)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUY',IMUY)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VY',LL4VY)
            CALL LCMIOF(IPTRK,'MUVY',IMUY)
            CALL LCMIOF(IPTRK,'IPVY',IIPVY)
            CALL LCMLEN(IPTRK,'NBLY',LONY,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLY',INBLY)
            CALL LCMIOF(IPTRK,'LBLY',ILBLY)
         ENDIF
      ENDIF
      IF(LL4Z.GT.0) THEN
         CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
         CALL LCMIOF(IPTRK,'ZB',IBZ)
         CALL LCMIOF(IPSYS,'ZI'//NAMT,IC11Z)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL LCMIOF(IPTRK,'MUZ',IMUZ)
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
            CALL LCMGET(IPTRK,'LL4VZ',LL4VZ)
            CALL LCMIOF(IPTRK,'MUVZ',IMUZ)
            CALL LCMIOF(IPTRK,'IPVZ',IIPVZ)
            CALL LCMLEN(IPTRK,'NBLZ',LONZ,ITYLCM)
            CALL LCMIOF(IPTRK,'NBLZ',INBLZ)
            CALL LCMIOF(IPTRK,'LBLZ',ILBLZ)
         ENDIF
      ENDIF
*----
*  PERFORM ADI ITERATIONS AND LEGENDRE ORDER SWAPPING
*----
      CALL SETARA(BASE,LL4F,IFL)
      CALL SETARA(BASE,LL4X,IFX)
      IF(LL4Y.GT.0) THEN
         CALL SETARA(BASE,LL4Y,IFY)
         CALL XDRSET(BASE(IFY),LL4Y,0.0)
      ENDIF
      IF(LL4Z.GT.0) THEN
         CALL SETARA(BASE,LL4Z,IFZ)
         CALL XDRSET(BASE(IFZ),LL4Z,0.0)
      ENDIF
      CALL SETARA(BASE,NUN,ISOUR)
      IF(ISEG.GT.0) CALL SETARA(BASE,ISEG,IT)
      DO 20 IUN=1,NUN
      BASE(ISOUR+IUN-1)=F1(IUN)
   20 F1(IUN)=0.0
      DO 410 IADI=1,NADI
      DO 410 IL=0,NLF-1
      CALL PNFL3E(IL,NREG,IELEM,ICOL,IDIM,BASE(IXX),BASE(IYY),
     1 BASE(IZZ),IBASE(IMAT),BASE(IVOL),NBMIX,NLF,NAN,SGD,L4,
     2 IBASE(IKN),BASE(IQFR),LC,BASE(IR),BASE(IV),BASE(ISOUR),F1)
      IF(MOD(IL,2).EQ.1) THEN
*----
*  X DIRECTION
*----
         DO 30 I0=1,LL4F
   30    BASE(IFL+I0-1)=F1((IL/2)*L4+I0)
         DO 50 I0=1,LL4Y
         DO 40 J0=1,2*IELEM
         JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 50
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBY+(I0-1)*2*IELEM+J0-1)*
     1   REAL(IL)*BASE(IFY+I0-1)
   40    CONTINUE
   50    CONTINUE
         DO 70 I0=1,LL4Z
         DO 60 J0=1,2*IELEM
         JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 70
         BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBZ+(I0-1)*2*IELEM+J0-1)*
     1   REAL(IL)*BASE(IFZ+I0-1)
   60    CONTINUE
   70    CONTINUE
         DO 90 I0=1,LL4X
         GGX=-F1((IL/2)*L4+LL4F+I0)
         DO 80 J0=1,2*IELEM
         JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
         IF(JJ.EQ.0) GO TO 90
         GGX=GGX+BASE(IBX+(I0-1)*2*IELEM+J0-1)*REAL(IL)*BASE(IFL+JJ-1)/
     1   BASE((IL/2)*LL4F+ITF+JJ-1)
   80    CONTINUE
   90    BASE(IFX+I0-1)=GGX
         MUMAX=IBASE(IMUX+LL4X-1)
         IF(ISEG.EQ.0) THEN
*           SCALAR SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL ALLDLS(LL4X,IBASE(IMUX),BASE((IL/2)*MUMAX+IC11X),
     1      BASE(IFX))
         ELSE IF(ISEG.GT.0) THEN
*           SUPERVECTORIAL SOLUTION FOR A X-ORIENTED LINEAR SYSTEM.
            CALL SETARA(BASE,LL4VX,IGAR)
            CALL XDRSET(BASE(IGAR),LL4VX,0.0)
            DO 100 I=1,LL4X
  100       BASE(IGAR+IBASE(IIPVX+I-1)-1)=BASE(IFX+I-1)
            CALL ALVDLS(LTSW,IBASE(IMUX),BASE((IL/2)*MUMAX+IC11X),
     1      BASE(IGAR),ISEG,LONX,IBASE(INBLX),IBASE(ILBLX),BASE(IT))
            DO 110 I=1,LL4X
  110       BASE(IFX+I-1)=BASE(IGAR+IBASE(IIPVX+I-1)-1)
            CALL RLSARA(BASE(IGAR))
         ENDIF
*----
*  Y DIRECTION
*----
         IF(LL4Y.GT.0) THEN
            DO 120 I0=1,LL4F
  120       BASE(IFL+I0-1)=F1((IL/2)*L4+I0)
            DO 140 I0=1,LL4X
            DO 130 J0=1,2*IELEM
            JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 140
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBX+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFX+I0-1)
  130       CONTINUE
  140       CONTINUE
            DO 160 I0=1,LL4Z
            DO 150 J0=1,2*IELEM
            JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 160
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBZ+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFZ+I0-1)
  150       CONTINUE
  160       CONTINUE
            DO 180 I0=1,LL4Y
            GGY=-F1((IL/2)*L4+LL4F+LL4X+I0)
            DO 170 J0=1,2*IELEM
            JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 180
            GGY=GGY+BASE(IBY+(I0-1)*2*IELEM+J0-1)*REAL(IL)*
     1      BASE(IFL+JJ-1)/BASE((IL/2)*LL4F+ITF+JJ-1)
  170       CONTINUE
  180       BASE(IFY+I0-1)=GGY
            MUMAX=IBASE(IMUY+LL4Y-1)
            IF(ISEG.EQ.0) THEN
*              SCALAR SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
               CALL ALLDLS(LL4Y,IBASE(IMUY),BASE((IL/2)*MUMAX+IC11Y),
     1         BASE(IFY))
            ELSE IF(ISEG.GT.0) THEN
*              SUPERVECTORIAL SOLUTION FOR A Y-ORIENTED LINEAR SYSTEM.
               CALL SETARA(BASE,LL4VY,IGAR)
               CALL XDRSET(BASE(IGAR),LL4VY,0.0)
               DO 190 I=1,LL4Y
  190          BASE(IGAR+IBASE(IIPVY+I-1)-1)=BASE(IFY+I-1)
               CALL ALVDLS(LTSW,IBASE(IMUY),BASE((IL/2)*MUMAX+IC11Y),
     1         BASE(IGAR),ISEG,LONY,IBASE(INBLY),IBASE(ILBLY),BASE(IT))
               DO 200 I=1,LL4Y
  200          BASE(IFY+I-1)=BASE(IGAR+IBASE(IIPVY+I-1)-1)
               CALL RLSARA(BASE(IGAR))
            ENDIF
         ENDIF
*----
*  Z DIRECTION
*----
         IF(LL4Z.GT.0) THEN
            DO 210 I0=1,LL4F
  210       BASE(IFL+I0-1)=F1((IL/2)*L4+I0)
            DO 230 I0=1,LL4X
            DO 220 J0=1,2*IELEM
            JJ=IBASE(IIPBX+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 230
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBX+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFX+I0-1)
  220       CONTINUE
  230       CONTINUE
            DO 250 I0=1,LL4Y
            DO 240 J0=1,2*IELEM
            JJ=IBASE(IIPBY+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 250
            BASE(IFL+JJ-1)=BASE(IFL+JJ-1)-BASE(IBY+(I0-1)*2*IELEM+J0-1)
     1      *REAL(IL)*BASE(IFY+I0-1)
  240       CONTINUE
  250       CONTINUE
            DO 270 I0=1,LL4Z
            GGZ=-F1((IL/2)*L4+LL4F+LL4X+LL4Y+I0)
            DO 260 J0=1,2*IELEM
            JJ=IBASE(IIPBZ+(I0-1)*2*IELEM+J0-1)
            IF(JJ.EQ.0) GO TO 270
            GGZ=GGZ+BASE(IBZ+(I0-1)*2*IELEM+J0-1)*REAL(IL)*
     1      BASE(IFL+JJ-1)/BASE((IL/2)*LL4F+ITF+JJ-1)
  260       CONTINUE
  270       BASE(IFZ+I0-1)=GGZ
            MUMAX=IBASE(IMUZ+LL4Z-1)
            IF(ISEG.EQ.0) THEN
*              SCALAR SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
               CALL ALLDLS(LL4Z,IBASE(IMUZ),BASE((IL/2)*MUMAX+IC11Z),
     1         BASE(IFZ))
            ELSE IF(ISEG.GT.0) THEN
*              SUPERVECTORIAL SOLUTION FOR A Z-ORIENTED LINEAR SYSTEM.
               CALL SETARA(BASE,LL4VZ,IGAR)
               CALL XDRSET(BASE(IGAR),LL4VZ,0.0)
               DO 280 I=1,LL4Z
  280          BASE(IGAR+IBASE(IIPVZ+I-1)-1)=BASE(IFZ+I-1)
               CALL ALVDLS(LTSW,IBASE(IMUZ),BASE((IL/2)*MUMAX+IC11Z),
     1         BASE(IGAR),ISEG,LONZ,IBASE(INBLZ),IBASE(ILBLZ),BASE(IT))
               DO 290 I=1,LL4Z
  290          BASE(IFZ+I-1)=BASE(IGAR+IBASE(IIPVZ+I-1)-1)
               CALL RLSARA(BASE(IGAR))
            ENDIF
         ENDIF
*----
*  COMPUTE FLUX AND RECOVER CURRENTS
*----
         DO 300 I0=1,LL4F
  300    BASE(IFL+I0-1)=F1((IL/2)*L4+I0)
         DO 320 J0=1,LL4X
         DO 310 I0=1,2*IELEM
         II=IBASE(IIPBX+(J0-1)*2*IELEM+I0-1)
         IF(II.EQ.0) GO TO 320
         BASE(IFL+II-1)=BASE(IFL+II-1)-BASE(IBX+(J0-1)*2*IELEM+I0-1)*
     1   REAL(IL)*BASE(IFX+J0-1)
  310    CONTINUE
  320    CONTINUE
         DO 340 J0=1,LL4Y
         DO 330 I0=1,2*IELEM
         II=IBASE(IIPBY+(J0-1)*2*IELEM+I0-1)
         IF(II.EQ.0) GO TO 340
         BASE(IFL+II-1)=BASE(IFL+II-1)-BASE(IBY+(J0-1)*2*IELEM+I0-1)*
     1   REAL(IL)*BASE(IFY+J0-1)
  330    CONTINUE
  340    CONTINUE
         DO 360 J0=1,LL4Z
         DO 350 I0=1,2*IELEM
         II=IBASE(IIPBZ+(J0-1)*2*IELEM+I0-1)
         IF(II.EQ.0) GO TO 360
         BASE(IFL+II-1)=BASE(IFL+II-1)-BASE(IBZ+(J0-1)*2*IELEM+I0-1)*
     1   REAL(IL)*BASE(IFZ+J0-1)
  350    CONTINUE
  360    CONTINUE
         DO 370 I0=1,LL4F
  370    F1((IL/2)*L4+I0)=BASE(IFL+I0-1)/BASE((IL/2)*LL4F+ITF+I0-1)
         DO 380 I0=1,LL4X
  380    F1((IL/2)*L4+LL4F+I0)=BASE(IFX+I0-1)
         IF(LL4Y.GT.0) THEN
            DO 390 I0=1,LL4Y
  390       F1((IL/2)*L4+LL4F+LL4X+I0)=BASE(IFY+I0-1)
         ENDIF
         IF(LL4Z.GT.0) THEN
            DO 400 I0=1,LL4Z
  400       F1((IL/2)*L4+LL4F+LL4X+LL4Y+I0)=BASE(IFZ+I0-1)
         ENDIF
      ENDIF
  410 CONTINUE
      IF(ISEG.GT.0) CALL RLSARA(BASE(IT))
      CALL RLSARA(BASE(IFL))
      CALL RLSARA(BASE(IFX))
      IF(LL4Y.GT.0) CALL RLSARA(BASE(IFY))
      IF(LL4Z.GT.0) CALL RLSARA(BASE(IFZ))
      CALL RLSARA(BASE(ISOUR))
      CALL RLSARA(BASE(IV))
      CALL RLSARA(BASE(IR))
      CALL RLSARA(BASE(IZZ))
      CALL RLSARA(BASE(IYY))
      CALL RLSARA(BASE(IXX))
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IVOL))
      CALL RLSARA(IBASE(IKEY))
      CALL RLSARA(IBASE(IMAT))
      RETURN
      END
