*DECK TRIASM
      SUBROUTINE TRIASM(HNAMT,IPTRK,IPSYS,IMPX,MAXMIX,NEL,NUN,IPR,MAT,
     1 VOL,IDL,SGD,XSGD,TITR)
*
*-----------------------------------------------------------------------
*
*Purpose:
* assembly of a single-group system matrix with leakage and removal
* cross sections.
*
*Copyright:
* Copyright (C) 2002 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* HNAMT   name of the matrix.
* IPTRK   L_TRACK pointer to the TRIVAC tracking information.
* IPSYS   L_SYSTEM pointer to system matrices.
* IMPX    print parameter (equal to zero for no print).
* MAXMIX  first dimension for matrices SGD and XSGD.
* NEL     total number of finite elements.
* NUN     total number of unknowns per group.
* IPR     type of assembly:
*         =0: calculation of the system matrices;
*         =1: calculation of the derivative of these matrices;
*         =2: calculation of the first variation of these matrices;
*         =3: identical to IPR=2, but these variation are added to
*         unperturbed system matrices.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* IDL     position of the average flux component associated with each
*         volume.
* SGD     nuclear properties per material mixture.
* XSGD    first variations or derivatives of nuclear properties:
*         if IPR.ge.1, XSGD contain first variations or derivatives
*         of nuclear properties in each material mixture;
*         if IPR=0, XSGD should be equivalenced with SGD. This is
*         obtained using 'CALL TRIASM(...,IDL,SGD,SGD,TITR)'.
* TITR    title.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER HNAMT*10,TITR*72
      INTEGER IPTRK,IPSYS,IMPX,MAXMIX,NEL,NUN,IPR,MAT(NEL),IDL(NEL)
      REAL VOL(NEL),SGD(MAXMIX,4),XSGD(MAXMIX,4)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40)
      LOGICAL CYLIND,CHEX,DIAG,LSGD
      CHARACTER TEXT10*10
      INTEGER IBASE(1),NCODE(6),ISTATE(NSTATE)
      REAL ZCODE(6)
      COMMON BASE(1)
      EQUIVALENCE (BASE(1),IBASE(1))
*----
*  RECOVER TRIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      ITYPE=ISTATE(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      CHEX=(ITYPE.EQ.8).OR.(ITYPE.EQ.9)
      IDIM=1
      IF((ITYPE.EQ.5).OR.(ITYPE.EQ.6).OR.(ITYPE.EQ.8)) IDIM=2
      IF((ITYPE.EQ.7).OR.(ITYPE.EQ.9)) IDIM=3
      IHEX=ISTATE(7)
      DIAG=(ISTATE(8).EQ.1)
      IELEM=ISTATE(9)
      ICOL=ISTATE(10)
      LL4=ISTATE(11)
      ICHX=ISTATE(12)
      ISPLH=ISTATE(13)
      LX=ISTATE(14)
      LY=ISTATE(15)
      LZ=ISTATE(16)
      ISEG=ISTATE(17)
      IMPV=ISTATE(18)
      NR0=ISTATE(24)
      LL4F=ISTATE(25)
      IF(ICHX.EQ.2) THEN
         ITY=3
         LL4W=ISTATE(26)
         LL4X=ISTATE(27)
         LL4Y=ISTATE(28)
         LL4Z=ISTATE(29)
      ELSE
         ITY=2
         LL4W=LL4
         LL4X=LL4
         LL4Y=LL4
         LL4Z=LL4
      ENDIF
      CALL LCMLEN(IPTRK,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK,'QFR',MAXQF,ITYLCM)
      CALL SETARA(BASE,LX*LY*LZ,IZZ)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL LCMGET(IPTRK,'ZZ',BASE(IZZ))
      CALL LCMGET(IPTRK,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK,'QFR',BASE(IQFR))
      IF(CHEX) THEN
         CALL LCMGET(IPTRK,'SIDE',SIDE)
         CALL SETARA(IBASE,LL4W,IMUW)
         CALL LCMGET(IPTRK,'MUW',IBASE(IMUW))
      ELSE
         CALL SETARA(BASE,LX*LY*LZ,IXX)
         CALL SETARA(BASE,LX*LY*LZ,IYY)
         CALL SETARA(BASE,LX*LY*LZ,IDD)
         CALL LCMGET(IPTRK,'XX',BASE(IXX))
         CALL LCMGET(IPTRK,'YY',BASE(IYY))
         CALL LCMGET(IPTRK,'DD',BASE(IDD))
      ENDIF
      IF(IDIM.GE.2) THEN
         CALL SETARA(IBASE,LL4X,IMUY)
         CALL LCMGET(IPTRK,'MUY',IBASE(IMUY))
      ENDIF
      IF(.NOT.DIAG) THEN
         CALL SETARA(IBASE,LL4X,IMUX)
         CALL LCMGET(IPTRK,'MUX',IBASE(IMUX))
      ELSE
         IMUX=IMUY
      ENDIF
      IF(IDIM.EQ.3) THEN
         CALL SETARA(IBASE,LL4Z,IMUZ)
         CALL LCMGET(IPTRK,'MUZ',IBASE(IMUZ))
      ENDIF
*----
*  RECOVER UNIT MATRICES
*----
      IF((ICHX.EQ.1).OR.(ICHX.EQ.2)) THEN
         CALL LCMSIX(IPTRK,'BIVCOL',1)
         CALL LCMLEN(IPTRK,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*LC,IRS)
         CALL SETARA(BASE,LC*LC,IQ)
         CALL SETARA(BASE,LC*LC,IQS)
         CALL SETARA(BASE,LC*(LC-1),IV)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,36,IQH)
         CALL SETARA(BASE,9,IRT)
         CALL SETARA(BASE,9,IQT)
         CALL LCMGET(IPTRK,'T',BASE(IT))
         CALL LCMGET(IPTRK,'TS',BASE(ITS))
         CALL LCMGET(IPTRK,'R',BASE(IR))
         CALL LCMGET(IPTRK,'RS',BASE(IRS))
         CALL LCMGET(IPTRK,'Q',BASE(IQ))
         CALL LCMGET(IPTRK,'QS',BASE(IQS))
         CALL LCMGET(IPTRK,'V',BASE(IV))
         IF((IELEM.EQ.1).AND.(ICOL.LE.2)) THEN
            CALL LCMGET(IPTRK,'RH',BASE(IRH))
            CALL LCMGET(IPTRK,'QH',BASE(IQH))
            CALL LCMGET(IPTRK,'RT',BASE(IRT))
            CALL LCMGET(IPTRK,'QT',BASE(IQT))
         ENDIF
         CALL LCMSIX(IPTRK,' ',2)
      ENDIF
*
      TEXT10=HNAMT
      IF(IMPX.GT.0) WRITE(6,'(/36H TRIASM: ASSEMBLY OF SYMMETRIC MATRI,
     1 3HX '',A10,38H'' IN COMPRESSED DIAGONAL STORAGE MODE.)') TEXT10
      CALL KDRCPU(TK1)
*----
*  COMPUTE THE INVERSE CROSS SECTIONS FOR DUAL FINITE ELEMENT CASES
*----
      IF(ICHX.EQ.2) THEN
         CALL SETARA(BASE,4*MAXMIX,IDSGD)
         IF(IPR.EQ.0) THEN
            DO 5 J=1,4
            DO 5 I=1,MAXMIX
            IF(SGD(I,J).NE.0.) BASE(IDSGD+(J-1)*MAXMIX+I-1)=1.0/SGD(I,J)
    5       CONTINUE
         ELSE IF(IPR.EQ.1) THEN
            DO 10 J=1,4
            DO 10 I=1,MAXMIX
            IF(SGD(I,J).NE.0.0) THEN
               BASE(IDSGD+(J-1)*MAXMIX+I-1)=-XSGD(I,J)/(SGD(I,J)**2)
            ENDIF
   10       CONTINUE
         ELSE
            DO 20 J=1,4
            DO 20 I=1,MAXMIX
            SIGMA=SGD(I,J)+XSGD(I,J)
            IF((SGD(I,J).NE.0.0).AND.(SIGMA.NE.0.0)) THEN
               BASE(IDSGD+(J-1)*MAXMIX+I-1)=1.0/SIGMA-1.0/SGD(I,J)
            ENDIF
   20       CONTINUE
         ENDIF
      ENDIF
*----
*  DETERMINATION OF THE PERTURBED ELEMENTS AND INCLUSION OF ELEMENTS
*  NEIGHBOUR TO PERTURBED ZONES IN MCFD CASES. NON-PERTURBED ELEMENTS
*  WILL HAVE BASE(IVOL+K-1)=0.0
*----
      CALL SETARA(BASE,NEL,IVOL)
      IF(IPR.EQ.0) THEN
         DO 25 K=1,NEL
   25    BASE(IVOL+K-1)=VOL(K)
      ELSE
         DO 26 K=1,NEL
   26    BASE(IVOL+K-1)=0.0
         IF(ICHX.EQ.3) THEN
*           MCFD CASE.
            NUM1=0
            DO 40 L=1,NEL
            IF(MAT(L).EQ.0) GO TO 40
            LSGD=.FALSE.
            DO 30 I=1,4
   30       LSGD=LSGD.OR.(XSGD(MAT(L),I).NE.0.0)
            IF(LSGD) THEN
               BASE(IVOL+L-1)=VOL(L)
               DO 35 I=1,6
               K=IBASE(IKN+(NUM1+I)-1)
               IF(K.GT.0) THEN
                  IF(K.GT.NEL) CALL XABORT('TRIASM: INVALID BOUNDARY E'
     1            //'LEMENT INDEX.')
                  BASE(IVOL+K-1)=VOL(K)
               ENDIF
   35          CONTINUE
            ENDIF
            NUM1=NUM1+6
   40       CONTINUE
         ELSE
            DO 50 L=1,NEL
            IF(MAT(L).EQ.0) GO TO 50
            LSGD=.FALSE.
            DO 45 I=1,4
   45       LSGD=LSGD.OR.(XSGD(MAT(L),I).NE.0.0)
            IF(LSGD) BASE(IVOL+L-1)=VOL(L)
   50       CONTINUE
         ENDIF
      ENDIF
*----
*  INTRODUCE THE CYLINDER BOUNDARY APPROXIMATION IN CARTESIAN GEOMETRY
*----
      IF(NR0.GT.0) THEN
         CALL SETARA(BASE,NR0,IRR0)
         CALL LCMGET(IPTRK,'RR0',BASE(IRR0))
         CALL SETARA(BASE,NR0,IXR0)
         CALL LCMGET(IPTRK,'XR0',BASE(IXR0))
         CALL SETARA(BASE,NR0,IANG)
         CALL LCMGET(IPTRK,'ANG',BASE(IANG))
         CALL LCMGET(IPTRK,'NCODE',NCODE)
         CALL LCMGET(IPTRK,'ZCODE',ZCODE)
         CALL SETARA(BASE,LX+1,IXXX)
         CALL SETARA(BASE,LY+1,IYYY)
         CALL SETARA(BASE,LZ+1,IZZZ)
         CALL TRICYL(MAXMIX,IMPX,ICHX,IDIM,LX,LY,LZ,BASE(IXX),BASE(IYY),
     1   BASE(IZZ),VOL,MAT,NCODE,ZCODE,NR0,BASE(IRR0),BASE(IXR0),
     2   BASE(IANG),SGD,BASE(IQFR),BASE(IXXX),BASE(IYYY),BASE(IZZZ))
         CALL RLSARA(BASE(IZZZ))
         CALL RLSARA(BASE(IYYY))
         CALL RLSARA(BASE(IXXX))
         CALL RLSARA(BASE(IANG))
         CALL RLSARA(BASE(IXR0))
         CALL RLSARA(BASE(IRR0))
      ENDIF
*----
*  ASSEMBLY OF THE ADI SPLITTED SYSTEM MATRICES
*----
*
* DIMENSION W
      IF(CHEX) THEN
         IF(ICHX.EQ.2) THEN
            CALL XABORT('TRIASM: NOT IMPLEMENTED.')
         ELSE IF((ICHX.EQ.3).AND.(ISPLH.GT.1)) THEN
            CALL SETARA(IBASE,LL4,IMATN)
            NUM1=0
            DO 2 I=1,LX*LZ
            IF(MAT(I).EQ.0) GO TO 2
            DO 1 J=1,6*(ISPLH-1)**2
            KEL=IBASE(IKN+NUM1+J-1)
   1        IBASE(IMATN+KEL-1)=MAT(I)
            NUM1=NUM1+18*(ISPLH-1)**2+8
   2        CONTINUE
         ENDIF
         IIMAW=IBASE(IMUW+LL4W-1)
         IF(IPR.EQ.3) THEN
            IF(ISEG.GT.0) CALL MTBLD('W '//TEXT10,IPTRK,IPSYS,1)
            CALL LCMIOF(IPSYS,'W '//TEXT10,IWA)
         ELSE
            CALL SETARA(BASE,IIMAW,IWA)
            DO 65 I=1,IIMAW
   65       BASE(IWA+I-1)=0.0
         ENDIF
         IF(ICHX.EQ.1) THEN
*           MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
            CALL LCMIOF(IPTRK,'IPW',IIPW)
            CALL TRIRWW(MAXMIX,NEL,LL4,VOL,MAT,XSGD,SIDE,BASE(IZZ),
     1      IBASE(IKN),BASE(IQFR),IBASE(IMUW),IBASE(IIPW),BASE(IWA),
     2      ISPLH,BASE(IR),BASE(IQ),BASE(IRH),BASE(IQH),BASE(IRT),
     3      BASE(IQT))
         ELSE IF(ICHX.EQ.3) THEN
*           MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
            CALL LCMIOF(IPTRK,'IPW',IIPW)
            IF(ISPLH.EQ.1) THEN
               CALL SETARA(BASE,NEL,IIGAR)
               CALL TRIMWW(MAXMIX,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,SIDE,
     1         BASE(IZZ),IBASE(IKN),BASE(IQFR),IBASE(IMUW),IBASE(IIPW),
     2         IPR,BASE(IWA),BASE(IIGAR))
               CALL RLSARA(BASE(IIGAR))
            ELSE
               CALL TRIMTW(ISPLH,MAXMIX,IDIM,NEL,LL4,VOL,MAT,
     1         IBASE(IMATN),SGD,XSGD,SIDE,BASE(IZZ),IBASE(IKN),
     2         BASE(IQFR),IBASE(IMUW),IBASE(IIPW),IPR,BASE(IWA))
            ENDIF
         ENDIF
         CALL LCMPOF(IPSYS,'W '//TEXT10,IIMAW,2,IWA)
      ENDIF
*
* DIMENSION X
      IIMAX=IBASE(IMUX+LL4X-1)
      IF(ICHX.EQ.2) THEN
*        THOMAS-RAVIART ADI ITERATIVE METHOD.
         IF(DIAG.OR.(IPR.NE.3)) THEN
            CALL SETARA(BASE,LL4F,ITF)
            CALL SETARA(BASE,IIMAX,IAX)
            CALL XDRSET(BASE(ITF),LL4F,0.0)
            CALL XDRSET(BASE(IAX),IIMAX,0.0)
         ELSE
            IF(ISEG.GT.0) CALL MTBLD('XA'//TEXT10,IPTRK,IPSYS,1)
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMIOF(IPSYS,'XA'//TEXT10,IAX)
         ENDIF
         CALL LCMIOF(IPTRK,'IPBBX',IIPBX)
         CALL LCMIOF(IPTRK,'XB',IBX)
         CALL SETARA(BASE,IIMAX,IXA)
         CALL TRIDXX(MAXMIX,CYLIND,IELEM,ICOL,IDIM,NEL,LL4,LL4F,LL4X,
     1   MAT,BASE(IVOL),BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),
     2   IBASE(IKN),BASE(IQFR),XSGD,BASE(IDSGD),IBASE(IMUX),
     3   IBASE(IIPBX),LC,BASE(IR),BASE(IV),BASE(IBX),BASE(ITF),
     4   BASE(IAX),BASE(IXA))
      ELSE
*        GENERIC ADI ITERATIVE METHOD.
         CALL LCMIOF(IPTRK,'IPX',IIPX)
         IF(DIAG.OR.(IPR.NE.3)) THEN
            CALL SETARA(BASE,IIMAX,IXA)
            CALL XDRSET(BASE(IXA),IIMAX,0.0)
         ELSE
            IF(ISEG.GT.0) CALL MTBLD('X '//TEXT10,IPTRK,IPSYS,1)
            CALL LCMIOF(IPSYS,'X '//TEXT10,IXA)
         ENDIF
         IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
            CALL TRIPXX(MAXMIX,MAXKN,NEL,LL4,BASE(IVOL),MAT,XSGD,
     1      BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),IBASE(IKN),
     2      BASE(IQFR),IBASE(IMUX),IBASE(IIPX),CYLIND,LC,BASE(IT),
     3      BASE(ITS),BASE(IQ),BASE(IQS),BASE(IXA))
         ELSE IF((ICHX.EQ.3).AND.(.NOT.CHEX)) THEN
            CALL SETARA(BASE,NEL,IIGAR)
            CALL TRIMXX(MAXMIX,CYLIND,IELEM,IDIM,NEL,LL4,BASE(IVOL),
     1      MAT,SGD,XSGD,BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),
     2      IBASE(IKN),BASE(IQFR),IBASE(IMUX),IBASE(IIPX),IPR,BASE(IXA),
     3      BASE(IIGAR))
            CALL RLSARA(BASE(IIGAR))
         ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
*           MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
            CALL TRIRWX(MAXMIX,NEL,LL4,VOL,MAT,XSGD,SIDE,BASE(IZZ),
     1      IBASE(IKN),BASE(IQFR),IBASE(IMUX),IBASE(IIPX),BASE(IXA))
         ELSE IF((ICHX.EQ.3).AND.CHEX) THEN
*           MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
            IF(ISPLH.EQ.1) THEN
               CALL SETARA(BASE,NEL,IIGAR)
               CALL TRIMWX(MAXMIX,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,SIDE,
     1         BASE(IZZ),IBASE(IKN),BASE(IQFR),IBASE(IMUX),IBASE(IIPX),
     2         IPR,BASE(IXA),BASE(IIGAR))
               CALL RLSARA(BASE(IIGAR))
            ELSE
               CALL TRIMTX(ISPLH,MAXMIX,IDIM,NEL,LL4,VOL,MAT,
     1         IBASE(IMATN),SGD,XSGD,SIDE,BASE(IZZ),IBASE(IKN),
     2         BASE(IQFR),IBASE(IMUX),IBASE(IIPX),IPR,BASE(IXA))
            ENDIF
         ENDIF
      ENDIF
      IF(.NOT.DIAG) THEN
         CALL LCMPOF(IPSYS,'X '//TEXT10,IIMAX,2,IXA)
         IF(ICHX.EQ.2) CALL LCMPOF(IPSYS,'XA'//TEXT10,IIMAX,2,IAX)
      ELSE
*        IN DIAGONAL SYMMETRY CASE, DO NOT SAVE THE X-DIRECTED ADI
*        MATRIX COMPONENT SINCE IT IS EQUAL TO THE Y-DIRECTED COMPONENT
         CALL RLSARA(BASE(IXA))
         IF(ICHX.EQ.2) CALL RLSARA(BASE(IAX))
      ENDIF
      IF(ICHX.EQ.2) CALL LCMPOF(IPSYS,'TF'//TEXT10,LL4F,2,ITF)
*
* DIMENSION Y
      IF(IDIM.GT.1) THEN
         IIMAY=IBASE(IMUY+LL4Y-1)
         IF(ICHX.EQ.2) THEN
*           THOMAS-RAVIART ADI ITERATIVE METHOD.
            IF(IPR.EQ.3) THEN
               IF(ISEG.GT.0) CALL MTBLD('YA'//TEXT10,IPTRK,IPSYS,1)
               CALL LCMIOF(IPSYS,'YA'//TEXT10,IAY)
            ELSE
               CALL SETARA(BASE,IIMAY,IAY)
               CALL XDRSET(BASE(IAY),IIMAY,0.0)
            ENDIF
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMIOF(IPTRK,'IPBBY',IIPBY)
            CALL LCMIOF(IPTRK,'YB',IBY)
            CALL SETARA(BASE,IIMAY,IYA)
            CALL TRIDXY(MAXMIX,IELEM,ICOL,IDIM,NEL,LL4,LL4F,LL4X,
     1      LL4Y,MAT,BASE(IVOL),BASE(IYY),IBASE(IKN),BASE(IQFR),
     2      XSGD,BASE(IDSGD),IBASE(IMUY),IBASE(IIPBY),LC,BASE(IR),
     3      BASE(IV),BASE(IBY),BASE(ITF),BASE(IAY),BASE(IYA))
         ELSE
*           GENERIC ADI ITERATIVE METHOD.
            CALL LCMIOF(IPTRK,'IPY',IIPY)
            IF(IPR.EQ.3) THEN
               IF(ISEG.GT.0) CALL MTBLD('Y '//TEXT10,IPTRK,IPSYS,1)
               CALL LCMIOF(IPSYS,'Y '//TEXT10,IYA)
            ELSE
               CALL SETARA(BASE,IIMAY,IYA)
               CALL XDRSET(BASE(IYA),IIMAY,0.0)
            ENDIF
            IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
               CALL TRIPXY(MAXMIX,MAXKN,NEL,LL4,BASE(IVOL),MAT,XSGD,
     1         BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),IBASE(IKN),
     2         BASE(IQFR),IBASE(IMUY),IBASE(IIPY),CYLIND,BASE(IYA))
            ELSE IF((ICHX.EQ.3).AND.(.NOT.CHEX)) THEN
               CALL SETARA(BASE,NEL,IIGAR)
               CALL TRIMXY(MAXMIX,CYLIND,IELEM,IDIM,NEL,LL4,BASE(IVOL),
     1         MAT,SGD,XSGD,BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),
     2         IBASE(IKN),BASE(IQFR),IBASE(IMUY),IBASE(IIPY),IPR,
     3         BASE(IYA),BASE(IIGAR))
               CALL RLSARA(BASE(IIGAR))
            ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
*              MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
               CALL TRIRWY(MAXMIX,NEL,LL4,VOL,MAT,XSGD,SIDE,BASE(IZZ),
     1         IBASE(IKN),BASE(IQFR),IBASE(IMUY),IBASE(IIPY),BASE(IYA))
            ELSE IF((ICHX.EQ.3).AND.CHEX) THEN
*              MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
               IF(ISPLH.EQ.1) THEN
                  CALL SETARA(BASE,NEL,IIGAR)
                  CALL TRIMWY(MAXMIX,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,
     1            SIDE,BASE(IZZ),IBASE(IKN),BASE(IQFR),IBASE(IMUY),
     2            IBASE(IIPY),IPR,BASE(IYA),BASE(IIGAR))
                  CALL RLSARA(BASE(IIGAR))
               ELSE
                  CALL TRIMTY(ISPLH,MAXMIX,IDIM,NEL,LL4,VOL,MAT,
     1            IBASE(IMATN),SGD,XSGD,SIDE,BASE(IZZ),IBASE(IKN),
     2            BASE(IQFR),IBASE(IMUY),IBASE(IIPY),IPR,BASE(IYA))
               ENDIF
            ENDIF
         ENDIF
         CALL LCMPOF(IPSYS,'Y '//TEXT10,IIMAY,2,IYA)
         IF(ICHX.EQ.2) CALL LCMPOF(IPSYS,'YA'//TEXT10,IIMAY,2,IAY)
      ENDIF
*
* DIMENSION Z
      IF(IDIM.GT.2) THEN
         IIMAZ=IBASE(IMUZ+LL4Z-1)
         IF(ICHX.EQ.2) THEN
*           THOMAS-RAVIART ADI ITERATIVE METHOD.
            IF(IPR.EQ.3) THEN
               IF(ISEG.GT.0) CALL MTBLD('ZA'//TEXT10,IPTRK,IPSYS,1)
               CALL LCMIOF(IPSYS,'ZA'//TEXT10,IAZ)
            ELSE
               CALL SETARA(BASE,IIMAZ,IAZ)
               CALL XDRSET(BASE(IAZ),IIMAZ,0.0)
            ENDIF
            CALL LCMIOF(IPSYS,'TF'//TEXT10,ITF)
            CALL LCMIOF(IPTRK,'IPBBZ',IIPBZ)
            CALL LCMIOF(IPTRK,'ZB',IBZ)
            CALL SETARA(BASE,IIMAZ,IZA)
            CALL TRIDXZ(MAXMIX,IELEM,ICOL,IDIM,NEL,LL4,LL4F,LL4X,
     1      LL4Y,LL4Z,MAT,BASE(IVOL),BASE(IZZ),IBASE(IKN),BASE(IQFR),
     2      XSGD,BASE(IDSGD),IBASE(IMUZ),IBASE(IIPBZ),LC,BASE(IR),
     3      BASE(IV),BASE(IBZ),BASE(ITF),BASE(IAZ),BASE(IZA))
         ELSE
            CALL LCMIOF(IPTRK,'IPZ',IIPZ)
            IF(IPR.EQ.3) THEN
               IF(ISEG.GT.0) CALL MTBLD('Z '//TEXT10,IPTRK,IPSYS,1)
               CALL LCMIOF(IPSYS,'Z '//TEXT10,IZA)
            ELSE
               CALL SETARA(BASE,IIMAZ,IZA)
               CALL XDRSET(BASE(IZA),IIMAZ,0.0)
            ENDIF
            IF((ICHX.EQ.1).AND.(.NOT.CHEX)) THEN
               CALL TRIPXZ(MAXMIX,MAXKN,NEL,LL4,BASE(IVOL),MAT,XSGD,
     1         BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),IBASE(IKN),
     2         BASE(IQFR),IBASE(IMUZ),IBASE(IIPZ),CYLIND,BASE(IZA))
            ELSE IF((ICHX.EQ.3).AND.(.NOT.CHEX)) THEN
               CALL SETARA(BASE,NEL,IIGAR)
               CALL TRIMXZ(MAXMIX,CYLIND,IELEM,IDIM,NEL,LL4,BASE(IVOL),
     1         MAT,SGD,XSGD,BASE(IXX),BASE(IYY),BASE(IZZ),BASE(IDD),
     2         IBASE(IKN),BASE(IQFR),IBASE(IMUZ),IBASE(IIPZ),IPR,
     3         BASE(IZA),BASE(IIGAR))
               CALL RLSARA(BASE(IIGAR))
            ELSE IF((ICHX.EQ.1).AND.CHEX) THEN
*              MESH CORNER FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
               CALL TRIRWZ(MAXMIX,NEL,LL4,VOL,MAT,XSGD,SIDE,BASE(IZZ),
     1         IBASE(IKN),BASE(IQFR),IBASE(IMUZ),IBASE(IIPZ),BASE(IZA))
            ELSE IF((ICHX.EQ.3).AND.CHEX) THEN
*              MESH CENTERED FINITE DIFFERENCES IN HEXAGONAL GEOMETRY.
               IF(ISPLH.EQ.1) THEN
                  CALL SETARA(BASE,NEL,IIGAR)
                  CALL TRIMWZ(MAXMIX,IDIM,NEL,LL4,VOL,MAT,SGD,XSGD,
     1            SIDE,BASE(IZZ),IBASE(IKN),BASE(IQFR),IBASE(IMUZ),
     2            IBASE(IIPZ),IPR,BASE(IZA),BASE(IIGAR))
                  CALL RLSARA(BASE(IIGAR))
               ELSE
                  CALL TRIMTZ(ISPLH,MAXMIX,IDIM,NEL,LL4,VOL,MAT,
     1            IBASE(IMATN),SGD,XSGD,SIDE,BASE(IZZ),IBASE(IKN),
     2            BASE(IQFR),IBASE(IMUZ),IBASE(IIPZ),IPR,BASE(IZA))
               ENDIF
            ENDIF
         ENDIF
         CALL LCMPOF(IPSYS,'Z '//TEXT10,IIMAZ,2,IZA)
         IF(ICHX.EQ.2) CALL LCMPOF(IPSYS,'ZA'//TEXT10,IIMAZ,2,IAZ)
      ENDIF
      CALL RLSARA(BASE(IVOL))
      IF(ICHX.EQ.2) CALL RLSARA(BASE(IDSGD))
      IF((ICHX.EQ.3).AND.(ISPLH.GT.1).AND.CHEX) THEN
         CALL RLSARA(IBASE(IMATN))
      ENDIF
*----
*  CHECK FOR MATRIX CONSISTENCY
*----
      IF(ICHX.NE.2) CALL TRICHK (TEXT10,IPTRK,IPSYS,IDIM,DIAG,CHEX,
     1 IPR,LL4)
      CALL KDRCPU(TK2)
      IF(IMPX.GT.1) WRITE(6,'(/35H TRIASM: CPU TIME FOR SYSTEM MATRIX,
     1 11H ASSEMBLY =,F9.2,3H S.)') TK2-TK1
*----
*  PERFORM SUPERVECTORIZATION REBUILD OF THE COEFFICIENT MATRICES
*----
      IF(ISEG.GT.0) THEN
         IF(CHEX) CALL MTBLD('W '//TEXT10,IPTRK,IPSYS,3)
         IF(.NOT.DIAG) CALL MTBLD('X '//TEXT10,IPTRK,IPSYS,3)
         IF(IDIM.GT.1) CALL MTBLD('Y '//TEXT10,IPTRK,IPSYS,3)
         IF(IDIM.GT.2) CALL MTBLD('Z '//TEXT10,IPTRK,IPSYS,3)
         IF(ICHX.EQ.2) THEN
            IF(CHEX) CALL MTBLD('WA'//TEXT10,IPTRK,IPSYS,3)
            IF(.NOT.DIAG) CALL MTBLD('XA'//TEXT10,IPTRK,IPSYS,3)
            IF(IDIM.GT.1) CALL MTBLD('YA'//TEXT10,IPTRK,IPSYS,3)
            IF(IDIM.GT.2) CALL MTBLD('ZA'//TEXT10,IPTRK,IPSYS,3)
         ENDIF
      ENDIF
*----
*  MATRIX FACTORIZATIONS
*----
      IF(IPR.EQ.0) THEN
         CALL KDRCPU(TK1)
         CALL MTLDLF(TEXT10,IPTRK,IPSYS,ITY,IMPX)
         CALL KDRCPU(TK2)
         IF(IMPX.GT.1) WRITE(6,'(/34H TRIASM: CPU TIME FOR LDLT FACTORI,
     1   18HZATION OF MATRIX '',A10,2H''=,F9.2,3H S.)') TEXT10,TK2-TK1
      ENDIF
*----
*  RELEASE UNIT MATRICES
*----
      IF((ICHX.EQ.1).OR.(ICHX.EQ.2)) THEN
         CALL RLSARA(BASE(IQT))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IQH))
         CALL RLSARA(BASE(IRH))
         CALL RLSARA(BASE(IV))
         CALL RLSARA(BASE(IQS))
         CALL RLSARA(BASE(IQ))
         CALL RLSARA(BASE(IRS))
         CALL RLSARA(BASE(IR))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
      ENDIF
*----
*  RELEASE TRIVAC SPECIFIC TRACKING INFORMATION
*----
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IZZ))
      IF(CHEX) THEN
         CALL RLSARA(IBASE(IMUW))
      ELSE
         CALL RLSARA(BASE(IDD))
         CALL RLSARA(BASE(IYY))
         CALL RLSARA(BASE(IXX))
      ENDIF
      IF(IDIM.GE.2) CALL RLSARA(IBASE(IMUY))
      IF(.NOT.DIAG) CALL RLSARA(IBASE(IMUX))
      IF(IDIM.EQ.3) CALL RLSARA(IBASE(IMUZ))
      RETURN
      END
