*DECK TRISPS
      SUBROUTINE TRISPS (IPTRK,IPMACR,IPMACP,IPSYS,IMPX,NGRP,NEL,NUN,
     1 NLF,NANI,NDG,IPR,MAT,VOL,IDL,TITLE,NBMIX,SGD,DSGD,WORK,IJJ,NJJ,
     2 IPOS,IND,SCAT,SCATI)
*
*-----------------------------------------------------------------------
*
*Purpose:
* recover the cross-section data in LCM object with pointer IPMACR,
* compute and store the corresponding trivac system matrices for a
* simplified PN approximation (or a perturbation to the system
* matrices).
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPTRK   L_TRACK pointer to the TRIVAC tracking information.
* IPMACR  L_MACROLIB pointer to the unperturbed cross sections.
* IPMACP  L_MACROLIB pointer to the perturbed cross sections if
*         IPR.gt.0. Equal to IPMACR if IPR=0.
* IPSYS   L_SYSTEM pointer to system matrices.
* IMPX    print parameter (equal to zero for no print).
* NGRP    number of energy groups.
* NEL     total number of finite elements.
* NUN     total number of unknowns per group.
* NLF     number of Legendre orders for the flux (even number).
* NANI    number of Legendre orders for the scattering cross sections.
* NDG     number of delayed neutron groups.
* IPR     type of assembly:
*         =0: calculation of the system matrices;
*         =1: calculation of the derivative of these matrices;
*         =2: calculation of the first variation of these matrices;
*         =3: identical to IPR=2, but these variation are added to
*         unperturbed system matrices.
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* IDL     position of the flux component associated with each volume.
* TITLE   title.
* NBMIX   total number of material mixtures in the macrolib.
*
*Parameters: scratch
* SGD
* DSGD
* WORK
* IJJ
* NJJ
* IPOS
* IND
* SCAT
* SCATI
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPTRK,IPMACR,IPMACP,IPSYS,IMPX,NGRP,NEL,NUN,NLF,NANI,
     1 NDG,IPR,MAT(NEL),IDL(NEL),NBMIX,IJJ(NBMIX),NJJ(NBMIX),
     2 IPOS(NBMIX),IND(NGRP)
      REAL VOL(NEL),SGD(NBMIX,2*(NANI+1)),DSGD(NBMIX,2*(NANI+1)),
     1 WORK(NBMIX*NGRP),SCAT(NGRP,NGRP,NBMIX),SCATI(NGRP,NGRP,NBMIX)
      CHARACTER TITLE*72
*----
*  LOCAL VARIABLES
*----
      CHARACTER TEXDIG*12,TEXT12*12,CM*2
      LOGICAL LFIS
*----
*  PROCESS MACROLIB INFORMATION FOR VARIOUS LEGENDRE ORDERS.
*----
      IF(NLF.EQ.0) CALL XABORT('TRISPS: SPN APPROXIMATION REQUESTED.')
      JPMACR=LCMGID(IPMACR,'GROUP')
      JPMACP=LCMGID(IPMACP,'GROUP')
      JPSYS=LCMLID(IPSYS,'REMOVALXS',NGRP)
      DO 110 IL=1,NANI+1
      WRITE(CM,'(I2.2)') IL-1
      CALL XDRSET(SCAT,NGRP*NGRP*NBMIX,0.0)
      DO 50 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      KPMACR=LCMGIL(JPMACR,IGR)
      KPMACP=LCMGIL(JPMACP,IGR)
      CALL XDRSET(SGD(1,1),NBMIX,0.0)
      IF(IL.EQ.1) THEN
         CALL LCMLEN(KPMACR,'NTOT0',LENGT,ITYLCM)
         IF(LENGT.EQ.0) THEN
            CALL XABORT('TRISPS: NO TOTAL CROSS SECTIONS.')
         ELSE IF(LENGT.GT.NBMIX) THEN
            CALL XABORT('TRISPS: INVALID LENGTH FOR TOTAL CROSS SECTIO'
     1      //'NS.')
         ENDIF
         CALL LCMGET(KPMACR,'NTOT0',SGD(1,2))
         CALL LCMLEN(KPMACR,'SIGW00',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            IF(LENGT.GT.NBMIX) CALL XABORT('TRISPS: INVALID LENGTH FOR'
     1      //' SIGW00 CROSS SECTIONS.')
            CALL LCMGET(KPMACR,'SIGW00',SGD(1,1))
         ENDIF
      ELSE
         CALL LCMLEN(KPMACR,'NTOT1',LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            IF(LENGT.GT.NBMIX) CALL XABORT('TRISPS: INVALID LENGTH FOR'
     1      //' TOTAL-P1 CROSS SECTIONS.')
            CALL LCMGET(KPMACR,'NTOT1',SGD(1,2))
         ELSE
            CALL LCMGET(KPMACR,'NTOT0',SGD(1,2))
         ENDIF
         CALL LCMLEN(KPMACR,'SIGW'//CM,LENGT,ITYLCM)
         IF(LENGT.GT.0) THEN
            IF(LENGT.GT.NBMIX) CALL XABORT('TRISPS: INVALID LENGTH FOR'
     1      //' SIGW'//CM//' CROSS SECTIONS.')
            CALL LCMGET(KPMACR,'SIGW'//CM,SGD(1,1))
         ENDIF
      ENDIF
      DO 10 IBM=1,NBMIX
   10 SCAT(IGR,IGR,IBM)=SGD(IBM,2)-SGD(IBM,1)
      CALL LCMLEN(KPMACR,'NJJS'//CM,LENGT,ITYLCM)
      IF(LENGT.GT.NBMIX) CALL XABORT('TRISPS: INVALID LENGTH FOR NJJS'
     1 //CM//' INFORMATION.')
      IF(LENGT.GT.0) THEN
         CALL LCMGET(KPMACR,'NJJS'//CM,NJJ)
         CALL LCMGET(KPMACR,'IJJS'//CM,IJJ)
         IGMIN=IGR
         IGMAX=IGR
         DO 20 IBM=1,NBMIX
         IGMIN=MIN(IGMIN,IJJ(IBM)-NJJ(IBM)+1)
   20    IGMAX=MAX(IGMAX,IJJ(IBM))
         CALL LCMGET(KPMACR,'IPOS'//CM,IPOS)
         CALL LCMGET(KPMACR,'SCAT'//CM,WORK)
         DO 40 JGR=IGMAX,IGMIN,-1
         IF(JGR.EQ.IGR) GO TO 40
         DO 30 IBM=1,NBMIX
         IF((JGR.GT.IJJ(IBM)-NJJ(IBM)).AND.(JGR.LE.IJJ(IBM))) THEN
            SCAT(IGR,JGR,IBM)=-WORK(IPOS(IBM)+IJJ(IBM)-JGR)
         ENDIF
   30    CONTINUE
   40    CONTINUE
      ENDIF
   50 CONTINUE
*----
*  INVERSION OF THE REMOVAL MATRIX FOR CASES WITH IELEM > 1.
*----
      DO 70 IBM=1,NBMIX
      DO 60 JGR=1,NGRP
      DO 60 IGR=1,NGRP
   60 SCATI(IGR,JGR,IBM)=SCAT(IGR,JGR,IBM)
      CALL ALINV(NGRP,SCATI(1,1,IBM),NGRP,IER,IND)
      IF(IER.NE.0) CALL XABORT('TRISPS: SINGULAR MATRIX.')
   70 CONTINUE
*
      DO 110 IGR=1,NGRP
      KPSYS=LCMDIL(JPSYS,IGR)
      IPOSDE=0
      DO 90 IBM=1,NBMIX
      IPOS(IBM)=IPOSDE+1
      IGMIN=IGR
      IGMAX=IGR
      DO 80 JGR=NGRP,1,-1
      IF((SCAT(IGR,JGR,IBM).NE.0.0).OR.(SCATI(IGR,JGR,IBM).NE.0.0)) THEN
         IGMIN=MIN(IGMIN,JGR)
         IGMAX=MAX(IGMAX,JGR)
      ENDIF
   80 CONTINUE
      IJJ(IBM)=IGMAX
      NJJ(IBM)=IGMAX-IGMIN+1
      DO 90 JGR=IGMAX,IGMIN,-1
      IPOSDE=IPOSDE+1
   90 WORK(IPOSDE)=SCAT(IGR,JGR,IBM)
      CALL LCMPUT(KPSYS,'NJJS'//CM,NBMIX,1,NJJ)
      CALL LCMPUT(KPSYS,'IJJS'//CM,NBMIX,1,IJJ)
      CALL LCMPUT(KPSYS,'IPOS'//CM,NBMIX,1,IPOS)
      CALL LCMPUT(KPSYS,'SCAT'//CM,IPOSDE,2,WORK)
      IPOSDE=0
      DO 100 IBM=1,NBMIX
      DO 100 JGR=IJJ(IBM),IJJ(IBM)-NJJ(IBM)+1,-1
      IPOSDE=IPOSDE+1
  100 WORK(IPOSDE)=SCATI(IGR,JGR,IBM)
      CALL LCMPUT(KPSYS,'SCATI'//CM,IPOSDE,2,WORK)
  110 CONTINUE
*----
*  COMPUTE AND FACTORIZE THE DIAGONAL SYSTEM MATRICES.
*----
      DO 160 IGR=1,NGRP
      KPSYS=LCMDIL(JPSYS,IGR)
      DO 140 IL=1,NANI+1
      WRITE(CM,'(I2.2)') IL-1
      CALL LCMGET(KPSYS,'IJJS'//CM,IJJ)
      CALL LCMGET(KPSYS,'IPOS'//CM,IPOS)
      CALL LCMGET(KPSYS,'SCAT'//CM,WORK)
      DO 130 IBM=1,NBMIX
  130 SGD(IBM,IL)=WORK(IPOS(IBM)+IJJ(IBM)-IGR)
      CALL LCMGET(KPSYS,'SCATI'//CM,WORK)
      DO 140 IBM=1,NBMIX
  140 SGD(IBM,NANI+1+IL)=WORK(IPOS(IBM)+IJJ(IBM)-IGR)
      WRITE(TEXT12,'(1HA,2I3.3)') IGR,IGR
      IF(IPR.EQ.0) THEN
*        COMPUTE UNPERTURBED SYSTEM MATRICES.
         CALL TRIASN(TEXT12,IPTRK,IPSYS,IMPX,NBMIX,NEL,NANI+1,NUN,0,
     1   MAT,VOL,IDL,SGD(1,1),SGD(1,2+NANI),TITLE)
      ELSE
*        COMPUTE A PERTURBATION TO THE SYSTEM MATRICES
         CALL XABORT('TRISPS: PERTURBATION NOT IMPLEMENTED.')
      ENDIF
*----
*  PUT A FLAG IN IPSYS TO IDENTIFY NON-ZERO SCATTERING TERMS.
*----
      DO 160 IL=1,NANI+1
      WRITE(CM,'(I2.2)') IL-1
      CALL LCMGET(KPSYS,'NJJS'//CM,NJJ)
      CALL LCMGET(KPSYS,'IJJS'//CM,IJJ)
      DO 160 IBM=1,NBMIX
      DO 160 JGR=IJJ(IBM),IJJ(IBM)-NJJ(IBM)+1,-1
      IF(JGR.NE.IGR) THEN
         WRITE (TEXT12,'(1HA,2I3.3)') IGR,JGR
         CALL LCMPUT(IPSYS,TEXT12,1,2,0.0)
      ENDIF
  160 CONTINUE
*----
*  PROCESS FISSION SPECTRUM TERMS
*----
      KPMACP=LCMGIL(JPMACP,1)
      CALL LCMLEN(KPMACP,'CHI',LENGT,ITYLCM)
      IF(LENGT.GT.NBMIX) CALL XABORT('TRISPS: INVALID LENGTH FOR CHI'
     1 //' INFORMATION.')
      IF(LENGT.GT.0) THEN
         DO 180 IGR=1,NGRP
         KPMACP=LCMGIL(JPMACP,IGR)
         CALL LCMGET(KPMACP,'CHI',WORK((IGR-1)*NBMIX+1))
  180    CONTINUE
      ELSE
         DO 190 IBM=1,NBMIX
         WORK(IBM)=1.0
         DO 190 IGR=2,NGRP
  190    WORK((IGR-1)*NBMIX+IBM)=0.0
      ENDIF
*----
*  PROCESS FISSION NUSIGF TERMS
*----
      DO 230 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      LFIS=.FALSE.
      DO 200 IBM=1,NBMIX
  200 LFIS=LFIS.OR.(WORK((IGR-1)*NBMIX+IBM).NE.0.0)
      IF(LFIS) THEN
         DO 220 JGR=1,NGRP
         KPMACP=LCMGIL(JPMACP,JGR)
         CALL LCMLEN(KPMACP,'NUSIGF',LENGT,ITYLCM)
         IF(LENGT.GT.NBMIX) CALL XABORT('TRISPS: INVALID LENGTH FOR '
     1    //'NUSIGF INFORMATION.')
         IF(LENGT.GT.0) THEN
            CALL LCMGET(KPMACP,'NUSIGF',SGD)
            DO 210 IBM=1,LENGT
  210       SGD(IBM,1)=SGD(IBM,1)*WORK((IGR-1)*NBMIX+IBM)
            WRITE (TEXDIG,'(1HB,2I3.3)') IGR,JGR
            CALL TRIDIG(TEXDIG,IPTRK,IPSYS,IMPX,NBMIX,NEL,NUN,IPR,MAT,
     1      VOL,IDL,SGD)
         ENDIF
  220    CONTINUE
      ENDIF
  230 CONTINUE
*----
*  PROCESS DELAYED FISSION SPECTRUM
*----
      DO 300 IDEL=1,NDG
      WRITE(TEXT12,'(3HCHI,I2.2,2H-D)') IDEL
      KPMACP=LCMGIL(JPMACP,1)
      CALL LCMLEN(KPMACP,TEXT12,LENGT,ITYLCM)
      IF(LENGT.GT.NBMIX) CALL XABORT('TRISPS: INVALID LENGTH FOR CHDL'
     1 //' INFORMATION.')
      IF(LENGT.GT.0) THEN
         DO 240 IGR=1,NGRP
         KPMACP=LCMGIL(JPMACP,IGR)
         CALL LCMGET(KPMACP,TEXT12,WORK((IGR-1)*NBMIX+1))
  240    CONTINUE
      ELSE
         NBMIX=NBMIX
         DO 250 IBM=1,NBMIX
         WORK(IBM)=1.0
         DO 250 IGR=2,NGRP
  250    WORK((IGR-1)*NBMIX+IBM)=0.0
      ENDIF
*----
*  PROCESS FISSION NUSIGF TERMS
*----
      WRITE(TEXT12,'(6HNUSIGF,I2.2,2H-D)') IDEL
      DO 290 IGR=1,NGRP
*     PROCESS SECONDARY GROUP IGR.
      LFIS=.FALSE.
      DO 260 IBM=1,NBMIX
  260 LFIS=LFIS.OR.(WORK((IGR-1)*NBMIX+IBM).NE.0.0)
      IF(LFIS) THEN
         DO 280 JGR=1,NGRP
         KPMACP=LCMGIL(JPMACP,JGR)
         CALL LCMLEN(KPMACP,TEXT12,LENGT,ITYLCM)
         IF(LENGT.GT.NBMIX) CALL XABORT('TRISPS: INVALID LENGTH FOR '
     1    //'NUSIGF INFORMATION.')
         IF(LENGT.GT.0) THEN
            CALL LCMGET(KPMACP,TEXT12,SGD)
            DO 270 IBM=1,LENGT
  270       SGD(IBM,1)=SGD(IBM,1)*WORK((IGR-1)*NBMIX+IBM)
            WRITE (TEXDIG,'(1HB,3I3.3)') IDEL,IGR,JGR
            CALL TRIDIG(TEXDIG,IPTRK,IPSYS,IMPX,NBMIX,NEL,NUN,IPR,MAT,
     1      VOL,IDL,SGD)
         ENDIF
  280    CONTINUE
      ENDIF
  290 CONTINUE
  300 CONTINUE
      RETURN
      END
