*DECK ALCACT
      SUBROUTINE ALCACT(LCACT,NG,XG,WG)
*
*-----------------------------------------------------------------------
*
* SET THE WEIGHTS AND BASE POINTS OF HALSALL (CACTUS) AND MCDANIEL.
*
* INPUT PARAMETERS:
*  LCACT  : =1,2: VALUES USED BY HALSALL IN CACTUS (1980);
*           =3: VALUES PROPOSED BY MCDANIEL NG=2 ONLY
*              (EXTENDED TO 3 AND 4 WITH THE SAME APPROACH
*              (R. LE TELLIER 06/2005)
*  NG     : NUMBER OF WEIGHTS/BASE POINTS.
*
* OUTPUT PARAMETERS:
*  XG     : BASE POINTS. DIMENSION XG(NG)
*  WG     : WEIGHTS. DIMENSION WG(NG)
*
* REFERENCE:
*  A. LEONARD and C.T. MC DANIEL, "OPTIMAL POLAR ANGLES AND WEIGHTS FOR
*  THE CHARACTERISTIC METHOD", TRANS. AM. NUCL. SOC., 73, 172 (1995).
*
*-------------------------------------------------- R. ROY (1999) ------
*
      IMPLICIT NONE
      INTEGER  LCACT, NG, IG
      REAL     XG(NG), WG(NG)
      DOUBLE PRECISION PI, PHIM, PHIP, DELPHI, WTEST, XTEST, YTEST,
     >                 ZTEST, ZERO, ONE, HALF, QUART
      PARAMETER( PI=3.14159265358979D0, ZERO=0.D0, ONE=1.D0,
     >           HALF=0.5D0, QUART=0.25D0 )
* for LCACT=3
      INTEGER I, I2, I3, I4, I5, IDEP, IFIN
      PARAMETER ( I2= 1, I3= 3, I4= 6, I5= 10)
      REAL     XMCD(I5-1), WMCD(I5-1)
      SAVE XMCD, WMCD
* NG=2
      DATA (XMCD(I),I=I2,I3-1) / 0.273658, 0.865714 /
      DATA (WMCD(I),I=I2,I3-1) / 0.139473, 0.860527 /
* NG=3
      DATA (XMCD(I),I=I3,I4-1) / 0.89143909158400, 0.39553366924293,
     >                           0.09981210150987 /
      DATA (WMCD(I),I=I3,I4-1) / 0.79381967942014, 0.18856045545959,
     >                           0.01761986512028 /
* NG=4
      DATA (XMCD(I),I=I4,I5-1) / 0.46416676028172, 0.90827423747595,
     >                           0.16600406601852, 0.04218057754179 /
      DATA (WMCD(I),I=I4,I5-1) / 0.21833136925738, 0.74642986712559,
     >                           0.03214120124681, 0.00309756237022 /
*
      IF( LCACT.EQ.1 )THEN
         DELPHI= ONE/DBLE(NG)
         PHIM  = ZERO
         XTEST = ZERO
         DO 10 IG= 1, NG
            PHIP  = DBLE(IG) * DELPHI
            WTEST = PHIP - PHIM
            WG(IG)= REAL( WTEST )
            YTEST = SQRT(ONE - PHIP * PHIP) * PHIP + ASIN(PHIP)
            ZTEST = HALF * (YTEST - XTEST) / WTEST
            XG(IG)= REAL( SQRT(ONE - ZTEST*ZTEST) )
            PHIM  = PHIP
            XTEST = YTEST
   10    CONTINUE
      ELSEIF( LCACT.EQ.2 )THEN
         DELPHI= PI/DBLE(2*NG)
         PHIM  = ZERO
         DO 20 IG= 1, NG
            PHIP  = DBLE(IG) * DELPHI
            WTEST = SIN(PHIP) - SIN(PHIM)
            WG(IG)= REAL( WTEST )
            XTEST = HALF * (PHIP - PHIM)
            YTEST = QUART * (SIN(PHIP+PHIP) - SIN(PHIM+PHIM))
            ZTEST = (XTEST + YTEST) / WTEST
            XG(IG)= REAL( SQRT(ONE - ZTEST*ZTEST) )
            PHIM= PHIP
   20    CONTINUE
      ELSEIF( LCACT.EQ.3 )THEN
c         IF( NG.NE.2 ) CALL XABORT('ALCACT: LCACA=3 => NG=2')
         IF( NG.EQ.2 ) THEN
            IDEP=I2
            IFIN=I3-1
         ELSE IF( NG.EQ.3 ) THEN
            IDEP=I3
            IFIN=I4-1
         ELSE IF( NG.EQ.4 ) THEN
            IDEP=I4
            IFIN=I5-1
         ELSE
            CALL XABORT('ALCACT: LCACA=3 => NG= 2, 3 OR 4')
         ENDIF
         DO 30 IG= IDEP, IFIN
            XG(IG-IDEP+1)= SQRT(ONE - XMCD(IG)*XMCD(IG))
            WG(IG-IDEP+1)= WMCD(IG)
   30    CONTINUE 
      ELSE
         CALL XABORT('ALCACT: *LCACA* IS 1, 2 OR 3') 
      ENDIF
      RETURN
      END
