*DECK ALTERI
      SUBROUTINE ALTERI(N,X,VAL0,VAL1,TERP,WK)
*
*-----------------------------------------------------------------------
*
*Purpose:
* determination of the TERP integration components using the order 4
* Ceschino method with cubic Hermite polynomials.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* N       number of points.
* X       abscissas
* VAL0    left integration limit.
* VAL1    right integration limit.
*
*Parameters: output
* TERP    integration components.
*
*Parameters: scratch
* WK
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER N
      REAL X(N),VAL0,VAL1,TERP(N),WK(3,N)
*----
*  LOCAL VARIABLES
*----
      REAL UU(2)
*
      IF(N.LE.1) CALL XABORT('ALTERI: INVALID NUMBER OF POINTS.')
      IF(VAL1.LE.VAL0) CALL XABORT('ALTERI: INVALID LIMITS.')
      IF((VAL0.LT.X(1)).OR.(VAL1.GT.X(N))) CALL XABORT('ALTERI: UNABLE'
     1 //' TO INTEGRATE.')
      IF(N.EQ.2) GO TO 90
      DO 10 I=1,N
   10 TERP(I)=0.0
*----
*  INTERVAL IDENTIFICATION.
*----
      DO 70 I0=1,N-1
      IF((VAL0.LT.X(I0+1)).AND.(VAL1.GT.X(I0))) THEN
         A=MAX(VAL0,X(I0))
         B=MIN(VAL1,X(I0+1))
         CC=0.5*(B-A)
         DX=X(I0+1)-X(I0)
         U1=(A-0.5*(X(I0)+X(I0+1)))/DX
         U2=(B-0.5*(X(I0)+X(I0+1)))/DX
         UU(1)=0.5*(-(U2-U1)*0.577350269189626+U1+U2)
         UU(2)=0.5*((U2-U1)*0.577350269189626+U1+U2)
         DO 60 JS=1,2
         H1=(3.0*(0.5-UU(JS))**2-2.0*(0.5-UU(JS))**3)*CC
         H2=((0.5-UU(JS))**2-(0.5-UU(JS))**3)*CC
         H3=(3.0*(0.5+UU(JS))**2-2.0*(0.5+UU(JS))**3)*CC
         H4=(-(0.5+UU(JS))**2+(0.5+UU(JS))**3)*CC
*----
*  LOOP OVER POINTS.
*----
         DO 60 J=1,N
         DO 20 I=1,N
   20    WK(3,I)=0.0
         WK(3,J)=1.0
         TERP(J)=TERP(J)+H1*WK(3,I0)+H3*WK(3,I0+1)
*----
*  COMPUTE THE SOURCE TERMS.
*----
         HP=1.0/(X(2)-X(1))
         WK(1,1)=HP
         WK(2,1)=HP
         YLAST=WK(3,1)
         WK(3,1)=2.0*HP*HP*(WK(3,2)-WK(3,1))
         DO 30 I=2,N-1
         HM=HP
         HP=1.0/(X(I+1)-X(I))
         WK(1,I)=2.0*(HM+HP)
         WK(2,I)=HP
         PMX=3.0*(HM*HM*(WK(3,I)-YLAST)+HP*HP*(WK(3,I+1)-WK(3,I)))
         YLAST=WK(3,I)
   30    WK(3,I)=PMX
         HM=HP
         WK(1,N)=HM
         WK(2,N)=HM
         WK(3,N)=2.0*HM*HM*(WK(3,N)-YLAST)
*----
*  FORWARD ELIMINATION.
*----
         PMX=WK(1,1)
         WK(3,1)=WK(3,1)/PMX
         DO 40 I=2,N
         GAR=WK(2,I-1)
         WK(2,I-1)=WK(2,I-1)/PMX
         PMX=WK(1,I)-GAR*WK(2,I-1)
   40    WK(3,I)=(WK(3,I)-GAR*WK(3,I-1))/PMX
*----
*  BACK SUBSTITUTION.
*----
         DO 50 I=N-1,1,-1
   50    WK(3,I)=WK(3,I)-WK(2,I)*WK(3,I+1)
   60    TERP(J)=TERP(J)+(H2*WK(3,I0)+H4*WK(3,I0+1))*DX
      ENDIF
   70 CONTINUE
      TEST=1.0
      DO 80 I=1,N
      IF(ABS(TERP(J)).LE.1.0E-7) TERP(J)=0.0
   80 TEST=TEST-TERP(I)/(VAL1-VAL0)
      IF(ABS(TEST).GT.1.0E-5) CALL XABORT('ALTERI: WRONG TERP FACTORS.')
      RETURN
*
   90 TERP(1)=(X(2)-0.5*(VAL0+VAL1))*(VAL1-VAL0)/(X(2)-X(1))
      TERP(2)=(0.5*(VAL0+VAL1)-X(1))*(VAL1-VAL0)/(X(2)-X(1))
      RETURN
      END
