*DECK ALTERP
      SUBROUTINE ALTERP(N,X,VAL,LDERIV,TERP,WK)
*
*-----------------------------------------------------------------------
*
*Purpose:
* determination of the TERP interpolation/derivation components using
* the order 4 Ceschino method with cubic Hermite polynomials.
*
*Copyright:
* Copyright (C) 2006 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* N       number of points.
* X       abscissas
* VAL     abscissa of the interpolated point.
* LDERIV  set to .true. to compute the first derivative with respect
*         to X. Set to .false. to interpolate.
*
*Parameters: output
* TERP    interpolation/derivation components.
*
*Parameters: scratch
* WK
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER N
      LOGICAL LDERIV
      REAL X(N),VAL,TERP(N),WK(3,N)
*
      IF(N.LE.1) CALL XABORT('ALTERP: INVALID NUMBER OF POINTS.')
      IF(N.EQ.2) GO TO 90
*----
*  INTERVAL IDENTIFICATION.
*----
      DO 20 I=1,N-1
      IF((VAL.GE.X(I)).AND.(VAL.LE.X(I+1))) THEN
         I0=I
         GO TO 30
      ENDIF
   20 CONTINUE
      CALL XABORT('ALTERP: UNABLE TO INTERPOLATE.')
   30 DX=X(I0+1)-X(I0)
      U=(VAL-0.5*(X(I0)+X(I0+1)))/DX
      IF(LDERIV) THEN
         H1=(-6.0*(0.5-U)+6.0*(0.5-U)**2)/DX
         H2=(-2.0*(0.5-U)+3.0*(0.5-U)**2)/DX
         H3=(6.0*(0.5+U)-6.0*(0.5+U)**2)/DX
         H4=(-2.0*(0.5+U)+3.0*(0.5+U)**2)/DX
         TEST=0.0
      ELSE
         H1=3.0*(0.5-U)**2-2.0*(0.5-U)**3
         H2=(0.5-U)**2-(0.5-U)**3
         H3=3.0*(0.5+U)**2-2.0*(0.5+U)**3
         H4=-(0.5+U)**2+(0.5+U)**3
         TEST=1.0
      ENDIF
*----
*  LOOP OVER POINTS.
*----
      DO 80 J=1,N
      DO 40 I=1,N
   40 WK(3,I)=0.0
      WK(3,J)=1.0
      TERP(J)=H1*WK(3,I0)+H3*WK(3,I0+1)
*----
*  COMPUTE THE SOURCE TERMS.
*----
      HP=1.0/(X(2)-X(1))
      WK(1,1)=HP
      WK(2,1)=HP
      YLAST=WK(3,1)
      WK(3,1)=2.0*HP*HP*(WK(3,2)-WK(3,1))
      DO 50 I=2,N-1
      HM=HP
      HP=1.0/(X(I+1)-X(I))
      WK(1,I)=2.0*(HM+HP)
      WK(2,I)=HP
      PMX=3.0*(HM*HM*(WK(3,I)-YLAST)+HP*HP*(WK(3,I+1)-WK(3,I)))
      YLAST=WK(3,I)
   50 WK(3,I)=PMX
      HM=HP
      WK(1,N)=HM
      WK(2,N)=HM
      WK(3,N)=2.0*HM*HM*(WK(3,N)-YLAST)
*----
*  FORWARD ELIMINATION.
*----
      PMX=WK(1,1)
      WK(3,1)=WK(3,1)/PMX
      DO 60 I=2,N
      GAR=WK(2,I-1)
      WK(2,I-1)=WK(2,I-1)/PMX
      PMX=WK(1,I)-GAR*WK(2,I-1)
   60 WK(3,I)=(WK(3,I)-GAR*WK(3,I-1))/PMX
*----
*  BACK SUBSTITUTION.
*----
      DO 70 I=N-1,1,-1
   70 WK(3,I)=WK(3,I)-WK(2,I)*WK(3,I+1)
      TERP(J)=TERP(J)+(H2*WK(3,I0)+H4*WK(3,I0+1))*DX
      IF(ABS(TERP(J)).LE.1.0E-7) TERP(J)=0.0
   80 TEST=TEST-TERP(J)
      IF(ABS(TEST).GT.1.0E-5) CALL XABORT('ALTERP: WRONG TERP FACTORS.')
      RETURN
*
   90 IF(LDERIV) THEN
         TERP(1)=-1.0/(X(2)-X(1))
         TERP(2)=1.0/(X(2)-X(1))
      ELSE
         TERP(1)=(X(2)-VAL)/(X(2)-X(1))
         TERP(2)=1.0-TERP(1)
      ENDIF
      RETURN
      END
